@@ -1,7 +1,7 @@
 ###  Fit a general nonlinear regression model with correlated and/or
 ###  heteroscedastic errors
 ###
-### Copyright 2007-2017  The R Core team
+### Copyright 2007-2023  The R Core team
 ### Copyright 1997-2003  Jose C. Pinheiro,
 ###                      Douglas M. Bates <bates@stat.wisc.edu>
 #
@@ -19,8 +19,7 @@
 #  http://www.r-project.org/Licenses/
 #
 
-gnls <-
-  function(model,
+gnls <- function(model,
 	   data = sys.frame(sys.parent()),
 	   params,
 	   start,
@@ -68,6 +67,8 @@ gnls <-
     stop("'object' must be a formula")
   if (length(form)!=3)
     stop("object formula must be of the form \"resp ~ pred\"")
+  ## if (length(attr(terms(form), "offset")))
+  ##   stop("offset() terms are not supported")
 
   ##
   ## checking if self-starting formula is given
@@ -81,7 +82,7 @@ gnls <-
       if (is.null(dim(data))) {
           stop("'data' must be given explicitly to use 'nls' to get initial estimates")
       }
-      start <- coef(eval(nlsCall))
+      start <- coef(eval.parent(nlsCall))
     } else {
       stop("no initial values for model parameters")
     }
@@ -93,29 +94,27 @@ gnls <-
   ##
   ## save writing list(...) when only one element
   ##
-
   if (missing(params)) {
     if (is.null(pNams <- names(start))) {
       stop("starting estimates must have names when 'params' is missing")
     }
-    params <- eval(parse(text = paste(paste(pNams, collapse = "+"), "1",
-                           sep = "~")))
+    params <- list(formula(paste(paste(pNams, collapse = "+"), "1", sep = "~")))
   }
-  if (!is.list(params)) {
+  else if (!is.list(params))
     params <- list(params)
-  }
-  val <- NULL
-  for(i in seq_along(params)) {
-    if (is.name(params[[i]][[2]])) {
-      val <- c(val, list(params[[i]]))
+
+  params <- unlist(lapply(params, function(pp) {
+    if (is.name(pp[[2]])) {
+      list(pp)
     } else {
       ## multiple parameters on left hand side
-      val <- c(val, eval(parse(text = paste("list(",
-           paste(paste(all.vars(params[[i]][[2]]), deparse(params[[i]][[3]]),
-                       sep = "~"), collapse=","),")"))))
+      eval(parse(text = paste("list(",
+           paste(paste(all.vars(pp[[2]]), deparse(pp[[3]]), sep = "~"),
+                 collapse = ","),
+           ")")))
     }
-  }
-  params <- as.list(val)
+  }), recursive=FALSE)
+
   pnames <- character(length(params))
   for (i in seq_along(params)) {
     this <- eval(params[[i]])
@@ -129,18 +128,6 @@ gnls <-
   }
   names(params) <- pnames
 
-  ##
-  ##  If data is a pframe, copy the parameters in the frame to frame 1
-  ##  Doesn't exist in R
-##  if (inherits(data, "pframe")) {
-##    pp <- parameters(data)
-##    for (i in names(pp)) {
-##      assign(i, pp[[i]])
-##    }
-##    attr(data,"parameters") <- NULL
-##    class(data) <- "data.frame"
-##  }
-
   ## check if correlation is present and has groups
   groups <- if (!is.null(correlation)) getGroupsFormula(correlation) # else NULL
 #  if (!is.null(correlation)) {
@@ -188,7 +175,7 @@ gnls <-
     ## sort the model.frame by groups and get the matrices and parameters
     ## used in the estimation procedures
     ## always use innermost level of grouping
-    groups <- eval(parse(text = paste("~1", deparse(groups[[2]]), sep = "|")))
+    groups <- eval(substitute( ~1 | GRP, list(GRP = groups[[2]])))
     grps <- getGroups(dataMod, groups,
                       level = length(getGroupsFormula(groups, asList = TRUE)))
     ## ordering data by groups
@@ -208,13 +195,12 @@ gnls <-
 
   dataModShrunk <- dataMod[naPat, , drop=FALSE]
   yShrunk <- eval(form[[2]], dataModShrunk)
-  if (!is.null(groups)) {
-##    ordShrunk <- ord[naPat]
-    grpShrunk <- grps[naPat]
-    revOrderShrunk <- match(origOrderShrunk, row.names(dataModShrunk))
-  } else {
-    grpShrunk <- NULL
-  }
+  grpShrunk <-
+    if (!is.null(groups)) {
+      ## ordShrunk <- ord[naPat]
+      revOrderShrunk <- match(origOrderShrunk, row.names(dataModShrunk))
+      grps[naPat]
+    } # else NULL
 
   ##
   ## defining list with parameter information
@@ -223,14 +209,21 @@ gnls <-
   plist <- vector("list", length(pnames))
   names(plist) <- pnames
   for (nm in pnames) {
-    if (deparse(params[[nm]][[3]]) != "1") {
-      form1s <- asOneSidedFormula(params[[nm]][[3]])
-      plist[[nm]] <- model.matrix(form1s, model.frame(form1s, dataModShrunk))
-      auxContr <- attr(plist[[nm]], "contrasts")
-      contr <- c(contr, auxContr[is.na(match(names(auxContr), names(contr)))])
-    } else
-      plist[[nm]] <- TRUE
+   rhs <- params[[nm]][[3]]
+   plist[[nm]] <-
+    if(identical(rhs, 1) || identical(rhs, 1L)) ## constant RHS
+      TRUE
+    else {
+      form1s <- asOneSidedFormula(rhs)
+      .X <- model.frame(form1s, dataModShrunk)
+      ## keeping the contrast matrices for later use in predict
+      auxContr <- lapply(.X, function(el) if (is.factor(el)) contrasts(el))
+      contr <- c(contr, auxContr[!vapply(auxContr, is.null, NA) &
+                                 is.na(match(names(auxContr), names(contr)))])
+      model.matrix(form1s, .X)
+    }
   }
+  
   ##
   ## Params effects names
   ##
@@ -264,7 +257,10 @@ gnls <-
   }
   pLen <- length(pn)
   if (length(start) != pLen)
-    stop ("starting values for parameters are not of the correct length")
+    stop(sprintf(ngettext(length(start),
+                          "supplied %d starting value, need %d",
+                          "supplied %d starting values, need %d"),
+                 length(start), pLen), domain = NA)
   spar <- start
   names(spar) <- pn
   NReal <- sum(naPat)
@@ -471,9 +467,9 @@ gnls <-
     lsig <- log(sigma) + 0.5 * log(1 - pLen/NReal)
     loglik <- ( - NReal * (1 + log(2 * pi) + 2 * lsig))/2 + c.L$logLik
   } else {
-    loglik <- - (NReal * (log(2 * pi)/2 + log(sigma)) +
-                 sum((c.L$Xy[, pLen + 1])^2) / (2 * sigma^2)) + c.L$logLik
     lsig <- log(sigma)
+    loglik <- - (NReal * (log(2 * pi)/2 + lsig) +
+                 sum((c.L$Xy[, pLen + 1])^2) / (2 * sigma^2)) + c.L$logLik
   }
   ## ####
   varBeta <- qr(c.L$Xy[ , 1:pLen, drop = FALSE])
@@ -539,11 +535,13 @@ gnls <-
 		 plist = plist,
                  pmap = pmap,
                  parAssign = parAssign,
+                 formula = form,
                  na.action = attr(dataMod, "na.action")),
 	    ## saving labels and units for plots
 	    units = if(grpDta) attr(data, "units"),
 	    labels= if(grpDta) attr(data, "labels"))
-}
+} ## end{gnls}
+      
 
 ### Auxiliary functions used internally in gls and its methods
 
@@ -638,7 +636,7 @@ getParsGnls <- function(plist, pmap, beta, N)
 
 coef.gnls <- function(object, ...) object$coefficients
 
-formula.gnls <- function(x, ...) eval(x$call[["model"]])
+formula.gnls <- function(x, ...) x$formula %||% eval(x$call[["model"]])
 
 getData.gnls <-
   function(object)
@@ -697,76 +695,52 @@ predict.gnls <-
   }
   newdata <- data.frame(newdata, check.names = FALSE)
   mCall <- object$call
-
-  mfArgs <- list(formula =
+  
+  ## use xlev to make sure factor levels are the same as in contrasts
+  ## and to support character-type 'newdata' for factors
+  contr <- object$contrasts
+  dataMod <- model.frame(formula =
                    asOneFormula(formula(object),
                                 mCall$params, naPattern,
                                 omit = c(names(object$plist), "pi",
                                          deparse(getResponseFormula(object)[[2]]))),
                  data = newdata, na.action = na.action,
-                 drop.unused.levels = TRUE)
-  dataMod <- do.call("model.frame", mfArgs)
+                 drop.unused.levels = TRUE,
+                 xlev = lapply(contr, rownames))
 
-  ## making sure factor levels are the same as in contrasts
-  contr <- object$contrasts
-  for(i in names(dataMod)) {
-    if (inherits(dataMod[,i], "factor") &&
-        !is.null(contr[[i]]) && is.matrix(contr[[i]]) ) {
-      levs <- levels(dataMod[,i])
-      levsC <- dimnames(contr[[i]])[[1]]
-      if (any(wch <- is.na(match(levs, levsC)))) {
-          stop(sprintf(ngettext(sum(wch),
-                                "level %s not allowed for %s",
-                                "levels %s not allowed for %s"),
-                       paste(levs[wch], collapse = ",")), domain = NA)
-      }
-      attr(dataMod[,i], "contrasts") <- contr[[i]][levs, , drop = FALSE]
-#      if (length(levs) < length(levsC)) {
-#        if (inherits(dataMod[,i], "ordered")) {
-#          dataMod[,i] <- ordered(as.character(dataMod[,i]), levels = levsC)
-#        } else {
-#          dataMod[,i] <- factor(as.character(dataMod[,i]), levels = levsC)
-#        }
-#      }
-    }
-  }
   N <- nrow(dataMod)
   ##
   ## evaluating the naPattern expression, if any
   ##
-  if (is.null(naPattern)) naPat <- rep(TRUE, N)
-  else naPat <- as.logical(eval(asOneSidedFormula(naPattern)[[2]], dataMod))
-
+  naPat <- if (is.null(naPattern)) rep(TRUE, N)
+           else as.logical(eval(asOneSidedFormula(naPattern)[[2]], dataMod))
   ##
   ## Getting  the plist for the new data frame
   ##
-  ##
   plist <- object$plist
   pnames <- names(plist)
   if (is.null(params <- eval(object$call$params))) {
-    params <- eval(parse(text = paste0(paste(pnames, collapse = "+"), "~ 1")))
+    params <- list(formula(paste0(paste(pnames, collapse = "+"), "~ 1")))
   }
-  if (!is.list(params)) params <- list(params)
-### FIXME: 2 x {very ugly}: 1) adding to val incrementally //  2) eval(parse(text = .))
-  val <- NULL
-  for(i in seq_along(params)) {
-    if (is.name(params[[i]][[2]])) {
-      val <- c(val, list(params[[i]]))
+  else if (!is.list(params)) params <- list(params)
+  params <- unlist(lapply(params, function(pp) {
+    if (is.name(pp[[2]])) {
+      list(pp)
     } else {
       ## multiple parameters on left hand side
-      val <- c(val, eval(parse(text = paste("list(",
-           paste(paste(all.vars(params[[i]][[2]]), deparse(params[[i]][[3]]),
-                       sep = "~"), collapse=","),")"))))
+      eval(parse(text = paste("list(",
+           paste(paste(all.vars(pp[[2]]), deparse(pp[[3]]), sep = "~"),
+                 collapse = ","),
+           ")")))
     }
-  }
-  params <- val
+  }), recursive=FALSE)
   names(params) <- pnames
   prs <- coef(object)
 ##  pn <- names(prs)
   for(nm in pnames) {
     if (!is.logical(plist[[nm]])) {
       form1s <- asOneSidedFormula(params[[nm]][[3]])
-      plist[[nm]] <- model.matrix(form1s, model.frame(form1s, dataMod))
+      plist[[nm]] <- model.matrix(form1s, model.frame(form1s, dataMod), contr)
     }
   }
   modForm <- getCovariateFormula(object)[[2]]
