@@ -44,7 +44,7 @@ Qidentical <- function(x,y, strictClass = TRUE) {
            return(FALSE)
         ## else try further
     }
-    slts <- slotNames(x)
+    slts <- slotNames(x) ## MJ: should be slotNames(y), since is(x, class(y)) ??
     if("Dimnames" %in% slts) { ## always (or we have no 'Matrix')
 	slts <- slts[slts != "Dimnames"]
 	if(!Qidentical.DN(x@Dimnames, y@Dimnames) &&
@@ -64,6 +64,42 @@ Qidentical <- function(x,y, strictClass = TRUE) {
     TRUE
 }
 
+## MJ: It seems intuitive to allow either of is(x, class(y))
+##     and is(y, class(x)) when strictClass=FALSE ...
+.MJ.Qidentical <- function(x, y, strictClass = TRUE, skipSlots = NULL) {
+    isxy <- identical(cx <- class(x), cy <- class(y))
+    if (!isxy) {
+        if (strictClass)
+            return(FALSE)
+        isxy <- is(x, cy)
+        if (!(isxy || is(y, cx)))
+            return(FALSE)
+        ## else try further
+    }
+    slts <- slotNames(if (isxy) y else x)
+    if (length(skipSlots))
+        slts <- setdiff(slts, skipSlots)
+    if ("Dimnames" %in% slts) { ## always, or we have no "Matrix"
+	## allow symmetrization of 'Dimnames' for "symmetricMatrix" :
+	slts <- slts[slts != "Dimnames"]
+        if(!Qidentical.DN(x@Dimnames, y@Dimnames) &&
+	   !Qidentical.DN(dimnames(x), dimnames(y)))
+	    return(FALSE)
+    }
+    if ("factors" %in% slts) {
+        ## allow one empty and one non-empty 'factors' :
+        slts <- slts[slts != "factors"]
+        ## if both are not empty, they must be the same:
+        if (length(xf <- x@factors) && length(yf <- y@factors) &&
+           !identical(xf, yf))
+            return(FALSE)
+    }
+    for (slt in slts)
+        if (!identical(slot(x, slt), slot(y, slt)))
+            return(FALSE)
+    TRUE
+}
+
 ##' quasi-identical()  for traditional ('matrix') matrices
 mQidentical <- function(x,y, strictClass = TRUE) {
     if(!identical(class(x), cy <- class(y))) {
@@ -120,7 +156,7 @@ Q.eq <- function(x, y,
         if((isDense <- extends(xcl,"denseMatrix")))
             function(m) as(m, "matrix")
         else function(m)
-            as(as(as(m,"CsparseMatrix"), "dMatrix"), "dgCMatrix")
+            as(as(as(m,"CsparseMatrix"), "dMatrix"), "generalMatrix") # => "dgC"
     if(is.na(tol)) {
 	if(isDense)
 	    all(x == y | (is.na(x) & is.na(y)))
@@ -182,11 +218,12 @@ sampleL <- function(n, size) {
 ##' @param m number of columns; default (=n)  ==> square matrix
 ##' @param density the desired sparseness density:
 ##' @param nnz number of non-zero entries; default from \code{density}
-##' @param giveCsparse logical specifying if result should be CsparseMatrix
-##' @return a [TC]sparseMatrix,  n x m
-##' @author Martin Maechler, Mar 2008
+##' @param repr character string specifying the sparseness kind of the result.
+##' @param giveCsparse *deprecated* logical specifying if result should be CsparseMatrix
+##' @return a [CTR]sparseMatrix,  n x m
+##' @author Martin Maechler, Mar 2008; July 2020 ('repr' instead og 'giveCsparse')
 rspMat <- function(n, m = n, density = 1/4, nnz = round(density * n*m),
-                   giveCsparse = TRUE)
+                   repr = c("C","T","R"), giveCsparse)
 {
     stopifnot(length(n) == 1, n == as.integer(n),
               length(m) == 1, m == as.integer(m),
@@ -196,7 +233,14 @@ rspMat <- function(n, m = n, density = 1/4, nnz = round(density * n*m),
     in0 <- sampleL(N, nnz)
     x <- sparseVector(i = in0, x = as.numeric(1L + seq_along(in0)), length = N)
     dim(x) <- c(n,m)#-> sparseMatrix
-    if (giveCsparse) as(x, "CsparseMatrix") else x
+    ## silent, back compatible (not yet warning about 'giveCsparse' deprecation):
+    repr <- if(missing(repr) && !missing(giveCsparse))
+		if(giveCsparse) "C" else "T"
+	    else match.arg(repr)
+    switch(repr,
+	   "C" = as(x, "CsparseMatrix"),
+	   "T" =    x,# TsparseMatrix
+	   "R" = as(x, "RsparseMatrix"))
 }
 
 ## __DEPRECATED__ !!
@@ -242,7 +286,7 @@ rUnitTri <- function(n, upper = TRUE, ...)
 ##' @param n matrix dimension \eqn{n \times n}{n x n}
 ##' @param density ratio of number of non-zero entries to total number
 ##' @param d0 The sqrt of the diagonal entries of D default \code{10}, to be
-##' \dQuote{different} from \code{L} entries.
+##' \dQuote{different} from \code{L} entries.  More generally these can be negative
 ##' @param rcond logical indicating if \code{\link{rcond}(A, useInv=TRUE)}
 ##' should be returned which requires non-singular A and D.
 ##' @param condest logical indicating if \code{\link{condest}(A)$est}
@@ -258,12 +302,13 @@ mkLDL <- function(n, density = 1/3,
     stopifnot(n == round(n), density <= 1)
     n <- as.integer(n)
     stopifnot(n >= 1, is.numeric(d.half),
-              length(d.half) == n, d.half >= 0)
+              length(d.half) == n)# no longer (2023-05-24): d.half >= 0
     L <- Matrix(0, n,n)
     nnz <- round(density * n*n)
     L[sample(n*n, nnz)] <- seq_len(nnz)
     L <- tril(L, -1L)
     diag(L) <- 1
+### FIXME: allow  *negative* d.half[] entries!
     dh2 <- d.half^2
     non.sing <- sum(dh2 > 0) == n
     D <- Diagonal(x = dh2)
@@ -311,7 +356,7 @@ allCholesky <- function(A, verbose = FALSE, silentTry = FALSE)
     ##' @return an is(perm,LDL,super) matrix with interesting and *named* rownames
     CHM_to_pLs <- function(r) {
         is.perm <- function(.)
-            if(inherits(., "try-error")) NA else !all(.@perm == 0:(.@Dim[1]-1))
+            if(inherits(., "try-error")) NA else .@type[1L] != 0L
         is.LDL <- function(.)if(inherits(., "try-error")) NA else isLDL(.)
 	r.st <-
 	    cbind(perm	= sapply(r, is.perm),
@@ -355,7 +400,23 @@ allCholesky <- function(A, verbose = FALSE, silentTry = FALSE)
 ##' Cheap  Boolean Arithmetic Matrix product
 ##' Should be equivalent to  %&%  which is faster [not for large dense!].
 ##' Consequently mainly used in  checkMatrix()
-boolProd <- function(x,y) as((abs(x) %*% abs(y)) > 0, "nMatrix")
+## The first version (up to Aug.2022)  -- possibly what we should use for dense case (!?)
+boolProd0 <- function(x,y) as((abs(x) %*% abs(y)) > 0, "nMatrix")
+## New since  Aug.13, 2022, ensuring that zeros are dropped
+isCRT <- function(x, cl = getClass(class(x)))
+    extends(cl, "CsparseMatrix") || extends(cl, "TsparseMatrix") || extends(cl, "RsparseMatrix")
+boolProd <- function(x,y) {
+    ## treat x & y, drop0() & coercing to "n" -- this treats  NA  <==>  1  (!)
+    x <- if(isCRT(x)) .sparse2kind(x, kind="n", drop0=TRUE) else as(drop0(x), "nMatrix")
+    y <- if(isCRT(y)) .sparse2kind(y, kind="n", drop0=TRUE) else as(drop0(y), "nMatrix")
+    r <- (abs(x) %*% abs(y)) > 0
+    if(isCRT(r))
+        .sparse2kind(r, kind="n", drop0=TRUE)
+    else # also for "sparseMatrix" cases "indMatrix" (incl "pMatrix") or "diagonalMatrix"
+        ## NB: "diagonalMatrix already *does* drop0(.) when coerced to "nMatrix"
+        as(r, "nMatrix")
+}
+.sparse2kind <- Matrix:::.sparse2kind # (FIXME -- a version of this should be exported!)
 
 ###----- Checking a "Matrix" -----------------------------------------
 
@@ -386,7 +447,10 @@ checkMatrix <- function(m, m.m = if(do.matrix) as(m, "matrix"),
                         doSummary = TRUE, doCoerce = TRUE,
 			doCoerce2 = doCoerce && !isRsp, doDet = do.matrix,
 			do.prod = do.t && do.matrix && !isRsp,
-			verbose = TRUE, catFUN = cat)
+			verbose = TRUE, catFUN = cat,
+                        MSG = if(interactive() || capabilities("long.double") ||
+                                 isTRUE(get0("doExtras"))) message else function(...) {}
+                        )
 {
     ## is also called from  dotestMat()  in ../tests/Class+Meth.R
 
@@ -395,15 +459,18 @@ checkMatrix <- function(m, m.m = if(do.matrix) as(m, "matrix"),
 
     clNam <- class(m)
     cld <- getClassDef(clNam) ## extends(cld, FOO) is faster than is(m, FOO)
-    isCor    <- extends(cld, "corMatrix")
-    isSym    <- extends(cld, "symmetricMatrix")
+    isGen <- extends(cld, "generalMatrix")
+    isSym <- extends(cld, "symmetricMatrix")
+    isTri <- extends(cld, "triangularMatrix")
+    isCor <- isSym && (extends(cld, "corMatrix") || extends(cld, "pcorMatrix"))
     if(isSparse <- extends(cld, "sparseMatrix")) { # also true for these
+        isCsp  <- extends(cld, "CsparseMatrix")
 	isRsp  <- extends(cld, "RsparseMatrix")
+        isTsp  <- extends(cld, "TsparseMatrix")
 	isDiag <- extends(cld, "diagonalMatrix")
 	isInd  <- extends(cld, "indMatrix")
 	isPerm <- extends(cld, "pMatrix")
-    } else isRsp <- isDiag <- isInd <- isPerm <- FALSE
-    isTri <- !isSym && !isDiag && !isInd && extends(cld, "triangularMatrix")
+    } else isCsp <- isRsp <- isTsp <- isDiag <- isInd <- isPerm <- FALSE
     is.n     <- extends(cld, "nMatrix")
     nonMatr  <- clNam != (Mcl <- MatrixClass(clNam, cld))
 
@@ -423,6 +490,7 @@ checkMatrix <- function(m, m.m = if(do.matrix) as(m, "matrix"),
     ina <- is.na(m)
     if(do.matrix) {
 	stopifnot(all(ina == is.na(m.m)),
+                  all(is.nan(m) == is.nan(m.m)),
 		  all(is.finite(m) == is.finite(m.m)),
 		  all(is.infinite(m) == is.infinite(m.m)),
 		  all(m == m | ina), ## check all() , "==" [Compare], "|" [Logic]
@@ -443,8 +511,7 @@ checkMatrix <- function(m, m.m = if(do.matrix) as(m, "matrix"),
 	ttm <- t(tm)
         ## notInd: "pMatrix" ok, but others inheriting from "indMatrix" are not
         notInd <- (!isInd || isPerm)
-	if(notInd && (extends(cld, "CsparseMatrix") ||
-	   extends(cld, "generalMatrix") || isDiag))
+	if(notInd && (isCsp || isGen || isDiag))
             stopifnot(Qidentical(m, ttm, strictClass = !nonMatr))
 	else if(do.matrix) {
 	    if(notInd) stopifnot(nonMatr || class(ttm) == clNam)
@@ -455,8 +522,8 @@ checkMatrix <- function(m, m.m = if(do.matrix) as(m, "matrix"),
 
 	## crossprod()	%*%  etc
 	if(do.prod) {
-	    c.m <-  crossprod(m)
-	    tcm <- tcrossprod(m)
+	    c.m <-  crossprod(m, boolArith = FALSE)
+	    tcm <- tcrossprod(m, boolArith = FALSE)
             tolQ <- if(isSparse) NA else eps16
 	    stopifnot(dim(c.m) == rep.int(ncol(m), 2),
 		      dim(tcm) == rep.int(nrow(m), 2),
@@ -486,12 +553,12 @@ checkMatrix <- function(m, m.m = if(do.matrix) as(m, "matrix"),
 	CatF(" Summary: ")
 	for(f in summList) {
 	    ## suppressWarnings():  e.g. any(<double>)	would warn here:
-	    r <- suppressWarnings(if(isCor) all.equal(f(m), f(m.m)) else
-				  identical(f(m), f(m.m)))
-	    if(!isTRUE(r)) {
+	    r <- suppressWarnings(identical(f(m), f(m.m)))
+	    if(!isTRUE(r)) { ## typically for prod()
 		f.nam <- sub("..$", '', sub("^\\.Primitive..", '', format(f)))
-		## prod() is delicate: NA or NaN can both happen
-		(if(f.nam == "prod") message else stop)(
+		## sum() and prod() are sensitive to order of f. p. operations
+		## particularly on systems where sizeof(long double) == sizeof(double)
+		(if(any(f.nam == c("sum", "prod"))) MSG else stop)(
 		    sprintf("%s(m) [= %g] differs from %s(m.m) [= %g]",
 			    f.nam, f(m), f.nam, f(m.m)))
 	    }
@@ -511,35 +578,52 @@ checkMatrix <- function(m, m.m = if(do.matrix) as(m, "matrix"),
     }
 
     if(!isInd)
-        m.d <- local({ m. <- m; diag(m.) <- diag(m); m. })
+        m.d <- local({ m. <- m
+            diag(m.) <- diag(m) ## << *assigning* to 'm.' now typically annihilates @factor
+            if(.hasSlot(m, "factors") && length(f <- m@factors))
+                m.@factors <- f
+            m. })
     if(do.matrix)
     stopifnot(identical(dim(m.m), dim(m)),
-	      ## base::diag() keeps names [Matrix FIXME]
+
 ## now that "pMatrix" subsetting gives *LOGICAL*
 ## 	      if(isPerm) {
 ## 		  identical(as.integer(unname(diag(m))), unname(diag(m.m)))
 ## 	      } else
-	      identical(unname(diag(m)),
-			unname(diag(m.m))),## not for NA: diag(m) == diag(m.m),
+	      identical(diag(m), # base:: *and* Matrix diag()  now keep names
+			diag(m.m)),## not for NA: diag(m) == diag(m.m),
 	      identical(nnzero(m), sum(m.m != 0)),
-	      identical(nnzero(m, na.= FALSE), sum(m.m != 0, na.rm = TRUE)),
-	      identical(nnzero(m, na.= TRUE),  sum(m.m != 0 | is.na(m.m)))
+	      identical(nnzero(m, na.counted = FALSE),
+                        sum(m.m != 0, na.rm = TRUE)),
+	      identical(nnzero(m, na.counted = TRUE),
+                        sum(m.m != 0 | is.na(m.m)))
 	      )
 
     if(isSparse) {
 	n0m <- drop0(m) #==> n0m is Csparse
 	has0 <- !Qidentical(n0m, as(m,"CsparseMatrix"))
-	if(!isInd && !isRsp &&
-           !(extends(cld, "TsparseMatrix") && anyDuplicatedT(m, di = d)))
-                                        # 'diag<-' is does not change attrib:
-	    stopifnot(Qidentical(m, m.d))# e.g., @factors may differ
-    }
-    else if(!identical(m, m.d)) { # dense : 'diag<-' is does not change attrib
-	if(isTri && m@diag == "U" && m.d@diag == "N" &&
-	   all(m == m.d))
-	    message("unitriangular m: diag(m) <- diag(m) lost \"U\" .. is ok")
-	else stop("diag(m) <- diag(m) has changed 'm' too much")
     }
+
+    if(isDiag)
+        stopifnot(exprs = {
+            .MJ.Qidentical(m, m.d, strictClass = FALSE,
+                           skipSlots = if(m@diag != "N") c("diag", "x"))
+            m@diag == "N" || (m.d@diag == "N" &&
+                              identical(m.d@x, diag(m, names = FALSE)))
+        })
+    else if(isTri && m@diag != "N")
+        stopifnot(exprs = {
+            is(m.d, "triangularMatrix") && m.d@diag == "N"
+            .MJ.Qidentical(m, m.d, strictClass = FALSE,
+                           skipSlots = c("diag", "p", "i", "j", "x"))
+            isSparse || all(m == m.d)
+        })
+    else if(!isInd)
+        stopifnot(.MJ.Qidentical(m, m.d, strictClass = FALSE,
+                                 skipSlots =
+                                     if(((isCsp || isRsp) && has0) || isTsp)
+                                         c("p", "i", "j", "x")))
+
     ## use non-square matrix when "allowed":
 
     ## m12: sparse and may have 0s even if this is not: if(isSparse && has0)
@@ -552,14 +636,18 @@ checkMatrix <- function(m, m.m = if(do.matrix) as(m, "matrix"),
 	    stopifnot(Qidentical(as(m11, "generalMatrix"),
 				 as(m12, "generalMatrix")))
     }
-    if(isSparse && !is.n) {
+    if(isSparse && !isDiag && !is.n) {
 	## ensure that as(., "nMatrix") gives nz-pattern
 	CatF("as(., \"nMatrix\") giving full nonzero-pattern: ")
 	n1 <- as(m, "nMatrix")
 	ns <- as(m, "nsparseMatrix")
 	stopifnot(identical(n1,ns),
-		  isDiag || ((if(isSym) Matrix:::nnzSparse else sum)(n1) ==
-			     length(if(isInd) m@perm else diagU2N(m)@x)))
+                  ## only testing [CR]sparseMatrix and indMatrix here ...
+                  ## sum(<n.T>) excludes duplicated (i,j) pairs whereas
+                  ## length(diagU2N(<[^n].T>)) includes them ...
+                  isTsp ||
+                  (if(isSym) length(if(.hasSlot(n1, "i")) n1@i else n1@j)
+                   else sum(n1)) == length(if(isInd) m@perm else diagU2N(m)@x))
         Cat("ok\n")
     }
 
@@ -601,7 +689,7 @@ checkMatrix <- function(m, m.m = if(do.matrix) as(m, "matrix"),
 		CatF("ok;  determinant(): ")
 		if(!doDet)
 		    Cat(" skipped (!doDet): ")
-		else if(any(is.na(m.m)) && extends(cld, "triangularMatrix"))
+		else if(any(is.na(m.m)) && isTri)
 		    Cat(" skipped: is triang. and has NA: ")
 		else
 		    stopifnot(eqDeterminant(m, m.m, NA.Inf.ok=TRUE))
@@ -626,8 +714,8 @@ checkMatrix <- function(m, m.m = if(do.matrix) as(m, "matrix"),
 	}
 	else if(extends(cld, "lMatrix")) { ## should fulfill even with NA:
 	    stopifnot(all(m | !m | ina), !any(!m & m & !ina))
-	    if(extends(cld, "TsparseMatrix")) # allow modify, since at end here
-		m <- uniqTsparse(m, clNam)
+	    if(isTsp) # allow modify, since at end here
+		m <- asUniqueT(m, isT = TRUE)
 	    stopifnot(identical(m, m & TRUE),
 		      identical(m, FALSE | m))
 	    ## also check the  coercions to [dln]Matrix
@@ -642,12 +730,8 @@ checkMatrix <- function(m, m.m = if(do.matrix) as(m, "matrix"),
 	}
 	else if(extends(cld, "dMatrix")) {
 	    m. <- if(isSparse && has0) n0m else m
-	    m1 <- (m. != 0)*1
-	    if(!isSparse && substr(clNam,1,3) == "dpp")
-		## no "nppMatrix" possible
-		m1 <- unpack(m1)
-
-	    m1. <- m1 # replace NA by 1 in m1. , carefully not changing class:
+	    m1 <- m1. <- (m. != 0)*1
+            ## replace NA by 1 in m1. , carefully not changing class:
 	    if(any(ina)) m1.@x[is.na(m1.@x)] <- 1
 	    ## coercion to n* (nz-pattern!) and back: only identical when no extra 0s and no NAs:
 	    stopifnot(Q.C.identical(m1., as(as(m., "nMatrix"),"dMatrix"),
@@ -656,18 +740,22 @@ checkMatrix <- function(m, m.m = if(do.matrix) as(m, "matrix"),
 				    isSparse, checkClass = FALSE))
 	}
 
-	if(extends(cld, "triangularMatrix")) {
+        maybeDense <- if(isSparse) identity else function(.) as(., "denseMatrix")
+	if(isTri) {
 	    mm. <- m
 	    i0 <- if(m@uplo == "L")
 		upper.tri(mm.) else lower.tri(mm.)
 	    n.catchWarn <- if(is.n) suppressWarnings else identity
 	    n.catchWarn( mm.[i0] <- 0 ) # ideally, mm. remained triangular, but can be dge*
-	    CatF("as(<triangular (ge)matrix>, ",clNam,"): ", sep='')
-	    tm <- as(as(mm., "triangularMatrix"), clNam)
+            ## Aug.2022 - Coercion deprecations: No longer do as(*, clNam):
+	    CatF("as(mm., \"triangularMatrix\"): ")
+	    tm <- as(mm., "triangularMatrix")
 	    Cat("valid:", validObject(tm), "\n")
-	    if(m@uplo == tm@uplo) ## otherwise, the matrix effectively was *diagonal*
-		## note that diagU2N(<dtr>) |-> dtC :
-		stopifnot(Qidentical(tm, as(diagU2N(m), clNam)))
+	    if(m@uplo == tm@uplo) { ## otherwise, the matrix effectively was *diagonal*
+                if(!isSparse && Matrix:::.isPacked(m)) m <- unpack(m) # to match tm
+		## note that diagU2N(<dtr>) |-> dtC, now dtT:
+		stopifnot(Qidentical(tm, maybeDense(diagU2N(m))))
+            }
 	}
 	else if(isDiag) {
 
@@ -687,4 +775,63 @@ checkMatrix <- function(m, m.m = if(do.matrix) as(m, "matrix"),
     }
 
     invisible(TRUE)
+} ## {checkMatrix}
+
+### --- These use
+
+##' Check QR-consistency of dense and sparse
+chk.qr.D.S <- function(d., s., y, Y = Matrix(y), force = FALSE, tol = 1e-10) {
+    stopifnot(is.qr(d.), is(s., "sparseQR"))
+    cc <- qr.coef(d.,y)
+    rank.def <- any(is.na(cc)) && d.$rank < length(d.$pivot)
+    if(rank.def && force) cc <- mkNA.0(cc) ## set NA's to 0 .. ok, in some case
+
+    ## when system is rank deficient, have differing cases, not always just NA <-> 0 coef
+    ## FIXME though:  resid & fitted should be well determined
+    if(force || !rank.def) stopifnot(
+	is.all.equal3(	    cc	     , qr.coef  (s.,y), drop(qr.coef  (s.,Y)), tol=tol),
+	is.all.equal3(qr.resid (d.,y), qr.resid (s.,y), drop(qr.resid (s.,Y)), tol=tol),
+	is.all.equal3(qr.fitted(d.,y), qr.fitted(s.,y), drop(qr.fitted(s.,Y)), tol=tol)
+	)
+}
+
+##' "Combi" calling chkQR() on both "(sparse)Matrix" and 'traditional' version
+##' ------  and combine the two qr decompositions using chk.qr.D.S()
+##'         [ chkQR() def. in >>>>> ./test-tools-1.R <<<<< ]
+##'
+##' @title check QR-decomposition, and compare sparse and dense one
+##' @param A a 'Matrix' , typically 'sparseMatrix'
+##' @param Qinv.chk
+##' @param QtQ.chk
+##' @param quiet
+##' @return list with 'qA' (sparse QR) and 'qa' (traditional (dense) QR)
+##' @author Martin Maechler
+checkQR.DS.both <- function(A, Qinv.chk, QtQ.chk=NA,
+                            quiet=FALSE, giveRE=TRUE, tol = 1e-13)
+{
+    stopifnot(is(A,"Matrix"))
+    if(!quiet) cat("classical: ")
+    qa <- chkQR(as(A, "matrix"), Qinv.chk=TRUE, QtQ.chk=TRUE, tol=tol, giveRE=giveRE)# works always
+    if(!quiet) cat("[Ok] ---  sparse: ")
+    qA <- chkQR(A, Qinv.chk=Qinv.chk, QtQ.chk=QtQ.chk, tol=tol, giveRE=giveRE)
+    validObject(qA)
+    if(!quiet) cat("[Ok]\n")
+    chk.qr.D.S(qa, qA, y = 10 + 1:nrow(A), tol = 256*tol)# ok [not done in rank deficient case!]
+    invisible(list(qA=qA, qa=qa))
+}
+
+non0.ij <- function(M) Matrix:::non0.i(as(M, "sparseMatrix"))
+
+triuChk <- function(x, k) {
+    ans <- triu(x, k)
+    ij <- non0.ij(ans)
+    stopifnot(identical(dim(x), dim(ans)), (ij %*% c(-1,1)) >= k)
+    ans
+}
+
+trilChk <- function(x, k) {
+    ans <- tril(x, k)
+    ij <- non0.ij(ans)
+    stopifnot(identical(dim(x), dim(ans)), (ij %*% c(-1,1)) <= k)
+    ans
 }
