@@ -4,7 +4,7 @@
    Copyright 1997-2005  Douglas M. Bates <bates@stat.wisc.edu>,
 			Jose C. Pinheiro,
 			Saikat DebRoy
-   Copyright 2007-2013  The R Core Team
+   Copyright 2007-2022  The R Core Team
 
    This file is part of the nlme package for R and related languages
    and is made available under the terms of the GNU General Public
@@ -25,44 +25,38 @@
 
 #include "matrix.h"
 #include <R_ext/Applic.h>
+#include <float.h> // for DBL_EPSILON
 
 
 /* Factor list and Recalc for general corStruct object */
 
 void
-corStruct_factList(double *mat, longint *pdims, double *FactorL, double *logdet)
+corStruct_factList(double *mat, int *pdims, double *FactorL, double *logdet)
 {
-    longint i, j, M = pdims[1], *len = pdims + 4, job = 11L, info;
-#ifndef USING_R
-    longint zero = 0L;
-#endif
+    int i, j, M = pdims[1], *len = pdims + 4, job = 11L, info;
     double *work, *work1;
 
     for(i = 0; i < M; i++) {
-	longint li = len[i], lisq = li * li, lip1 = li + 1;
-	work = Calloc(li, double);
-	work1 = Calloc(lisq, double);
-#ifdef USING_R
+	int li = len[i], lisq = li * li, lip1 = li + 1;
+	work = R_Calloc(li, double);
+	work1 = R_Calloc(lisq, double);
 	F77_CALL(chol) (mat, &li, &li, mat, &info);
-#else
-	F77_CALL(chol) (mat, &li, work, &zero, &zero, &info);
-#endif
 	for(j = 0; j < li; j++) {
 	    work1[j * lip1] = 1;
 	    F77_CALL(dtrsl) (mat, &li, &li, work1 + j * li, &job, &info);
 	    *logdet -= log(fabs(mat[j * lip1]));
 	}
 	Memcpy(FactorL, work1, lisq);
-	Free(work); Free(work1);
+	R_Free(work); R_Free(work1);
 	FactorL += lisq;
 	mat += lisq;
     }
 }
 
 void
-corStruct_recalc(double *Xy, longint *pdims, longint *ZXcol, double *Factor)
+corStruct_recalc(double *Xy, int *pdims, int *ZXcol, double *Factor)
 {
-    longint N = pdims[0], M = pdims[1], *len = pdims + 4, *start = len + M, i;
+    int N = pdims[0], M = pdims[1], *len = pdims + 4, *start = len + M, i;
     for(i = 0; i < M;  i++) {
 	mult_mat(Xy + start[i], N, Factor, len[i], len[i], len[i],
 		 Xy + start[i], N, *ZXcol);
@@ -74,19 +68,19 @@ corStruct_recalc(double *Xy, longint *pdims, longint *ZXcol, double *Factor)
    parametrization */
 
 void
-symm_fullCorr(double *par, longint *maxC, double *crr)
+symm_fullCorr(double *par, int *maxC, double *crr)
     /* calculates full correlation structure  */
 {
     double *work, aux, aux1, *src = par, *src1, *dest;
-    longint i, j, n = *maxC;
+    int i, j, n = *maxC;
 
     /* first get upper-triangular factor */
-    dest = work = Calloc(n * (n + 1) / 2, double);
+    dest = work = R_Calloc(n * (n + 1) / 2, double);
     for(i = 0; i < n; i++) {
 	aux = 1.0;
 	for(j = 0; j < i; j++) {
 	    aux1 = exp(*src);
-	    aux1 = PI * aux1/(1 + aux1); /* untransforming */
+	    aux1 = M_PI * aux1/(1 + aux1); /* untransforming */
 	    *dest = aux * cos(aux1);
 	    aux *= sin(aux1);
 	    dest++; src++;
@@ -97,7 +91,7 @@ symm_fullCorr(double *par, longint *maxC, double *crr)
 
     /* getting the correlations */
     for(i = 0, dest = crr, src = work; i < n - 1; i++) {
-	longint ip1 = i + 1;
+	int ip1 = i + 1;
 	src += i;
 	for(j = ip1, src1 = src; j < n; j++) {
 	    src1 += j;
@@ -105,13 +99,13 @@ symm_fullCorr(double *par, longint *maxC, double *crr)
 	    dest++;
 	}
     }
-    Free(work);
+    R_Free(work);
 }
 
 static void
-symm_mat(double *crr, longint *time, longint *n, longint *maxC, double *mat)
+symm_mat(double *crr, int *time, int *n, int *maxC, double *mat)
 {
-    longint i, j, k, np1 = *n + 1, n1, n2;
+    int i, j, k, np1 = *n + 1, n1, n2;
     for(i = 0; i < *n; i++) {
 	mat[i * np1] = 1.0;
 	for(j = i + 1; j < *n; j++) {
@@ -124,11 +118,10 @@ symm_mat(double *crr, longint *time, longint *n, longint *maxC, double *mat)
 }
 
 void
-symm_matList(double *pars, longint *time, longint *maxC,
-	     longint *pdims, double *mat)
+symm_matList(double *pars, int *time, int *maxC, int *pdims, double *mat)
 {
-    double *crr = Calloc(*maxC * (*maxC - 1) / 2, double);
-    longint i, M = pdims[1], *len = pdims + 4;
+    double *crr = R_Calloc(*maxC * (*maxC - 1) / 2, double);
+    int i, M = pdims[1], *len = pdims + 4;
     /* parameters assumed in unconstrained form */
     symm_fullCorr(pars, maxC, crr);
     for(i = 0; i < M;  i++) {
@@ -136,40 +129,33 @@ symm_matList(double *pars, longint *time, longint *maxC,
 	mat += len[i] * len[i];
 	time += len[i];
     }
-    Free(crr);
+    R_Free(crr);
 }
 
 static void
-symm_fact(double *crr, longint *time, longint *n, longint *maxC, double *mat,
+symm_fact(double *crr, int *time, int *n, int *maxC, double *mat,
 	  double *logdet)
 {
-    longint job = 11L, info, i, nsq = *n * (*n), np1 = *n + 1;
-#ifndef USING_R
-    longint zero = 0L;
-#endif
-    double *work = Calloc(*n, double), *work1 = Calloc(nsq, double);
+    int job = 11L, info, i, nsq = *n * (*n), np1 = *n + 1;
+    double *work = R_Calloc(*n, double), *work1 = R_Calloc(nsq, double);
 
     symm_mat(crr, time, n, maxC, mat);
-#ifdef USING_R
     F77_CALL(chol) (mat, n, n, mat, &info);
-#else
-    F77_CALL(chol) (mat, n, work, &zero, &zero, &info);
-#endif
     for(i = 0; i < *n; i++) {
 	work1[i * np1] = 1;
 	F77_CALL(dtrsl) (mat, n, n, work1 + i * (*n), &job, &info);
 	*logdet -= log(fabs(mat[i * np1]));
     }
     Memcpy(mat, work1, nsq);
-    Free(work); Free(work1);
+    R_Free(work); R_Free(work1);
 }
 
 void
-symm_factList(double *pars, longint *time, longint *maxC, longint *pdims,
+symm_factList(double *pars, int *time, int *maxC, int *pdims,
 	      double *FactorL, double *logdet)
 {
-    double *crr = Calloc(*maxC * (*maxC - 1L) / 2L, double);
-    longint i, M = pdims[1], *len = pdims + 4;
+    double *crr = R_Calloc(*maxC * (*maxC - 1L) / 2L, double);
+    int i, M = pdims[1], *len = pdims + 4;
     /* parameters assumed in unconstrained form */
     symm_fullCorr(pars, maxC, crr);
     for(i = 0; i < M;  i++) {
@@ -177,35 +163,35 @@ symm_factList(double *pars, longint *time, longint *maxC, longint *pdims,
 	FactorL += len[i] * len[i];
 	time += len[i];
     }
-    Free(crr);
+    R_Free(crr);
 }
 
 void
-symm_recalc(double *Xy, longint *pdims, longint *ZXcol, double *pars,
-	    longint *time, longint *maxC, double *logdet)
+symm_recalc(double *Xy, int *pdims, int *ZXcol, double *pars,
+	    int *time, int *maxC, double *logdet)
 {
-    longint N = pdims[0], M = pdims[1], *len = pdims + 4, *start = len + M, i;
-    double *crr = Calloc(*maxC * (*maxC - 1) / 2, double);
+    int N = pdims[0], M = pdims[1], *len = pdims + 4, *start = len + M, i;
+    double *crr = R_Calloc(*maxC * (*maxC - 1) / 2, double);
     /* parameters assumed in unconstrained form */
     symm_fullCorr(pars, maxC, crr);
     for(i = 0; i < M;  i++) {
-	double *Factor = Calloc((len[i] * len[i]), double);
+	double *Factor = R_Calloc((len[i] * len[i]), double);
 	symm_fact(crr, time + start[i], &len[i], maxC, Factor, logdet);
 	mult_mat(Xy + start[i], N, Factor, len[i], len[i], len[i],
 		 Xy + start[i], N, *ZXcol);
-	Free(Factor);
+	R_Free(Factor);
     }
-    Free(crr);
+    R_Free(crr);
 }
 
 /* nat class - unstructured correlation - natural parametrization */
 
 void
-nat_fullCorr(double *par, longint *maxC, double *crr)
+nat_fullCorr(double *par, int *maxC, double *crr)
     /* calculates full correlation structure  */
 {
     double aux;
-    longint i, npar = *maxC * (*maxC - 1) / 2;
+    int i, npar = *maxC * (*maxC - 1) / 2;
 
     for(i = 0; i < npar; i++) {
 	aux = exp(par[i]);
@@ -214,11 +200,10 @@ nat_fullCorr(double *par, longint *maxC, double *crr)
 }
 
 void
-nat_matList(double *pars, longint *time, longint *maxC,
-	    longint *pdims, double *mat)
+nat_matList(double *pars, int *time, int *maxC, int *pdims, double *mat)
 {
-    double *crr = Calloc(*maxC * (*maxC - 1) / 2, double);
-    longint i, M = pdims[1], *len = pdims + 4;
+    double *crr = R_Calloc(*maxC * (*maxC - 1) / 2, double);
+    int i, M = pdims[1], *len = pdims + 4;
     /* parameters assumed in unconstrained form */
     nat_fullCorr(pars, maxC, crr);
     for(i = 0; i < M;  i++) {
@@ -226,15 +211,15 @@ nat_matList(double *pars, longint *time, longint *maxC,
 	mat += len[i] * len[i];
 	time += len[i];
     }
-    Free(crr);
+    R_Free(crr);
 }
 
 void
-nat_factList(double *pars, longint *time, longint *maxC, longint *pdims,
+nat_factList(double *pars, int *time, int *maxC, int *pdims,
 	     double *FactorL, double *logdet)
 {
-    double *crr = Calloc(*maxC * (*maxC - 1L) / 2L, double);
-    longint i, M = pdims[1], *len = pdims + 4;
+    double *crr = R_Calloc(*maxC * (*maxC - 1L) / 2L, double);
+    int i, M = pdims[1], *len = pdims + 4;
     /* parameters assumed in unconstrained form */
     nat_fullCorr(pars, maxC, crr);
     for(i = 0; i < M;  i++) {
@@ -242,33 +227,33 @@ nat_factList(double *pars, longint *time, longint *maxC, longint *pdims,
 	FactorL += len[i] * len[i];
 	time += len[i];
     }
-    Free(crr);
+    R_Free(crr);
 }
 
 void
-nat_recalc(double *Xy, longint *pdims, longint *ZXcol, double *pars,
-	   longint *time, longint *maxC, double *logdet)
+nat_recalc(double *Xy, int *pdims, int *ZXcol, double *pars,
+	   int *time, int *maxC, double *logdet)
 {
-    longint N = pdims[0], M = pdims[1], *len = pdims + 4, *start = len + M, i;
-    double *crr = Calloc(*maxC * (*maxC - 1) / 2, double);
+    int N = pdims[0], M = pdims[1], *len = pdims + 4, *start = len + M, i;
+    double *crr = R_Calloc(*maxC * (*maxC - 1) / 2, double);
     /* parameters assumed in unconstrained form */
     nat_fullCorr(pars, maxC, crr);
     for(i = 0; i < M;  i++) {
-	double *Factor = Calloc((len[i] * len[i]), double);
+	double *Factor = R_Calloc((len[i] * len[i]), double);
 	symm_fact(crr, time + start[i], &len[i], maxC, Factor, logdet);
 	mult_mat(Xy + start[i], N, Factor, len[i], len[i], len[i],
 		 Xy + start[i], N, *ZXcol);
-	Free(Factor);
+	R_Free(Factor);
     }
-    Free(crr);
+    R_Free(crr);
 }
 
 /* AR1 class */
 
 static void
-AR1_mat(double *par, longint *n, double *mat)
+AR1_mat(double *par, int *n, double *mat)
 {
-    longint i, j;
+    int i, j;
     double aux;
     for(i = 0; i < *n; i++) {
 	*(mat + i * (*n + 1)) = 1.0;
@@ -292,9 +277,9 @@ safe_phi(double x)		/* returns (exp(x) - 1)/(exp(x) + 1), x < 0 */
 }
 
 void
-AR1_matList(double *par, longint *pdims, double *mat)
+AR1_matList(double *par, int *pdims, double *mat)
 {
-    longint i, M = pdims[1], *len = pdims + 4;
+    int i, M = pdims[1], *len = pdims + 4;
     /* par assumed in unconstrained form */
 
     *par = safe_phi( *par );
@@ -305,9 +290,9 @@ AR1_matList(double *par, longint *pdims, double *mat)
 }
 
 static void
-AR1_fact(double *par, longint *n, double *mat, double *logdet)
+AR1_fact(double *par, int *n, double *mat, double *logdet)
 {
-    longint i, np1 = *n + 1;
+    int i, np1 = *n + 1;
     double aux = sqrt(1 - *par * (*par)), aux1 = - (*par)/aux;
 
     *logdet -= (*n - 1) * log(aux);
@@ -320,10 +305,9 @@ AR1_fact(double *par, longint *n, double *mat, double *logdet)
 }
 
 void
-AR1_factList(double *par, longint *pdims, double *FactorL,
-	     double *logdet)
+AR1_factList(double *par, int *pdims, double *FactorL, double *logdet)
 {
-    longint i, M = pdims[1], *len = pdims + 4;
+    int i, M = pdims[1], *len = pdims + 4;
     /* par assumed in unconstrained form */
 
     *par = safe_phi( *par );
@@ -334,19 +318,18 @@ AR1_factList(double *par, longint *pdims, double *FactorL,
 }
 
 void
-AR1_recalc(double *Xy, longint *pdims, longint *ZXcol, double *par,
-	   double *logdet)
+AR1_recalc(double *Xy, int *pdims, int *ZXcol, double *par, double *logdet)
 {
-    longint N = pdims[0], M = pdims[1], *len = pdims + 4,  *start = len + M, i;
+    int N = pdims[0], M = pdims[1], *len = pdims + 4,  *start = len + M, i;
     double *Factor;
     /* par assumed in unconstrained form */
     *par = safe_phi( *par );
     for(i = 0; i < M;  i++) {
-	Factor = Calloc(len[i] * len[i], double);
+	Factor = R_Calloc(len[i] * len[i], double);
 	AR1_fact(par, &len[i], Factor, logdet);
 	mult_mat(Xy + start[i], N, Factor, len[i], len[i], len[i],
 		 Xy + start[i], N, *ZXcol);
-	Free(Factor);
+	R_Free(Factor);
     }
 }
 
@@ -354,9 +337,9 @@ AR1_recalc(double *Xy, longint *pdims, longint *ZXcol, double *par,
 /* Continuous AR1 class */
 
 static void
-CAR1_mat(double *par, double *time, longint *n, double *mat)
+CAR1_mat(double *par, double *time, int *n, double *mat)
 {
-    longint i, j;
+    int i, j;
     double aux;
     for(i = 0; i < *n; i++) {
 	*(mat + i * (*n + 1)) = 1.0;
@@ -368,9 +351,9 @@ CAR1_mat(double *par, double *time, longint *n, double *mat)
 }
 
 void
-CAR1_matList(double *par, double *time, longint *pdims, double *mat)
+CAR1_matList(double *par, double *time, int *pdims, double *mat)
 {
-    longint i, M = pdims[1], *len = pdims + 4;
+    int i, M = pdims[1], *len = pdims + 4;
     double aux = exp(*par);
     /* parameter assumed in unconstrained form */
     *par = aux / (1.0 + aux);
@@ -382,31 +365,26 @@ CAR1_matList(double *par, double *time, longint *pdims, double *mat)
 }
 
 static void
-CAR1_fact(double *par, double *time, longint *n, double *mat, double *logdet)
+CAR1_fact(double *par, double *time, int *n, double *mat, double *logdet)
 {
-    longint job = 11L, info, i, nsq = *n * (*n), np1 = *n + 1;
-    double *work = Calloc(*n, double), *work1 = Calloc(nsq, double);
+    int job = 11L, info, i, nsq = *n * (*n), np1 = *n + 1;
+    double *work = R_Calloc(*n, double), *work1 = R_Calloc(nsq, double);
     CAR1_mat(par, time, n, mat);
-#ifdef USING_R
     F77_CALL(chol) (mat, n, n, mat, &info);
-#else
-    longint zero = 0L;
-    F77_CALL(chol) (mat, n, work, &zero, &zero, &info);
-#endif
     for(i = 0; i < *n; i++) {
 	work1[i * np1] = 1;
 	F77_CALL(dtrsl) (mat, n, n, work1 + i * (*n), &job, &info);
 	*logdet -= log(fabs(mat[i * np1]));
     }
     Memcpy(mat, work1, nsq);
-    Free(work); Free(work1);
+    R_Free(work); R_Free(work1);
 }
 
 void
-CAR1_factList(double *par, double *time, longint *pdims,
+CAR1_factList(double *par, double *time, int *pdims,
 	      double *FactorL, double *logdet)
 {
-    longint i, M = pdims[1], *len = pdims + 4;
+    int i, M = pdims[1], *len = pdims + 4;
     double aux = exp(*par);
     /* parameter assumed in unconstrained form */
     *par = aux / (1.0 + aux);
@@ -418,28 +396,28 @@ CAR1_factList(double *par, double *time, longint *pdims,
 }
 
 void
-CAR1_recalc(double *Xy, longint *pdims, longint *ZXcol,
+CAR1_recalc(double *Xy, int *pdims, int *ZXcol,
 	    double *par, double *time, double *logdet)
 {
-    longint N = pdims[0], M = pdims[1], *len = pdims + 4, *start = len + M, i;
+    int N = pdims[0], M = pdims[1], *len = pdims + 4, *start = len + M, i;
     double aux = exp(*par);
     /* parameter assumed in unconstrained form */
     *par = aux / (1.0 + aux);
     for(i = 0; i < M;  i++) {
-	double *Factor = Calloc(len[i] * len[i], double);
+	double *Factor = R_Calloc(len[i] * len[i], double);
 	CAR1_fact(par, time + start[i], &len[i], Factor, logdet);
 	mult_mat(Xy + start[i], N, Factor, len[i], len[i], len[i],
 		 Xy + start[i], N, *ZXcol);
-	Free(Factor);
+	R_Free(Factor);
     }
 }
 
 /* ARMA class */
 
 static void
-ARMA_transPar(longint N, double *pars, double sgn)
+ARMA_transPar(int N, double *pars, double sgn)
 {
-    longint i, j, n, n2;
+    int i, j, n, n2;
     double ps, D, aux;
     for(n = N - 1; n > -1; n--) {
 	if ((ps = pars[n] * pars[n]) >= 1.0)
@@ -462,19 +440,19 @@ ARMA_transPar(longint N, double *pars, double sgn)
 }
 
 void
-ARMA_unconstCoef(longint *p, longint *q, double *pars)
+ARMA_unconstCoef(int *p, int *q, double *pars)
 {
     ARMA_transPar(*p, pars, 1.0);
     ARMA_transPar(*q, pars + *p, -1.0);
 }
 
 static void
-ARMA_untransPar(longint N, double *pars, double sgn)
+ARMA_untransPar(int N, double *pars, double sgn)
 {
-    longint i, j;
+    int i, j;
     double *aux;
     if (N) {
-	aux = Calloc(N, double);
+	aux = R_Calloc(N, double);
 	for(i = 0; i < N; i++) {
 	    pars[i] = exp(-pars[i]);
 	    aux[i] = pars[i] = (1 - pars[i])/(1 + pars[i]);
@@ -485,21 +463,21 @@ ARMA_untransPar(longint N, double *pars, double sgn)
 		Memcpy(aux, pars, i);
 	    }
 	}
-	Free(aux);
+	R_Free(aux);
     }
 }
 
 void
-ARMA_constCoef(longint *p, longint *q, double *pars)
+ARMA_constCoef(int *p, int *q, double *pars)
 {
     ARMA_untransPar(*p, pars, -1.0);
     ARMA_untransPar(*q, pars + *p, 1.0);
 }
 
 static void
-ARMA_cross(longint *p, longint *q, double *pars, double *psi)
+ARMA_cross(int *p, int *q, double *pars, double *psi)
 {
-    longint i, j, M = *q + 1, PM;
+    int i, j, M = *q + 1, PM;
     M = (*p > M ? *p : M);
     psi[0] = 1;
     for(i = 1; i < M; i++) {
@@ -511,16 +489,16 @@ ARMA_cross(longint *p, longint *q, double *pars, double *psi)
     }
 }
 
-static void
-ARMA_corr(longint *p, longint *q, longint *maxlag, double *pars, double *psi,
-	  double *crr)
-{
-    longint P = *p + 1, Pp1 = P + 1, i, j, k, minPQ, Mlag, maxPQ,
-	*pivot = Calloc(P, longint);
-    double *coef = Calloc(P * P, double), *src, *qraux = Calloc(P, double),
-	*work = Calloc(P * P, double), *work1;
 
-    if (!sqrt_eps) sqrt_eps = sqrt(DOUBLE_EPS);
+static void
+ARMA_corr(int *p, int *q, int *maxlag, double *pars, double *psi, double *crr)
+{
+    int P = *p + 1, Pp1 = P + 1, i, j, k, minPQ, Mlag, maxPQ,
+	*pivot = R_Calloc(P, int);
+    double *coef = R_Calloc(P * P, double), *src, *qraux = R_Calloc(P, double),
+	*work = R_Calloc(P * P, double), *work1;
+
+    if (sqrt_eps == 0.0) sqrt_eps = sqrt(DBL_EPSILON);
     if ((maxPQ = ((*p > *q) ? *p : *q))) {
 	for(i = 0, src = coef; i < P; i++, src += Pp1) {
 	    crr[i] = 0;
@@ -528,7 +506,7 @@ ARMA_corr(longint *p, longint *q, longint *maxlag, double *pars, double *psi,
 	}
 	Mlag = ((*maxlag > *q) ? *maxlag : *q);
 	Mlag = ((Mlag > *p) ? Mlag : *p) + 1;
-	work1 = Calloc(Mlag, double);
+	work1 = R_Calloc(Mlag, double);
 	for(i = P; i < Mlag; i++) {
 	    crr[i] = 0;
 	}
@@ -576,28 +554,27 @@ ARMA_corr(longint *p, longint *q, longint *maxlag, double *pars, double *psi,
 	for(i = 1; i < Mlag; i++) {
 	    crr[i] /= crr[0];
 	}
-	Free(qraux); Free(work); Free(coef); Free(pivot); Free(work1);
+	R_Free(qraux); R_Free(work); R_Free(coef); R_Free(pivot); R_Free(work1);
     }
     crr[0] = 1;
 }
 
 static void
-ARMA_fullCorr(longint *p, longint *q, longint *maxlag, double *pars,
-	      double *crr)
+ARMA_fullCorr(int *p, int *q, int *maxlag, double *pars, double *crr)
 {
-    longint M = *q + 1;
+    int M = *q + 1;
     double *psi;
     M = ((M < *p) ? *p : M);
-    psi = Calloc(M, double);
+    psi = R_Calloc(M, double);
     ARMA_cross(p, q, pars, psi);
     ARMA_corr(p, q, maxlag, pars, psi, crr);
-    Free(psi);
+    R_Free(psi);
 }
 
 static void
-ARMA_mat(double *crr, longint *time, longint *n, double *mat)
+ARMA_mat(double *crr, int *time, int *n, double *mat)
 {
-    longint i, j, k;
+    int i, j, k;
     for(i = 0; i < *n; i++) {
 	for(j = i; j < *n; j++) {
 	    k = time[j] - time[i];
@@ -608,11 +585,11 @@ ARMA_mat(double *crr, longint *time, longint *n, double *mat)
 }
 
 void
-ARMA_matList(double *pars, longint *p, longint *q, longint *time,
-	     longint *maxlag, longint *pdims, double *mat)
+ARMA_matList(double *pars, int *p, int *q, int *time, int *maxlag,
+	     int *pdims, double *mat)
 {
-    double *crr = Calloc(*maxlag + 1L, double);
-    longint i, M = pdims[1], *len = pdims + 4;
+    double *crr = R_Calloc(*maxlag + 1L, double);
+    int i, M = pdims[1], *len = pdims + 4;
     /* parameters assumed in unconstrained form */
     ARMA_constCoef(p, q, pars);
     ARMA_fullCorr(p, q, maxlag, pars, crr);
@@ -621,39 +598,32 @@ ARMA_matList(double *pars, longint *p, longint *q, longint *time,
 	mat += len[i] * len[i];
 	time += len[i];
     }
-    Free(crr);
+    R_Free(crr);
 }
 
 static void
-ARMA_fact(double *crr, longint *time, longint *n, double *mat, double *logdet)
+ARMA_fact(double *crr, int *time, int *n, double *mat, double *logdet)
 {
-    longint job = 11L, info, i, nsq = *n * (*n), np1 = *n + 1;
-    double *work = Calloc(*n, double), *work1 = Calloc(nsq, double);
-#ifndef USING_R
-    longint zero = 0L;
-#endif
+    int job = 11L, info, i, nsq = *n * (*n), np1 = *n + 1;
+    double *work = R_Calloc(*n, double), *work1 = R_Calloc(nsq, double);
     ARMA_mat(crr, time, n, mat);
-#ifdef USING_R
     F77_CALL(chol) (mat, n, n, mat, &info);
-#else
-    F77_CALL(chol) (mat, n, work, &zero, &zero, &info);
-#endif
     for(i = 0; i < *n; i++) {
 	work1[i * np1] = 1;
 	F77_CALL(dtrsl) (mat, n, n, work1 + i * (*n), &job, &info);
 	*logdet -= log(fabs(mat[i * np1]));
     }
     Memcpy(mat, work1, nsq);
-    Free(work); Free(work1);
+    R_Free(work); R_Free(work1);
 }
 
 void
-ARMA_factList(double *pars, longint *p, longint *q, longint *time,
-	      longint *maxlag, longint *pdims, double *FactorL,
+ARMA_factList(double *pars, int *p, int *q, int *time,
+	      int *maxlag, int *pdims, double *FactorL,
 	      double *logdet)
 {
-    double *crr = Calloc(*maxlag + 1L, double);
-    longint i, M = pdims[1], *len = pdims + 4;
+    double *crr = R_Calloc(*maxlag + 1L, double);
+    int i, M = pdims[1], *len = pdims + 4;
     /* parameters assumed in unconstrained form */
     ARMA_constCoef(p, q, pars);
     ARMA_fullCorr(p, q, maxlag, pars, crr);
@@ -662,34 +632,34 @@ ARMA_factList(double *pars, longint *p, longint *q, longint *time,
 	FactorL += len[i] * len[i];
 	time += len[i];
     }
-    Free(crr);
+    R_Free(crr);
 }
 
 void
-ARMA_recalc(double *Xy, longint *pdims, longint *ZXcol, double *pars,
-	    longint *p, longint *q, longint *time, longint *maxlag,
+ARMA_recalc(double *Xy, int *pdims, int *ZXcol, double *pars,
+	    int *p, int *q, int *time, int *maxlag,
 	    double *logdet)
 {
-    longint N = pdims[0], M = pdims[1], *len = pdims + 4, *start = len + M, i;
-    double *crr = Calloc(*maxlag + 1L, double);
+    int N = pdims[0], M = pdims[1], *len = pdims + 4, *start = len + M, i;
+    double *crr = R_Calloc(*maxlag + 1L, double);
     /* parameters assumed in unconstrained form */
     ARMA_constCoef(p, q, pars);
     ARMA_fullCorr(p, q, maxlag, pars, crr);
     for(i = 0; i < M;  i++) {
-	double *Factor = Calloc(len[i] * len[i], double);
+	double *Factor = R_Calloc(len[i] * len[i], double);
 	ARMA_fact(crr, time + start[i], &len[i], Factor, logdet);
 	mult_mat(Xy + start[i], N, Factor, len[i], len[i], len[i],
 		 Xy + start[i], N, *ZXcol);
-	Free(Factor);
+	R_Free(Factor);
     }
 }
 
 /* Compound symmetry */
 
 static void
-compSymm_mat(double *par, longint *n, double *mat)
+compSymm_mat(double *par, int *n, double *mat)
 {
-    longint i, j;
+    int i, j;
     for(i = 0; i < *n; i++) {
 	mat[(*n + 1) * i] = 1.0;
 	for(j = i + 1; j < *n; j++) {
@@ -699,9 +669,9 @@ compSymm_mat(double *par, longint *n, double *mat)
 }
 
 void
-compSymm_matList(double *par, double *inf, longint *pdims, double *mat)
+compSymm_matList(double *par, double *inf, int *pdims, double *mat)
 {
-    longint i, M = pdims[1], *len = pdims + 4;
+    int i, M = pdims[1], *len = pdims + 4;
     /* parameter assumed in unconstrained form */
     double aux = exp(*par);
     *par = (aux + *inf)/(aux + 1.0);
@@ -712,10 +682,10 @@ compSymm_matList(double *par, double *inf, longint *pdims, double *mat)
 }
 
 static void
-compSymm_fact(double *par, longint *n, double *mat, double *logdet)
+compSymm_fact(double *par, int *n, double *mat, double *logdet)
 {
-    longint i, j, np1 = *n + 1, nsq = *n * (*n);
-    double aux, aux1, *work = Calloc(nsq, double);
+    int i, j, np1 = *n + 1, nsq = *n * (*n);
+    double aux, aux1, *work = R_Calloc(nsq, double);
     aux = 1 + (*n - 1) * (*par);
     *logdet -= log(aux)/2;
     aux = 1/sqrt(aux * (*n));
@@ -732,14 +702,14 @@ compSymm_fact(double *par, longint *n, double *mat, double *logdet)
 	work[i * np1] = -i * aux1;
     }
     Memcpy(mat, work, nsq);
-    Free(work);
+    R_Free(work);
 }
 
 void
-compSymm_factList(double *par, double *inf, longint *pdims,
+compSymm_factList(double *par, double *inf, int *pdims,
 		  double *FactorL, double *logdet)
 {
-    longint i, M = pdims[1], *len = pdims + 4;
+    int i, M = pdims[1], *len = pdims + 4;
     /* parameter assumed in unconstrained form */
     double aux = exp(*par);
     *par = (aux + *inf)/(aux + 1.0);
@@ -750,28 +720,28 @@ compSymm_factList(double *par, double *inf, longint *pdims,
 }
 
 void
-compSymm_recalc(double *Xy, longint *pdims, longint *ZXcol, double *par,
+compSymm_recalc(double *Xy, int *pdims, int *ZXcol, double *par,
 		double *inf, double *logdet)
 {
-    longint N = pdims[0], M = pdims[1], *len = pdims + 4, *start = len + M, i;
+    int N = pdims[0], M = pdims[1], *len = pdims + 4, *start = len + M, i;
     double aux = exp(*par);
     /* parameter assumed in unconstrained form */
     *par = (aux + *inf)/(aux + 1.0);
     for(i = 0; i < M;  i++) {
-	double *Factor = Calloc(len[i] * len[i], double);
+	double *Factor = R_Calloc(len[i] * len[i], double);
 	compSymm_fact(par, &len[i], Factor, logdet);
 	mult_mat(Xy + start[i], N, Factor, len[i], len[i], len[i],
 		 Xy + start[i], N, *ZXcol);
-	Free(Factor);
+	R_Free(Factor);
     }
 }
 
 /* Huyn-Feldt class */
 
 static void
-HF_mat(double *par, longint *time, longint *n, double *mat)
+HF_mat(double *par, int *time, int *n, double *mat)
 {
-    longint i, j, np1 = *n + 1;
+    int i, j, np1 = *n + 1;
     for(i = 0; i < *n; i++) {
 	mat[i * np1] = par[time[i]];
 	for(j = i + 1; j < *n; j++) {
@@ -782,10 +752,10 @@ HF_mat(double *par, longint *time, longint *n, double *mat)
 }
 
 void
-HF_matList(double *par, longint *maxC, longint *time, longint *pdims,
+HF_matList(double *par, int *maxC, int *time, int *pdims,
 	   double *mat)
 {
-    longint i, M = pdims[1], *len = pdims + 4;
+    int i, M = pdims[1], *len = pdims + 4;
     /* parameter assumed in unconstrained form */
     double inf = -1.0/(2.0 * ((double) *maxC));
     for(i = 0; i < *maxC; i++) {
@@ -799,33 +769,26 @@ HF_matList(double *par, longint *maxC, longint *time, longint *pdims,
 }
 
 static void
-HF_fact(double *par, longint *time, longint *n, double *mat, double *logdet)
+HF_fact(double *par, int *time, int *n, double *mat, double *logdet)
 {
-    longint job = 11L, info, i, nsq = *n * (*n), np1 = *n + 1;
-    double *work = Calloc(*n, double), *work1 = Calloc(nsq, double);
-#ifndef USING_R
-    longint zero = 0L;
-#endif
+    int job = 11L, info, i, nsq = *n * (*n), np1 = *n + 1;
+    double *work = R_Calloc(*n, double), *work1 = R_Calloc(nsq, double);
     HF_mat(par, time, n, mat);
-#ifdef USING_R
     F77_CALL(chol) (mat, n, n, mat, &info);
-#else
-    F77_CALL(chol) (mat, n, work, &zero, &zero, &info);
-#endif
     for(i = 0; i < *n; i++) {
 	work1[i * np1] = 1;
 	F77_CALL(dtrsl) (mat, n, n, work1 + i * (*n), &job, &info);
 	*logdet -= log(fabs(mat[i * np1]));
     }
     Memcpy(mat, work1, nsq);
-    Free(work); Free(work1);
+    R_Free(work); R_Free(work1);
 }
 
 void
-HF_factList(double *par, longint *maxC, longint *time, longint *pdims,
+HF_factList(double *par, int *maxC, int *time, int *pdims,
 	    double *FactorL, double *logdet)
 {
-    longint i, M = pdims[1], *len = pdims + 4;
+    int i, M = pdims[1], *len = pdims + 4;
     /* parameter assumed in unconstrained form */
     double inf = -1.0/(2.0 * ((double) *maxC));
     for(i = 0; i < *maxC; i++) {
@@ -839,21 +802,21 @@ HF_factList(double *par, longint *maxC, longint *time, longint *pdims,
 }
 
 void
-HF_recalc(double *Xy, longint *pdims, longint *ZXcol, double *par,
-	  longint *time, longint *maxC, double *logdet)
+HF_recalc(double *Xy, int *pdims, int *ZXcol, double *par,
+	  int *time, int *maxC, double *logdet)
 {
-    longint N = pdims[0], M = pdims[1], *len = pdims + 4,  *start = len + M, i;
+    int N = pdims[0], M = pdims[1], *len = pdims + 4,  *start = len + M, i;
     double inf = -1.0/(2.0 * ((double) *maxC));
     /* parameter assumed in unconstrained form */
     for(i = 0; i < *maxC; i++) {
 	par[i] = 2.0 * (exp(par[i]) + inf) + 1.0;
     }
     for(i = 0; i < M;  i++) {
-	double *Factor = Calloc(len[i] * len[i], double);
+	double *Factor = R_Calloc(len[i] * len[i], double);
 	HF_fact(par, time + start[i], &len[i], Factor, logdet);
 	mult_mat(Xy + start[i], N, Factor, len[i], len[i], len[i],
 		 Xy + start[i], N, *ZXcol);
-	Free(Factor);
+	R_Free(Factor);
     }
 }
 
@@ -913,10 +876,10 @@ dummy_corr(double val)
 /* methods for the virtual class */
 
 static void
-spatial_mat(double *par, double *dist, longint *n, longint *nug,
+spatial_mat(double *par, double *dist, int *n, int *nug,
 	    double (*corr)(double ), double *mat)
 {
-    longint i, j, np1 = *n + 1;
+    int i, j, np1 = *n + 1;
     double aux, *sdist, ratio = 1.0;
     sdist = dist;
     if (*nug) ratio = par[1];
@@ -930,10 +893,10 @@ spatial_mat(double *par, double *dist, longint *n, longint *nug,
 }
 
 void
-spatial_matList(double *par, longint *nug, double *dist, longint *pdims,
+spatial_matList(double *par, int *nug, double *dist, int *pdims,
 		double *minD, double *mat)
 {
-    longint i, M = pdims[1], spClass = pdims[2], *len = pdims + 4,
+    int i, M = pdims[1], spClass = pdims[2], *len = pdims + 4,
 	*start = len + M;
     double aux, (*corr)(double ) = dummy_corr;
     /* parameter assumed in unconstrained form */
@@ -971,35 +934,28 @@ spatial_matList(double *par, longint *nug, double *dist, longint *pdims,
 }
 
 static void
-spatial_fact(double *par, double *dist, longint *n, longint *nug,
+spatial_fact(double *par, double *dist, int *n, int *nug,
 	     double (*corr) (double ), double *mat,
 	     double *logdet)
 {
-    longint job = 11L, info, i, nsq = *n * (*n), np1 = *n + 1;
-    double *work = Calloc(*n, double), *work1 = Calloc(nsq, double);
-#ifndef USING_R
-    longint zero = 0L;
-#endif
+    int job = 11L, info, i, nsq = *n * (*n), np1 = *n + 1;
+    double *work = R_Calloc(*n, double), *work1 = R_Calloc(nsq, double);
     spatial_mat(par, dist, n, nug, corr, mat);
-#ifdef USING_R
     F77_CALL(chol) (mat, n, n, mat, &info);
-#else
-    F77_CALL(chol) (mat, n, work, &zero, &zero, &info);
-#endif
     for(i = 0; i < *n; i++) {
 	work1[i * np1] = 1;
 	F77_CALL(dtrsl) (mat, n, n, work1 + i * (*n), &job, &info);
 	*logdet -= log(fabs(mat[i * np1]));
     }
     Memcpy(mat, work1, nsq);
-    Free(work); Free(work1);
+    R_Free(work); R_Free(work1);
 }
 
 void
-spatial_factList(double *par, longint *nug, double *dist, longint *pdims,
+spatial_factList(double *par, int *nug, double *dist, int *pdims,
 		 double *minD, double *FactorL, double *logdet)
 {
-    longint i, M = pdims[1], spClass = pdims[2], *len = pdims + 4,
+    int i, M = pdims[1], spClass = pdims[2], *len = pdims + 4,
 	*start = len + M;
     double aux, (*corr)(double ) = dummy_corr;
     /* parameter assumed in unconstrained form */
@@ -1038,10 +994,10 @@ spatial_factList(double *par, longint *nug, double *dist, longint *pdims,
 }
 
 void
-spatial_recalc(double *Xy, longint *pdims, longint *ZXcol, double *par,
-	       double *dist, double *minD, longint *nug, double *logdet)
+spatial_recalc(double *Xy, int *pdims, int *ZXcol, double *par,
+	       double *dist, double *minD, int *nug, double *logdet)
 {
-    longint N = pdims[0], M = pdims[1], spClass = pdims[2],
+    int N = pdims[0], M = pdims[1], spClass = pdims[2],
 	*len = pdims + 4, *start = len + M, i;
     double aux, (*corr)(double ) = dummy_corr, *sXy;
     /* parameter assumed in unconstrained form */
@@ -1075,10 +1031,10 @@ spatial_recalc(double *Xy, longint *pdims, longint *ZXcol, double *par,
     }
 
     for(i = 0, sXy = Xy; i < M;  i++) {
-	double *Factor = Calloc(len[i] * len[i], double);
+	double *Factor = R_Calloc(len[i] * len[i], double);
 	spatial_fact(par, dist + start[i], &len[i], nug, corr, Factor, logdet);
 	mult_mat(sXy, N, Factor, len[i], len[i], len[i], sXy, N, *ZXcol);
 	sXy += len[i];
-	Free(Factor);
+	R_Free(Factor);
     }
 }
