@@ -498,11 +498,7 @@ window newwindow(const char *name, rect r, long flags)
     return (window) obj;
 }
 
-/*
- *  Find the next valid window, start looking from the given one.
- */
-PROTECTED
-object find_valid_sibling(object obj)
+static object find_valid_sibling(object obj, int next)
 {
     object first = obj;
 
@@ -515,18 +511,36 @@ object find_valid_sibling(object obj)
 	    if (obj->kind != LabelObject)
 		return obj;
 	}
-	obj = obj->next;
+	obj = next ? obj->next : obj->prev;
     } while (obj != first);
 
     return first;
 }
 
+/*
+ *  Find the next valid window, start looking from the given one.
+ */
+PROTECTED
+object find_next_valid_sibling(object obj)
+{
+    return find_valid_sibling(obj, 1);
+}
+
+/*
+ *  Find the previus valid window, start looking from the given one.
+ */
+PROTECTED
+object find_prev_valid_sibling(object obj)
+{
+    return find_valid_sibling(obj, 0);
+}
+
 /*
  *  Set focus to an object. It must be visible and enabled.
  */
 static void select_sibling(object obj)
 {
-    obj = find_valid_sibling(obj);
+    obj = find_next_valid_sibling(obj);
     if (obj) {
 	if (obj->flags & Document)
 	    sendmessage(hwndClient, WM_MDIACTIVATE,
