@@ -17,7 +17,7 @@
  *   Edin Hodzic, Eric J Bivona, Kai Uwe Rommel, Danny Quah, Ulrich Betzler
  */
 
- /* Copyright (C) 2018-2021 The R Core Team */
+ /* Copyright (C) 2018-2022 The R Core Team */
 
 #include       "getline.h"
 
@@ -264,11 +264,13 @@ gl_getc(void)
 	    c = gl_alt_to_ucs(bbb);
 	    bbb = 0;
 	    nAlt = 0;
+	    AltIsDown = 0;
 	  } else if (hex && nAlt==8) {
             c = bbb;
             bbb = 0;
             nAlt = 0;
             hex = 0;
+	    AltIsDown = 0;
           }
 	}
 	/* Originally, these (LEFT, RIGHT, HOME, END, UP, DOWN, DELETE) were
@@ -307,12 +309,17 @@ gl_getc(void)
 	    high = 0;
 	} else if (hex && wc==0) {
 	    c = bbb;
-	} else if (wc == 0) {
-	    /* Handle Alt+xxx */
+	} else if (bbb) {
+	    /* Handle Alt+xxx (Alt+xx).
+	       Console implementations differ in whether and how they interpret
+	       Alt+xxx sequences. Some translate internally and do not send
+	       Alt+xxx to R. Some send the Alt+xxx to R but also interpret and
+	       send the result in Alt key up event (wc). See PR#18391. */
 	    c = gl_alt_to_ucs(bbb);
-	} else 
-	    /* E.g. combining diacritical marks appear to arrive this way. */
+	} else
 	    c = wc;
+	/* This may have to be re-visited when extending support for combining
+	   marks, which have been seen arriving in Alt key up event as well. */
 	AltIsDown = 0;
 	nAlt = 0;
 	bbb = 0;
@@ -341,11 +348,13 @@ gl_getc(void)
 	  c = gl_alt_to_ucs(bbb);
 	  bbb = 0;
 	  nAlt = 0;
+	  AltIsDown = 0;
         } else if (hex && nAlt==8) {
 	  c = bbb;
 	  bbb = 0;
 	  nAlt = 0;
 	  hex = 0;
+	  AltIsDown = 0;
         }
       } 
     }
@@ -1298,7 +1307,7 @@ size_t (*func)();
     }
 }
 
-/* lenght of string in widths */
+/* length of string in widths */
 static size_t
 gl_w_strlen(const char *s)
 {
