@@ -139,7 +139,7 @@ void wgl_savehistory(const char *file, int size)
     fp = fopen(file, "w");
     if (!fp) {
        char msg[256];
-       sprintf(msg, "Unable to open %s", file);
+       snprintf(msg, 256, "Unable to open %s", file);
        R_ShowMessage(msg);
        return;
     }
@@ -175,7 +175,7 @@ void wgl_savehistoryW(const wchar_t *file, int size)
     fp = _wfopen(file, L"w");
     if (!fp) {
        char msg[256];
-       sprintf(msg, "Unable to open %ls", file);
+       snprintf(msg, 256, "Unable to open %ls", file);
        R_ShowMessage(msg);
        return;
     }
