@@ -1,12 +1,16 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
- *  Copyright (C) 1998-2013   The R Core Team
+ *  Copyright (C) 1998-2018   The R Core Team
  *
- *  This program is free software; you can redistribute it and/or modify
+ *  This header file is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License as published by
  *  the Free Software Foundation; either version 2.1 of the License, or
  *  (at your option) any later version.
  *
+ *  This file is part of R. R is distributed under the terms of the
+ *  GNU General Public License, either Version 2, June 1991 or Version 3,
+ *  June 2007. See doc/COPYRIGHTS for details of the copyright status of R.
+ *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
@@ -84,6 +88,7 @@ void samin(int n, double *pb, double *yb, optimfn fn, int maxit,
 int findInterval(double *xt, int n, double x,
 		 Rboolean rightmost_closed,  Rboolean all_inside, int ilo,
 		 int *mflag);
+// findInterval2() is only in Utils.h (and hence Rinternals.h)
 
 
 /* ------------------ Entry points NOT in the R API --------------- */
@@ -108,7 +113,7 @@ void F77_NAME(dqrxb)(double *x, int *n, int *k, double *qraux,
 
 /* appl/pretty.c: for use in engine.c and util.c */
 double R_pretty(double *lo, double *up, int *ndiv, int min_n,
-		double shrink_sml, double high_u_fact[],
+		double shrink_sml, const double high_u_fact[],
 		int eps_correction, int return_bounds);
 
 
