@@ -196,6 +196,21 @@
 	 }							        \
     } while (0)
 
+#define ITERATE_BY_REGION_PARTIAL_REV0(sx, px, idx, nb, etype, vtype,	\
+				       strt, nfull, expr) do {		\
+	etype __ibr_buf__[GET_REGION_BUFSIZE];				\
+	R_xlen_t __ibr_n__ = strt + nfull;				\
+	R_xlen_t nb = nfull > GET_REGION_BUFSIZE ?			\
+	    GET_REGION_BUFSIZE : nfull;					\
+	for (R_xlen_t idx = __ibr_n__ - nb; nb > 0; idx -= nb) {	\
+	    etype *px = (etype *) GET_REGION_PTR(sx, idx, nb,		\
+	                                         __ibr_buf__, vtype);	\
+	    expr							\
+	    nb = idx - strt > GET_REGION_BUFSIZE ?			\
+		GET_REGION_BUFSIZE :  idx - strt;			\
+	 }							        \
+    } while (0)
+
 #define ITERATE_BY_REGION_PARTIAL(sx, px, idx, nb, etype, vtype,	\
 				  strt, nfull, expr) do {		\
 	const etype *px = (etype *) DATAPTR_OR_NULL(sx);		\
@@ -210,6 +225,19 @@
 					strt, nfull, expr);		\
     } while (0)
 
+#define ITERATE_BY_REGION_PARTIAL_REV(sx, px, idx, nb, etype, vtype,	\
+				      strt, nfull, expr) do {		\
+	const etype *px = (etype *) DATAPTR_OR_NULL(sx);		\
+	if (px != NULL) {						\
+	    R_xlen_t idx = strt;					\
+	    (void) idx; /* variable may be unused in expr */		\
+	    R_xlen_t nb = nfull;					\
+	    px += strt;							\
+	    expr							\
+	}								\
+	else ITERATE_BY_REGION_PARTIAL_REV0(sx, px, idx, nb, etype,	\
+					    vtype, strt, nfull, expr);	\
+    } while (0)
 
 #define ITERATE_BY_REGION(sx, px, idx, nb, etype, vtype, expr) do {	\
 	ITERATE_BY_REGION_PARTIAL(sx, px, idx, nb, etype, vtype,	\
