@@ -1,7 +1,7 @@
 #  File src/library/base/R/Defunct.R
 #  Part of the R package, https://www.R-project.org
 #
-#  Copyright (C) 1995-2014 The R Core Team
+#  Copyright (C) 1995-2023 The R Core Team
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -180,32 +180,54 @@
 ##     .Defunct(msg = "namespaces should be specified via the 'NAMESPACE' file")
 ## .Export <- function(...)
 ##     .Defunct(msg = "namespaces should be specified via the 'NAMESPACE' file")
-## .S3method <- function(generic, class, method)
-##     .Defunct(msg = "namespaces should be specified via the 'NAMESPACE' file")
 ## </entry>
 
 ## <entry>
 ## Deprecated in 2.14.0
 ## Defunct in 2.15.0
-mem.limits <- function(nsize=NA, vsize=NA) .Defunct("gc")
+## Removed for 4.0.0
+## mem.limits <- function(nsize=NA, vsize=NA) .Defunct("gc")
 ## </entry>
 
 ## <entry>
 ## Deprecated in 2.13.1
 ## Defunct in 2.15.0
-.readRDS <- function(...) .Defunct("readRDS")
-.saveRDS <- function(...) .Defunct("saveRDS")
+## Removed for 4.0.0
+## .readRDS <- function(...) .Defunct("readRDS")
+## .saveRDS <- function(...) .Defunct("saveRDS")
 ## </entry>
 
 ## <entry>
 ## Deprecated in 2.5.0
-## Removed in 2.15.0
+## Defunct and Removed in 2.15.0
 # Sys.putenv <- function(...) .Defunct("Sys.setenv")
 ## </entry>
 
 ## <entry>
 ## Deprecated in 3.0.0
 ## Defunct in 3.1.0
-.find.package <- function(...).Defunct("find.package")
-.path.package <- function(...).Defunct("path.package")
+## Removed for 4.1.0
+## .find.package <- function(...).Defunct("find.package")
+## .path.package <- function(...).Defunct("path.package")
 ## </entry>
+
+## <entry>
+## Deprecated in 3.4.0
+## Defunct in 4.2.0
+unix.time <- function(...) .Defunct("system.time")
+## </entry>
+
+## <entry>
+## Deprecated in 4.2.0
+## Defunct in 4.3.0
+default.stringsAsFactors <- function()
+    .Defunct("`stringsAsFactors = FALSE`")
+## </entry>
+
+## <entry>
+## Deprecated in 3.5.0
+## Defunct in 4.3.0
+.__H__.cbind <- function(..., deparse.level = 1) .Defunct("base::cbind")
+.__H__.rbind <- function(..., deparse.level = 1) .Defunct("base::rbind")
+## </entry>
+
