@@ -61,7 +61,7 @@ getNativeSymbolInfo <- function(name, PACKAGE, unlist = TRUE,
 		msg <- paste(msg, "in package", pkgName)
 	    stop(msg, domain = NA)
 	}
-	names(v) <- c("name", "address", "package", "numParameters")[seq_along(v)]
+	names(v) <- c("name", "address", "dll", "numParameters")[seq_along(v)]
 	v
     })
 
@@ -125,9 +125,13 @@ getDLLRegisteredRoutines.DLLInfo <- function(dll, addNames = TRUE)
 print.NativeRoutineList <-
 function(x, ...)
 {
-    m <- data.frame(numParameters = sapply(x, function(x) x$numParameters),
-                    row.names = sapply(x, function(x) x$name))
-    print(m, ...)
+    if(length(x)) {    
+        m <- data.frame(numParameters =
+                            sapply(x, function(x) x$numParameters),
+                        row.names =
+                            sapply(x, function(x) x$name))
+        print(m, ...)
+    }
     invisible(x)
 }
 
@@ -210,7 +214,3 @@ print.DLLInfoList <- function(x, ...)
 
 `$.DLLInfo` <- function(x, name)
     getNativeSymbolInfo(as.character(name), PACKAGE = x)
-
-
-
-
