@@ -1,7 +1,7 @@
 #  File src/library/base/R/methodsSupport.R
 #  Part of the R package, https://www.R-project.org
 #
-#  Copyright (C) 1995-2014 The R Core Team
+#  Copyright (C) 1995-2015 The R Core Team
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -16,14 +16,14 @@
 #  A copy of the GNU General Public License is available at
 #  https://www.R-project.org/Licenses/
 
-trace <- function(what, tracer, exit, at, print, signature, where = topenv(parent.frame()), edit = FALSE)
+trace <- function(what, tracer, exit, at, print, signature,
+                  where = topenv(parent.frame()), edit = FALSE)
 {
-    needsAttach <- nargs() > 1L && !.isMethodsDispatchOn()
-    if(needsAttach) {
+    if(nargs() > 1L && !.isMethodsDispatchOn()) {
         ns <- try(loadNamespace("methods"))
         if(isNamespace(ns))
             message("(loaded the methods namespace)", domain = NA)
-        else
+        else ## (should not be possible)
             stop("tracing functions requires the 'methods' package, but unable to load the 'methods' namespace")
     }
     else if(nargs() == 1L)
@@ -33,34 +33,24 @@ trace <- function(what, tracer, exit, at, print, signature, where = topenv(paren
     ## now call the version in the methods package, to ensure we get
     ## the correct namespace (e.g., correct version of class())
     call <- sys.call()
-    call[[1L]] <- quote(methods::.TraceWithMethods)
+    call[[1L]] <- quote(methods:::.TraceWithMethods)
     call$where <- where
-    value <- eval.parent(call)
-    on.exit() ## no error
-    tracingState(tState)
-    value
+    eval.parent(call)
 }
 
 untrace <- function(what, signature = NULL, where = topenv(parent.frame())) {
-    ## NOTE: following test is TRUE after loadNamespace("methods") (even if not in search())
-    MethodsDispatchOn <- .isMethodsDispatchOn()
-    if(MethodsDispatchOn) {
-        tState <- tracingState(FALSE)
-        on.exit(tracingState(tState))
-    }
-    if(!MethodsDispatchOn)
-        return(.primUntrace(what)) ## can't have called trace except in primitive form
+    if(!.isMethodsDispatchOn()) ## can't have called trace except in primitive form
+        return(.primUntrace(what))
     ## at this point we can believe that the methods namespace was successfully loaded
+    tState <- tracingState(FALSE)
+    on.exit(tracingState(tState))
     ## now call the version in the methods package, to ensure we get
     ## the correct namespace (e.g., correct version of class())
     call <- sys.call()
-    call[[1L]] <- quote(methods::.TraceWithMethods)
+    call[[1L]] <- quote(methods:::.TraceWithMethods)
     call$where <- where
     call$untrace <- TRUE
-    value <- eval.parent(call)
-    on.exit() ## no error
-    tracingState(tState)
-    invisible(value)
+    invisible(eval.parent(call))
 }
 
 
