@@ -1,7 +1,7 @@
 #  File src/library/base/R/split.R
 #  Part of the R package, https://www.R-project.org
 #
-#  Copyright (C) 1995-2016 The R Core Team
+#  Copyright (C) 1995-2018 The R Core Team
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -30,18 +30,17 @@ split.default <- function(x, f, drop = FALSE, sep = ".", lex.order = FALSE, ...)
     if (is.null(attr(x, "class")))
 	return(.Internal(split(x, f)))
     ## else
-    lf <- levels(f)
-    y <- vector("list", length(lf))
-    names(y) <- lf
     ind <- .Internal(split(seq_along(x), f))
-    for(k in lf) y[[k]] <- x[ind[[k]]]
-    y
+    lapply(ind, function(i) x[i])
 }
 
 ## This is documented to work for matrices too
 split.data.frame <- function(x, f, drop = FALSE, ...)
+{
+    if (inherits(f, "formula")) f <- .formula2varlist(f, x)
     lapply(split(x = seq_len(nrow(x)), f = f, drop = drop, ...),
            function(ind) x[ind, , drop = FALSE])
+}
 
 `split<-` <- function(x, f, drop = FALSE, ..., value) UseMethod("split<-")
 
@@ -60,6 +59,7 @@ split.data.frame <- function(x, f, drop = FALSE, ...)
 ## This is documented to work for matrices too
 `split<-.data.frame` <- function(x, f, drop = FALSE, ..., value)
 {
+    if (inherits(f, "formula")) f <- eval(attr(stats::terms(f), "variables"), x, environment(f))
     ix <- split(seq_len(nrow(x)), f, drop = drop, ...)
     n <- length(value)
     j <- 0
@@ -70,14 +70,39 @@ split.data.frame <- function(x, f, drop = FALSE, ...)
     x
 }
 
+## (Note: use rep(NA_integer_, len) for indexing here. Logical NA confuses 
+## tibbles and only coincidentally does the right thing otherwise, 
+## because len is longer than value[[1]]. Otherwise recycling would kick in.) 
 unsplit <- function (value, f, drop = FALSE)
 {
     len <- length(if (is.list(f)) f[[1L]] else f)
     if (is.data.frame(value[[1L]])) {
-        x <- value[[1L]][rep(NA, len),, drop = FALSE]
+        x <- value[[1L]][rep(NA_integer_, len),, drop = FALSE]
         rownames(x) <- unsplit(lapply(value, rownames), f, drop = drop)
     } else
-        x <- value[[1L]][rep(NA, len)]
+        x <- value[[1L]][rep(NA_integer_, len)]
     split(x, f, drop = drop) <- value
     x
 }
+
+## utility to convert formula to list suitable for split(f = )
+
+
+.formula2varlist <- function(formula, data, warnLHS = TRUE, ignoreLHS = warnLHS)
+{
+    if (!inherits(formula, "formula")) stop("'formula' must be a formula")
+    if (!is.list(data) && !is.environment(data)) data <- as.data.frame(data)
+    if (length(formula) == 3) {
+        if (warnLHS) {
+            if (ignoreLHS)
+                warning("Unexpected LHS in 'formula' has been ignored.")
+            else
+                warning("Unexpected LHS in 'formula' has been combined with RHS.")
+        }
+        if (ignoreLHS) formula <- formula[-2]
+    }
+    fterms <- stats::terms(formula)
+    ans <- eval(attr(fterms, "variables"), data, environment(formula))
+    names(ans) <- attr(fterms, "term.labels")
+    ans
+}
