@@ -1,7 +1,7 @@
 #  File src/library/base/R/version.R
 #  Part of the R package, https://www.R-project.org
 #
-#  Copyright (C) 1995-2015, 2017 The R Core Team
+#  Copyright (C) 1995-2023 The R Core Team
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -162,8 +162,6 @@ function(x)
 
 ## <NOTE>
 ## Currently unused.
-## </NOTE>
-
 .decode_numeric_version <-
 function(x)
 {
@@ -179,6 +177,7 @@ function(x)
     class(y) <-  unique(c(attr(x, ".classes"), "numeric_version"))
     y
 }
+## </NOTE>
 
 ## Methods.
 
@@ -188,11 +187,11 @@ function(x, i, j)
     y <- if(missing(j))
         unclass(x)[i]
     else
-        lapply(unclass(x)[i], "[", j)
+        lapply(unclass(x)[i], `[`, j)
     ## Change sequences which are NULL or contains NAs to integer().
     bad <- vapply(y, function(t) is.null(t) || anyNA(t), NA)
     if(any(bad))
-        y[bad] <- rep.int(list(integer()), length(bad))
+        y[bad] <- rep.int(list(integer()), sum(bad))
     class(y) <- class(x)
     y
 }
@@ -222,7 +221,7 @@ function(x, i, j, value)
 `[[.numeric_version` <-
 function(x, ..., exact = NA)
 {
-   if(length(list(...)) < 2L)
+   if(...length() < 2L)
       structure(list(unclass(x)[[..., exact=exact]]), class = oldClass(x))
    else
       unclass(x)[[..1, exact=exact]][..2]
@@ -266,14 +265,8 @@ function(e1, e2)
                       .Generic), domain = NA)
     if(!is.numeric_version(e1)) e1 <- as.numeric_version(e1)
     if(!is.numeric_version(e2)) e2 <- as.numeric_version(e2)
-    n1 <- length(e1)
-    n2 <- length(e2)
-    if(!n1 || !n2) return(logical())
-    e <- split(.encode_numeric_version(c(e1, e2)),
-               rep.int(c(1L, 2L), c(n1, n2)))
-    e1 <- e[[1L]]
-    e2 <- e[[2L]]
-    NextMethod(.Generic)
+    op <- get(.Generic, mode = "function")
+    op(.Internal(compareNumericVersion(e1, e2)), 0L)
 }
 
 Summary.numeric_version <-
@@ -283,7 +276,7 @@ function(..., na.rm)
     if(!ok)
         stop(gettextf("%s not defined for \"numeric_version\" objects",
                       .Generic), domain = NA)
-    x <- do.call("c", lapply(list(...), as.numeric_version))
+    x <- do.call(c, lapply(list(...), as.numeric_version))
     v <- xtfrm(x)
     if(!na.rm && length(pos <- which(is.na(v)))) {
         y <- x[pos[1L]]
@@ -354,7 +347,7 @@ function(x)
 `is.na<-.numeric_version` <-
 function(x, value)
 {
-    x[value] <- rep.int(list(integer()), length(value))
+    x[value] <- list(integer())
     x
 }
 
@@ -369,15 +362,13 @@ function(x, recursive = FALSE)
 }
 
 print.numeric_version <-
-function(x, ...)
+function(x, quote = FALSE, ...)
 {
     y <- as.character(x)
     if(!length(y))
         writeLines(gettext("<0 elements>"))
-    else if(any("quote" == names(list(...))))
-	print(ifelse(is.na(y), NA_character_, sQuote(y)), ...)
     else
-	print(ifelse(is.na(y), NA_character_, sQuote(y)), quote = FALSE, ...)
+        print(ifelse(is.na(y), NA_character_, sQuote(y)), quote=quote, ...)
     invisible(x)
 }
 
@@ -408,7 +399,7 @@ function(x, name)
     name <- pmatch(name, c("major", "minor", "patchlevel"))
     x <- unclass(x)
     switch(name,
-	   major = vapply(x, "[", 0L, 1L),
-	   minor = vapply(x, "[", 0L, 2L),
-	   patchlevel = vapply(x, "[", 0L, 3L))
+	   major = vapply(x, `[`, 0L, 1L),
+	   minor = vapply(x, `[`, 0L, 2L),
+	   patchlevel = vapply(x, `[`, 0L, 3L))
 }
