@@ -1,7 +1,7 @@
 #  File src/library/grDevices/R/postscript.R
 #  Part of the R package, https://www.R-project.org
 #
-#  Copyright (C) 1995-2019 The R Core Team
+#  Copyright (C) 1995-2023 The R Core Team
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -259,7 +259,7 @@ postscript <- function(file = if(onefile) "Rplots.ps" else "Rplot%03d.ps",
             stop("invalid 'family' argument")
         old$family <- family
     }
-    if(grepl("[\n\r\f\127]", old$title))
+    if(grepl("[\n\r\f\177]", old$title))
         ## title with these characters generates corrupt postscript file
         stop(gettextf("'title' argument \"%s\" contains invalid characters",
                       old$title), domain = NA)
@@ -267,6 +267,8 @@ postscript <- function(file = if(onefile) "Rplots.ps" else "Rplot%03d.ps",
     onefile <- old$onefile # for 'file'
     if(!checkIntFormat(file))
         stop(gettextf("invalid 'file' argument '%s'", file), domain = NA)
+    if (!is.na(pmatch("MacRoman", old$encoding)))
+        warning("encoding MacRoman is deprecated")
     .External(C_PostScript,
               file, old$paper, old$family, old$encoding, old$bg, old$fg,
               old$width, old$height, old$horizontal, old$pointsize,
@@ -369,6 +371,8 @@ pdf <- function(file = if(onefile) "Rplots.pdf" else "Rplot%03d.pdf",
     onefile <- old$onefile # needed to set 'file'
     if(!checkIntFormat(file))
         stop(gettextf("invalid 'file' argument '%s'", file), domain = NA)
+    if (!is.na(pmatch("MacRoman", old$encoding)))
+        warning("encoding MacRoman is deprecated")
     .External(C_PDF,
               file, old$paper, old$family, old$encoding, old$bg, old$fg,
               old$width, old$height, old$pointsize, onefile, old$pagecentre,
@@ -989,3 +993,44 @@ embedFonts <- function(file, # The ps or pdf file to convert
     file.copy(tmpfile, outfile, overwrite = TRUE)
     invisible(cmd)
 }
+
+## 'file' is the pdf file to convert
+## 'glyphInfo' is RGlyphInfo
+## 'outfile' is the new pdf file
+## 'options' are additional options to ghostscript
+embedGlyphs <- function(file, glyphInfo, outfile = file,
+                        options = character()) {
+    if (!is.character(file) || length(file) != 1L || !nzchar(file))
+        stop("'file' must be a non-empty character string")
+    if (!inherits(glyphInfo, "RGlyphInfo"))
+        stop("Invalid 'glyphInfo'")
+    gsexe <- tools::find_gs_cmd()
+    if(!nzchar(gsexe)) stop("GhostScript was not found")
+    if(.Platform$OS.type == "windows") gsexe <- shortPathName(gsexe)
+    format <- "pdfwrite"
+    check_gs_type(gsexe, format)
+    tmpfile <- tempfile("Rembed")
+    ## Generate cidfmap to relate font names to font files
+    cidfmap <- file.path(tempdir(), "cidfmap")
+    fontfile <- unique(sapply(glyphInfo$fonts, function(x) x$file))
+    fontname <- unique(sapply(glyphInfo$fonts, function(x) x$PSname))
+    writeLines(paste0("/", fontname,
+                      " << /FileType /TrueType /Path (", fontfile,
+                      ") /SubfontID 0 /CSI [(Identity) 0] >>;"),
+               cidfmap)
+    args <- c(paste0("-dNOPAUSE -dBATCH -q -dAutoRotatePages=/None ",
+                     "-sDEVICE=", format),
+              paste0("-sOutputFile=", shQuote(tmpfile)),
+              ## Make sure ghostscript can see the cidfmap
+              paste0("-I", shQuote(tempdir())),
+              options, shQuote(file))
+    ret <- system2(gsexe, args)
+    if (ret != 0)
+        stop(gettextf("status %d in running command '%s'", ret, cmd),
+             domain = NA)
+    if (outfile != file)
+        args[2] <- paste0(" -sOutputFile=", shQuote(outfile))
+    cmd <- paste(c(shQuote(gsexe), args), collapse = " ")
+    file.copy(tmpfile, outfile, overwrite = TRUE)
+    invisible(cmd)
+}
