@@ -1,7 +1,7 @@
 #  File src/library/graphics/R/legend.R
 #  Part of the R package, https://www.R-project.org
 #
-#  Copyright (C) 1995-2014 The R Core Team
+#  Copyright (C) 1995-2022 The R Core Team
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -25,12 +25,12 @@ function(x, y = NULL, legend, fill = NULL, col = par("col"), border="black",
 	 text.width = NULL, text.col = par("col"), text.font = NULL,
 	 merge = do.lines && has.pch, trace = FALSE,
 	 plot = TRUE, ncol = 1, horiz = FALSE, title = NULL,
-	 inset = 0, xpd, title.col = text.col, title.adj = 0.5,
+	 inset = 0, xpd, title.col = text.col[1], title.adj = 0.5, 
+	 title.cex = cex[1], title.font = text.font[1],
          seg.len = 2)
 {
     ## the 2nd arg may really be `legend'
-    if(missing(legend) && !missing(y) &&
-       (is.character(y) || is.expression(y))) {
+    if(missing(legend) && !missing(y) && (is.character(y) || is.expression(y))) {
 	legend <- y
 	y <- NULL
     }
@@ -41,10 +41,14 @@ function(x, y = NULL, legend, fill = NULL, col = par("col"), border="black",
         on.exit(par(xpd=op))
         par(xpd=xpd)
     }
+    text.font <- if(is.null(text.font)) par("font") else text.font    
     title <- as.graphicsAnnot(title)
     if(length(title) > 1) stop("invalid 'title'")
     legend <- as.graphicsAnnot(legend)
-    n.leg <- if(is.call(legend)) 1 else length(legend)
+	## if legend is a length one call or a list containing at least one call, 
+	## we want to convert to an expression:
+	if(any(sapply(legend, is.language))) legend <- as.expression(legend)
+    n.leg <- length(legend)
     if(n.leg == 0) stop("'legend' is of length 0")
     auto <-
 	if (is.character(x))
@@ -53,14 +57,21 @@ function(x, y = NULL, legend, fill = NULL, col = par("col"), border="black",
 	else NA
 
     if (is.na(auto)) {
-	xy <- xy.coords(x, y); x <- xy$x; y <- xy$y
+	xy <- xy.coords(x, y, setLab = FALSE); x <- xy$x; y <- xy$y
 	nx <- length(x)
 	if (nx < 1 || nx > 2) stop("invalid coordinate lengths")
     } else nx <- 0
 
+    reverse.xaxis <- par("xaxp")[1] > par("xaxp")[2]
+    reverse.yaxis <- par("yaxp")[1] > par("yaxp")[2]    
     xlog <- par("xlog")
     ylog <- par("ylog")
 
+    ## recycle
+    cex <- rep(cex, length.out=n.leg)
+    x.intersp <- rep(x.intersp, length.out=n.leg)
+    seg.len <- rep(seg.len, length.out=n.leg)
+
     rect2 <- function(left, top, dx, dy, density = NULL, angle, ...) {
 	r <- left + dx; if(xlog) { left <- 10^left; r <- 10^r }
 	b <- top  - dy; if(ylog) {  top <- 10^top;  b <- 10^b }
@@ -82,42 +93,29 @@ function(x, y = NULL, legend, fill = NULL, col = par("col"), border="black",
 	if(ylog) y <- 10^y
 	text(x, y, ...)
     }
-    if(trace)
+    colwise <- function(x, n, ncol, n.legpercol, fun, reverse=FALSE){
+	## needed for legend with >1 columns
+	xmat <- matrix(c(rep(x, length.out=n), rep(0L, n.legpercol * ncol - n)), ncol=ncol)
+	res <- apply(xmat, 2, fun)
+	res[res == 0L] <- max(res) # space for empty columns
+	if(reverse) -res else res
+    }
+    rowwise <- function(x, n, ncol, n.legpercol, fun, reverse=FALSE){
+	## needed for legend with >1 rows
+	xmat <- matrix(c(rep(x, length.out=n), rep(0L, n.legpercol * ncol - n)), ncol=ncol)
+	res <- apply(xmat, 1, fun)
+	if(reverse) -res else res
+    }
+    if(trace) {
 	catn <- function(...)
-	    do.call("cat", c(lapply(list(...),formatC), list("\n")))
-
-    cin <- par("cin")
-    Cex <- cex * par("cex")		# = the `effective' cex for text
-
-    ## at this point we want positive width even for reversed x axis.
-    if(is.null(text.width))
-	text.width <- max(abs(strwidth(legend, units="user",
-				       cex=cex, font = text.font)))
-    else if(!is.numeric(text.width) || text.width < 0)
-	stop("'text.width' must be numeric, >= 0")
-
-    xc <- Cex * xinch(cin[1L], warn.log=FALSE) # [uses par("usr") and "pin"]
-    yc <- Cex * yinch(cin[2L], warn.log=FALSE)
-    if(xc < 0) text.width <- -text.width
-
-    xchar  <- xc
-    xextra <- 0
-    yextra <- yc * (y.intersp - 1)
-    ## watch out for reversed axis here: heights can be negative
-    ymax   <- yc * max(1, strheight(legend, units="user", cex=cex)/yc)
-    ychar <- yextra + ymax
-    if(trace) catn("  xchar=", xchar, "; (yextra,ychar)=", c(yextra,ychar))
-
-    if(mfill) {
-	##= sizes of filled boxes.
-	xbox <- xc * 0.8
-	ybox <- yc * 0.5
-	dx.fill <- xbox ## + x.intersp*xchar
+	    do.call(cat, c(lapply(list(...),formatC), "\n"))
+        fv <- function(...)
+            paste(vapply(lapply(list(...), formatC),
+                         paste, collapse=",", ""),
+                  collapse=", ")
     }
-    do.lines <- (!missing(lty) && (is.character(lty) || any(lty > 0))
-		 ) || !missing(lwd)
 
-    ## legends per column:
+    ## number of ("rbinded") legends _per_ column:
     n.legpercol <-
 	if(horiz) {
 	    if(ncol != 1)
@@ -127,6 +125,54 @@ function(x, y = NULL, legend, fill = NULL, col = par("col"), border="black",
 	    1
 	} else ceiling(n.leg / ncol)
 
+    Cex <- cex * par("cex") # = the `effective' cex for text
+
+    ## at this point we want positive width even for reversed x axis.
+    if(is.null(text.width))
+	text.width <- max(abs(mapply(strwidth, legend, cex = cex,
+				     font = text.font, MoreArgs = list(units = "user"))))
+    else if((length(text.width) > 1L && any(is.na(text.width)))  ||
+            (all(!is.na(text.width)) && (!is.numeric(text.width) || any(text.width < 0))))
+	stop("'text.width' must be numeric, >= 0, or a scalar NA")
+
+    if(auto.text.width <- all(is.na(text.width))) {
+	## textwidth for each legend element
+	text.width <- abs(mapply(strwidth, legend, cex = cex, 
+				 font = text.font, MoreArgs = list(units = "user")))
+	ncol <- ceiling(n.leg / n.legpercol) # adjust ncol to the really needed number
+    }
+
+    xyc <- xyinch(par("cin"), warn.log=FALSE) # [uses par("usr") and "pin"]
+    xc <- Cex * xyc[1L]
+    yc <- Cex * xyc[2L]
+    if(any(xc < 0)) text.width <- -text.width
+
+    xchar  <- xc
+    xextra <- 0
+
+    y.intersp <- rep(y.intersp, length.out=n.legpercol)
+    yextra <- rowwise(yc, n=n.leg, ncol=ncol, n.legpercol=n.legpercol, 
+                      fun=function(x) max(abs(x)), reverse = reverse.yaxis) * (y.intersp - 1)
+    ## watch out for reversed axis here: heights can be negative
+    ymax <- sign(yc[1]) * max(abs(yc)) *
+		max(1, mapply(strheight, legend, cex = cex, font = text.font,
+			      MoreArgs = list(units = "user"))/yc)
+    ychar <- yextra + ymax
+    ymaxtitle <- title.cex * par("cex") * xyc[2L] * 
+    	max(1, strheight(title, cex = title.cex, font = title.font, units = "user") / 
+		(title.cex * par("cex") * xyc[2L]))
+    ychartitle <- yextra[1] + ymaxtitle
+    if(trace) catn("  xchar=", fv(xchar), "; (yextra, ychar)=", fv(yextra,ychar))
+
+    if(mfill) {
+	##= sizes of filled boxes.
+	xbox <- xc * 0.8
+	ybox <- yc * 0.5
+	dx.fill <- max(xbox) ## + x.intersp*xchar
+    }
+    do.lines <- (!missing(lty) && (is.character(lty) || any(lty > 0))
+		 ) || !missing(lwd)
+
     has.pch <- !missing(pch) && length(pch) > 0 # -> default 'merge' is available
     if(do.lines) {
 	x.off <- if(merge) -0.7 else 0
@@ -164,17 +210,32 @@ function(x, y = NULL, legend, fill = NULL, col = par("col"), border="black",
 	y <- mean(y)
 	if(missing(xjust)) xjust <- 0.5
 	if(missing(yjust)) yjust <- 0.5
-
     }
     else {## nx == 1  or  auto
 	## -- (w,h) := (width,height) of the box to draw -- computed in steps
-	h <- (n.legpercol + !is.null(title)) * ychar + yc
-	w0 <- text.width + (x.intersp + 1) * xchar
+	yc <- rowwise(yc, n.leg, ncol, n.legpercol,
+		      fun = function(x) max(abs(x)), reverse = reverse.yaxis)
+	h <- sum(ychar) + yc[length(yc)] + (!is.null(title)) * ychartitle
+
+	## calculate optimal width per column, and other widths
+	xch1 <- colwise(xchar, n.leg, ncol, n.legpercol,
+			fun = function(x) max(abs(x)), reverse=reverse.xaxis)
+	x.interspCol <- colwise(x.intersp, n.leg, ncol, n.legpercol, fun = max)
+	seg.lenCol <- colwise(seg.len, n.leg, ncol, n.legpercol, fun = max)
+	text.width <- colwise(text.width, n = if(auto.text.width) n.leg else ncol, ncol,
+			      n.legpercol = if(auto.text.width) n.legpercol else 1,
+			      fun = function(x) max(abs(x)), reverse=reverse.xaxis)
+
+	w0 <- text.width + (x.interspCol + 1) * xch1
 	if(mfill)	w0 <- w0 + dx.fill
-	if(do.lines)		w0 <- w0 + (seg.len + x.off)*xchar
-	w <- ncol*w0 + .5* xchar
+	if(do.lines)	w0 <- w0 + (seg.lenCol + x.off)*xch1
+
+	w <- sum(w0) + 0.5 * xch1[ncol]   # width of box
+
 	if (!is.null(title)
-	    && (abs(tw <- strwidth(title, units="user", cex=cex) + 0.5*xchar)) > abs(w)) {
+	    && (abs(tw <- strwidth(title, units="user", 
+                               cex = title.cex, font = title.font) + 
+			0.5 * title.cex * par("cex") * xyc[1L])) > abs(w)) {
 	    xextra <- (tw - w)/2
 	    w <- tw
 	}
@@ -209,12 +270,13 @@ function(x, y = NULL, legend, fill = NULL, col = par("col"), border="black",
     }
 
     ## (xt[],yt[]) := `current' vectors of (x/y) legend text
-    xt <- left + xchar + xextra +
-	(w0 * rep.int(0:(ncol-1), rep.int(n.legpercol,ncol)))[1L:n.leg]
-    yt <- top -	0.5 * yextra - ymax -
-	(rep.int(1L:n.legpercol,ncol)[1L:n.leg] - 1 + !is.null(title)) * ychar
+    xt <- left + xc + xextra +  
+	rep(c(0, cumsum(w0))[1L:ncol], each=n.legpercol, length.out=n.leg)
+    topspace <- 0.5 * ymax + (!is.null(title)) * ychartitle
+    yt <- top -	topspace - cumsum((c(0, ychar)/2 + c(ychar, 0)/2)[1L:n.legpercol])
+    yt <- rep(yt, length.out=n.leg)
 
-    if (mfill) {		#- draw filled boxes -------------
+    if (mfill) {			#- draw filled boxes -------------
 	if(plot) {
 	    if(!is.null(fill)) fill <- rep_len(fill, n.leg)
 	    rect2(left = xt, top=yt+ybox/2, dx = xbox, dy = ybox,
@@ -236,11 +298,11 @@ function(x, y = NULL, legend, fill = NULL, col = par("col"), border="black",
 	lwd <- rep_len(lwd, n.leg)
 	ok.l <- !is.na(lty) & (is.character(lty) | lty > 0) & !is.na(lwd)
 	if(trace)
-	    catn("  segments2(",xt[ok.l] + x.off*xchar, ",", yt[ok.l],
-		 ", dx=", seg.len*xchar, ", dy=0, ...)")
+	    catn("  segments2(",xt[ok.l] + x.off*xchar[ok.l], ",", yt[ok.l],
+		 ", dx=", (seg.len*xchar)[ok.l], ", dy=0, ...)")
 	if(plot)
-	    segments2(xt[ok.l] + x.off*xchar, yt[ok.l],
-                      dx = seg.len*xchar, dy = 0,
+	    segments2(xt[ok.l] + x.off*xchar[ok.l], yt[ok.l],
+                      dx = (seg.len*xchar)[ok.l], dy = 0,
 		      lty = lty[ok.l], lwd = lwd[ok.l], col = col[ok.l])
 	# if (!merge)
 	xt <- xt + (seg.len+x.off) * xchar
@@ -265,14 +327,14 @@ function(x, y = NULL, legend, fill = NULL, col = par("col"), border="black",
 	if(plot)
 	    points2(x1, y1, pch = pch[ok], col = col[ok],
 		    cex = pt.cex[ok], bg = pt.bg[ok], lwd = pt.lwd[ok])
-##D	if (!merge) xt <- xt + dx.pch
     }
 
-    xt <- xt + x.intersp * xchar
+    xt <- xt + x.intersp * xc
     if(plot) {
 	if (!is.null(title))
-            text2(left + w*title.adj, top - ymax, labels = title,
-                  adj = c(title.adj, 0), cex = cex, col = title.col)
+            text2(left + w*title.adj, top - ymaxtitle, labels = title,
+                  adj = c(title.adj, 0), cex = title.cex, 
+                  col = title.col, font = title.font)
 
 	text2(xt, yt, labels = legend, adj = adj, cex = cex,
 	      col = text.col, font = text.font)
