@@ -1,7 +1,7 @@
 #  File src/library/methods/R/NextMethod.R
 #  Part of the R package, https://www.R-project.org
 #
-#  Copyright (C) 1995-2015 The R Core Team
+#  Copyright (C) 1995-2016 The R Core Team
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -24,19 +24,20 @@ callNextMethod <- function(...) {
     ## Because of the .local mechanism used to allow variable argument lists
     ## in methods (see rematchDefinition) these may be different.
     parent <- sys.parent(1)
-    maybeMethod <- sys.function(parent)
+    methodFun <- maybeMethod <- sys.function(parent)
     if(is(maybeMethod, "MethodDefinition")) {
         callEnv <- methodEnv <- parent.frame(1)
         mcall <- sys.call(parent)
         dotsenv <- parent.frame(2)
-        i <- 1
+        i <- 1L
     }
     else {
         callEnv <- parent.frame(1)
         methodEnv <- parent.frame(2)
         mcall <- sys.call(sys.parent(2))
         dotsenv <- parent.frame(3)
-        i <- 2
+        maybeMethod <- sys.function(sys.parent(2))
+        i <- 2L
     }
     ## set up the nextMethod object, load it
     ## into the calling environment, and cache it
@@ -47,10 +48,14 @@ callNextMethod <- function(...) {
     }
     else if(identical(mcall[[1L]], dotNextMethod)) {
         ## a call from another callNextMethod()
-        nextMethodEnv <- parent.frame(i+1)
+        nextMethodEnv <- parent.frame(i+1L)
         nextMethod <- nextMethodEnv$.nextMethod
         f <- nextMethodEnv$.Generic
     }
+    else if (is(maybeMethod, "MethodDefinition")) {
+        f <- maybeMethod@generic
+        method <- maybeMethod
+    }
     else {
         ## may be a method call for a primitive; not available as .Method
         if (is.primitive(mcall[[1L]])) {
@@ -58,7 +63,7 @@ callNextMethod <- function(...) {
         } else {
             f <- as.character(mcall[[1L]])
         }
-        fdef <- genericForPrimitive(f)
+        fdef <- genericForBasic(f)
         ## check that this could be a basic function with methods
         if(is.null(fdef))
             stop(gettextf("a call to callNextMethod() appears in a call to %s, but the call does not seem to come from either a generic function or another 'callNextMethod'",
@@ -119,16 +124,18 @@ callNextMethod <- function(...) {
                if(is.na(i) || i > length(call))
                    length(fnames) <- length(call)
                else {
-                   i <- i-1
+                   i <- i-1L
                    length(fnames) <- i
                    fnames <- c(fnames, rep("", length(call) - i))
                }
+               if (endsWith(f, "<-"))
+                   fnames[length(fnames)] <- "value"
                names(call) <- fnames
                call <- as.call(call)
            }
         }
         else
-            call <- match.call(maybeMethod, mcall, expand.dots = FALSE,
+            call <- match.call(methodFun, mcall, expand.dots = FALSE,
                                envir = dotsenv)
         .Call(C_R_nextMethodCall, call, callEnv)
     }
@@ -151,7 +158,10 @@ loadMethod <- function(method, fname, envir) method
     else {
         i <- i-1
         length(fnames) <- i
-        fnames <- c(fnames, rep("", length(call) - i))
+        cnames <- if (is.null(names(call)))
+                      rep("", length(call) - i)
+                  else utils::tail(names(call), -i)
+        fnames <- c(fnames, cnames)
     }
     names(call) <- fnames
     if(hasDrop)
