@@ -233,10 +233,9 @@ generic.skeleton <- function(name, fdef, fdefault)
     }
     if(is.null(fdefault)) {
         fdefault <- fdef
-	msg <- gettextf("invalid call in method dispatch to '%s' (no default method)",
-			name)
-	body(fdefault) <- substitute(stop(MESSAGE, domain = NA),
-				     list(MESSAGE = msg))
+	msg <- "invalid call in method dispatch to '%s' (no default method)"
+	body(fdefault) <- substitute(stop(gettextf(MSG, NAME), domain=NA),
+				     list(MSG = msg, NAME = name))
         environment(fdefault) <- baseenv()
     }
     skeleton[[1L]] <- fdefault
@@ -383,7 +382,7 @@ rematchDefinition <- function(definition, generic, mnames, fnames, signature)
              call. = FALSE)
     clArgs <- fnames[useNames]
     ## leave newCall as a list while checking the trailing args
-    if(keepsDots && dotsPos < length(fnames)) {
+    if((keepsDots || added) && dotsPos < length(fnames)) {
 	## Trailing arguments (those after "...") are required to match.  This is a little
 	## stronger than necessary, but this is a dicey case, because
 	## the argument-matching may not be consistent otherwise (in
