@@ -1,7 +1,7 @@
 #  File src/library/stats/R/confint.R
 #  Part of the R package, https://www.R-project.org
 #
-#  Copyright (C) 2003-2018 The R Core Team
+#  Copyright (C) 2003-2023 The R Core Team
 #  Copyright (C) 1994-2003 W. N. Venables and B. D. Ripley
 #
 #  This program is free software; you can redistribute it and/or modify
@@ -19,9 +19,8 @@
 
 confint <- function(object, parm, level = 0.95, ...) UseMethod("confint")
 
-format.perc <- function(probs, digits)
-    ## Not yet exported, maybe useful in other contexts:
-    ## quantile.default() sometimes uses a version of it
+## *different* and simpler than  format_perc()  in ./quantile.R for quantile.default()
+.format_perc <- function(probs, digits)
     paste(format(100 * probs, trim = TRUE, scientific = FALSE, digits = digits),
 	  "%")
 
@@ -37,7 +36,7 @@ confint.lm <- function(object, parm, level = 0.95, ...)
     a <- (1 - level)/2
     a <- c(a, 1 - a)
     fac <- qt(a, object$df.residual) # difference from default method
-    pct <- format.perc(a, 3)
+    pct <- .format_perc(a, 3)
     ci <- array(NA_real_,
                 dim = c(length(parm), 2L), dimnames = list(parm, pct))
     ci[] <- cf[parm] + ses[parm] %o% fac
@@ -50,16 +49,14 @@ confint.glm <- function(object, parm, level = 0.95, ...)
 {
     if(!requireNamespace("MASS", quietly = TRUE))
         stop("package 'MASS' must be installed")
-    confint.glm <- get("confint.glm", asNamespace("MASS"), inherits = FALSE)
-    confint.glm(object, parm, level, ...)
+    asNamespace("MASS")$confint.glm(object, parm, level, ...)
 }
 
 confint.nls <- function(object, parm, level = 0.95, ...)
 {
     if(!requireNamespace("MASS", quietly = TRUE))
         stop("package 'MASS' must be installed")
-    confint.nls <- get("confint.nls", asNamespace("MASS"), inherits = FALSE)
-    confint.nls(object, parm, level, ...)
+    asNamespace("MASS")$confint.nls(object, parm, level, ...)
 }
 
 confint.default <- function (object, parm, level = 0.95, ...)
@@ -70,7 +67,7 @@ confint.default <- function (object, parm, level = 0.95, ...)
     else if(is.numeric(parm)) parm <- pnames[parm]
     a <- (1 - level)/2
     a <- c(a, 1 - a)
-    pct <- format.perc(a, 3)
+    pct <- .format_perc(a, 3)
     fac <- qnorm(a)
     ci <- array(NA, dim = c(length(parm), 2L),
 		dimnames = list(parm, pct))
