@@ -1,7 +1,7 @@
 #  File src/library/stats/R/density.R
 #  Part of the R package, https://www.R-project.org
 #
-#  Copyright (C) 1995-2019 The R Core Team
+#  Copyright (C) 1995-2023 The R Core Team
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -23,7 +23,8 @@ density.default <-
 	     kernel = c("gaussian", "epanechnikov", "rectangular",
 	     "triangular", "biweight", "cosine", "optcosine"),
 	     weights = NULL, window = kernel, width,
-	     give.Rkern = FALSE,
+	     give.Rkern = FALSE, subdensity = FALSE,
+             warnWbw = var(weights) > 0,# sd(weights) > mean(weights) "better", but arbitrary
 	     n = 512, from, to, cut = 3, na.rm = FALSE, ...)
 {
     chkDots(...)
@@ -46,12 +47,25 @@ density.default <-
         stop("argument 'x' must be numeric")
     name <- deparse1(substitute(x))
     x <- as.vector(x)
+    N <- length(x)
+    if(has.wts <- !is.null(weights)) {
+        if(length(weights) != N)
+            stop("'x' and 'weights' have unequal length")
+    }
     x.na <- is.na(x)
     if (any(x.na)) {
-        if (na.rm) x <- x[!x.na]
+        if (na.rm) {
+            N <- length(x <- x[!x.na])
+            if(has.wts) {
+                trueD <- isTRUE(all.equal(1, sum(weights)))
+                weights <- weights[!x.na]
+                if(trueD) ## keep weights summing to one
+                    weights <- weights/sum(weights)
+            }
+        }
         else stop("'x' contains missing values")
     }
-    N <- nx <- as.integer(length(x))
+    nx <- N <- as.integer(N)
     if(is.na(N)) stop(gettextf("invalid value of %s", "length(x)"), domain = NA)
     x.finite <- is.finite(x)
     if(any(!x.finite)) {
@@ -60,13 +74,11 @@ density.default <-
     }
 
     ## Handle 'weights'
-    if(is.null(weights))  {
+    if(!has.wts)  {
         weights <- rep.int(1/nx, nx)
         totMass <- nx/N
     }
     else {
-        if(length(weights) != N)
-            stop("'x' and 'weights' have unequal length")
         if(!all(is.finite(weights)))
             stop("'weights' must all be finite")
         if(any(weights < 0))
@@ -78,7 +90,7 @@ density.default <-
         } else totMass <- 1
 
         ## No error, since user may have wanted "sub-density"
-        if (!isTRUE(all.equal(1, wsum)))
+        if (!subdensity && !isTRUE(all.equal(1, wsum)))
             warning("sum(weights) != 1  -- will not get true density")
     }
 
@@ -107,6 +119,9 @@ density.default <-
     if (is.character(bw)) {
         if(nx < 2)
             stop("need at least 2 points to select a bandwidth automatically")
+        ##cat(sprintf("density(): sd(wts)/mean(wts) = %g\n", sd(weights)/mean(weights)))
+        if(has.wts && warnWbw)
+            warning("Selecting bandwidth *not* using 'weights'")
         bw <- switch(tolower(bw),
                      nrd0 = bw.nrd0(x),
                      nrd = bw.nrd(x),
@@ -125,9 +140,9 @@ density.default <-
     if (missing(to))
 	to   <- max(x) + cut * bw
     if (!is.finite(from)) stop("non-finite 'from'")
-    if (!is.finite(to)) stop("non-finite 'to'")
+    if (!is.finite(to))   stop("non-finite 'to'")
     lo <- from - 4 * bw
-    up <- to + 4 * bw
+    up <- to   + 4 * bw
     ## This bins weighted distances
     y <- .Call(C_BinDist, x, weights, lo, up, n) * totMass
 
@@ -170,9 +185,9 @@ plot.density <- function(x, main = NULL, xlab = NULL, ylab = "Density",
 {
     if(is.null(xlab))
 	xlab <- paste("N =", x$n, "  Bandwidth =", formatC(x$bw))
-    if(is.null(main)) main <- deparse(x$call)
+    if(is.null(main)) main <- sub("[.]default", "", deparse(x$call))
     plot.default(x, main = main, xlab = xlab, ylab = ylab, type = type, ...)
-    if(zero.line) abline(h = 0, lwd = 0.1, col = "gray")
+    if(zero.line) abline(h = 0, lwd = 0.25, col = "gray")
     invisible(NULL)
 }
 
