@@ -533,10 +533,13 @@ anova.glm <- function(object, ..., dispersion = NULL, test = NULL)
     ## calculate test statistics if needed
 
     df.dispersion <- Inf
-    if(is.null(dispersion)) {
-	dispersion <- summary(object, dispersion=dispersion)$dispersion
-	df.dispersion <- if (dispersion == 1) Inf else object$df.residual
+    if (is.null(dispersion)) {
+        dispersion <- summary(object)$dispersion
+        if (!is.null(object$family$dispersion) && is.na(object$family$dispersion)) {
+            df.dispersion <- object$df.residual
+        }
     }
+
     if(!is.null(test)) {
         if(test == "F" && df.dispersion == Inf) {
             fam <- object$family$family
@@ -628,8 +631,12 @@ anova.glmlist <- function(object, ..., dispersion=NULL, test=NULL)
 
     if(!is.null(test)) {
 	bigmodel <- object[[order(resdf)[1L]]]
-	dispersion <- summary(bigmodel, dispersion=dispersion)$dispersion
-	df.dispersion <- if (dispersion == 1) Inf else min(resdf)
+        if (is.null(dispersion)) {
+            dispersion <- summary(bigmodel)$dispersion
+        }
+	df.dispersion <- if (dispersion == 1) Inf
+                         else if (!is.null(bigmodel$family$dispersion) && !is.na(bigmodel$family$dispersion)) Inf
+                         else min(resdf)
         if(test == "F" && df.dispersion == Inf) {
             fam <- bigmodel$family$family
             if(fam == "binomial" || fam == "poisson")
@@ -647,15 +654,16 @@ anova.glmlist <- function(object, ..., dispersion=NULL, test=NULL)
 	      class = c("anova", "data.frame"))
 }
 
-
 summary.glm <- function(object, dispersion = NULL,
 			correlation = FALSE, symbolic.cor = FALSE, ...)
 {
     est.disp <- FALSE
     df.r <- object$df.residual
-    if(is.null(dispersion))	# calculate dispersion if needed
+    if(is.null(dispersion)) {	# calculate dispersion if needed
+        fam <- object$family
 	dispersion <-
-	    if(object$family$family %in% c("poisson", "binomial"))  1
+            if (!is.null(fam$dispersion) && !is.na(fam$dispersion)) fam$dispersion
+            else if(fam$family %in% c("poisson", "binomial"))  1
 	    else if(df.r > 0) {
                 est.disp <- TRUE
 		if(any(object$weights==0))
@@ -665,7 +673,7 @@ summary.glm <- function(object, dispersion = NULL,
                 est.disp <- TRUE
                 NaN
             }
-
+    }
     ## calculate scaled and unscaled covariance matrix
 
     aliased <- is.na(coef(object))  # used in print method
@@ -737,18 +745,20 @@ summary.glm <- function(object, dispersion = NULL,
 print.summary.glm <-
     function (x, digits = max(3L, getOption("digits") - 3L),
 	      symbolic.cor = x$symbolic.cor,
-	      signif.stars = getOption("show.signif.stars"), ...)
+	      signif.stars = getOption("show.signif.stars"),
+              show.residuals = FALSE, ...)
 {
     cat("\nCall:\n",
-	paste(deparse(x$call), sep = "\n", collapse = "\n"), "\n\n", sep = "")
-    cat("Deviance Residuals: \n")
-    if(x$df.residual > 5) {
-	x$deviance.resid <- setNames(quantile(x$deviance.resid, na.rm = TRUE),
-				     c("Min", "1Q", "Median", "3Q", "Max"))
+	paste(deparse(x$call), sep = "\n", collapse = "\n"), "\n", sep = "")
+    if (show.residuals) {
+        cat("\nDeviance Residuals: \n")
+        if(x$df.residual > 5) {
+            x$deviance.resid <- setNames(quantile(x$deviance.resid, na.rm = TRUE),
+                                         c("Min", "1Q", "Median", "3Q", "Max"))
+        }
+        xx <- zapsmall(x$deviance.resid, digits + 1L)
+        print.default(xx, digits = digits, na.print = "", print.gap = 2L)
     }
-    xx <- zapsmall(x$deviance.resid, digits + 1L)
-    print.default(xx, digits = digits, na.print = "", print.gap = 2L)
-
     if(length(x$aliased) == 0L) {
         cat("\nNo Coefficients\n")
     } else {
