@@ -346,7 +346,7 @@ runmedint(R_xlen_t n, int k, int k2, const double *data, double *median,
 	for(int i = 0; i <  k2; median[i] = data[i], i++);
 	median[k2] = window[k];
     }
-    R_xlen_t every_i;
+    R_xlen_t every_i = 1; // -Wall: only used for print_level >= 2.
     if(print_level >= 2)
 	every_i = (n > 100) ? n/10 : 10;
 
