@@ -1,6 +1,6 @@
 #  File src/library/tools/R/Rd2HTML.R
 #
-#  Copyright (C) 1995-2021 The R Core Team
+#  Copyright (C) 1995-2023 The R Core Team
 #  Part of the R package, https://www.R-project.org
 #
 #  This program is free software; you can redistribute it and/or modify
@@ -116,18 +116,32 @@ shtmlify <- function(s) {
     s
 }
 
-## URL encode anything other than alphanumeric, . - _ $ and reserved
-## characters in URLs.
-urlify <- function(x) {
-    ## Like utils::URLencode(reserved = FALSE), but with '&' replaced by
-    ## '&amp;' and hence directly usable for href attributes.
-    ## See
-    ##   <http://www.w3.org/TR/html4/appendix/notes.html#h-B.2.1>
-    ##   <http://www.w3.org/TR/html4/appendix/notes.html#h-B.2.2>
-    ##   RFC 3986 <http://tools.ietf.org/html/rfc3986>
-
-    ## We do not want to mess with already-encoded URLs
-    if(grepl("%[[:xdigit:]]{2}", x, useBytes = TRUE)) {
+## URL encode for use in href attributes.
+urlify <- function(x, reserved = FALSE, repeated = FALSE) {
+    ## When reserved is a logical, like
+    ##   utils::URLencode(x, reserved)
+    ## with '&' replaced by '&amp;' and hence directly usable for href
+    ## attributes.  Equivalently, one could use
+    ##   escapeAmpersand(utils::URLencode(x, reserved))
+    ## Alternatively, reserved can be a string giving the reserved chars
+    ## not to percent encode if it starts with a '^', and to percent
+    ## encode otherwise (perhaps utils::URLencode() should be enhanced
+    ## accordingly?).
+    ##
+    ## According to RFC 3986 <https://www.rfc-editor.org/rfc/rfc3986>, the
+    ## reserved characters are
+    ##   c(gendelims, subdelims)
+    ## with
+    ##   gendelims <- c(":", "/", "?", "#", "[", "]", "@")
+    ##   subdelims <- c("!", "$", "&", "'", "(", ")", "*", "+", ",", ";", "=")
+    ## The following is
+    ##   paste(c(gendelims, subdelims), collapse = "")
+    ## re-arranged for convenient use in regexp (negated) character
+    ## classes:
+    alldelims <- "][!$&'()*+,;=:/?@#"
+    ## See also <https://url.spec.whatwg.org/#valid-url-string>.
+    
+    if(!repeated && grepl("%[[:xdigit:]]{2}", x, useBytes = TRUE)) {
         gsub("&", "&amp;", x, fixed = TRUE)
     } else {
         chars <- unlist(strsplit(x, ""))
@@ -136,16 +150,53 @@ urlify <- function(x) {
                       paste0("%", toupper(as.character(charToRaw(x))),
                              collapse = ""),
                       "")
+        if(is.character(reserved)) {
+            reserved <- paste(reserved, collapse = "")
+            reserved <- if(startsWith(reserved, "^"))
+                            substring(reserved, 2L)
+                        else      
+                            rawToChar(setdiff(charToRaw(alldelims),
+                                              charToRaw(reserved)))
+            escape <- any(charToRaw(reserved) == charToRaw("&"))
+        } else if(!reserved) {
+            reserved <- alldelims
+            escape <- TRUE
+        } else {
+            reserved <- ""
+            escape <- FALSE
+        }
         todo <- paste0("[^",
-                       "][!$&'()*+,;=:/?@ #",
+                       reserved,
                        "ABCDEFGHIJKLMNOPQRSTUVWXYZ",
                        "abcdefghijklmnopqrstuvwxyz0123456789._~-",
                        "]")
-        mixed <- ifelse(grepl(todo, chars), hex, chars)
-        gsub("&", "&amp;", paste(mixed, collapse = ""), fixed = TRUE)
+        x <- paste(ifelse(grepl(todo, chars), hex, chars), collapse = "")
+        if(escape)
+            x <- gsub("&", "&amp;", x, fixed = TRUE)
+        x
     }
 }
-## (Equivalently, could use escapeAmpersand(utils::URLencode(x)).)
+
+urlify_email_address <- function(x) {
+    ## As per RFC 6068
+    ## <https://www.rfc-editor.org/rfc/rfc6068#section-2> we must
+    ## percent encode
+    ##   "%"
+    ##   from gendelims:   c("/", "?", "#", "[", "]")  
+    ##   from subdelims:   c("&", ";", "=")
+    urlify(x, reserved = "][#?/&;=%", repeated = TRUE)
+}
+
+urlify_doi <- function(x) {
+    ## According to
+    ##   <https://www.doi.org/doi_handbook/2_Numbering.html#2.2>
+    ## a DOI name can "incorporate any printable characters from the
+    ## legal graphic characters of Unicode".  The subsequent
+    ##   <https://www.doi.org/doi_handbook/2_Numbering.html#htmlencoding>
+    ## discussed encoding issues but is a bit vague.
+    ## For now, percent encode all reserved characters but the slash.
+    urlify(x, reserved = "^/", repeated = TRUE)
+}
 
 ## Ampersands should be escaped in proper HTML URIs
 escapeAmpersand <- function(x) gsub("&", "&amp;", x, fixed = TRUE)
@@ -153,9 +204,35 @@ escapeAmpersand <- function(x) gsub("&", "&amp;", x, fixed = TRUE)
 invalid_HTML_chars_re <-
     "[\u0001-\u0008\u000b\u000c\u000e-\u001f\u007f-\u009f]"
 
+## topics can contain weird characters like % / & !, so need to be
+## encoded. We allow for different encodings for URLs and filenames
+## (although mostly the same for now).
 
-topic2filename <- function(x) gsub("%", "+", utils::URLencode(x, reserved=TRUE))
-
+## & -> %26F is OK, & -> &amp; is NOT OK with dynamic help
+topic2url <- function(x)
+{
+    if(config_val_to_logical(Sys.getenv("_R_HELP_USE_URLENCODE_",
+                                        "FALSE")))
+        utils::URLencode(x, reserved = TRUE)
+    else
+        vapply(x, urlify, "", reserved = TRUE) # to vectorize (used in toHTML.R)
+}
+topic2filename <- function(x)
+    gsub("%", "+", utils::URLencode(x, reserved = TRUE))
+## The next 3 are for generating URL fragment ids
+name2id <- function(x)
+    gsub("%", "+", utils::URLencode(x, reserved = TRUE))
+topic2id <- function(x)
+    sprintf("topic+%s", gsub("%", "+", utils::URLencode(x, reserved = TRUE)))
+topic2href <- function(x, destpkg = NULL, FUN = NULL)
+{
+    if (is.null(destpkg)) sprintf("#%s", topic2id(x))
+    else {
+        ## FIXME: Do we want to allow FUN to be user-supplied through options()?
+        if (is.null(FUN)) FUN <- function(id, pkg) sprintf("%s.html#%s", pkg, id)
+        FUN(topic2id(x), destpkg)
+    }
+}
 
 ## Create HTTP redirect files for aliases; called only during package
 ## installation if static help files are enabled. Files are named
@@ -170,7 +247,9 @@ createRedirects <- function(file, Rdobj)
         config_val_to_logical(Sys.getenv("_R_HELP_LINKS_TO_TOPICS_", "TRUE"))
     if (!linksToTopics) return(invisible()) # do nothing
     ## create a HTTP redirect for each 'alias' in .../pkg/help/
-    redirHTML <- sprintf("<html><head><meta http-equiv='refresh' content='0; url=../html/%s'></head></html>\n", urlify(basename(file)))
+    redirHTML <-
+        sprintf("<!DOCTYPE html>\n<html><head><meta http-equiv='refresh' content='0; url=../html/%s'><title>HTTP redirect</title></head><body></body></html>\n",
+                urlify(basename(file), reserved = TRUE))
     toProcess <- which(RdTags(Rdobj) == "\\alias")
     helpdir <- paste0(dirname(dirname(file)), "/help") # .../pkg/help/
     aliasName <- function(i) trimws(Rdobj[[i]][[1]])
@@ -193,16 +272,16 @@ createRedirects <- function(file, Rdobj)
             message(msg, appendLF = FALSE)
         }
         try(suppressWarnings(cat(redirHTML, file = afile)), silent = TRUE) # Fails for \alias{%/%}
-        ## redirMsg("topic", aname, basename(file), if (file.exists(afile)) "SUCCESS" else "FAIL")
-        if (!file.exists(afile)) redirMsg("topic", aname, basename(file), "FAIL")
+        ## redirMsg("topic", aname, basename(file), if (file.exists(afile)) "SUCCESS" else "FAILURE")
+        if (!file.exists(afile)) redirMsg("topic", aname, basename(file), "FAILURE")
     }
     ## Also add .../pkg/help/file.html -> ../pkg/html/file.html as fallback
     ## when topic is not found (but do not overwrite)
     file.fallback <- file.path(helpdir, basename(file))
     if (!file.exists(file.fallback)) {
         try(cat(redirHTML, file = file.fallback), silent = TRUE)
-        ## redirMsg("file", basename(file), basename(file), if (file.exists(file.fallback)) "SUCCESS" else "FAIL")
-        if (!file.exists(file.fallback)) redirMsg("file", basename(file), basename(file),  "FAIL")
+        ## redirMsg("file", basename(file), basename(file), if (file.exists(file.fallback)) "SUCCESS" else "FAILURE")
+        if (!file.exists(file.fallback)) redirMsg("file", basename(file), basename(file),  "FAILURE")
     }
 }
 
@@ -227,12 +306,19 @@ createRedirects <- function(file, Rdobj)
 ##    and missing links (those without an explicit package, and
 ##    those topics not in Links[2]) don't get linked anywhere.
 
+
+
 Rd2HTML <-
     function(Rd, out = "", package = "", defines = .Platform$OS.type,
              Links = NULL, Links2 = NULL,
              stages = "render", outputEncoding = "UTF-8",
              dynamic = FALSE, no_links = FALSE, fragment=FALSE,
-             stylesheet = "R.css", ...)
+             stylesheet = if (dynamic) "/doc/html/R.css" else "R.css",
+             texmath = getOption("help.htmlmath"),
+             concordance = FALSE,
+             standalone = TRUE,
+             Rhtml = FALSE, # TODO: guess from 'out' if non-missing
+             ...)
 {
     ## Is this package help, as opposed to from Rdconv or similar?
     ## Used to decide whether redirect files should be created when
@@ -241,6 +327,12 @@ Rd2HTML <-
     if (missing(no_links) && is.null(Links) && !dynamic) no_links <- TRUE
     linksToTopics <-
         config_val_to_logical(Sys.getenv("_R_HELP_LINKS_TO_TOPICS_", "TRUE"))
+    enhancedHTML <-
+        config_val_to_logical(Sys.getenv("_R_HELP_ENABLE_ENHANCED_HTML_", "TRUE"))
+    if (!no_links && !linksToTopics && !standalone) {
+        warning("links not supported for 'standalone = FALSE' when _R_HELP_LINKS_TO_TOPICS_=false")
+        no_links <- TRUE
+    }
     version <- ""
     if(!identical(package, "")) {
         if(length(package) > 1L) {
@@ -275,11 +367,19 @@ Rd2HTML <-
             writeLines(x, con, useBytes = TRUE, ...)
         }
     }
-
+    
+    if (concordance)
+    	conc <- activeConcordance()
+    else
+    	conc <- NULL
+    
     of0 <- function(...)
-        writeLinesUTF8(paste0(...), con, outputEncoding, sep = "")
-    of1 <- function(text)
+        of1(paste0(...))
+    of1 <- function(text) {
+    	if (concordance)
+    	    conc$addToConcordance(text)
         writeLinesUTF8(text, con, outputEncoding, sep = "")
+    }
 
     pendingClose <- pendingOpen <- character()  # Used for infix methods
 
@@ -301,31 +401,31 @@ Rd2HTML <-
                   "\\strong"="strong",
                   "\\var"="var")
     # These have simple substitutions
-    HTMLEscapes <- c("\\R"='<span style="font-family: Courier New, Courier; color: #666666;"><b>R</b></span>',
+    HTMLEscapes <- c("\\R"='<span class="rlang"><b>R</b></span>',
     		     "\\cr"="<br />",
     		     "\\dots"="...",
     		     "\\ldots"="...")
     ## These correspond to idiosyncratic wrappers
-    HTMLLeft <- c("\\acronym"='<acronym><span class="acronym">',
+    HTMLLeft <- c("\\acronym"='<abbr><span class="acronym">',
     		  "\\donttest"="",
     		  "\\env"='<span class="env">',
                   "\\file"='&lsquo;<span class="file">',
                   "\\option"='<span class="option">',
                   "\\pkg"='<span class="pkg">',
-                  "\\samp"='<span class="samp">',
+                  "\\samp"='&lsquo;<span class="samp">&#8288;',
                   "\\sQuote"="&lsquo;",
                   "\\dQuote"="&ldquo;",
-                  "\\verb"='<code style="white-space: pre;">')
-    HTMLRight <- c("\\acronym"='</span></acronym>',
+                  "\\verb"='<code style="white-space: pre;">&#8288;')
+    HTMLRight <- c("\\acronym"='</span></abbr>',
     		   "\\donttest"="",
     		   "\\env"="</span>",
                    "\\file"='</span>&rsquo;',
                    "\\option"="</span>",
                    "\\pkg"="</span>",
-                   "\\samp"="</span>",
+                   "\\samp"="&#8288;</span>&rsquo;",
                    "\\sQuote"="&rsquo;",
                    "\\dQuote"="&rdquo;",
-                   "\\verb"="</code>")
+                   "\\verb"="&#8288;</code>")
 
     addParaBreaks <- function(x) {
 	if (isBlankLineRd(x) && isTRUE(inPara)) {
@@ -343,16 +443,44 @@ Rd2HTML <-
 
     enterPara <- function(enter = TRUE) {
 	if (enter && isFALSE(inPara)) {
-            of0("<p>")
+            of1("<p>")
             inPara <<- TRUE
         }
     }
 
     leavePara <- function(newval) {
-    	if (isTRUE(inPara)) of0("</p>\n")
+    	if (isTRUE(inPara)) of1("</p>\n")
     	inPara <<- newval
     }
 
+    writeItemAsCode <- function(blocktag, block, addID = blocktag == "\\arguments") {
+        ## Usually RdTags(block)[1L] == "TEXT", except when it is
+        ## \\dots, \\ldots, etc. Potentially more complicated in cases
+        ## like \item{foo, \dots, bar}, where block will have length >
+        ## 1. We want to (a) split on comma and treat each part as a
+        ## separate argument / value, and (b) for blocktag==arguments
+        ## only, add an id tag so that we can have internal links.
+
+        ## We do this by 'deparsing' block[[1L]], using as.character.Rd()
+        s <- as.character.Rd(block)
+        toEsc <- s %in% names(HTMLEscapes)
+        if (any(toEsc)) s[toEsc] <- HTMLEscapes[s[toEsc]]
+
+        ## Now just join, split on comma, wrap individually inside
+        ## </code>, and unsplit. This will be problematic if any
+        ## TeX-like macros remain, but that should not happen in
+        ## practice for \item-s inside \arguments or \value.
+
+        s <- trimws(strsplit(paste(s, collapse = ""), ",", fixed = TRUE)[[1]])
+        s <- s[nzchar(s)] # unlikely to matter, but just to be safe
+        s <- if (addID) sprintf('<code id="%s">%s</code>',
+                                gsub("[[:space:]]+", "-", paste(name2id(name), s, sep = "_:_")),
+                                vhtmlify(s))
+             else sprintf('<code>%s</code>', vhtmlify(s))
+        s <- paste0(s, collapse = ", ")
+        of1(s)
+    }
+
     writeWrapped <- function(tag, block, doParas) {
     	if (!doParas || HTMLTags[tag] == "pre")
             leavePara(NA)
@@ -383,12 +511,11 @@ Rd2HTML <-
             if (!no_links) of1('</a>')
             inPara <<- savePara
         }
-
     	if (is.null(parts$targetfile)) {
             ## ---------------- \link{topic} and \link[=topic]{foo}
             topic <- parts$dest
     	    if (dynamic) { # never called with package=""
-                htmlfile <- paste0("../../", urlify(package), "/help/", urlify(topic))
+                htmlfile <- paste0("../../", urlify(package), "/help/", topic2url(topic))
                 writeHref()
                 return()
             }
@@ -399,7 +526,11 @@ Rd2HTML <-
                 ## package, but also those in base+recommended
                 ## packages. We do this branch only if this is a
                 ## within-package link
-                htmlfile <- paste0("../../", urlify(package), "/help/", topic2filename(topic), ".html")
+                htmlfile <-
+                    if (standalone)
+                        paste0("../../", urlify(package), "/help/", topic2filename(topic), ".html")
+                    else
+                        topic2href(topic)
                 writeHref()
                 return()
 
@@ -421,10 +552,15 @@ Rd2HTML <-
                     warnRd(block, Rdfile, "missing link ", sQuote(topic))
                 writeContent(block, tag)
             } else {
-                ## treat links in the same package specially -- was needed for CHM
-                pkg_regexp <- paste0("^../../", urlify(package), "/html/")
-                if (grepl(pkg_regexp, htmlfile)) {
-                    htmlfile <- sub(pkg_regexp, "", htmlfile)
+                if (!standalone) {
+                    htmlfile <- topic2href(topic, destpkg = strsplit(htmlfile, "/", fixed = TRUE)[[1]][[3]])
+                }
+                else {
+                    ## treat links in the same package specially -- was needed for CHM
+                    pkg_regexp <- paste0("^../../", urlify(package), "/html/")
+                    if (grepl(pkg_regexp, htmlfile)) {
+                        htmlfile <- sub(pkg_regexp, "", htmlfile)
+                    }
                 }
                 writeHref()
             }
@@ -433,9 +569,9 @@ Rd2HTML <-
             if (!dynamic && !linksToTopics && !no_links &&
                 nzchar(pkgpath <- system.file(package = parts$pkg))) {
                 ## old-style static HTML: prefer filename over topic,
-                ## so treat as filename and urlify() instead of
+                ## so treat as filename and topic2url() instead of
                 ## topic2filename()
-                htmlfile <- paste0(urlify(parts$targetfile), ".html")
+                htmlfile <- paste0(topic2url(parts$targetfile), ".html")
                 ## check the link, only if the package is found
                 OK <- FALSE
                 if (!file.exists(file.path(pkgpath, "html", htmlfile))) {
@@ -464,23 +600,24 @@ Rd2HTML <-
             if (parts$pkg == package) { # within same package
                 if (linksToTopics)
                     htmlfile <-
-                        if (dynamic) paste0("../help/", urlify(parts$targetfile))
-                        else paste0("../help/", topic2filename(parts$targetfile), ".html")
+                        if (dynamic) paste0("../help/", topic2url(parts$targetfile))
+                        else if (standalone) paste0("../help/", topic2filename(parts$targetfile), ".html")
+                        else topic2href(parts$targetfile)
                 else # use href = "file.html"
-                    htmlfile <- paste0(urlify(parts$targetfile), ".html")
+                    htmlfile <- paste0(topic2url(parts$targetfile), ".html")
                 writeHref()
             } else {  # link to different package
                 ## href = "../../pkg/html/file.html"
                 if (linksToTopics)
                     htmlfile <-
                         if (dynamic) paste0("../../", urlify(parts$pkg), "/help/",
-                                            urlify(parts$targetfile))
-                        else paste0("../../", urlify(parts$pkg), "/help/",
-                                    topic2filename(parts$targetfile), ".html")
-
+                                            topic2url(parts$targetfile))
+                        else if (standalone) paste0("../../", urlify(parts$pkg), "/help/",
+                                                    topic2filename(parts$targetfile), ".html")
+                        else topic2href(parts$targetfile, destpkg = urlify(parts$pkg))
                 else
                     htmlfile <- paste0("../../", urlify(parts$pkg), "/html/",
-                                       urlify(parts$targetfile), ".html")
+                                       topic2url(parts$targetfile), ".html") # FIXME Is this always OK ??
                 writeHref()
             }
         }
@@ -494,22 +631,25 @@ Rd2HTML <-
     }
 
     writeDR <- function(block, tag) {
+        if (Rhtml && length(block) > 1L)
+            of1("\nend.rcode-->\n\n<!--begin.rcode eval=FALSE\n")
+        of1('## Not run: ')
+        writeContent(block, tag)
         if (length(block) > 1L) {
-            of1('## Not run: ')
-            writeContent(block, tag)
             of1('\n## End(Not run)')
-        } else {
-            of1('## Not run: ')
-            writeContent(block, tag)
-       }
+            if (Rhtml) of1("\nend.rcode-->\n\n<!--begin.rcode\n")
+        }
     }
 
     writeBlock <- function(block, tag, blocktag) {
+    	if (concordance)
+    	    conc$saveSrcref(block)
         doParas <- (blocktag %notin% c("\\tabular"))
 	switch(tag,
                UNKNOWN =,
-               VERB = of1(vhtmlify(block, inEqn)),
-               RCODE = of1(vhtmlify(block)),
+               VERB = if (Rhtml && blocktag == "\\dontrun") of1(block)
+                      else of1(vhtmlify(block, inEqn)),
+               RCODE = if (Rhtml) of1(block) else of1(vhtmlify(block)),
                TEXT = of1(if(doParas && !inAsIs) addParaBreaks(htmlify(block)) else vhtmlify(block)),
                USERMACRO =,
                "\\newcommand" =,
@@ -539,7 +679,8 @@ Rd2HTML <-
                "\\email" = if (length(block)) {
                    url <- lines2str(as.character(block))
                    enterPara(doParas)
-                   of0('<a href="mailto:', urlify(url), '">',
+                   ## FIXME: urlify
+                   of0('<a href="mailto:', urlify_email_address(url), '">',
                        htmlify(url), '</a>')},
                ## watch out for empty URLs (TeachingDemos had one)
                "\\url" = if(length(block)) {
@@ -583,24 +724,36 @@ Rd2HTML <-
                "\\dontrun"= writeDR(block, tag),
                "\\enc" = writeContent(block[[1L]], tag),
                "\\eqn" = {
-                   enterPara(doParas)
-                   inEqn <<- TRUE
-                   of1("<i>")
-                   block <- block[[length(block)]];
-                   ## FIXME: space stripping needed: see Special.html
-                   writeContent(block, tag)
-                   of1("</i>")
-                   inEqn <<- FALSE
+                   block <-
+                       if (doTexMath) block[[1L]]
+                       else block[[length(block)]]
+                   if(length(block)) {
+                       enterPara(doParas)
+                       inEqn <<- !doTexMath
+                       if (doTexMath) of1('<code class="reqn">') # safer than of1('\\(') etc.
+                       else of1("<i>")
+                       ## FIXME: space stripping needed: see Special.html
+                       writeContent(block, tag)
+                       if (doTexMath) of1('</code>') # of1('\\)')
+                       else of1("</i>")
+                       inEqn <<- FALSE
+                   }
                },
                "\\deqn" = {
-                   inEqn <<- TRUE
-                   leavePara(TRUE)
-                   of1('<p style="text-align: center;"><i>')
-                   block <- block[[length(block)]];
-                   writeContent(block, tag)
-                   of0('</i>')
-                   leavePara(FALSE)
-                   inEqn <<- FALSE
+                   block <-
+                       if (doTexMath) block[[1L]]
+                       else block[[length(block)]]
+                   if(length(block)) {
+                       inEqn <<- !doTexMath
+                       leavePara(TRUE)
+                       if (doTexMath) of1('<p style="text-align: center;"><code class="reqn">')
+                       else of1('<p style="text-align: center;"><i>')
+                       writeContent(block, tag)
+                       if (doTexMath) of1('</code>\n')
+                       else of1('</i>')
+                       leavePara(FALSE)
+                       inEqn <<- FALSE
+                   }
                },
                "\\figure" = {
                    enterPara(doParas)
@@ -611,10 +764,19 @@ Rd2HTML <-
                    of1('" ')
                	   if (length(block) > 1L
                	       && length(imgoptions <- .Rd_get_latex(block[[2]]))
-		       && startsWith(imgoptions, "options: ")) {
-		       # There may be escaped percent signs within
-		       imgoptions <- gsub("\\%", "%", imgoptions, fixed=TRUE)
+		       && startsWith(imgoptions[1L], "options: ")) {
+		       ## There may be escaped percent signs within
+		       imgoptions <- gsub("\\%", "%",
+                                          paste(imgoptions,
+                                                collapse = " "),
+                                          fixed=TRUE)
                        of1(sub("^options: ", "", imgoptions))
+                       ## Expert use may have forgotten alt ...
+                       if(!grepl("\\balt *=", imgoptions)) {
+                           of1(' alt="')
+                           writeContent(block[[1L]], tag)
+                           of1('"')
+                       }
 	           } else {
 		       of1('alt="')
 		       writeContent(block[[length(block)]], tag)
@@ -657,10 +819,14 @@ Rd2HTML <-
         tags <- RdTags(content)
 
 	leavePara(NA)
-	of1('\n<table summary="Rd table">\n')
+	of1('\n<table>\n')
+	if (concordance)
+	    conc$saveSrcref(table)
         newrow <- TRUE
         newcol <- TRUE
         for (i in seq_along(tags)) {
+            if (concordance)
+                conc$saveSrcref(content[[i]])
             if (newrow) {
             	of1("<tr>\n ")
             	newrow <- FALSE
@@ -705,6 +871,8 @@ Rd2HTML <-
 	    i <- i + 1
             tag <- tags[i]
             block <- blocks[[i]]
+            if (concordance)
+            	conc$saveSrcref(block)
             if (length(pendingOpen)) { # Handle $, [ or [[ methods
             	if (tag == "RCODE" && startsWith(block, "(")) {
             	    block <- sub("^\\(", "", block)
@@ -736,8 +904,8 @@ Rd2HTML <-
     	    	leavePara(FALSE)
     	    	if (!inlist) {
     	    	    switch(blocktag,
-                           "\\value" =  of1('<table summary="R valueblock">\n'),
-                           "\\arguments" = of1('<table summary="R argblock">\n'),
+                           "\\value" =  of1('<table>\n'),
+                           "\\arguments" = of1('<table>\n'),
                            "\\itemize" = of1("<ul>\n"),
                            "\\enumerate" = of1("<ol>\n"),
                            "\\describe" = of1("<dl>\n"))
@@ -751,11 +919,17 @@ Rd2HTML <-
     		}
     		switch(blocktag,
    		"\\value"=,
-     		"\\arguments"={
-    		    of1('<tr valign="top"><td><code>')
+     		"\\arguments"= {
+    		    of1('<tr><td>')
     		    inPara <<- NA
-    		    writeContent(block[[1L]], tag)
-    		    of1('</code></td>\n<td>\n')
+                    ## Argh.  Quite a few packages put the items in
+                    ## their value section inside \code.
+                    if(identical(RdTags(block[[1L]])[1L], "\\code")) {
+                        writeContent(block[[1L]], tag)
+                    } else {
+                        writeItemAsCode(blocktag, block[[1L]])
+                    }
+    		    of1('</td>\n<td>\n')
     		    inPara <<- FALSE
     		    writeContent(block[[2L]], tag)
     		    leavePara(FALSE)
@@ -779,6 +953,7 @@ Rd2HTML <-
     	    },
     	    { # default
     	    	if (inlist && (blocktag %notin% c("\\itemize", "\\enumerate"))
+    	    	           && tag != "COMMENT"
     	    	           && !(tag == "TEXT" && isBlankRd(block))) {
     	    	    switch(blocktag,
     	    	    "\\arguments" =,
@@ -818,7 +993,8 @@ Rd2HTML <-
         save <- sectionLevel
         sectionLevel <<- sectionLevel + 1L
     	of1(paste0("\n\n<h", sectionLevel+2L, ">"))
-
+        if (concordance)
+            conc$saveSrcref(section)
     	if (tag == "\\section" || tag == "\\subsection") {
     	    title <- section[[1L]]
     	    section <- section[[2L]]
@@ -827,8 +1003,15 @@ Rd2HTML <-
     	} else
     	    of1(sectionTitles[tag])
         of1(paste0("</h", sectionLevel+2L, ">\n\n"))
-        if (tag %in% c("\\examples", "\\usage")) {
-            of1("<pre>")
+        if (tag == "\\usage") {
+            of1("<pre><code class='language-R'>")
+            inPara <<- NA
+            pre <- TRUE
+        } else if (tag == "\\examples") {
+            if (dynamic && enhancedHTML && !Rhtml && !is.null(firstAlias))
+                of1(sprintf("<p><a href='../Example/%s'>Run examples</a></p>",
+                            topic2url(firstAlias)))
+            if (Rhtml) of1("\n\n<!--begin.rcode\n") else of1("<pre><code class='language-R'>")
             inPara <<- NA
             pre <- TRUE
         } else {
@@ -838,15 +1021,18 @@ Rd2HTML <-
     	if (length(section)) {
 	    ## There may be an initial \n, so remove that
 	    s1 <- section[[1L]][1L]
-	    if (RdTags(section)[1] == "TEXT" && s1 == "\n") section <- section[-1L]
+	    if (RdTags(section)[1] %in% c("TEXT", "RCODE") && s1 == "\n") section <- section[-1L]
 	    writeContent(section, tag)
 	}
 	leavePara(FALSE)
-	if (pre) of0("</pre>\n")
+        if (pre) # must be \usage or \examples
+            if (Rhtml && tag == "\\examples") of1("\nend.rcode-->\n\n")
+            else of1("</code></pre>\n")
     	sectionLevel <<- save
     }
 
     ## ----------------------- Continue in main function -----------------------
+    info <- list() # attribute to be returned if standalone = FALSE
     create_redirects <- FALSE
     if (is.character(out)) {
         if (out == "") {
@@ -860,9 +1046,74 @@ Rd2HTML <-
     	con <- out
     	out <- summary(con)$description
     }
-
     Rd <- prepare_Rd(Rd, defines = defines, stages = stages,
                      fragment = fragment, ...)
+    ## Check if man page already uses mathjaxr package
+    ## (then skip mathjax processing)
+    uses_mathjaxr <- function(rd)
+    {
+        done <- TRUE
+        ## go through one by one until we hit \description
+        for (frag in rd) {
+            if (attr(frag, "Rd_tag") == "\\description") {
+                done <- FALSE
+                break
+            }
+        }
+        if (done) return(FALSE)
+        ## go through one by one until we hit \loadmathjax
+        for (subfrag in frag) {
+            if (identical(attr(subfrag, "Rd_tag"), "USERMACRO") &&
+                identical(attr(subfrag, "macro"), "\\loadmathjax"))
+                return(TRUE)
+        }
+        return(FALSE)
+    }
+    ## Both katex and mathjax need custom config scripts. For dynamic
+    ## HTML these are in /doc/html/*-config.js (as well as the main
+    ## katex script and CSS), but for static HTML, the appropriate
+    ## relative path is not computable in general. So, for static HTML
+    ## we only support katex, using a CDN for the main files and
+    ## embedding the config in the output file itself
+    if (is.null(texmath)) texmath <- "katex"
+    if (texmath == "mathjax" && !dynamic) texmath <- "katex"
+    doTexMath <- enhancedHTML && !uses_mathjaxr(Rd) &&
+        texmath %in% c("katex", "mathjax")
+
+    ## KaTeX / Mathjax resources (if they are used)
+    if (doTexMath && texmath == "katex") {
+        KATEX_JS <-
+            if (dynamic) "/doc/html/katex/katex.js"
+            else "https://cdn.jsdelivr.net/npm/katex@0.15.3/dist/katex.min.js"
+        KATEX_CSS <- if (dynamic) "/doc/html/katex/katex.css"
+                     else "https://cdn.jsdelivr.net/npm/katex@0.15.3/dist/katex.min.css"
+        KATEX_CONFIG <-
+            if (dynamic) "/doc/html/katex-config.js"
+            else c("const macros = { \"\\\\R\": \"\\\\textsf{R}\", \"\\\\code\": \"\\\\texttt\"};", 
+                   "function processMathHTML() {",
+                   "    var l = document.getElementsByClassName('reqn');", 
+                   "    for (let e of l) { katex.render(e.textContent, e, { throwOnError: false, macros }); }", 
+                   "    return;",
+                   "}")
+    }
+    if (doTexMath && texmath == "mathjax") {
+        MATHJAX_JS <-
+            if (dynamic && requireNamespace("mathjaxr", quietly = TRUE))
+                "/library/mathjaxr/doc/mathjax/es5/tex-chtml-full.js"
+            else
+                "https://cdn.jsdelivr.net/npm/mathjax@3/es5/tex-chtml-full.js"
+        MATHJAX_CONFIG <-
+            if (dynamic) "/doc/html/mathjax-config.js"
+            else "../../../doc/html/mathjax-config.js"
+    }
+    if (enhancedHTML) {
+        PRISM_JS <- 
+            if (dynamic) "/doc/html/prism.js"
+            else NULL # "../../../doc/html/prism.js"
+        PRISM_CSS <- 
+            if (dynamic) "/doc/html/prism.css"
+            else NULL # "../../../doc/html/prism.css"
+    }
     Rdfile <- attr(Rd, "Rdfile")
     sections <- RdTags(Rd)
     if (fragment) {
@@ -875,54 +1126,87 @@ Rd2HTML <-
     } else {
         if (create_redirects) createRedirects(out, Rd)
 	name <- htmlify(Rd[[2L]][[1L]])
-
-        of0('<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">',
-            '<html xmlns="http://www.w3.org/1999/xhtml">',
-	    '<head><title>')
+        firstAlias <-
+            trimws(Rd[[ which(sections == "\\alias")[1] ]][[1]])
+	if (concordance)
+            conc$saveSrcref(.Rd_get_section(Rd, "title"))
 	headtitle <- strwrap(.Rd_format_title(.Rd_get_title(Rd)),
 	                     width=65, initial="R: ")
 	if (length(headtitle) > 1) headtitle <- paste0(headtitle[1], "...")
-	of1(htmlify(headtitle))
-	of0('</title>\n',
-	    '<meta http-equiv="Content-Type" content="text/html; charset=',
-	    mime_canonical_encoding(outputEncoding),
-	    '" />\n')
-        of1('<meta name="viewport" content="width=device-width, initial-scale=1.0, user-scalable=yes" />\n')
 
-	of0('<link rel="stylesheet" type="text/css" href="',
-	    urlify(stylesheet),
-	    '" />\n',
-	    '</head><body><div class="container">\n\n',
-	    '<table width="100%" summary="page for ', htmlify(name))
-	if (nchar(package))
-	    of0(' {', package, '}"><tr><td>',name,' {', package,'}')
-	else
-	    of0('"><tr><td>',name)
-	of0('</td><td style="text-align: right;">R Documentation</td></tr></table>\n\n')
+        ## Create HTML header and footer
+        if (standalone) {
+            hfcomps <- # should we be able to specify static URLs here?
+                HTMLcomponents(title = headtitle, logo = FALSE,
+                               up = NULL,
+                               top = NULL,
+                               css = stylesheet,
+                               outputEncoding = outputEncoding,
+                               dynamic = dynamic, prism = enhancedHTML,
+                               doTexMath = doTexMath, texmath = texmath,
+                               PRISM_CSS_STATIC = NULL, PRISM_JS_STATIC = NULL)
+            of1(paste(hfcomps$header, collapse = "")) # write out header
+            of0('\n\n<table style="width: 100%;">',
+                '<tr><td>',
+                name)
+            if (nchar(package))
+                of0(' {', package, '}')
+            of0('</td><td style="text-align: right;">R Documentation</td></tr></table>\n\n')
+        }
 
-	of1("<h2>")
+        ## id can identify help page when combined with others, and
+        ## also needed to form argument id-s programmatically
+        of0("<h2 id='", name2id(name), "'>")
 	inPara <- NA
 	title <- Rd[[1L]]
-	writeContent(title,sections[1])
+        info$name <- name
+        info$title <- trimws(paste(as.character(title), collapse = "\n"))
+	if (concordance)
+	    conc$saveSrcref(title)
+	writeContent(title, sections[1])
 	of1("</h2>")
 	inPara <- FALSE
-
+        if (!standalone) {
+            ## create empty spans with aliases as id, so that we can link
+            for (a in trimws(unlist(Rd[ which(sections == "\\alias") ]))) {
+                of0("<span id='", topic2id(a), "'></span>")
+            }
+        }
 	for (i in seq_along(sections)[-(1:2)])
 	    writeSection(Rd[[i]], sections[i])
 
 	if(nzchar(version))
-	    version <- paste0('Package <em>',package,'</em> version ',version,' ')
-	of0('\n')
-	if(nzchar(version))
-	    of0('<hr /><div style="text-align: center;">[', version,
-		if (!no_links) '<a href="00Index.html">Index</a>',
-		']</div>')
-	of0('\n',
-	    '</div></body></html>\n')
+	    version <- paste0('Package <em>', package, '</em> version ', version, ' ')
+	of1('\n')
+        if (standalone) {
+            if(nzchar(version))
+                of0('<hr /><div style="text-align: center;">[', version,
+                    if (!no_links) '<a href="00Index.html">Index</a>',
+                    ']</div>')
+            of1(paste(hfcomps$footer, collapse = "")) # write out footer
+        }
+        else attr(out, "info") <- info
+    }
+    if (concordance) {
+    	conc$srcFile <- Rdfile
+    	concdata <- followConcordance(conc$finish(), attr(Rd, "concordance"))
+    	# NB:  Prior to R 4.3.0, the srcFile could be
+    	#      an absolute path, possibly containing a 
+    	#      colon.  This strips the leading part up to
+    	#      "/man/".
+    	concdata$srcFile <- stripPathTo(concdata$srcFile, "man")
+    	attr(out, "concordance") <- concdata
+    	of0('<!-- ',
+    	    as.character(concdata),
+    	    ' -->\n')
     }
     invisible(out)
 } ## Rd2HTML()
 
+
+## The following functions return 'relative' links assuming that all
+## packages are installed in the same virtual library tree.
+
 findHTMLlinks <- function(pkgDir = "", lib.loc = NULL, level = 0:2)
 {
     ## The priority order is
@@ -952,23 +1236,38 @@ findHTMLlinks <- function(pkgDir = "", lib.loc = NULL, level = 0:2)
     gsub("[Rr]d$", "html", Links)
 }
 
+## These helper functions can optionally return the absolute path as
+## well (in the local file system)
+
 .find_HTML_links_in_package <-
-function(dir)
+function(dir, absolute = FALSE)
 {
-    if (file_test("-f", f <- file.path(dir, "Meta", "links.rds")))
-        readRDS(f)
-    else if (file_test("-f", f <- file.path(dir, "Meta", "Rd.rds")))
-        .build_links_index(readRDS(f), basename(dir))
-    else character()
+    ans <- 
+        if (file_test("-f", f <- file.path(dir, "Meta", "links.rds")))
+            readRDS(f)
+        else if (file_test("-f", f <- file.path(dir, "Meta", "Rd.rds")))
+            .build_links_index(readRDS(f), basename(dir))
+        else character()
+    if (absolute)
+        structure(file.path(dir, "html", basename(ans), fsep = "/"),
+                  names = names(ans))
+    else
+        ans
 }
 
 .find_HTML_links_in_library <-
-function(dir)
+function(dir, absolute = FALSE)
 {
-    if (file_test("-f", f <- file.path(dir, ".Meta", "links.rds")))
-        readRDS(f)
+    ans <- 
+        if (file_test("-f", f <- file.path(dir, ".Meta", "links.rds")))
+            readRDS(f)
+        else
+            .build_library_links_index(dir)
+    if (absolute)
+        structure(file.path(dir, substring(ans, first = 7), fsep = "/"), # drop initial "../../"
+                  names = names(ans))
     else
-        .build_library_links_index(dir)
+        ans
 }
 
 .build_library_links_index <-
