@@ -1,7 +1,7 @@
 #  File src/library/tools/R/Rd2latex.R
 #  Part of the R package, https://www.R-project.org
 #
-#  Copyright (C) 1995-2021 The R Core Team
+#  Copyright (C) 1995-2022 The R Core Team
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -61,7 +61,8 @@ latex_canonical_encoding  <- function(encoding)
 Rd2latex <- function(Rd, out = "", defines = .Platform$OS.type,
                      stages = "render",
 		     outputEncoding = "UTF-8", fragment = FALSE, ...,
-                     writeEncoding = TRUE)
+                     writeEncoding = TRUE,
+		     concordance = FALSE)
 {
     encode_warn <- FALSE
     WriteLines <-
@@ -81,9 +82,16 @@ Rd2latex <- function(Rd, out = "", defines = .Platform$OS.type,
             }
     }
 
+    if (concordance)
+    	conc <- activeConcordance()
+    else
+    	conc <- NULL
+    
     last_char <- ""
     of0 <- function(...) of1(paste0(...))
     of1 <- function(text) {
+    	if (concordance)
+    	    conc$addToConcordance(text)
         nc <- nchar(text)
         last_char <<- substr(text, nc, nc)
         WriteLines(text, con, outputEncoding, sep = "")
@@ -104,7 +112,6 @@ Rd2latex <- function(Rd, out = "", defines = .Platform$OS.type,
 
     sectionExtras <-
     c("\\usage"="verbatim",
-      "\\arguments"="ldescription",
       "\\examples"="ExampleCode")
 
     inCodeBlock <- FALSE ## used to indicate to texify where we are
@@ -170,7 +177,7 @@ Rd2latex <- function(Rd, out = "", defines = .Platform$OS.type,
 		## avoid conversion to guillemets
 		x <- fsub("<<", "<{}<", x)
 		x <- fsub(">>", ">{}>", x)
-		x <- fsub(",,", ",{},", x) # ,, is a ligature in the ae font.
+		x <- fsub(",,", ",{},", x) # ,, is a ligature in the ec and lmodern fonts.
 		## cat(sprintf("\ntexify out: '%s'\n", x))
 	    }
 	}
@@ -188,6 +195,8 @@ Rd2latex <- function(Rd, out = "", defines = .Platform$OS.type,
     	wrapper <- wrappers[[tag]]
     	if (is.null(wrapper))
     	    wrapper <- c(paste0(tag, "{"), "}")
+    	if (concordance)
+    	    conc$saveSrcref(block)
     	of1(wrapper[1L])
     	writeContent(block, tag)
     	of1(wrapper[2L])
@@ -203,10 +212,18 @@ Rd2latex <- function(Rd, out = "", defines = .Platform$OS.type,
         }
         ## cleanup URL
         url <- lines2str(url)
-        ## escape % for LaTeX
+        ## escape special characters for LaTeX
+        url <- fsub("\\", "\\\\", url)
+        ## support \href inside \tabular
         url <- fsub("%", "\\%",  url)
+        url <- fsub("#", "\\#",  url)
+        url <- fsub("&", "\\&",  url)  # needs hyperref >= 6.78n (2008-12-26)
+        if (concordance)
+            conc$saveSrcref(block)
     	of0(tag, "{", url, "}")
         if (tag == "\\Rhref") {
+            if (concordance)
+                conc$saveSrcref(block[[2L]])	
             of1("{")
             writeContent(block[[2L]], tag)
             of1("}")
@@ -217,11 +234,15 @@ Rd2latex <- function(Rd, out = "", defines = .Platform$OS.type,
     ## (as documented)
     writeLink <- function(tag, block) {
         parts <- get_link(block, tag)
+        if (concordance)
+            conc$saveSrcref(block)
         of0("\\LinkA{", latex_escape_link(parts$topic), "}{",
             latex_link_trans0(parts$dest), "}")
     }
 
     writeDR <- function(block, tag) {
+    	if (concordance)
+    	    conc$saveSrcref(block)
         if (length(block) > 1L) {
             of1('## Not run: ')
             writeContent(block, tag)
@@ -243,13 +264,12 @@ Rd2latex <- function(Rd, out = "", defines = .Platform$OS.type,
 
     latex_escape_name <- function(x)
     {
-        x <- psub("([$#_&])", "\\\\\\1", x) #- escape them
-        x <- fsub("{", "\\textbraceleft{}", x)
-        x <- fsub("}", "\\textbraceright{}", x)
+        x <- fsub("\\", "\\textbackslash", x)
+        x <- psub("([$#_&{}])", "\\\\\\1", x) #- escape them
+        x <- fsub("\\textbackslash", "\\textbackslash{}", x)
         x <- fsub("^", "\\textasciicircum{}", x)
         x <- fsub("~", "\\textasciitilde{}", x)
         x <- fsub("%", "\\Rpercent{}", x)
-        x <- fsub("\\\\", "\\textbackslash{}", x)
         ## avoid conversion to guillemets
         x <- fsub("<<", "<{}<", x)
         x <- fsub(">>", ">{}>", x)
@@ -316,6 +336,10 @@ Rd2latex <- function(Rd, out = "", defines = .Platform$OS.type,
 
     writeAlias <- function(block, tag) {
         alias <- as.character(block)
+        if(length(alias) > 1L)
+            stop("alias:\n",
+                 sQuote(paste(alias, collapse = "\n")),
+                 "\nis not one line")
         aa <- "\\aliasA{"
         ## Some versions of hyperref (from 6.79d) have trouble indexing these
         ## |, || in base, |.bit, %||% in ggplot2 ...
@@ -328,11 +352,15 @@ Rd2latex <- function(Rd, out = "", defines = .Platform$OS.type,
         ## 'name' is linked from the header
         if (alias == name) return()
         alias2 <- latex_link_trans0(alias)
+        if (concordance)
+            conc$saveSrcref(block)
         of0(aa, latex_code_alias(alias), "}{",
             latex_escape_name(name), "}{", alias2, "}\n")
     }
 
     writeBlock <- function(block, tag, blocktag) {
+    	if (concordance)
+    	    conc$saveSrcref(block)
 	switch(tag,
                UNKNOWN =,
                VERB = of1(texify(block, TRUE)),
@@ -403,9 +431,9 @@ Rd2latex <- function(Rd, out = "", defines = .Platform$OS.type,
 	       	          },
 
                "\\verb"= {
-                   of0("\\AsIs{")
+                   of0("\\AsIs{\\texttt{")
                    writeContent(block, tag)
-                   of1("}")
+                   of1("}}")
                },
                "\\special"= writeContent(block, tag), ## FIXME, verbatim?
                "\\linkS4class" =,
@@ -426,6 +454,8 @@ Rd2latex <- function(Rd, out = "", defines = .Platform$OS.type,
                } ,
                "\\eqn" =,
                "\\deqn" = {
+               	   if (concordance)
+               		conc$saveSrcref(block[[1L]])
                    of0(tag, "{")
                    inEqn <<- TRUE
                    writeContent(block[[1L]], tag)
@@ -433,14 +463,20 @@ Rd2latex <- function(Rd, out = "", defines = .Platform$OS.type,
                    of0('}{}')
                },
                "\\figure" = {
+               	   if (concordance)
+               		conc$saveSrcref(block[[1L]])
                	   of0('\\Figure{')
                	   writeContent(block[[1L]], tag)
                	   of0('}{')
                	   if (length(block) > 1L) {
-		       includeoptions <- .Rd_get_latex(block[[2]])
-		       if (length(includeoptions)
-			   && startsWith(includeoptions, "options: "))
-			   of0(sub("^options: ", "", includeoptions))
+               	       if (concordance)
+               	   	   conc$saveSrcref(block[[2L]])	
+		       includeoptions <- .Rd_get_latex(block[[2L]])
+                       ## this was wrong if length(includeopptions) > 1
+		       if (length(includeoptions))
+                           for (z in includeoptions)
+                               if(startsWith(z, "options: "))
+                                   of0(sub("^options: ", "", z))
                    }
                	   of0('}')
                	   hasFigures <<- TRUE
@@ -460,8 +496,11 @@ Rd2latex <- function(Rd, out = "", defines = .Platform$OS.type,
                		writeContent(block[[2L]], tag)
                	    else if (tag == "\\ifelse")
                	    	writeContent(block[[3L]], tag),
-               "\\out" = for (i in seq_along(block))
-		   of1(block[[i]]),
+               "\\out" = for (i in seq_along(block)) {
+               	   if (concordance)
+               		conc$saveSrcref(block[[i]])
+		   of1(block[[i]])
+		   },
                stopRd(block, Rdfile, "Tag ", tag, " not recognized")
                )
     }
@@ -473,8 +512,12 @@ Rd2latex <- function(Rd, out = "", defines = .Platform$OS.type,
     	if (length(format) != 1L || RdTags(format) != "TEXT")
     	    stopRd(table, Rdfile, "\\tabular format must be simple text")
         tags <- RdTags(content)
-        of0('\n\\Tabular{', format, '}{')
+	of0('\n\\Tabular{', format, '}{')
+        if (concordance)
+            conc$saveSrcref(table[[1L]])
         for (i in seq_along(tags)) {
+            if (concordance)
+                conc$saveSrcref(content[[i]])	
             switch(tags[i],
                    "\\tab" = of1("&"),
                    "\\cr" = of1("\\\\{}"),
@@ -505,24 +548,32 @@ Rd2latex <- function(Rd, out = "", defines = .Platform$OS.type,
                    	i <- i - 1
                    },
                    "\\item" = {
-                       if (blocktag == "\\value" && !inList) {
+                       if (blocktag %in% c("\\value", "\\arguments") && !inList) {
                            of1("\\begin{ldescription}\n")
                            inList <- TRUE
                        }
                        switch(blocktag,
                               "\\describe" = {
+                              	  if (concordance)
+                              	      conc$saveSrcref(block[[1L]])
                                   of1('\\item[')
                                   writeContent(block[[1L]], tag)
                                   of1('] ')
+                                  if (concordance)
+                                      conc$saveSrcref(block[[2L]])
                                   writeContent(block[[2L]], tag)
                               },
                               "\\value"=,
                               "\\arguments"={
+                              	  if (concordance)
+                              	      conc$saveSrcref(block[[1L]])
                                   of1('\\item[\\code{')
                                   inCode <<- TRUE
                                   writeContent(block[[1L]], tag)
                                   inCode <<- FALSE
                                   of1('}] ')
+                                  if (concordance)
+                                      conc$saveSrcref(block[[2L]])
                                   writeContent(block[[2L]], tag)
                               },
                               "\\enumerate" =,
@@ -534,7 +585,8 @@ Rd2latex <- function(Rd, out = "", defines = .Platform$OS.type,
                    },
                    "\\cr" = of1("\\\\{}"), ## might be followed by [
                { # default
-                   if (inList && !(tag == "TEXT" && isBlankRd(block))) {
+                   if (inList && tag != "COMMENT"
+                              && !(tag == "TEXT" && isBlankRd(block))) {
                        of1("\\end{ldescription}\n")
                        inList <- FALSE
                    }
@@ -556,10 +608,11 @@ Rd2latex <- function(Rd, out = "", defines = .Platform$OS.type,
         if (length(section)) {
 	    ## need \n unless one follows, so
 	    nxt <- section[[1L]]
-	    if (attr(nxt, "Rd_tag") %notin% c("TEXT", "RCODE") ||
+	    if (is.null(nxttag <- attr(nxt, "Rd_tag"))) # erroneous Rd file
+		return()
+	    if (nxttag %notin% c("TEXT", "RCODE") ||
 		!startsWith(as.character(nxt), "\n")) of1("\n")
 	    writeContent(section, tag)
-	    inCodeBlock <<- FALSE
 	    if (last_char != "\n") of1("\n")
 	}
     }
@@ -567,6 +620,8 @@ Rd2latex <- function(Rd, out = "", defines = .Platform$OS.type,
     writeSection <- function(section, tag) {
         if (tag == "\\encoding")
             return()
+    	if (concordance)
+    	    conc$saveSrcref(section)
         save <- sectionLevel
         sectionLevel <<- sectionLevel + 1
         if (tag == "\\alias")
@@ -591,9 +646,6 @@ Rd2latex <- function(Rd, out = "", defines = .Platform$OS.type,
     	} else {
             title <- envTitles[tag]
             of0("%\n\\begin{", title, "}")
-            if(tag %in% c("\\author", "\\description", "\\details", "\\note",
-                          "\\references", "\\seealso", "\\source"))
-                of1("\\relax")
             extra <- sectionExtras[tag]
             if(!is.na(extra)) of0("\n\\begin{", extra, "}")
             if(tag %in% c("\\usage", "\\examples")) inCodeBlock <<- TRUE
@@ -650,7 +702,8 @@ Rd2latex <- function(Rd, out = "", defines = .Platform$OS.type,
         title <- paste(title[nzchar(title)], collapse = " ")
 
 	name <- Rd[[2L]]
-
+	if (concordance)
+	    conc$saveSrcref(name)
 	name <- trim(as.character(Rd[[2L]][[1L]]))
 	ltxname <- latex_escape_name(name)
 
@@ -664,6 +717,12 @@ Rd2latex <- function(Rd, out = "", defines = .Platform$OS.type,
     if (encode_warn)
 	warnRd(Rd, Rdfile, "Some input could not be re-encoded to ",
 	       outputEncoding)
+    if (concordance) {
+    	conc$srcFile <- Rdfile
+        concdata <- followConcordance(conc$finish(), attr(Rd, "concordance"))
+        attr(out, "concordance") <- concdata
+    }
+
     invisible(structure(out, latexEncoding = latexEncoding,
                         hasFigures = hasFigures))
 }
