@@ -1,7 +1,7 @@
 #  File src/library/tools/R/Rd2pdf.R
 #  Part of the R package, https://www.R-project.org
 #
-#  Copyright (C) 1995-2015 The R Core Team
+#  Copyright (C) 1995-2022 The R Core Team
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -19,62 +19,141 @@
 #### R based engine for  R CMD Rdconv|Rd2pdf
 ####
 
-##' @param args
-
-##' @return ...
-
 ## base packages do not have versions and this is called on
 ## DESCRIPTION.in
 ## encodings are tricky: this may be done in a foreign encoding
 ## (e.g., Latin-1 in UTF-8)
 .DESCRIPTION_to_latex <- function(descfile, outfile, version = "Unknown")
 {
-    desc <- read.dcf(descfile)[1, ]
-    ## Using
-    ##   desc <- .read_description(descfile)
-    ## would preserve leading white space in Description and Author ...
+    mytrfm <- .gsub_with_transformed_matches
+    mygsub <- function(pattern, replacement, x)
+        .Internal(gsub(pattern, replacement, x, FALSE, FALSE, FALSE, FALSE))
+    ## Unlike tools:::psub and tools:::fsub, don't use useBytes = TRUE:
+    mypsub <- function(pattern, replacement, x)
+        .Internal(gsub(pattern, replacement, x, FALSE, TRUE,  FALSE, FALSE))
+    myfsub <- function(pattern, replacement, x)
+        .Internal(gsub(pattern, replacement, x, FALSE, FALSE,  TRUE, FALSE))
+    texify <- function(x, one = TRUE, two = FALSE) {
+        ## Handle LaTeX special characters.
+        ## one: handle # $ % & _ ^ ~
+        ##      backslash escape the first five
+        ##      replace ^ by \textasciicircum{}
+        ##      replace ~ by \textasciitilde{}
+        ## two: handle { } \
+        ##      backslash escape the first two
+        ##      replace \ by \textbackslash{}
+        if(two)
+            x <- myfsub("\\", "\\textbackslash", x)
+        if(one) {
+            x <- mypsub("([#$%&_])", "\\\\\\1", x)
+            x <- myfsub("^", "\\textasciicircum", x)
+            x <- myfsub("~", "\\textasciitilde", x)
+        }
+        if(two) {
+            x <- mypsub("([{}])", "\\\\\\1", x)
+            x <- myfsub("\\textbackslash", "\\textbackslash{}", x)
+        }
+        if(one) {
+            x <- myfsub("\\textasciicircum", "\\textasciicircum{}", x)
+            x <- myfsub("\\textasciitilde", "\\textasciitilde{}", x)
+        }
+        x
+    }
+    mytrim <- function(x) {
+        y <- unlist(strsplit(x, "\n", fixed = TRUE))
+        lines2trim <- setdiff(which(nzchar(y)), 1L)
+        if(!length(lines2trim))
+            x
+        else
+            paste(replace(y, lines2trim,
+                          .trim_common_leading_whitespace(y[lines2trim])),
+                  collapse = "\n")
+    }
+
+    desc <- enc2utf8(.read_description(descfile))
     if (is.character(outfile)) {
         out <- file(outfile, "a")
         on.exit(close(out))
     } else out <- outfile
-    cat("\\begin{description}", "\\raggedright{}", sep="\n", file=out)
     fields <- names(desc)
     fields <- fields %w/o% c("Package", "Packaged", "Built")
-    if ("Encoding" %in% fields)
-        cat("\\inputencoding{", latex_canonical_encoding(desc["Encoding"]),
-            "}\n", sep = "", file = out)
+    enc <- desc["Encoding"]
+    if(!is.na(enc)) {
+        cat("\\inputencoding{utf8}\n", file = out)
+    }
+    ## Also try adding PDF title and author metadata.
+    tit <- desc["Title"]
+    tit <- paste0(desc["Package"], ": ",
+                  texify(mygsub("[[:space:]]+", " ", tit), two = TRUE))
+    tit <- paste0("\\ifthenelse{\\boolean{Rd@use@hyper}}",
+                  "{\\hypersetup{pdftitle = {", tit, "}}}{}")
+    writeLines(tit, con = out, useBytes = TRUE)
+    ## Only try author from Authors@R.
+    if(!is.na(aar <- desc["Authors@R"])) {
+        aar <- tryCatch(utils:::.read_authors_at_R_field(aar),
+                        error = identity)
+        if(!inherits(aar, "error")) {
+            aar <- Filter(utils:::.person_has_author_role, aar)
+            aut <- format(aar, include = c("given", "family"))
+            aut <- paste(aut[nzchar(aut)], collapse = "; ")
+            aut <- texify(mygsub("[[:space:]]+", " ", aut), two = TRUE)
+            if(nzchar(aut)) {
+                aut <- paste0("\\ifthenelse{\\boolean{Rd@use@hyper}}",
+                              "{\\hypersetup{pdfauthor = {", aut, "}}}{}")
+                writeLines(aut, con = out, useBytes = TRUE)
+            }
+        }
+    }
+    ## And now the actual content.
+    cat("\\begin{description}", "\\raggedright{}", sep="\n", file=out)
     for (f in fields) {
         ## Drop 'Authors@R' for now: this is formatted badly by \AsIs,
         ## and ideally was used for auto-generating the Author and
         ## Maintainer fields anyways ...
         if(f == "Authors@R") next
         text <- desc[f]
+        if(f %in% c("Author", "Description"))
+            text <- mytrim(text)
         ## munge 'text' appropriately (\\, {, }, "...")
         ## not sure why just these: copied from Perl Rd2dvi, then added to.
         ## KH: the LaTeX special characters are
         ##   # $ % & _ ^ ~ { } \
         ## \Rd@AsIs@dospecials in Rd.sty handles the first seven, so
         ## braces and backslashes need explicit handling.
-        text <- gsub('"([^"]*)"', "\\`\\`\\1''", text, useBytes = TRUE)
-        text <- gsub("\\", "\\textbackslash{}", text, fixed = TRUE, useBytes = TRUE)
-        text <- gsub("([{}$#_])", "\\\\\\1", text, useBytes = TRUE)
-        text <- gsub("@VERSION@", version, text, fixed = TRUE, useBytes = TRUE)
-        ## text can have paras, and digest/DESCRIPTION does.
-        ## \AsIs is per-para.
-        text <- strsplit(text, "\n\n", fixed = TRUE, useBytes = TRUE)[[1L]]
-        Encoding(text) <- "unknown"
+        text <- mygsub('"([^"]*)"', "\\`\\`\\1''", text)
+        text <- texify(text, one = FALSE, two = TRUE)
+        text <- myfsub("@VERSION@", version, text)
         if(f %in% c("Author", "Maintainer", "Contact"))
-            text <- gsub("<([^@ ]+)@([^> ]+)>",
-                         "}\\\\email{\\1@\\2}\\\\AsIs{",
-                         text, useBytes = TRUE)
-        if(f %in% c("URL", "BugReports", "Contact"))
-            text <- gsub("(http://|ftp://|https://)([^[:space:],]+)",
-                         "}\\\\url{\\1\\2}\\\\AsIs{",
-                         text, useBytes = TRUE)
+            text <- mytrfm("<([^@ ]+)@([^> ]+)>",
+                           "}\\\\email{%s@%s}\\\\AsIs{",
+                           text,
+                           list(texify, texify),
+                           c(1L, 2L))
+        if(f %in% c("URL", "BugReports", "Additional_repositories"))
+            text <- mygsub("(http://|ftp://|https://)([^[:space:],]+)",
+                           "}\\\\url{\\1\\2}\\\\AsIs{",
+                           text)
+        if(f %in% c("Author",       # possibly with ORCID URLs inside <>
+                    "Description")) {
+            text <- mygsub("<(http://|ftp://|https://)([^[:space:],>]+)>",
+                           "<}\\\\url{\\1\\2}\\\\AsIs{>",
+                           text)
+        }
+        if(f == "Description") {   # DOI and arXiv identifiers inside <>
+            text <- mytrfm("<(DOI:|doi:)([[:space:]]*)([^[:space:]]+)>",
+                           "<}\\\\Rhref{https://doi.org/%s}{\\1%s}\\\\AsIs{>",
+                           text,
+                           list(identity, texify),
+                           c(3L, 3L))
+            ## Fancy escaping should not be needed for arXiv ids.
+            text <- mygsub("<(arXiv:|arxiv:)([[:alnum:]/.-]+)([[:space:]]*\\[[^]]+\\])?>",
+                           "<}\\\\Rhref{https://arxiv.org/abs/\\2}{\\1\\2}\\\\AsIs{\\3>",
+                           text)
+        }
         text <- paste0("\\AsIs{", text, "}")
-        ## Not entirely safe: in theory, tags could contain \ ~ ^.
-        cat("\\item[", gsub("([#$%&_{}])", "\\\\\\1", f),
-            "]", paste(text, collapse = "\n\n"),  "\n", sep = "", file=out)
+        writeLines(paste0("\\item[", texify(f, TRUE, TRUE), "]",
+                          text),
+                   con = out, useBytes = TRUE)
     }
     cat("\\end{description}\n", file = out)
 }
@@ -85,12 +164,12 @@
              append = FALSE, extraDirs = NULL, internals = FALSE,
              silent = FALSE, pkglist = NULL)
 {
-    if (dir.exists(files))
-        .pkg2tex(files, outfile, encoding = encoding, append = append,
-                 asChapter = FALSE, extraDirs = extraDirs,
-                 internals = internals, silent = silent,
-                 pkglist = pkglist)
-    else {
+    if (dir.exists(files)) {
+        .pkg2tex(files, outfile, internals = internals, asChapter = FALSE, 
+                 encoding = encoding, outputEncoding = outputEncoding,
+                 extraDirs = extraDirs, append = append,
+                 silent = silent, pkglist = pkglist)
+    } else {
         files <- strsplit(files, "[[:space:]]+")[[1L]]
         latexdir <- tempfile("ltx")
         dir.create(latexdir)
@@ -99,8 +178,8 @@
             outfile <- file(outfile, if (append) "at" else "wt")
             on.exit(close(outfile))
         }
-        latexEncodings <- character()
-        hasFigures <- FALSE
+        latexEncodings <- character() # Record any encodings used in the output
+        hasFigures <- FALSE           # and whether graphics is used
         macros <- initialRdMacros(pkglist = pkglist)
         for(f in files) {
             if (!silent) cat("  ", basename(f), "\n", sep="")
@@ -182,9 +261,7 @@
                 return(invisible(character()))
             }
             cnt <- 0L
-            macros <- initialRdMacros(pkglist)
             for(f in names(Rd)) {
-##                bf <- basename(f)
                 cnt <- cnt + 1L
                 if (!silent && cnt %% 10L == 0L)
                     message(".", appendLF=FALSE, domain=NA)
@@ -192,11 +269,9 @@
                 outfilename <- file.path(latexdir, out)
                 res <- Rd2latex(Rd[[f]],
 				  outfilename,
-				  encoding = encoding,
 				  outputEncoding = outputEncoding,
 				  defines = NULL,
-				  writeEncoding = !asChapter,
-				  macros = macros)
+				  writeEncoding = !asChapter)
                 latexEncodings <- c(latexEncodings,
                                     attr(res, "latexEncoding"))
                 if (attr(res, "hasFigures")) {
@@ -226,13 +301,18 @@
                 if (!length(files))
                     stop("this package has a ", sQuote("man"), " directory but no .Rd files",
                          domain = NA)
+                macros <- loadPkgRdMacros(pkgdir)
+                macros <- initialRdMacros(pkglist, macros)
            } else {
+                ## (Be nice and find Rd files & system macros also when 'pkgdir' is
+                ## not a package root directory.)
                 files <- c(Sys.glob(file.path(pkgdir, "*.Rd")),
                            Sys.glob(file.path(pkgdir, "*.rd")))
                 if (!length(files))
                     stop("this package does not have either a ", sQuote("latex"),
                          " or a (source) ", sQuote("man"), " directory",
                          domain = NA)
+                macros <- initialRdMacros(pkglist)
             }
             paths <- files
             ## Use a partial Rd db if there is one.
@@ -251,10 +331,6 @@
             if (!silent) message("Converting Rd files to LaTeX ",
                                  appendLF = FALSE, domain = NA)
             cnt <- 0L
-            macros <- loadPkgRdMacros(pkgdir, initialRdMacros())
-            ## (Be nice and give the system macros also when 'pkgdir' is
-            ## not a package root directory.)
-            macros <- initialRdMacros(pkglist, macros)
             for(i in seq_along(paths)) {
                 cnt <- cnt + 1L
                 if(!silent && cnt %% 10L == 0L)
@@ -402,7 +478,7 @@
                 R.version[["major"]], ".",  R.version[["minor"]],
                 " (r", R.version[["svn rev"]], ")\n", sep = "")
             cat("",
-                "Copyright (C) 1997-2015 The R Core Team.",
+                .R_copyright_msg(1997),
                 "This is free software; see the GNU General Public License version 2",
                 "or later for copying conditions.  There is NO warranty.",
                 sep="\n")
@@ -477,7 +553,7 @@
 ### * .Rd2pdf
 
 .Rd2pdf <-
-function(pkgdir, outfile, title, batch = FALSE,
+function(pkgdir, outfile, title, silent = FALSE,
          description = TRUE, only_meta = FALSE,
          enc = "unknown", outputEncoding = "UTF-8", files_or_dir, OSdir,
          internals = FALSE, index = TRUE, pkglist = NULL)
@@ -495,13 +571,12 @@ function(pkgdir, outfile, title, batch = FALSE,
             pkg_enc <- desc["Encoding"]
             if (!is.na(pkg_enc)) {
             	enc <- pkg_enc
-            	outputEncoding <- pkg_enc
             }
         }
     }
 
     ## Rd2.tex part 1: header
-    if (batch) writeLines("\\nonstopmode{}", out)
+    writeLines("\\nonstopmode{}", out)  # for texinfo < 6.7 and MikTeX's texify
     cat("\\documentclass[", Sys.getenv("R_PAPERSIZE"), "paper]{book}\n",
         "\\usepackage[", Sys.getenv("R_RD4PDF", "times,inconsolata,hyper"), "]{Rd}\n",
         sep = "", file = out)
@@ -563,7 +638,7 @@ function(pkgdir, outfile, title, batch = FALSE,
         "\\Rdcontents{\\R{} topics documented:}"
     } else ""
 
-    latexEncodings <- character()
+    latexEncodings <- if(description) "utf8" else character()
     hasFigures <- FALSE
     ## if this looks like a package with no man pages, skip body
     if (file.exists(file.path(pkgdir, "DESCRIPTION")) &&
@@ -572,15 +647,14 @@ function(pkgdir, outfile, title, batch = FALSE,
           dir.exists(file.path(pkgdir, "latex")))) only_meta <- TRUE
     if (!only_meta) {
         if (nzchar(toc)) writeLines(toc, out)
-        res <- .Rdfiles2tex(files_or_dir, out, encoding = enc, append = TRUE,
-                         extraDirs = OSdir, internals = internals,
-                         silent = batch, pkglist = pkglist)
+        res <- .Rdfiles2tex(files_or_dir, out, encoding = enc,
+                            outputEncoding = outputEncoding,
+                            append = TRUE, extraDirs = OSdir, 
+                            internals = internals, silent = silent,
+                            pkglist = pkglist)
         if(length(res)) {
-            latexEncodings <- res$latexEncodings
+            latexEncodings <- c(latexEncodings, res$latexEncodings)
             hasFigures <- res$hasFigures
-        } else {
-            latexEncodings <- character()
-            hasFigures <- FALSE
         }
     }
 
@@ -637,23 +711,18 @@ function(pkgdir, outfile, title, batch = FALSE,
     ## 'success'), and 1 otherwise, so that the return value can be used
     ## for shell 'if' tests.
 
-    ## <NOTE>
-    ## For now only used for the R sources (/doc/manual/Makefile.in)
-    ## hence no need to also look for Rd files with '.rd' extension.
-    ## </NOTE>
-
     if (!file.exists(file)) return(0L)
     age <- file.mtime(file)
 
     if (any(file.mtime(c(Sys.glob(file.path(dir, "man", "*.Rd")),
-                        Sys.glob(file.path(dir, "man", "*.rd"))))
-                       > age))
+                         Sys.glob(file.path(dir, "man", "*.rd"))))
+            > age))
         return(0L)
 
     if (dir.exists(file.path(dir, OS))) {
         if (any(file.mtime(c(Sys.glob(file.path(dir, "man", OS, "*.Rd")),
-                            Sys.glob(file.path(dir, "man", OS, "*.rd"))))
-                           > age))
+                             Sys.glob(file.path(dir, "man", OS, "*.rd"))))
+                > age))
             return(0L)
     }
 
@@ -667,11 +736,11 @@ function(pkgdir, outfile, title, batch = FALSE,
 
 ..Rd2pdf <- function(args = NULL, quit = TRUE)
 {
-    do_cleanup <- function() {
+    do_cleanup <- function(quiet = FALSE) {
         if(clean) {
             setwd(startdir)
             unlink(build_dir, recursive = TRUE)
-        } else {
+        } else if (!quiet) {
             cat("You may want to clean up by 'rm -Rf ", build_dir, "'\n", sep="")
         }
     }
@@ -690,7 +759,7 @@ function(pkgdir, outfile, title, batch = FALSE,
             "The Rd sources are assumed to be ASCII unless they contain \\encoding",
             "declarations (which take priority) or --encoding is supplied or if using",
             "package sources, if the package DESCRIPTION file has an Encoding field.",
-            "The output encoding defaults to the package encoding then to 'UTF-8'.",
+            "The output encoding defaults to 'UTF-8'.",
             "",
             "Files are listed in the order given: for a package they are in alphabetic",
             "order of the \\name sections.",
@@ -698,7 +767,7 @@ function(pkgdir, outfile, title, batch = FALSE,
             "Options:",
             "  -h, --help		print short help message and exit",
             "  -v, --version		print version info and exit",
-            "      --batch		no interaction",
+            "  -q, --quiet		no output unless errors",
             "      --no-clean	do not remove created temporary files",
             "      --no-preview	do not preview generated PDF file",
             "      --encoding=enc    use 'enc' as the default input encoding",
@@ -712,7 +781,7 @@ function(pkgdir, outfile, title, batch = FALSE,
             "      --no-index	do not index output",
             "      --no-description	do not typeset the description of a package",
             "      --internals	typeset 'internal' documentation (usually skipped)",
-            "      --build_dir=DIR	use DIR as the working directory",
+            "      --build-dir=DIR	use DIR as the working directory",
             "      --RdMacros=pkglist",
             "             		packages from which to get Rd macros",
             "",
@@ -736,13 +805,13 @@ function(pkgdir, outfile, title, batch = FALSE,
         stop("current working directory cannot be ascertained")
     build_dir <- paste0(".Rd2pdf", Sys.getpid())
     title <- ""
-    batch <- FALSE
+    quiet <- FALSE
     clean <- TRUE
     only_meta <- FALSE
     out_ext <- "pdf"
     output <- ""
     enc <- "unknown"
-    outenc <- "latin1"
+    outenc <- "UTF-8"
     index <- TRUE
     description <- TRUE
     internals <- FALSE
@@ -766,13 +835,15 @@ function(pkgdir, outfile, title, batch = FALSE,
                 R.version[["major"]], ".",  R.version[["minor"]],
                 " (r", R.version[["svn rev"]], ")\n", sep = "")
             cat("",
-                "Copyright (C) 2000-2011 The R Core Team.",
+                .R_copyright_msg(2000),
                 "This is free software; see the GNU General Public License version 2",
                 "or later for copying conditions.  There is NO warranty.",
                 sep="\n")
             q("no", runLast = FALSE)
         } else if (a == "--batch") {
-            batch <- TRUE
+            # ignore for back-compatibility (now always use batch)
+        } else if (a %in% c("-q", "--quiet")) {
+            quiet <- TRUE
         } else if (a == "--no-clean") {
             clean <- FALSE
         } else if (a == "--no-preview") {
@@ -821,17 +892,17 @@ function(pkgdir, outfile, title, batch = FALSE,
     if(WINDOWS) files[1L] <- sub("[\\/]$", "", files[1L])
     if(dir.exists(files[1L])) {
         if(file.exists(file.path(files[1L], "DESCRIPTION"))) {
-            cat("Hmm ... looks like a package\n")
+            if (!quiet) cat("Hmm ... looks like a package\n")
             dir <- files[1L]
             if(!nzchar(output)) output <- paste(basename(dir), out_ext, sep = ".")
         } else if (file.exists(f <- file.path(files[1L], "DESCRIPTION.in"))
                    && any(grepl("^Priority: *base", readLines(f)))) {
-            cat("Hmm ... looks like a package from the R distribution\n")
+            if (!quiet) cat("Hmm ... looks like a package from the R distribution\n")
             dir <- files[1L]
             if(!nzchar(output)) output <- paste(basename(dir), out_ext, sep = ".")
             if(index && basename(dir) == "base") {
                 index <- FALSE
-                cat("_not_ indexing 'base' package\n")
+                if (!quiet) cat("_not_ indexing 'base' package\n")
             }
         } else {
             dir <- if(dir.exists(d <- file.path(files[1L], "man"))) d else files[1L]
@@ -844,31 +915,31 @@ function(pkgdir, outfile, title, batch = FALSE,
     if(!nzchar(dir)) dir <- paste(files, collapse = " ")
 
     ## Prepare for building the documentation.
-    if(dir.exists(build_dir) && unlink(build_dir, recursive = TRUE)) {
-        cat("cannot write to build dir\n")
-        q("no", status = 2L, runLast = FALSE)
-    }
-    dir.create(build_dir, FALSE)
     if(!nzchar(output)) output <- paste0("Rd2.", out_ext)
     if(file.exists(output) && !force) {
         cat("file", sQuote(output), "exists; please remove it first\n")
         q("no", status = 1L, runLast = FALSE)
     }
+    if(dir.exists(build_dir) && unlink(build_dir, recursive = TRUE)) {
+        cat("cannot write to build dir\n")
+        q("no", status = 2L, runLast = FALSE)
+    }
+    dir.create(build_dir, FALSE)
 
     res <-
         try(.Rd2pdf(files[1L], file.path(build_dir, "Rd2.tex"),
-                    title, batch, description, only_meta,
+                    title, quiet, description, only_meta,
                     enc, outenc, dir, OSdir, internals, index,
                     pkglist))
     if (inherits(res, "try-error"))
         q("no", status = 11L, runLast = FALSE)
 
-    if (!batch)  cat("Creating", out_ext, "output from LaTeX ...\n")
+    if (!quiet)  cat("Creating", out_ext, "output from LaTeX ...\n")
     setwd(build_dir)
 
-    res <- try(texi2pdf('Rd2.tex', quiet = FALSE, index = index))
+    res <- try(texi2pdf('Rd2.tex', quiet = quiet, index = index))
     if(inherits(res, "try-error")) {
-        res <- try(texi2pdf('Rd2.tex', quiet = FALSE, index = index))
+        res <- try(texi2pdf('Rd2.tex', quiet = quiet, index = index))
         if(inherits(res, "try-error")) {
             message("Error in running tools::texi2pdf()")
             do_cleanup()
@@ -877,12 +948,12 @@ function(pkgdir, outfile, title, batch = FALSE,
     }
 
     setwd(startdir)
-    cat("Saving output to", sQuote(output), "...\n")
+    if (!quiet)  cat("Saving output to", sQuote(output), "...\n")
     file.copy(file.path(build_dir, paste0("Rd2.", out_ext)), output,
               overwrite = force)
-    cat("Done\n")
+    if (!quiet)  cat("Done\n")
 
-    do_cleanup()
+    do_cleanup(quiet)
     if(preview != "false") system(paste(preview, output))
     if (quit)
     	q("no", runLast = FALSE)
