@@ -1,7 +1,7 @@
 #  File src/library/tools/R/admin.R
 #  Part of the R package, https://www.R-project.org
 #
-#  Copyright (C) 1995-2021 The R Core Team
+#  Copyright (C) 1995-2023 The R Core Team
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -308,6 +308,12 @@ function(dir, outDir)
                outFile)
     enc <- as.vector(db["Encoding"])
     need_enc <- !is.na(enc) # Encoding was specified
+    testParse <- function(...) { # parse only to detect errors
+        op <- options(showErrorCalls=FALSE)
+        on.exit(options(op))
+        parse(...)
+        invisible()
+    }
     ## assume that if locale is 'C' we can used 8-bit encodings unchanged.
     if(need_enc && (Sys.getlocale("LC_CTYPE") %notin% c("C", "POSIX"))) {
         con <- file(outFile, "a")
@@ -331,7 +337,9 @@ function(dir, outDir)
                                 paste(bad2, collapse = ", ")),
                         domain = NA, call. = FALSE)
             }
-            writeLines(paste0("#line 1 \"", f, "\""), con)
+            line1 <- paste0("#line 1 \"", f, "\"")
+            testParse(text = c(line1, tmp))
+            writeLines(line1, con)
             writeLines(tmp, con)
         }
 	close(con); on.exit()
@@ -341,16 +349,11 @@ function(dir, outDir)
         ##   writeLines(sapply(codeFiles, readLines), outFile)
         ## instead, but this would be much slower ...
         ## use fast version of file.append that ensures LF between files
+        lapply(codeFiles, testParse)
         if(!all(.file_append_ensuring_LFs(outFile, codeFiles)))
             stop("unable to write code files")
         ## </NOTE>
     }
-    ## A syntax check here, so that we do not install a broken package.
-    ## FIXME:  this is only needed if we don't lazy load, as the lazy loader
-    ## would detect the error.
-    op <- options(showErrorCalls=FALSE)
-    on.exit(options(op))
-    parse(outFile)
     invisible()
 }
 
@@ -483,10 +486,13 @@ function(dir, outDir)
 }
 
 ### * .install_package_vignettes2
-## called from R CMD INSTALL for pre 3.0.2-built tarballs, and for base packages
+## called from R CMD INSTALL for pre 3.0.2-built tarballs
+## and for installation from package sources (missing build/vignette.rds),
+## including for the temporary package installation during R CMD build,
+## and when building base packages (where we need to tangle vignettes)
 
 .install_package_vignettes2 <-
-function(dir, outDir, encoding = "")
+function(dir, outDir, encoding = "", tangle = FALSE)
 {
     dir <- file_path_as_absolute(dir)
     subdirs <- c("vignettes", file.path("inst", "doc"))
@@ -501,7 +507,7 @@ function(dir, outDir, encoding = "")
     outDir <- file_path_as_absolute(outDir)
     packageName <- basename(outDir)
     outVignetteDir <- file.path(outDir, "doc")
-    ## --fake  and --no-inst installs do not have a outVignetteDir.
+    ## --no-inst installs do not have a outVignetteDir.
     if(!dir.exists(outVignetteDir)) return(invisible())
 
     ## If there is an HTML index in the @file{inst/doc} subdirectory of
@@ -534,7 +540,7 @@ function(dir, outDir, encoding = "")
     })
 
     vignetteIndex <- .build_vignette_index(vigns)
-    if(NROW(vignetteIndex) > 0L) {
+    if(tangle && NROW(vignetteIndex) > 0L) {
         cwd <- getwd()
         if (is.null(cwd))
             stop("current working directory cannot be ascertained")
@@ -542,10 +548,7 @@ function(dir, outDir, encoding = "")
 
 	loadVignetteBuilder(dir, mustwork = FALSE)
 
-        ## install tangled versions of Sweave vignettes.  FIXME:  Vignette
-        ## *.R files should have been included when the package was built,
-        ## but in the interim before they are all built with the new code,
-        ## this is needed.
+        ## install tangled versions of Sweave vignettes.
         for(i in seq_along(vigns$docs)) {
             file <- vigns$docs[i]
             if (!is.null(vigns$sources) && !is.null(vigns$sources[file][[1]]))
@@ -557,6 +560,9 @@ function(dir, outDir, encoding = "")
                 if(nzchar(enc)) paste("using", sQuote(enc)), "\n")
 
 	    engine <- try(vignetteEngine(vigns$engines[i]), silent = TRUE)
+	    ## tangling in outVignetteDir would fail if the vignette relied
+	    ## on SweaveInput/child documents (not copied over),
+	    ## but base packages currently don't do that
 	    if (!inherits(engine, "try-error"))
             	engine$tangle(file, quiet = TRUE, encoding = enc)
             setwd(outVignetteDir) # just in case some strange tangle function changed it
@@ -753,13 +759,16 @@ function(dir, outDir, keep.source = TRUE)
                           outVignetteDir),
                  domain = NA)
     }
+
+    compactPDF(outVignetteDir, gs_quality = "ebook")
+
     ## Need to change out of this dir before we delete it,
     ## at least on Windows.
     setwd(cwd)
     unlink(buildDir, recursive = TRUE)
     ## Now you need to update the HTML index!
     ## This also creates the .R files
-    .install_package_vignettes2(dir, outDir)
+    .install_package_vignettes2(dir, outDir, tangle = TRUE)
     invisible()
 }
 
@@ -1037,7 +1046,7 @@ compactPDF <-
     function(paths, qpdf = Sys.which(Sys.getenv("R_QPDF", "qpdf")),
              gs_cmd = Sys.getenv("R_GSCMD", ""),
              gs_quality = Sys.getenv("GS_QUALITY", "none"),
-             gs_extras = character())
+             gs_extras = character(), verbose = FALSE)
 {
     use_qpdf <- nzchar(qpdf)
     qpdf_flags <- "--object-streams=generate"
@@ -1057,15 +1066,20 @@ compactPDF <-
     }
     gs_quality <- match.arg(gs_quality, c("none", "printer", "ebook", "screen"))
     use_gs <- if(gs_quality != "none") nzchar(gs_cmd <- find_gs_cmd(gs_cmd)) else FALSE
+    if(verbose) cat(sprintf("qs_quality=\"%s\" : use_gs=%s, use_qpdf=%s\n",
+                            gs_quality, use_gs, use_qpdf))
     if (!use_gs && !use_qpdf) return()
     if(length(paths) == 1L && dir.exists(paths))
         paths <- Sys.glob(file.path(paths, "*.pdf"))
+    if(verbose) cat(sprintf("#{pdf}s = length(paths) = %d\n", length(paths)))
     dummy <- rep.int(NA_real_, length(paths))
     ans <- data.frame(old = dummy, new = dummy, row.names = paths)
     ## These should not have spaces, but quote below to be safe.
     tf <- tempfile("pdf"); tf2 <- tempfile("pdf")
+    verb2 <- verbose >= 2
     for (p in paths) {
         res <- 0
+        if(verbose) cat(sprintf("- %s:%s", p, if(verb2)"\n" else " "))
         if (use_gs) {
             res <- system2(gs_cmd,
                            c("-q -dNOPAUSE -dBATCH -sDEVICE=pdfwrite",
@@ -1074,24 +1088,43 @@ compactPDF <-
                              "-dAutoRotatePages=/None",
                              "-dPrinted=false",
                              sprintf("-sOutputFile=%s", shQuote(tf)),
-                             gs_extras, shQuote(p)), FALSE, FALSE)
+                             gs_extras, shQuote(p)), verb2, verb2)
+            if(verbose) {
+                res0 <- (verb2 && !length(res))
+                cat(sprintf("   gs: res=%s; ", if(res0) "<>" else res))
+                if(verb2) res <- if(res0) 0 else attr(res, "status")
+            }
             if(!res && use_qpdf) {
                 unlink(tf2) # precaution
                 file.rename(tf, tf2)
                 res <- system2(qpdf, c(qpdf_flags, shQuote(tf2), shQuote(tf)),
-                               FALSE, FALSE)
+                               verb2, verb2)
+                if(verbose) {
+                    res0 <- (verb2 && !length(res))
+                    cat(sprintf(" + qpdf: res=%s; ", if(res0) "<>" else res))
+                    if(verb2) res <- if(res0) 0 else attr(res, "status")
+                }
                 unlink(tf2)
             }
         } else if(use_qpdf) {
             res <- system2(qpdf, c(qpdf_flags, shQuote(p), shQuote(tf)),
-                           FALSE, FALSE)
+                           verb2, verb2)
+            if(verbose) {
+                res0 <- (verb2 && !length(res))
+                cat(sprintf(" only qpdf: res=%s; ", if(res0) "<>" else res))
+                if(verb2) res <- if(res0) 0 else attr(res, "status")
+            }
         }
         if(!res && file.exists(tf)) {
             old <- file.size(p); new <-  file.size(tf)
+            if(verbose)
+                cat(sprintf("\n    ==> (new=%g)/(old=%g) = %g", new,old, new/old))
             if(new/old < 0.9 && new < old - 1e4) {
+                if(verbose) cat(" =====> using it !!\n")
                 file.copy(tf, p, overwrite = TRUE)
                 ans[p, ] <- c(old, new)
-            }
+            } else if(verbose) cat(" .. not worth using\n")
+
         }
         unlink(tf)
     }
