@@ -1,7 +1,7 @@
 #  File src/library/tools/R/build.R
 #  Part of the R package, https://www.R-project.org
 #
-#  Copyright (C) 1995-2021 The R Core Team
+#  Copyright (C) 1995-2022 The R Core Team
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -193,10 +193,7 @@ inRbuildignore <- function(files, pkgdir) {
 
     temp_install_pkg <- function(pkgdir, libdir) {
 	dir.create(libdir, mode = "0755", showWarnings = FALSE)
-        install_missing_dependencies <-
-            config_val_to_logical(Sys.getenv("_R_BUILD_INSTALL_MISSING_DEPENDENCIES_",
-                                             "no"))
-        if(install_missing_dependencies &&
+        if(nzchar(install_dependencies) &&
            all((repos <- getOption("repos")) != "@CRAN@")) {
             ## try installing missing dependencies too
             available <- utils::available.packages(repos = repos)
@@ -207,9 +204,9 @@ inRbuildignore <- function(files, pkgdir) {
                                 drop = FALSE],
                       db[colnames(available)])
             depends <- package_dependencies(package, available,
-                                            which = "most")
+                                            which = install_dependencies)
             depends <- setdiff(unlist(depends),
-                               utils::installed.packages())
+                               rownames(utils::installed.packages()))
             if(length(depends)) {
                 message(paste(strwrap(sprintf("installing dependencies %s",
                                               paste(sQuote(sort(depends)),
@@ -449,17 +446,17 @@ inRbuildignore <- function(files, pkgdir) {
             messageLog(Log, "cleaning src")
             if (WINDOWS) {
                 have_make <- nzchar(Sys.which(Sys.getenv("MAKE", "make")))
-                if (file.exists("Makefile.win")) {
+                if (file.exists(fn <- "Makefile.ucrt") || file.exists(fn <- "Makefile.win")) {
                     if (have_make)
-                        Ssystem(Sys.getenv("MAKE", "make"), "-f Makefile.win clean")
+                        Ssystem(Sys.getenv("MAKE", "make"), paste0("-f ", fn, " clean"))
                     else warning("unable to run 'make clean' in 'src'",
                                  domain = NA)
                 } else {
-                    if (file.exists("Makevars.win")) {
+                    if (file.exists(fn <- "Makevars.ucrt") || file.exists(fn <- "Makevars.win")) {
                         if (have_make) {
                             makefiles <- paste("-f",
                                                shQuote(file.path(R.home("share"), "make", "clean.mk")),
-                                           "-f Makevars.win")
+                                           "-f", fn)
                             Ssystem(Sys.getenv("MAKE", "make"),
                                     c(makefiles, "clean"))
                         } else warning("unable to run 'make clean' in 'src'",
@@ -504,14 +501,20 @@ inRbuildignore <- function(files, pkgdir) {
         ## to 'Writing R Extensions', but were not in Perl version (nor
         ## was cleanup.win used).
         if (WINDOWS) {
-            if (file.exists("cleanup.win")) {
+            has_cleanup_ucrt <- file.exists("cleanup.ucrt")
+            if (has_cleanup_ucrt || file.exists("cleanup.win")) {
                 ## check we have sh.exe first
                 if (nzchar(Sys.which("sh.exe"))) {
                     Sys.setenv(R_PACKAGE_NAME = pkgname)
                     Sys.setenv(R_PACKAGE_DIR = pkgdir)
                     Sys.setenv(R_LIBRARY_DIR = dirname(pkgdir))
-                    messageLog(Log, "running 'cleanup.win'")
-                    Ssystem("sh", "./cleanup.win")
+                    if (has_cleanup_ucrt) {
+                        messageLog(Log, "running 'cleanup.ucrt'")
+                        Ssystem("sh", "./cleanup.ucrt")
+                    } else {
+                        messageLog(Log, "running 'cleanup.win'")
+                        Ssystem("sh", "./cleanup.win")
+                    }
                 }
             }
         } else if (file_test("-x", "cleanup")) {
@@ -521,6 +524,7 @@ inRbuildignore <- function(files, pkgdir) {
             messageLog(Log, "running 'cleanup'")
             Ssystem("./cleanup")
         }
+        revert_install_time_patches()
     }
 
     update_Rd_index <- function(oldindex, Rd_files, Log)
@@ -576,14 +580,29 @@ inRbuildignore <- function(files, pkgdir) {
                            os = c("unix", "windows"), step = 1)
     	if (!length(db)) return(FALSE)
 
-    	# Strip the pkgdir off the names
+    	## Strip the pkgdir off the names
     	names(db) <- substring(names(db),
                                nchar(file.path(pkgdir, "man")) + 2L)
 
-	containsSexprs <-
-            which(sapply(db, function(Rd) getDynamicFlags(Rd)["\\Sexpr"]))
-	if (!length(containsSexprs)) return(FALSE)
-
+        btinfo <- do.call(rbind,
+                          lapply(db, .Rd_get_Sexpr_build_time_info))
+        if(!any(btinfo[, "\\Sexpr"])) {
+            return(FALSE)
+        } else {
+            ## <FIXME>
+            ## Remove eventually.
+            ## If we only have Sexprs we never process when building,
+            ## for now create an empty partial db to make older versions
+            ## of the CRAN incoming check code happy.
+            if(!any(btinfo[, c("build", "later")])) {
+                dir.create("build", showWarnings = FALSE)
+                saveRDS(structure(list(), names = character()),
+                        build_partial_Rd_db_path, version = 2L)
+                return(FALSE)
+            }
+            ## </FIXME>
+        }
+                    
 	messageLog(Log, "installing the package to process help pages")
 
         dir.create(libdir, mode = "0755", showWarnings = FALSE)
@@ -593,29 +612,44 @@ inRbuildignore <- function(files, pkgdir) {
 
         temp_install_pkg(pkgdir, libdir)
 
-	containsBuildSexprs <-
-            which(sapply(db, function(Rd) getDynamicFlags(Rd)["build"]))
-
-	if (length(containsBuildSexprs)) {
-	    for (i in containsBuildSexprs)
+        containsBuildSexprs <- which(btinfo[, "build"])
+	if(length(containsBuildSexprs)) {
+	    for (i in containsBuildSexprs) {
 		db[[i]] <- prepare_Rd(db[[i]], stages = "build",
                                       stage2 = FALSE, stage3 = FALSE)
+                ## There could be build Sexprs giving install/render
+                ## Sexprs ...
+                btinfo[i, ] <- .Rd_get_Sexpr_build_time_info(db[[i]])
+            }
 	    messageLog(Log, "saving partial Rd database")
 	    partial <- db[containsBuildSexprs]
 	    dir.create("build", showWarnings = FALSE)
 	    ## version = 2L for maximal back-compatibility
 	    saveRDS(partial, build_partial_Rd_db_path, version = 2L)
 	}
+
+        containsLaterSexprs <- which(btinfo[, "later"])
+        if(length(containsLaterSexprs)) {
+	    for (i in containsLaterSexprs) {
+		db[[i]] <- prepare_Rd(db[[i]], stages = c("install", "render"),
+                                      stage2 = FALSE, stage3 = FALSE)
+            }
+            stage23 <- db[containsLaterSexprs]
+            dir.create("build", showWarnings = FALSE)
+            build_stage23_Rd_db_path <-
+                file.path("build", "stage23.rdb")
+            if(file.exists(build_stage23_Rd_db_path))
+                unlink(build_stage23_Rd_db_path)
+            saveRDS(stage23, build_stage23_Rd_db_path, version = 2L)
+        }
+        
 	needRefman <- manual &&
             parse_description_field(desc, "BuildManual", TRUE) &&
-            any(vapply(db,
-                       function(Rd)
-                           any(getDynamicFlags(Rd)[c("install", "render")]),
-                       NA))
+            any(btinfo[, "later"])
 	if (needRefman) {
 	    messageLog(Log, "building the PDF package manual")
 	    dir.create("build", showWarnings = FALSE)
-	    ..Rd2pdf(c("--force", "--no-preview",
+	    ..Rd2pdf(c("--force", "--no-preview", "--quiet",
 	               paste0("--output=", build_refman_path),
 	               pkgdir), quit = FALSE)
         }
@@ -643,8 +677,8 @@ inRbuildignore <- function(files, pkgdir) {
     fix_nonLF_in_make_files <- function(pkgname, Log) {
         fix_nonLF_in_files(pkgname,
                            paste0("^(",
-                                  paste(c("Makefile", "Makefile.in", "Makefile.win",
-                                          "Makevars", "Makevars.in", "Makevars.win"),
+                                  paste(c("Makefile", "Makefile.in", "Makefile.win", "Makefile.ucrt",
+                                          "Makevars", "Makevars.in", "Makevars.win", "Makevars.ucrt"),
                                         collapse = "|"), ")$"), Log)
         ## Other Makefiles
         makes <- dir(pkgname, pattern = "^Makefile$",
@@ -853,6 +887,12 @@ inRbuildignore <- function(files, pkgdir) {
     keep_empty <-
         config_val_to_logical(Sys.getenv("_R_BUILD_KEEP_EMPTY_DIRS_", "FALSE"))
 
+    install_dependencies <- Sys.getenv("_R_BUILD_INSTALL_DEPENDENCIES_")
+    if(nzchar(install_dependencies) &&
+       (install_dependencies %notin% c("strong", "most", "all")))
+        install_dependencies <-
+            if(config_val_to_logical(install_dependencies)) "most" else ""
+
     if (is.null(args)) {
         args <- commandArgs(TRUE)
         ## it seems that splits on spaces, so try harder.
@@ -902,6 +942,10 @@ inRbuildignore <- function(files, pkgdir) {
             with_md5 <- TRUE
         } else if (a == "--log") {
             with_log <- TRUE
+        } else if (substr(a, 1, 23) == "--install-dependencies=") {
+            install_dependencies <- substr(a, 24, 1000)
+        } else if (a == "--install-dependencies") {
+            install_dependencies <- "most"
         } else if (substr(a, 1, 14) == "--compression=") {
             compression <- match.arg(substr(a, 15, 1000),
                                      c("none", "gzip", "bzip2", "xz"))
