@@ -91,8 +91,6 @@ function(package, lib.loc = NULL, format = NULL, reader = NULL)
 .news_reader_default <-
 function(file)
 {
-    verbose <- getOption("verbose")
-
     .collapse <- function(s) paste(s, collapse = "\n")
 
     lines <- readLines(file, warn = FALSE)
@@ -111,7 +109,7 @@ function(file)
         if(!is.na(encoding))
             lines[ind] <- iconv(lines[ind], encoding, "")
         ## Last resort.
-        if(any(is.na(nchar(lines[ind], allowNA = TRUE))))
+        if(anyNA(nchar(lines[ind], allowNA = TRUE)))
             lines[ind] <- iconv(lines[ind], "", "", sub = "byte")
     }
 
@@ -243,9 +241,9 @@ function(file)
     out <- lapply(chunks, do_chunk)
     ## Now assemble pieces.
     reporter <- function(x) {
-        if(verbose)
-            message(gettextf("Cannot process chunk/lines:\n%s",
-                             .collapse(x)))
+        warning(gettextf("Cannot process chunk/lines:\n%s",
+                         .collapse(paste0("  ", x))),
+                domain = NA, call. = FALSE)
         NULL
     }
     finisher <- function(x) {
@@ -358,7 +356,7 @@ function(f, pdf_file)
              outputEncoding = "UTF-8", writeEncoding = FALSE,
              macros = file.path(R.home("share"), "Rd", "macros", "system.Rd"))
     cat("\\documentclass[", Sys.getenv("R_PAPERSIZE"), "paper]{book}\n",
-        "\\usepackage[ae,hyper]{Rd}\n",
+        "\\usepackage[hyper]{Rd}\n",
         "\\usepackage[utf8]{inputenc}\n",
         "\\usepackage{graphicx}\n",
         "\\setkeys{Gin}{width=0.7\\textwidth}\n",
@@ -394,8 +392,6 @@ function(file, out = stdout(), codify = FALSE)
     ## can use the DESCRIPTION metadata to obtain the package name and
     ## encoding.
 
-    format <- "default"
-
     file <- file_path_as_absolute(file)
 
     if(file_test("-d", file)) {
@@ -438,38 +434,9 @@ function(file, out = stdout(), codify = FALSE)
         on.exit(close(out))
     }
 
-    if(format == "R") {
-        news <- readNEWS(chop = "keepAll")
-        if(!length(news))
-            stop("No news found in given file using old-style R-like format.")
-        wto(c("\\newcommand{\\PR}{\\Sexpr[results=rd]{tools:::Rd_expr_PR(#1)}}",
-              "\\name{NEWS}",
-              "\\title{R News}",
-              "\\encoding{UTF-8}"))
-        for(y in news) {
-            for(i in seq_along(y)) {
-                wto(sprintf("\\section{CHANGES IN R VERSION %s}{",
-                            names(y)[i]))
-                z <- y[[i]]
-                for(j in seq_along(z)) {
-                    wto(c(sprintf("  \\subsection{%s}{", names(z)[j]),
-                          "    \\itemize{"))
-                    for(chunk in z[[j]]) {
-                        chunk <- toRd(paste(chunk, collapse = "\n      "))
-                        if(codify) {
-                            chunk <- gsub(cre, "\\1\\\\code{\\2}\\3",
-                                          chunk)
-                        }
-                        chunk <- gsub("PR#([[:digit:]]+)", "\\\\PR{\\1}",
-                                      chunk)
-                        wto(paste("      \\item", enc2utf8(chunk)))
-                    }
-                    wto(c("    }", "  }"))
-                }
-                wto("}")
-            }
-        }
-    } else {
+    ## had   if(format == "R") {
+    ## and this was   } else { format == "default" :
+    {
         news <- .news_reader_default(file)
         bad <- attr(news, "bad")
         if(!length(bad))
@@ -503,7 +470,7 @@ function(file, out = stdout(), codify = FALSE)
         ## Re-order according to decreasing version.
         vchunks <- vchunks[order(as.numeric_version(names(vchunks)),
                                  decreasing = TRUE)]
-        dates <- sapply(vchunks, function(v) v$Date[1L])
+        dates <- vapply(vchunks, function(v) v$Date[1L], "")
         if(any(ind <- !is.na(dates)))
             names(vchunks)[ind] <-
                 sprintf("%s (%s)", names(vchunks)[ind], dates[ind])
@@ -585,8 +552,9 @@ function(file)
     nms <- db[, 1L]
     ind <- grepl(re_v, nms, ignore.case = TRUE)
     if(!all(ind))
-        warning("Cannot extract version info from the following section titles:\n",
-		paste(unique(nms[!ind]), collapse = "  "))
+        warning(gettextf("Cannot extract version info from the following section titles:\n%s",
+                         paste0("  ", unique(nms[!ind]), collapse = "\n")),
+                domain = NA, call. = FALSE)
     .make_news_db(cbind(ifelse(ind,
 			       sub(re_v, "\\1", nms, ignore.case = TRUE),
 			       NA_character_),
@@ -625,7 +593,7 @@ function(x)
                              substr(sub("^[[:space:]]*", "",
                                         .Rd_deparse(x)),
                                     1L, 60L)),
-                    domain = NA)
+                    domain = NA, call. = FALSE)
             pos <- pos[1L]
         }
 
@@ -645,7 +613,7 @@ function(x)
                              substr(sub("^[[:space:]]*", "",
                                         .Rd_deparse(x)),
                                     1L, 60L)),
-                    domain = NA)
+                    domain = NA, call. = FALSE)
             return(matrix(character(), 0L, 2L,
                           dimnames = list(NULL, c("Text", "HTML"))))
         }
@@ -695,8 +663,13 @@ function(x)
     y <- do.call(rbind, y)
     ## Sanitze HTML.
     s <- trimws(y[, "HTML"])
-    i <- startsWith(s, "<p>") & !endsWith(s, "</p>")
-    s[i] <- paste0(s[i], "</p>")
+    i <- which(startsWith(s, "<p>") & !endsWith(s, "</p>"))
+    if(length(i)) {
+        z <- s[i]
+        j <- which((lengths(gregexpr("</?p>", z)) %% 2L) > 0L)
+        if(length(j))
+            s[i[j]] <- paste0(z[j], "</p>")
+    }
     y[, "HTML"] <- s
 
     y
@@ -707,6 +680,8 @@ function(x)
 function(f)
 {
     md <- readLines(f, encoding = "UTF-8", warn = FALSE)
+    ## Maybe complain?
+    if(!length(md)) return()
 
     ## Handle YAML header.
     if(md[1L] == "---") {
@@ -841,8 +816,9 @@ function(f)
     nms <- db[, 1L]
     ind <- grepl(re_v, nms, ignore.case = TRUE)
     if(!all(ind))
-        warning("Cannot extract version info from the following section titles:\n",
-                paste(unique(nms[!ind]), collapse = "  "))
+        warning(gettextf("Cannot extract version info from the following section titles:\n%s",
+                         paste0("  ", unique(nms[!ind]), collapse = "\n")),
+                domain = NA, call. = FALSE)
 
     .make_news_db(cbind(ifelse(ind,
                                sub(re_v, "\\2", nms, ignore.case = TRUE),
@@ -876,7 +852,7 @@ function(x, ...)
     if(!length(vchunks))
         return(character())
 
-    dates <- sapply(vchunks, function(v) v$Date[1L])
+    dates <- vapply(vchunks, function(v) v$Date[1L], "")
     vheaders <-
         format(sprintf("Changes in version %s%s",
                        names(vchunks),
