@@ -1,7 +1,7 @@
 #  File src/library/tools/R/urltools.R
 #  Part of the R package, https://www.R-project.org
 #
-#  Copyright (C) 2015-2019 The R Core Team
+#  Copyright (C) 2015-2022 The R Core Team
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -16,6 +16,9 @@
 #  A copy of the GNU General Public License is available at
 #  https://www.R-project.org/Licenses/
 
+## See RFC 3986 <https://www.rfc-editor.org/rfc/rfc3986> and
+## <https://url.spec.whatwg.org/>.
+
 get_IANA_URI_scheme_db <-
 function()
 {
@@ -25,13 +28,13 @@ function()
     db <- utils::read.csv(url(paste0(baseurl, "uri-schemes-1.csv")),
                           stringsAsFactors = FALSE, encoding = "UTF-8")
     names(db) <- chartr(".", "_", names(db))
+    db$URI_Scheme <- sub(" .*", "", db$URI_Scheme)
     db
 }
 
 parse_URI_reference <-
 function(x)
 {
-    ## See RFC_3986 <http://www.ietf.org/rfc/rfc3986.txt>.
     re <- "^(([^:/?#]+):)?(//([^/?#]*))?([^?#]*)(\\?([^#]*))?(#(.*))?"
     if(length(x)) {
         y <- do.call(rbind, regmatches(x, regexec(re, x)))
@@ -44,7 +47,7 @@ function(x)
 }
 
 .get_urls_from_Rd <-
-function(x)
+function(x, href = TRUE, ifdef = FALSE)
 {
     urls <- character()
     recurse <- function(e) {
@@ -52,8 +55,22 @@ function(x)
         ## Rd2HTML and Rd2latex remove whitespace and \n from URLs.
         if(identical(tag, "\\url")) {
             urls <<- c(urls, lines2str(.Rd_deparse(e, tag = FALSE)))
-        } else if(identical(tag, "\\href")) {
+        } else if(href && identical(tag, "\\href")) {
+            ## One could also record the \href text argument in the
+            ## names, but then one would need to process named and
+            ## unnamed extracted URLs separately.
             urls <<- c(urls, lines2str(.Rd_deparse(e[[1L]], tag = FALSE)))
+        } else if(ifdef && length(tag) && (tag %in% c("\\if", "\\ifelse"))) {
+            ## cf. testRdConditional()
+            condition <- e[[1L]]
+            if(all(RdTags(condition) == "TEXT")) {
+                if(any(c("TRUE", "html") %in%
+                       trimws(strsplit(paste(condition, collapse = ""), 
+                                       ",")[[1L]])))
+                    recurse(e[[2L]])
+                else if(tag == "\\ifelse")
+                    recurse(e[[3L]])
+            }
         } else if(is.list(e))
             lapply(e, recurse)
     }
@@ -97,9 +114,8 @@ url_db <-
 function(urls, parents)
 {
     ## Some people get leading LFs in URLs, so trim before checking.
-    db <- data.frame(URL = trimws(as.character(urls)),
-                     Parent = as.character(parents),
-                     stringsAsFactors = FALSE)
+    db <- list2DF(list(URL = trimws(as.character(urls)),
+                       Parent = as.character(parents)))
     class(db) <- c("url_db", "data.frame")
     db
 }
@@ -138,8 +154,6 @@ function(dir, recursive = FALSE, files = NULL, verbose = FALSE)
         files <- list.files(dir, pattern = "[.]pdf$",
                             full.names = TRUE,
                             recursive = recursive)
-    ## FIXME: this is simpler to do with full.names = FALSE and without
-    ## tools:::.file_path_relative_to_dir().
     urls <-
         lapply(files,
                function(f) {
@@ -342,9 +356,9 @@ function()
 }
 
 ## See <https://en.wikipedia.org/wiki/List_of_FTP_server_return_codes>
-## and <http://tools.ietf.org/html/rfc959>,
+## and <https://www.rfc-editor.org/rfc/rfc959>,
 ## Section 4.2.2 "Numeric Order List of Reply Codes",
-## and <https://tools.ietf.org/html/rfc2228>,
+## and <https://www.rfc-editor.org/rfc/rfc2228>,
 ## Section 5 "New FTP Replies".
 ## Only need those >= 400.
 table_of_FTP_server_return_codes <-
@@ -379,13 +393,17 @@ table_of_FTP_server_return_codes <-
       )
 
 check_url_db <-
-function(db, remote = TRUE, verbose = FALSE)
+function(db, remote = TRUE, verbose = FALSE, parallel = FALSE, pool = NULL)
 {
     use_curl <-
+        !parallel &&
         config_val_to_logical(Sys.getenv("_R_CHECK_URLS_USE_CURL_",
                                          "TRUE")) &&
         requireNamespace("curl", quietly = TRUE)
 
+    if(parallel && is.null(pool))
+        pool <- curl::new_pool()    
+
     .gather <- function(u = character(),
                         p = list(),
                         s = rep.int("", length(u)),
@@ -394,33 +412,21 @@ function(db, remote = TRUE, verbose = FALSE)
                         cran = rep.int("", length(u)),
                         spaces = rep.int("", length(u)),
                         R = rep.int("", length(u))) {
-        y <- data.frame(URL = u, From = I(p), Status = s, Message = m,
-                        New = new, CRAN = cran, Spaces = spaces, R = R,
-                        row.names = NULL, stringsAsFactors = FALSE)
-        y$From <- p
+        y <- list2DF(list(URL = u, From = p, Status = s, Message = m,
+                          New = new, CRAN = cran, Spaces = spaces, R = R))
         class(y) <- c("check_url_db", "data.frame")
         y
     }
+    
+    .fetch_headers <-
+        if(parallel)
+            function(urls)
+                .fetch_headers_via_curl(urls, verbose, pool)
+        else
+            function(urls)
+                .fetch_headers_via_base(urls, verbose)
 
-    .fetch <- function(u) {
-        if(verbose) message(sprintf("processing %s", u))
-        h <- tryCatch(curlGetHeaders(u), error = identity)
-        if(inherits(h, "error")) {
-            ## Currently, this info is only used in .check_http().
-            ## Might be useful for checking ftps too, so simply leave it
-            ## here instead of moving to .check_http().
-            msg <- conditionMessage(h)
-            if (grepl("libcurl error code (51|60)", msg)) {
-                h2 <- tryCatch(curlGetHeaders(u, verify = FALSE),
-                               error = identity)
-                attr(h, "no-verify") <- h2
-            }
-        }
-        h
-    }
-
-    .check_ftp <- function(u) {
-        h <- .fetch(u)
+    .check_ftp <- function(u, h) {
         if(inherits(h, "error")) {
             s <- "-1"
             msg <- sub("[[:space:]]*$", "", conditionMessage(h))
@@ -432,18 +438,25 @@ function(db, remote = TRUE, verbose = FALSE)
     }
 
     .check_http <- if(remote)
-        function(u) c(.check_http_A(u), .check_http_B(u))
-    else
-        function(u) c(rep.int("", 3L), .check_http_B(u))
+                       function(u, h) c(.check_http_A(u, h),
+                                        .check_http_B(u))
+                   else
+                       function(u, h) c(rep.int("", 3L),
+                                        .check_http_B(u))
 
-    .check_http_A <- function(u) {
-        h <- .fetch(u)
+    .check_http_A <- function(u, h) {
         newLoc <- ""
         if(inherits(h, "error")) {
             s <- "-1"
             msg <- sub("[[:space:]]*$", "", conditionMessage(h))
-            if (!is.null(v <- attr(h, "no-verify"))) {
-                s2 <- as.character(attr(v, "status"))
+            if(grepl(paste(c("server certificate verification failed",
+                             "failed to get server cert",
+                             "libcurl error code (51|60)"),
+                           collapse = "|"),
+                     msg)) {
+                h2 <- tryCatch(curlGetHeaders(u, verify = FALSE),
+                               error = identity)
+                s2 <- as.character(attr(h2, "status"))
                 msg <- paste0(msg, "\n\t(Status without verification: ",
                               table_of_HTTP_status_codes[s2], ")")
             }
@@ -452,10 +465,46 @@ function(db, remote = TRUE, verbose = FALSE)
             msg <- table_of_HTTP_status_codes[s]
         }
         ## Look for redirected URLs
-        if (any(grepl("301 Moved Permanently", h, useBytes = TRUE))) {
-            ind <- grep("^[Ll]ocation: ", h, useBytes = TRUE)
-            if (length(ind))
-                newLoc <- sub("^[Ll]ocation: ([^\r]*)\r\n", "\\1", h[max(ind)])
+        ## According to
+        ## <https://www.rfc-editor.org/rfc/rfc7230#section-3.1.2> the first
+        ## line of a response is the status-line, with "a possibly empty
+        ## textual phrase describing the status code", so only look for
+        ## a 301 status code in the first line.
+        if(grepl(" 301 ", h[1L], useBytes = TRUE)) {
+            ## Get the new location from the last consecutive 301
+            ## obtained.
+            h <- split(h, c(0L, cumsum(h == "\r\n")[-length(h)]))
+            i <- vapply(h,
+                        function(e)
+                            grepl(" 301 ", e[1L], useBytes = TRUE),
+                        NA)
+            h <- h[[which(!i)[1L] - 1L]]
+            pos <- grep("^[Ll]ocation: ", h, useBytes = TRUE)
+            if(length(pos)) {
+                loc <- sub("^[Ll]ocation: ([^\r]*)\r\n", "\\1",
+                           h[pos[1L]])
+                ## Ouch.  According to RFC 7231, the location is a URI
+                ## reference, and may be relative in which case it needs
+                ## resolving against the effect request URI.
+                ## <https://www.rfc-editor.org/rfc/rfc7231#section-7.1.2>.
+                ## Not quite straightforward, hence do not report such
+                ## 301s. 
+                ## (Alternatively, could try reporting the 301 but no
+                ## new location.)
+                newParts <- parse_URI_reference(loc)
+                if(nzchar(newParts[1L, "scheme"])) {
+                    newLoc <- loc
+                    ## Handle fragments. If the new URL does have one,
+                    ## use it. Otherwise, if the old has one, use that.
+                    ## (From section 7.1.2).
+                    if (newParts[1L, "fragment"] == "") {
+                        uParts <- parse_URI_reference(u)
+                        if (nzchar(uFragment <- uParts[1L, "fragment"])) {
+                            newLoc <- paste0(newLoc, "#", uFragment)
+                        }
+                    }
+                }
+            }
         }
         ##
         if((s != "200") && use_curl) {
@@ -474,7 +523,7 @@ function(db, remote = TRUE, verbose = FALSE)
     .check_http_B <- function(u) {
         ul <- tolower(u)
         cran <- ((grepl("^https?://cran.r-project.org/web/packages", ul) &&
-                  !grepl("^https?://cran.r-project.org/web/packages/[.[:alnum:]]+(html|pdf|rds)$",
+                  !grepl("^https?://cran.r-project.org/web/packages/([.[:alnum:]_]+(html|pdf|rds))?$",
                          ul)) ||
                  (grepl("^https?://cran.r-project.org/web/views/[[:alnum:]]+[.]html$",
                         ul)) ||
@@ -540,7 +589,9 @@ function(db, remote = TRUE, verbose = FALSE)
     ## ftp.
     pos <- which(schemes == "ftp")
     if(length(pos) && remote) {
-        results <- do.call(rbind, lapply(urls[pos], .check_ftp))
+        urlspos <- urls[pos]
+        headers <- .fetch_headers(urlspos)
+        results <- do.call(rbind, Map(.check_ftp, urlspos, headers))
         status <- as.numeric(results[, 1L])
         ind <- (status < 0L) | (status >= 400L)
         if(any(ind)) {
@@ -557,11 +608,33 @@ function(db, remote = TRUE, verbose = FALSE)
     ## http/https.
     pos <- which(schemes == "http" | schemes == "https")
     if(length(pos)) {
-        results <- do.call(rbind, lapply(urls[pos], .check_http))
+        urlspos <- urls[pos]
+        ## Check DOI URLs via the DOI handle API, as we nowadays do for
+        ## checking DOIs.
+        myparts <- parts[pos, , drop = FALSE]
+        ind <- (((myparts[, 2L] == "doi.org") | 
+                 (myparts[, 2L] == "dx.doi.org")) &
+                startsWith(myparts[, 3L], "/10.") &
+                !nzchar(myparts[, 4L]) &
+                !nzchar(myparts[, 5L]))
+        if(any(ind))
+            urlspos[ind] <- paste0("https://doi.org/api/handles",
+                                   myparts[ind, 3L])
+        ## Could also use regexps, e.g.
+        ##    pat <- "^https?://(dx[.])?doi.org/10[.]([^?#]+)$"
+        ##    ind <- grep(pat, urlspos)
+        ##    if(length(ind))
+        ##         urlspos[ind] <-
+        ##             paste0("https://doi.org/api/handles/10.",
+        ##                     sub(pat, "\\2", urlspos[ind]))
+        ## but using the parts is considerably faster ...
+        headers <- .fetch_headers(urlspos)
+        results <- do.call(rbind, Map(.check_http, urlspos, headers))
         status <- as.numeric(results[, 1L])
         ## 405 is HTTP not allowing HEAD requests
         ## maybe also skip 500, 503, 504 as likely to be temporary issues
         ind <- is.na(match(status, c(200L, 405L, NA))) |
+            nzchar(results[, 3L]) |
             nzchar(results[, 4L]) |
             nzchar(results[, 5L]) |
             nzchar(results[, 6L])
@@ -591,7 +664,14 @@ function(x, ...)
     u <- x$URL
     new <- x$New
     ind <- nzchar(new)
-    u[ind] <- sprintf("%s (moved to %s)", u[ind], new[ind])
+    if(any(ind)) {
+        u[ind] <- sprintf("%s (moved to %s)", u[ind], new[ind])
+        if(config_val_to_logical(Sys.getenv("_R_CHECK_URLS_SHOW_301_STATUS_",
+                                            "FALSE"))) {
+            x$Message[ind] <- "Moved Permanently"
+            x$Status[ind] <- "301"
+        }
+    }
 
     paste0(sprintf("URL: %s", u),
            sprintf("\nFrom: %s",
@@ -634,6 +714,114 @@ function(x, ...)
     y
 }
 
+.fetch_headers_via_base <-
+function(urls, verbose = FALSE, ids = urls)
+    Map(function(u, verbose, i) {
+            if(verbose) message(sprintf("processing %s", i))
+            tryCatch(curlGetHeaders(u), error = identity)
+        },
+        urls, verbose, ids)
+
+.fetch_headers_via_curl <-
+function(urls, verbose = FALSE, pool = NULL) {
+    out <- .curl_multi_run_worker(urls, TRUE, verbose, pool)
+    ind <- !vapply(out, inherits, NA, "error")
+    if(any(ind))
+        out[ind] <- lapply(out[ind],
+                           function(x) {
+                               y <- strsplit(rawToChar(x$headers),
+                                             "(?<=\r\n)",
+                                             perl = TRUE)[[1L]]
+                               attr(y, "status") <- x$status_code
+                               y
+                           })
+    out
+}
+
+
+.curl_multi_run_worker <-
+function(urls, nobody = FALSE, verbose = FALSE, pool = NULL)
+{
+    ## Use 'nobody = TRUE' to fetch only headers.
+    
+    .progress_bar <- function(length, msg = "") {
+        bar <- new.env(parent = baseenv())
+        if(is.null(length)) {
+            length <- 0L
+        }
+        done <- fmt <- NULL             # make codetools happy
+        bar$length <- length
+        bar$done <- -1L
+        digits <- trunc(log10(length)) + 1L
+        bar$fmt <- paste0("\r", msg, "[ %", digits, "i / %", digits, "i ]")
+        bar$update <- function() {
+            assign("done", inherits = TRUE, done + 1L)
+            if (length <= 0L) {
+                return()
+            }
+            if (done >= length) {
+                cat("\r", strrep(" ", nchar(fmt)), "\r", sep = "")
+            } else {
+                cat(sprintf(fmt, done, length), sep = "")
+            }
+        }
+        environment(bar$update) <- bar
+        bar$update()
+        bar
+    }
+
+    if(is.null(pool))
+        pool <- curl::new_pool()
+
+    bar <- .progress_bar(if (verbose) length(urls), msg = "fetching ")    
+
+    out <- vector("list", length(urls))
+
+    for(i in seq_along(out)) {
+        u <- urls[[i]]
+        h <- curl::new_handle(url = u)
+        curl::handle_setopt(h,
+                            nobody = nobody,
+                            cookiesession = 1L,
+                            followlocation = 1L,
+                            http_version = 2L,
+                            ssl_enable_alpn = 0L)
+        timeout <- as.integer(getOption("timeout"))
+        if(!is.na(timeout) && (timeout > 0L))
+            curl::handle_setopt(h,
+                                connecttimeout = timeout,
+                                timeout = timeout)
+        if(grepl("^https?://github[.]com", u) &&
+           nzchar(a <- Sys.getenv("GITHUB_PAT", ""))) {
+            curl::handle_setheaders(h, "Authorization" = paste("token", a))
+        }
+        handle_result <- local({
+            i <- i
+            function(x) {
+                out[[i]] <<- x
+                bar$update()
+            }
+        })
+        handle_error <- local({
+            i <- i
+            function(x) {
+                out[[i]] <<-
+                    structure(list(message = x),
+                              class = c("curl_error", "error", "condition"))
+                bar$update()
+            }
+        })
+        curl::multi_add(h,
+                        done = handle_result,
+                        fail = handle_error,
+                        pool = pool)
+    }
+
+    curl::multi_run(pool = pool)
+   
+    out
+}
+
 .curl_GET_status <-
 function(u, verbose = FALSE)
 {
@@ -649,6 +837,11 @@ function(u, verbose = FALSE)
                         followlocation = 1,
                         http_version = 2L,
                         ssl_enable_alpn = 0)
+    timeout <- as.integer(getOption("timeout"))
+    if(!is.na(timeout) && (timeout > 0L))
+        curl::handle_setopt(h,
+                            connecttimeout = timeout,
+                            timeout = timeout)
     if(startsWith(u, "https://github.com") &&
        nzchar(a <- Sys.getenv("GITHUB_PAT", "")))
         curl::handle_setheaders(h, "Authorization" = paste("token", a))
