@@ -1,7 +1,7 @@
 #  File src/library/tools/R/utils.R
 #  Part of the R package, https://www.R-project.org
 #
-#  Copyright (C) 1995-2020 The R Core Team
+#  Copyright (C) 1995-2023 The R Core Team
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -44,6 +44,44 @@ function(x)
     normalizePath(epath, "/", TRUE)
 }
 
+### ** file_path_relative_to
+
+file_path_relative_to <-
+function(x, start = getwd(), parent = TRUE)
+{
+    x <- normalizePath(x, "/", mustWork = FALSE)
+    if(!parent) {
+        p <- normalizePath(start[1L], "/", mustWork = TRUE)
+        if(any(i <- startsWith(x, p))) {
+            ## Assume .Platform$file.sep is a single character.
+            x[i] <- substring(x[i], nchar(p) + 2L)
+        }
+        x
+    } else {
+        p <- strsplit(normalizePath(start, "/", mustWork = FALSE),
+                      "/", fixed = TRUE)[[1L]]
+        y <- strsplit(x, "/", fixed = TRUE)
+        f <- function(u, v) {
+            i <- 1L
+            while(i <= min(length(v), length(p))) {
+                if(v[i] == p[i])
+                    i <- i + 1L
+                else
+                    break
+            }
+            if(i == 1L) {
+                ## Paths start differently, so relative cannot work
+                u
+            } else {
+                i <- i - 1L
+                paste(c(rep_len("..", length(p) - i), v[-seq_len(i)]),
+                      collapse = .Platform$file.sep)
+            }
+        }
+        unlist(Map(f, x, y, USE.NAMES = FALSE))
+    }
+}
+
 ### ** file_path_sans_ext
 
 file_path_sans_ext <-
@@ -63,7 +101,8 @@ function(x, compression = FALSE)
 file_test <-
 function(op, x, y)
 {
-    ## Provide shell-style '-f', '-d', '-x', '-nt' and '-ot' tests.
+    ## Provide shell-style '-f', '-d', '-h'/'-L', '-x', '-w', '-r',
+    ## '-nt' and '-ot' tests.
     ## Note that file.exists() only tests existence ('test -e' on some
     ## systems), and that our '-f' tests for existence and not being a
     ## directory (the GNU variant tests for being a regular file).
@@ -71,6 +110,8 @@ function(op, x, y)
     switch(op,
            "-f" = !is.na(isdir <- file.info(x, extra_cols = FALSE)$isdir) & !isdir,
            "-d" = dir.exists(x),
+           "-h" = (!is.na(y <- Sys.readlink(x)) & nzchar(y)),
+           "-L" = (!is.na(y <- Sys.readlink(x)) & nzchar(y)),
            "-nt" = (!is.na(mt.x <- file.mtime(x))
                     & !is.na(mt.y <- file.mtime(y))
                     & (mt.x > mt.y)),
@@ -78,6 +119,8 @@ function(op, x, y)
                     & !is.na(mt.y <- file.mtime(y))
                     & (mt.x < mt.y)),
            "-x" = (file.access(x, 1L) == 0L),
+           "-w" = (file.access(x, 2L) == 0L),
+           "-r" = (file.access(x, 4L) == 0L),
            stop(gettextf("test '%s' is not available", op),
                 domain = NA))
 }
@@ -160,7 +203,7 @@ reQuote <-
 function(x)
 {
     escape <- function(s) paste0("\\", s)
-    re <- "[.*?+^$\\[]"
+    re <- "[.*?+^$\\()[]"
     m <- gregexpr(re, x)
     regmatches(x, m) <- lapply(regmatches(x, m), escape)
     x
@@ -176,12 +219,15 @@ function(x)
     ## The only one we guarantee to exist is 'latin1'.
     ## The default sub=NA is faster, but on some platforms
     ## some characters used just to lose their accents, so two tests.
-    asc <- iconv(x, "latin1", "ASCII")
-    ind <- is.na(asc) | asc != x
-    if(any(ind))
+    ##    asc <- iconv(x, "latin1", "ASCII")
+    ##    ind <- is.na(asc) | asc != x
+
+    ind <- .Call(C_nonASCII, x)
+    if(any(ind)) {
         message(paste0(which(ind), ": ",
-                       iconv(x[ind], "latin1", "ASCII", sub = "byte"),
+                       iconv(x[ind], "", "ASCII", sub = "byte"),
                        collapse = "\n"), domain = NA)
+    }
     invisible(x[ind])
 }
 
@@ -210,9 +256,11 @@ function(x, delim = c("{", "}"), syntax = "Rd")
 
 ### ** lines2str
 lines2str <-
-function(txt, sep = "")
-    trimws(gsub("\n", sep, paste(txt, collapse = sep),
-                fixed = TRUE, useBytes = TRUE))
+function(txt, sep = "") {
+    bytes <- gsub("\n", sep, paste(txt, collapse = sep),
+                  fixed = TRUE, useBytes = TRUE)
+    trimws(iconv(bytes, to = "UTF-8", sub = "byte"))
+}
 
 
 ### * LaTeX utilities
@@ -511,13 +559,6 @@ function(file, pdf = FALSE, clean = FALSE, quiet = TRUE,
 
 ### * Internal utility variables.
 
-### ** .BioC_version_associated_with_R_version
-
-.BioC_version_associated_with_R_version <-
-    function() numeric_version(Sys.getenv("R_BIOC_VERSION", "3.13"))
-## Things are more complicated from R-2.15.x with still two BioC
-## releases a year, so we do need to set this manually.
-
 ### ** .ORCID_iD_regexp
 
 .ORCID_iD_regexp <-
@@ -668,10 +709,8 @@ function(txt)
     txt <- as.character(txt)
     if(!length(txt)) return(txt)
     enc <- Encoding(txt)
-    txt <- gsub(paste0("(", intToUtf8(0x2018), "|", intToUtf8(0x2019), ")"),
-                "'", txt, perl = TRUE, useBytes = TRUE)
-    txt <- gsub(paste0("(", intToUtf8(0x201c), "|", intToUtf8(0x201d), ")"),
-                '"', txt, perl = TRUE, useBytes = TRUE)
+    txt <- gsub("(\u2018|\u2019)", "'", txt, perl = TRUE, useBytes = TRUE)
+    txt <- gsub("(\u201c|\u201d)", '"', txt, perl = TRUE, useBytes = TRUE)
     Encoding(txt) <- enc
     txt
 }
@@ -722,8 +761,8 @@ function(expr, type = NULL)
 
     value <- eval(expr)
     list(value = value,
-         output = readLines(outcon, encoding = "UTF-8", warn = FALSE),
-         message = readLines(msgcon, encoding = "UTF-8", warn = FALSE))
+         output = readLines(outcon, warn = FALSE),
+         message = readLines(msgcon, warn = FALSE))
 }
 
 ### ** .expand_anchored_Rd_xrefs
@@ -778,21 +817,35 @@ function(x, dir, add = FALSE)
 .find_calls <-
 function(x, predicate = NULL, recursive = FALSE)
 {
-    x <- if(is.call(x)) list(x) else as.list(x)
+    calls <- list()
+
+    if(!is.recursive(x) || isS4(x)) return(calls)
+
+    x <- if(is.call(x))
+             list(x)
+         else {
+             if(is.object(x))
+                 class(x) <- NULL
+             as.list(x)
+         }
 
     f <- if(is.null(predicate))
         function(e) is.call(e)
-    else
-        function(e) is.call(e) && predicate(e)
+    else ## no check predicate returns a scalar, so any() added for 4.2.0
+        function(e) is.call(e) && any(predicate(e))
 
     if(!recursive) return(Filter(f, x))
 
-    calls <- list()
     gatherer <- function(e) {
         if(f(e)) calls <<- c(calls, list(e))
-        if(is.recursive(e))
+        if(is.recursive(e) && !is.environment(e) && !isS4(e)) {
+            if(is.object(e))
+                class(e) <- NULL
+            e <- as.list(e)
             for(i in seq_along(e)) gatherer(e[[i]])
+        }
     }
+
     gatherer(x)
 
     calls
@@ -809,7 +862,8 @@ function(file, encoding = NA, predicate = NULL, recursive = FALSE)
 ### ** .find_calls_in_package_code
 
 .find_calls_in_package_code <-
-function(dir, predicate = NULL, recursive = FALSE, .worker = NULL)
+function(dir, predicate = NULL, recursive = FALSE, .worker = NULL,
+         which = "code")
 {
     dir <- file_path_as_absolute(dir)
 
@@ -821,9 +875,24 @@ function(dir, predicate = NULL, recursive = FALSE, .worker = NULL)
         .worker <- function(file, encoding)
             .find_calls_in_file(file, encoding, predicate, recursive)
 
+    which <- match.arg(which,
+                       c("code", "vignettes", "NAMESPACE", "CITATION"),
+                       several.ok = TRUE)
     code_files <-
-        list_files_with_type(file.path(dir, "R"), "code",
-                             OS_subdirs = c("unix", "windows"))
+        c(character(),
+          if("code" %in% which)
+              list_files_with_type(file.path(dir, "R"), "code",
+                                   OS_subdirs = c("unix", "windows")),
+          if(("vignettes" %in% which) &&
+             dir.exists(fp <- file.path(dir, "inst", "doc")))
+              list_files_with_type(fp, "code"),
+          if(("NAMESPACE" %in% which) &&
+             file.exists(fp <- file.path(dir, "NAMESPACE")))
+              fp,
+          if(("CITATION" %in% which) &&
+             file.exists(fp <- file.path(dir, "inst", "CITATION")))
+              fp)
+
     calls <- lapply(code_files, .worker, encoding)
     names(calls) <-
         .file_path_relative_to_dir(code_files, dirname(dir))
@@ -831,6 +900,20 @@ function(dir, predicate = NULL, recursive = FALSE, .worker = NULL)
     calls
 }
 
+.predicate_for_calls_with_names <-
+function(nms)
+{
+    function(e) {
+        (is.call(e) &&
+         ((is.name(x <- e[[1L]]) &&
+           as.character(x) %in% nms)) ||
+         ((is.call(x <- e[[1L]]) &&
+           is.name(x[[1L]]) &&
+           (as.character(x[[1L]]) %in% c("::", ":::")) &&
+           as.character(x[[3L]]) %in% nms)))
+    }
+}
+
 ### ** .find_owner_env
 
 .find_owner_env <-
@@ -904,7 +987,9 @@ function(primitive = TRUE) # primitive means 'include primitives'
     out <-
         ## Get the names of R internal S3 generics (via DispatchOrEval(),
         ## cf. ?InternalMethods).
-        c("[", "[[", "$", "[<-", "[[<-", "$<-", "@<-",
+        c("[", "[[", "$", "[<-", "[[<-", "$<-", "@", "@<-",
+          ## The above are actually primitive but not listed in
+          ## base::.S3PrimitiveGenerics et al: not sure why?
           "as.vector", "cbind", "rbind", "unlist",
           "is.unsorted", "lengths", "nchar", "rep.int", "rep_len",
           .get_S3_primitive_generics()
@@ -945,7 +1030,7 @@ function(nsInfo)
     ## names of the generic, class and method (as a function).
     S3_methods_db <- nsInfo$S3methods
     if(!length(S3_methods_db))
-        return(matrix(character(), ncol = 3L))
+        return(matrix(character(), ncol = 4L))
     idx <- is.na(S3_methods_db[, 3L])
     S3_methods_db[idx, 3L] <-
         paste(S3_methods_db[idx, 1L],
@@ -966,11 +1051,10 @@ function(package, lib.loc = NULL)
     if(!nzchar(path)) return(NULL)
     if(package == "base") {
         len <- nrow(.S3_methods_table)
-        return(data.frame(generic = .S3_methods_table[, 1L],
-                          home = rep_len("base", len),
-                          class = .S3_methods_table[, 2L],
-                          delayed = rep_len(FALSE, len),
-                          stringsAsFactors = FALSE))
+        return(list2DF(list(generic = .S3_methods_table[, 1L],
+                            home = rep_len("base", len),
+                            class = .S3_methods_table[, 2L],
+                            delayed = rep_len(FALSE, len))))
     }
     lib.loc <- dirname(path)
     nsinfo <- parseNamespaceFile(package, lib.loc)
@@ -981,11 +1065,10 @@ function(package, lib.loc = NULL)
     if(!all(ind)) {
         ## Delayed registrations can be handled directly.
         pos <- which(!ind)
-        tab <- data.frame(generic = S3methods[pos, 1L],
-                          home = S3methods[pos, 4L],
-                          class = S3methods[pos, 2L],
-                          delayed = rep_len(TRUE, length(pos)),
-                          stringsAsFactors = FALSE)
+        tab <- list2DF(list(generic = S3methods[pos, 1L],
+                            home = S3methods[pos, 4L],
+                            class = S3methods[pos, 2L],
+                            delayed = rep_len(TRUE, length(pos))))
         S3methods <- S3methods[ind, , drop = FALSE]
     }
     generic <- S3methods[, 1L]
@@ -1003,11 +1086,10 @@ function(package, lib.loc = NULL)
                use.names = FALSE)
     ## S3 group generics belong to base.
     homes[!ind] <- "base"
-    home <- homes[match(generic, generics)]
-    class <- S3methods[, 2L]
-    delayed <- rep_len(FALSE, length(class))
-    rbind(data.frame(generic, home, class, delayed,
-                     stringsAsFactors = FALSE),
+    rbind(list2DF(list(generic = generic,
+                       home = homes[match(generic, generics)],
+                       class = S3methods[, 2L],
+                       delayed = rep_len(FALSE, length(generic)))),
           tab)
 }
 
@@ -1033,20 +1115,6 @@ function(dir, installed = FALSE)
     stop("invalid package layout")
 }
 
-### ** .get_repositories
-
-.get_repositories <-
-function()
-{
-    rfile <- Sys.getenv("R_REPOSITORIES", unset = NA_character_)
-    if(is.na(rfile) || !file_test("-f", rfile)) {
-        rfile <- file.path(Sys.getenv("HOME"), ".R", "repositories")
-        if(!file_test("-f", rfile))
-            rfile <- file.path(R.home("etc"), "repositories")
-    }
-    .read_repositories(rfile)
-}
-
 ### ** .get_requires_from_package_db
 
 .get_requires_from_package_db <-
@@ -1083,51 +1151,79 @@ function(db,
 
 ### ** .get_S3_generics_as_seen_from_package
 
-.get_S3_generics_as_seen_from_package <-
-function(dir, installed = TRUE, primitive = FALSE)
+## .get_S3_generics_as_seen_from_package <-
+## function(dir, installed = TRUE, primitive = FALSE)
+## {
+##     ## Get the S3 generics "as seen from a package" rooted at
+##     ## @code{dir}.  Tricky ...
+##     if(basename(dir) == "base")
+##         env_list <- list()
+##     else {
+##         ## Always look for generics in the whole of the former base.
+##         ## (Not right, but we do not perform run time analyses when
+##         ## working off package sources.)  Maybe change this eventually,
+##         ## but we still cannot rely on packages to fully declare their
+##         ## dependencies on base packages.
+##         env_list <-
+##             list(baseenv(),
+##                  as.environment("package:graphics"),
+##                  as.environment("package:stats"),
+##                  as.environment("package:utils"))
+##         if(installed) {
+##             ## Also use the loaded namespaces and attached packages
+##             ## listed in the DESCRIPTION Depends and Imports fields.
+##             ## Not sure if this is the best approach: we could also try
+##             ## to determine which namespaces/packages were made
+##             ## available by loading the package (which should work at
+##             ## least when run from R CMD check), or we could simply
+##             ## attach every package listed as a dependency ... or
+##             ## perhaps do both.
+##             db <- .read_description(file.path(dir, "DESCRIPTION"))
+##             depends <- .get_requires_from_package_db(db, "Depends")
+##             imports <- .get_requires_from_package_db(db, "Imports")
+##             reqs <- intersect(c(depends, imports), loadedNamespaces())
+##             if(length(reqs))
+##                 env_list <- c(env_list, lapply(reqs, getNamespace))
+##             reqs <- intersect(setdiff(depends, loadedNamespaces()),
+##                               .packages())
+##             if(length(reqs))
+##                 env_list <- c(env_list, lapply(reqs, .package_env))
+##             env_list <- unique(env_list)
+##         }
+##     }
+##     ## some BioC packages warn here
+##     suppressWarnings(
+##     unique(c(.get_internal_S3_generics(primitive),
+##              unlist(lapply(env_list, .get_S3_generics_in_env))))
+##     )
+## }
+
+### ** .get_S3_generics_in_base
+
+.get_S3_generics_in_base <-
+function()
 {
-    ## Get the S3 generics "as seen from a package" rooted at
-    ## @code{dir}.  Tricky ...
-    if(basename(dir) == "base")
-        env_list <- list()
-    else {
-        ## Always look for generics in the whole of the former base.
-        ## (Not right, but we do not perform run time analyses when
-        ## working off package sources.)  Maybe change this eventually,
-        ## but we still cannot rely on packages to fully declare their
-        ## dependencies on base packages.
-        env_list <-
-            list(baseenv(),
-                 as.environment("package:graphics"),
-                 as.environment("package:stats"),
-                 as.environment("package:utils"))
-        if(installed) {
-            ## Also use the loaded namespaces and attached packages
-            ## listed in the DESCRIPTION Depends and Imports fields.
-            ## Not sure if this is the best approach: we could also try
-            ## to determine which namespaces/packages were made
-            ## available by loading the package (which should work at
-            ## least when run from R CMD check), or we could simply
-            ## attach every package listed as a dependency ... or
-            ## perhaps do both.
-            db <- .read_description(file.path(dir, "DESCRIPTION"))
-            depends <- .get_requires_from_package_db(db, "Depends")
-            imports <- .get_requires_from_package_db(db, "Imports")
-            reqs <- intersect(c(depends, imports), loadedNamespaces())
-            if(length(reqs))
-                env_list <- c(env_list, lapply(reqs, getNamespace))
-            reqs <- intersect(setdiff(depends, loadedNamespaces()),
-                              .packages())
-            if(length(reqs))
-                env_list <- c(env_list, lapply(reqs, .package_env))
-            env_list <- unique(env_list)
-        }
-    }
-    ## some BioC packages warn here
-    suppressWarnings(
-    unique(c(.get_internal_S3_generics(primitive),
-             unlist(lapply(env_list, .get_S3_generics_in_env))))
-    )
+    ## .get_S3_generics_in_env(.BaseNamespaceEnv) gets all UseMethod
+    ## generics.
+    ## .get_internal_S3_generics() gets the internal S3 generics.  By
+    ## default this also adds the primitive generics.
+    ## .get_S3_group_generics() gets the S3 group generics.
+    ## Note that
+    ##   .make_S3_group_generic_env()
+    ## generates an env with the group generics and appropriate
+    ## signatures, so we should always have
+    ##    identical(sort(.get_S3_group_generics()),
+    ##              sort(names(.make_S3_group_generic_env())))
+    ## and that
+    ##    .make_S3_primitive_generic_env()
+    ##  generates and env with the primitive generics and appropriate
+    ##  signatures (in turn using base::.GenericArgsEnv), so we should
+    ##  always have
+    ##    identical(sort(.get_S3_primitive_generics()),
+    ##              sort(names(.make_S3_primitive_generic_env())))
+    c(.get_S3_generics_in_env(.BaseNamespaceEnv),
+      .get_internal_S3_generics(),
+      .get_S3_group_generics())
 }
 
 ### ** .get_S3_generics_in_env
@@ -1147,7 +1243,7 @@ function(env, nms = NULL)
 
 .get_S3_group_generics <-
 function()
-    c("Ops", "Math", "Summary", "Complex")
+    c("Ops", "Math", "Summary", "Complex", "matrixOps")
 
 ### ** .get_S3_primitive_generics
 
@@ -1178,7 +1274,9 @@ function(include_group_generics = TRUE)
           ## Group 'Summary':
           "all", "any", "sum", "prod", "max", "min", "range",
           ## Group 'Complex':
-          "Arg", "Conj", "Im", "Mod", "Re")
+          "Arg", "Conj", "Im", "Mod", "Re",
+          ## Group 'matrixOps'
+          "%*%")
     else
         base::.S3PrimitiveGenerics
 }
@@ -1205,7 +1303,7 @@ local({
     out <- strsplit(sub("^R_PKGS_[[:upper:]]+ *= *", "", lines), " +")
     names(out) <-
         tolower(sub("^R_PKGS_([[:upper:]]+) *=.*", "\\1", lines))
-    eval(substitute(function() {out}, list(out=out)), envir=NULL)
+    eval(substitute(function() {out}, list(out=out)), envir = topenv())
     })
 
 ### ** .get_standard_package_dependencies
@@ -1230,31 +1328,43 @@ function(reverse = FALSE, recursive = FALSE)
 
 ### ** .get_standard_repository_URLs
 
+## Usage in e.g. CRAN_baseurl_for_web_area assumes this returns a
+## valid CRAN mirror as its first element.
+## That used not to be guaranteed, and it is still unchecked.
 .get_standard_repository_URLs <-
-function()
-{
-    repos <- Sys.getenv("_R_CHECK_XREFS_REPOSITORIES_", "")
-    if(nzchar(repos)) {
-        repos <-
-            .expand_BioC_repository_URLs(strsplit(repos, " +")[[1L]])
-    } else {
-        nms <- c("CRAN", "BioCsoft", "BioCann", "BioCexp")
-        repos <- getOption("repos")
-        ## This is set by utils:::.onLoad(), hence may be NULL.
-        if(!is.null(repos) &&
-           !anyNA(repos[nms]) &&
-           (repos["CRAN"] != "@CRAN@"))
-            repos <- repos[nms]
-        else {
-            repos <- .get_repositories()[nms, "URL"]
-            names(repos) <- nms
-            if(repos["CRAN"] == "@CRAN@")
-                repos["CRAN"] <- "https://CRAN.R-project.org"
-        }
-    }
-    repos
+function(ForXrefs = FALSE)
+ {
+     if(ForXrefs &&
+        nzchar(repos <- Sys.getenv("_R_CHECK_XREFS_REPOSITORIES_", "")))
+         return(utils:::.expand_BioC_repository_URLs(strsplit(repos, " +")[[1L]]))
+
+     nms <- c("CRAN", "BioCsoft", "BioCann", "BioCexp")
+     repos <- getOption("repos")
+     ## This is set by utils:::.onLoad(), hence may be NULL.
+     if(!is.null(repos) && !anyNA(repos[nms]) && (repos["CRAN"] != "@CRAN@"))
+         repos <- repos[nms]
+     else {
+         repos <- utils:::.get_repositories()[nms, "URL"]
+         names(repos) <- nms
+         ## That might not contain an entry for CRAN
+         if(is.na(repos["CRAN"]) || repos["CRAN"] == "@CRAN@")
+             repos["CRAN"] <- "https://CRAN.R-project.org"
+     }
+     repos
 }
 
+.get_CRAN_repository_URL <-
+function()
+ {
+     repos <- getOption("repos")
+     if(!is.null(repos) && !is.na(cr <- repos["CRAN"]) && (cr != "@CRAN@"))
+         return(cr)
+     cr <- utils:::.get_repositories()["CRAN", "URL"]
+     ## That might not contain an entry for CRAN
+     if(is.na(cr) || cr == "@CRAN@") cr <- "https://CRAN.R-project.org"
+     cr
+ }
+
 ### ** .get_standard_repository_db_fields
 
 .get_standard_repository_db_fields <-
@@ -1325,6 +1435,7 @@ function()
                "Title",
                "Type",
                "URL",
+               "UseLTO",
                "Version",
                "VignetteBuilder",
                "ZipData"),
@@ -1529,6 +1640,14 @@ function(fname, envir, mustMatch = TRUE)
     if(mustMatch) res == fname else nzchar(res)
 }
 
+### ** .load_namespace_quietly
+
+.load_namespace_quietly <-
+function(package, lib.loc) {
+    if(package != "base")
+        .try_quietly(loadNamespace(package, lib.loc))
+}
+
 ### ** .load_namespace_rather_quietly
 
 .load_namespace_rather_quietly <-
@@ -1625,6 +1744,8 @@ function(parent = parent.frame())
            envir = env)
     assign("Ops", function(e1, e2) UseMethod("Ops"),
            envir = env)
+    assign("matrixOps", function(x, y) UseMethod("matrixOps"),
+           envir = env)
     assign("Summary", function(..., na.rm = FALSE) UseMethod("Summary"),
            envir = env)
     assign("Complex", function(z) UseMethod("Complex"),
@@ -1665,6 +1786,26 @@ function(parent = parent.frame())
              hash=TRUE, parent=parent)
 }
 
+### ** .make_KaTeX_checker
+
+.make_KaTeX_checker <- local({
+    fun <- NULL
+    ctx <- NULL
+    function() {
+        if(is.null(fun) && requireNamespace("V8", quietly = TRUE)) {
+            dir <- file.path(R.home("doc"), "html")
+            ctx <<- V8::v8("window")
+            ctx$source(file.path(dir, "katex", "katex.js"))
+            ## Provides additional macros:
+            ctx$source(file.path(dir, "katex-config.js"))
+            ## Provides checkTex():
+            ctx$source(file.path(dir, "katex-check.js"))
+            fun <<- function(tex) ctx$call('checkTex', tex)
+        }
+        fun
+    }
+})
+
 ### ** nonS3methods [was .make_S3_methods_stop_list ]
 
 nonS3methods <- function(package)
@@ -1674,17 +1815,18 @@ nonS3methods <- function(package)
     ## Using package = NULL returns all known examples
 
     stopList <-
-        list(base = c("all.equal", "all.names", "all.vars", "expand.grid",
-             "format.char", "format.info", "format.pval",
+        list(base = c("all.equal", "all.names", "all.vars",
+             "as.data.frame.vector",
+             "format.info", "format.pval",
              "max.col",
-             "pmax.int", "pmin.int",
              ## the next two only exist in *-defunct.Rd.
-             "print.atomic", "print.coefmat",
+             ## "print.atomic", "print.coefmat",
              "qr.Q", "qr.R", "qr.X", "qr.coef", "qr.fitted", "qr.qty",
              "qr.qy", "qr.resid", "qr.solve",
-             "rep.int", "sample.int", "seq.int", "sort.int", "sort.list"),
+             "rep.int", "seq.int", "sort.int", "sort.list"),
              AMORE = "sim.MLPnet",
              BSDA = "sign.test",
+             BiocGenerics = "rep.int",
              ChemometricsWithR = "lda.loofun",
              ElectoGraph = "plot.wedding.cake",
              FrF2 = "all.2fis.clear.catlg",
@@ -1784,7 +1926,7 @@ function()
     ind <- grepl("^[[:alpha:]]", generics)
     generics <- c(generics[!ind], generics[ind])
     ## The foo.bar objects in base:
-    objects <- grep("[^.]+[.]", objects, value = TRUE)
+    objects <- grep("[^.]+[.][[:alpha:]]", objects, value = TRUE)
     ## Make our lives easier ...
     objects <- setdiff(objects, nonS3methods("base"))
     ## Find the ones matching GENERIC.CLASS from the list of generics.
@@ -1804,6 +1946,8 @@ function()
 .deparse_S3_methods_table_for_base <-
 function()
 {
+    if(!identical("C", Sys.getlocale("LC_COLLATE")))
+        warning("*not* using 'C' for LC_COLLATE locale")
     mdb <- .make_S3_methods_table_for_base()
     n <- nrow(mdb)
     c(sprintf("%s\"%s\", \"%s\"%s",
@@ -1941,7 +2085,7 @@ function(txt)
     c("Description", "Authors@R", "Author", "Built", "Packaged")
 
 .read_description <-
-function(dfile)
+function(dfile, keep.white = .keep_white_description_fields)
 {
     ## Try reading in package metadata from a DESCRIPTION file.
     ## (Never clear whether this should work on the path of the file
@@ -1952,16 +2096,14 @@ function(dfile)
     ## </NOTE>
     if(!file_test("-f", dfile))
         stop(gettextf("file '%s' does not exist", dfile), domain = NA)
-    out <- tryCatch(read.dcf(dfile,
-                             keep.white =
-                             .keep_white_description_fields),
+    out <- tryCatch(read.dcf(dfile, keep.white = keep.white),
                     error = function(e)
                     stop(gettextf("file '%s' is not in valid DCF format",
                                   dfile),
                          domain = NA, call. = FALSE))
-    if (nrow(out) != 1)
+    if (nrow(out) != 1L)
         stop("contains a blank line", call. = FALSE)
-    out <- out[1,]
+    out <- out[1L, ]
     if(!is.na(encoding <- out["Encoding"])) {
         ## could convert everything (valid) to UTF-8
         if(encoding == "UTF-8") {
@@ -2008,8 +2150,7 @@ function(x, dfile)
         asc <- iconv(x, "latin1", "ASCII")
         ## fields might have been NA to start with, so use identical.
         if(!identical(asc, x)) {
-            warning(gettext("Unknown encoding with non-ASCII data: converting to ASCII"),
-                    domain = NA)
+            warning("Unknown encoding with non-ASCII data: converting to ASCII")
 	    ind <- is.na(asc) | (asc != x)
             x[ind] <- iconv(x[ind], "latin1", "ASCII", sub = "byte")
         }
@@ -2044,32 +2185,6 @@ function(x, dfile)
     }
 }
 
-### ** .read_repositories
-
-.read_repositories <-
-function(file)
-{
-    db <- utils::read.delim(file, header = TRUE, comment.char = "#",
-                            colClasses =
-                            c(rep.int("character", 3L),
-                              rep.int("logical", 4L))) # allow for win64.binary
-    db[, "URL"] <- .expand_BioC_repository_URLs(db[, "URL"])
-    db
-}
-
-### default changed to https: for R 3.3.0
-.expand_BioC_repository_URLs <-
-function(x)
-{
-    x <- sub("%bm",
-             as.character(getOption("BioC_mirror",
-                                    "https://bioconductor.org")),
-             x, fixed = TRUE)
-    sub("%v",
-        as.character(.BioC_version_associated_with_R_version()),
-        x, fixed = TRUE)
-}
-
 .expand_package_description_db_R_fields <-
 function(x)
 {
@@ -2138,7 +2253,8 @@ function(file, envir, enc = NA)
     exprs <- parse(n = -1L, file = con)
     exprs <- exprs[lengths(exprs) > 0L]
     for(e in exprs) {
-	if(is.call(e) && as.character(e[[1L]]) %in% assignmentSymbols)
+	if(is.call(e) &&
+           as.character(e[[1L]])[1L] %in% assignmentSymbols)
             tryCatch(eval(e, envir), error = identity)
     }
     invisible()
@@ -2230,6 +2346,33 @@ function(command, args = character(), env = character(),
          stderr = readLines(errfile, warn = FALSE))
 }
 
+### ** .trim_common_leading_whitespace
+
+.trim_common_leading_whitespace <-
+function(x)
+{
+    y <- sub("^([ \t]*).*", "\\1", x)
+    n <- nchar(y)
+    if(any(n == 0))
+        return(x)
+    i <- grep("\t", y, fixed = TRUE)
+    if(length(i)) {
+        ## Need to convert tabs to spaces.
+        ## Ideally nchar(y, "width") would do things for us ...
+        wids <- vapply(strsplit(y[i], ""),
+                       function(e) {
+                           p <- which(e == "\t")
+                           d <- diff(c(0, p))
+                           sum(d + 8 - (d %% 8)) + length(e) -
+                               p[length(p)]
+                       },
+                       0)
+        x[i] <- paste0(strrep(" ", wids), substring(x[i], n[i] + 1L))
+        n[i] <- wids
+    }
+    substring(x, min(n) + 1L)
+}
+
 ### ** .try_quietly
 
 .try_quietly <-
@@ -2325,6 +2468,72 @@ function(args, msg)
 
 ### * Miscellania
 
+### ** R
+
+R <-
+function(fun, args = list(), opts = character(), env = character(),
+         arch = "", drop = TRUE, timeout = 0)
+{
+    .safe_repositories <- function() {
+        x <- getOption("repos")
+        y <- .get_standard_repository_URLs()
+        i <- which(names(x) == "CRAN")[1L]
+        if(is.na(i) || x[i] == "@CRAN@")
+            x[i] <- y["CRAN"]
+        c(x, y[match(names(y), names(x), 0L) == 0L])
+    }
+
+    tfi <- tempfile("runri")
+    tfo <- tempfile("runro")
+    wrk <- c(sprintf("x <- readRDS(\"%s\")", tfi),
+             "options(repos = x$repos)",
+             ## need quote = TRUE in case some of args are not self-evaluating
+             ## could catch other conditions also
+             "y <- tryCatch(list(do.call(x$fun, x$args, quote = TRUE)), error = identity)",
+             sprintf("saveRDS(y, \"%s\")", tfo))
+    saveRDS(list(fun = fun, args = args, repos = .safe_repositories()),
+            tfi)
+    cmd <- if(.Platform$OS.type == "windows") {
+               if(nzchar(arch))
+                   ## R.home("bin") might be better, but Windows
+                   ## installation is monolithic
+                   file.path(R.home(), "bin", arch, "Rterm.exe")
+               else
+                   file.path(R.home("bin"), "Rterm.exe")
+           } else {
+               if(nzchar(arch))
+                   opts <- c(paste0("--arch=", arch), opts)
+               file.path(R.home("bin"), "R")
+           }
+    res <- .system_with_capture(cmd, opts, env, input = wrk,
+                                timeout = timeout)
+    ## FIXME: what should the "value" be in case of error?
+    if(file.exists(tfo)) {
+        val <- readRDS(tfo)
+        if (inherits(val, "condition")) {
+            ## maybe wrap in a classed error and include some of res
+            msg <- paste0("error in inferior call:\n  ", conditionMessage(val))
+            stop(errorCondition(msg,
+                                class = "inferiorCallError",
+                                res = res,
+                                error = val))
+        }
+        else {
+            val <- val[[1L]]
+            if(drop)
+                val
+            else
+                c(list(value = val), res)
+        }
+    }
+    else
+        ## again maybe wrap in a classed error  and include some of res
+        ## might want to distinguish two errors by sub-classes
+        stop(errorCondition("inferior call failed",
+                            class = "inferiorCallError",
+                            res = res))
+}
+
 ### ** Rcmd
 
 Rcmd <-
