@@ -1,7 +1,7 @@
 #  File src/library/utils/R/indices.R
 #  Part of the R package, https://www.R-project.org
 #
-#  Copyright (C) 1995-2017 The R Core Team
+#  Copyright (C) 1995-2023 The R Core Team
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -83,9 +83,9 @@ packageDescription <-
 	    if(encoding != enc) { # try to translate from 'enc' to 'encoding' --------
 		## might have an invalid encoding ...
 		newdesc <- try(lapply(desc, iconv, from = enc, to = encoding))
-		dOk <- function(nd) !inherits(nd, "error") && !anyNA(nd)
+		dOk <- function(nd) !inherits(nd, "try-error") && !anyNA(nd)
 		ok <- dOk(newdesc)
-		if(!ok) # try again
+		if(!ok && !endsWith(encoding, "//TRANSLIT")) # try again
 		    ok <- dOk(newdesc <- try(lapply(desc, iconv, from = enc,
 						    to = paste0(encoding,"//TRANSLIT"))))
 		if(!ok) # try again
@@ -127,9 +127,14 @@ print.packageDescription <-
     xx[] <- lapply(xx, function(x) if(is.na(x)) "NA" else x)
     if(abbrCollate > 0 && any(names(xx) == "Collate")) {
         ## trim a long "Collate" field -- respecting word boundaries
-	wrds <- strsplit(xx$Collate,"[ \n]")[[1L]]
-	k <- which.max(cumsum(nchar(wrds)) > abbrCollate) - 1L
-	xx$Collate <- paste(c(wrds[seq_len(k)], "....."), collapse=" ")
+	wrds <- strsplit(xx$Collate,"[ \t\n]")[[1L]]
+        if(n <- length(wrds)) {
+            k <- sum((cumsum(nchar(wrds, "w")) + seq.int(0, n - 1)) <=
+                     abbrCollate)
+            if(k < n)
+                xx$Collate <-
+                    paste(c(wrds[seq_len(k)], "....."), collapse=" ")
+        }
     }
     write.dcf(as.data.frame.list(xx, optional = TRUE))
     cat("\n-- File:", attr(x, "file"), "\n")
@@ -171,6 +176,8 @@ packageDate <- function(pkg, lib.loc = NULL,
 	desc = packageDescription(pkg, lib.loc=lib.loc, fields=date.fields))
 {
     useDesc <- is.list(desc) && length(names(desc)) >= 1
+    if(!useDesc && is.na(desc)) # also got warning
+        return(.Date(NA))
     for (fld in date.fields) {
 	res <- if(useDesc) {
 		   r <- desc[[fld]]
@@ -184,7 +191,7 @@ packageDate <- function(pkg, lib.loc = NULL,
 				NA_character_
 			    })
 	       else as.Date(res, tryFormats=tryFormats,
-			    optional = TRUE)# NA instead of errror
+			    optional = TRUE)# NA instead of error
 	if (!is.na(res))
 	    break
     }
