@@ -1,7 +1,7 @@
 #  File src/library/utils/R/readtable.R
 #  Part of the R package, https://www.R-project.org
 #
-#  Copyright (C) 1995-2014 The R Core Team
+#  Copyright (C) 1995-2022 The R Core Team
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -35,14 +35,20 @@ function(x, ...)
     UseMethod("type.convert")
 
 type.convert.default <-
-function(x, na.strings = "NA", as.is = FALSE, dec = ".",
-         numerals = c("allow.loss", "warn.loss", "no.loss"), ...)
+function(x, na.strings = "NA", as.is, dec = ".",
+         numerals = c("allow.loss", "warn.loss", "no.loss"), tryLogical = TRUE, ...)
 {
     if(is.array(x))
         storage.mode(x) <- "character"
     else
         x <- as.character(x)
-    .External2(C_typeconvert, x, na.strings, as.is, dec, match.arg(numerals))
+    ## as.is = FALSE  was default in R <= 4.0.0, but the default for 'stringsAsFactors',
+    ## hence 'as.is' was changed for/in read.table() with R 4.0.0
+    if(missing(as.is)) {
+        warning("'as.is' should be specified by the caller; using TRUE")
+        as.is <- TRUE
+    }
+    .External2(C_typeconvert, x, na.strings, as.is, dec, match.arg(numerals), tryLogical)
 }
 
 type.convert.list <-
@@ -58,13 +64,13 @@ type.convert.data.frame <- type.convert.list
 read.table <-
 function(file, header = FALSE, sep = "", quote = "\"'", dec = ".",
 	 numerals = c("allow.loss", "warn.loss", "no.loss"),
-         row.names, col.names, as.is = !stringsAsFactors,
+         row.names, col.names, as.is = !stringsAsFactors, tryLogical = TRUE,
          na.strings = "NA", colClasses = NA,
          nrows = -1, skip = 0,
          check.names = TRUE, fill = !blank.lines.skip,
          strip.white = FALSE, blank.lines.skip = TRUE,
          comment.char = "#", allowEscapes = FALSE, flush = FALSE,
-         stringsAsFactors = default.stringsAsFactors(),
+         stringsAsFactors = FALSE,
          fileEncoding = "", encoding = "unknown", text, skipNul = FALSE)
 {
     if (missing(file) && !missing(text)) {
@@ -108,6 +114,7 @@ function(file, header = FALSE, sep = "", quote = "\"'", dec = ".",
                       nlines = 1, quiet = TRUE, skip = 0,
                       strip.white = TRUE,
                       blank.lines.skip = blank.lines.skip,
+                      na.strings=character(0),# NA colname if !check.names
                       comment.char = comment.char, allowEscapes = allowEscapes,
                       encoding = encoding, skipNul = skipNul)
         col1 <- if(missing(col.names)) length(first) else length(col.names)
@@ -178,7 +185,7 @@ function(file, header = FALSE, sep = "", quote = "\"'", dec = ".",
     colClasses[colClasses %in% c("real", "double")] <- "numeric"
     known <- colClasses %in% c("logical", "integer", "numeric", "complex",
                                "character", "raw")
-    what[known] <- sapply(colClasses[known], do.call, list(0))
+    what[known] <- lapply(colClasses[known], do.call, list(0))
     what[colClasses %in% "NULL"] <- list(NULL)
     keep <- !sapply(what, is.null)
 
@@ -228,7 +235,8 @@ function(file, header = FALSE, sep = "", quote = "\"'", dec = ".",
         data[[i]] <-
             if (is.na(colClasses[i]))
                 type.convert(data[[i]], as.is = as.is[i], dec=dec,
-			     numerals=numerals, na.strings = character(0L))
+			     numerals=numerals, na.strings = character(0L),
+                             tryLogical=tryLogical)
         ## as na.strings have already been converted to <NA>
             else if (colClasses[i] == "factor") as.factor(data[[i]])
             else if (colClasses[i] == "Date") as.Date(data[[i]])
