@@ -1,7 +1,7 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
- *  Copyright (C) 1995, 1996  Robert Gentleman and Ross Ihaka
- *  Copyright (C) 1998-2018   The R Core Team.
+ *  Copyright (C) 1998-2022  The R Core Team.
+ *  Copyright (C) 1995,1996  Robert Gentleman and Ross Ihaka
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -73,7 +73,7 @@ typedef struct {
     int quiet;
     int sepchar; /*  = 0 */      /* This gets compared to ints */
     char decchar; /* = '.' */    /* This only gets compared to chars */
-    char *quoteset; /* = NULL */
+    char quoteset[10]; /* = "" */
     int comchar; /* = NO_COMCHAR */
     int ttyflag; /* = 0 */
     Rconnection con; /* = NULL */
@@ -135,12 +135,10 @@ static int ConsoleGetchar(void)
 /* used by scan() */
 static int ConsoleGetcharWithPushBack(Rconnection con)
 {
-    char *curLine;
-    int c;
-
+    // -fanalyzer says this can free curLine twice.
     if(con->nPushBack > 0) {
-	curLine = con->PushBack[con->nPushBack-1];
-	c = curLine[con->posPushBack++];
+	char *curLine = con->PushBack[con->nPushBack-1];
+	int c = curLine[con->posPushBack++];
 	if(con->posPushBack >= strlen(curLine)) {
 	    /* last character on a line, so pop the line */
 	    free(curLine);
@@ -312,8 +310,8 @@ SEXP countfields(SEXP args)
     int nfields, nskip, i, c, inquote, quote = 0;
     int blocksize, nlines, blskip;
     const char *p;
-    Rboolean dbcslocale = (MB_CUR_MAX == 2);
-    LocalData data = {NULL, 0, 0, '.', NULL, NO_COMCHAR, 0, NULL, FALSE,
+    Rboolean dbcslocale = (R_MB_CUR_MAX == 2);
+    LocalData data = {NULL, 0, 0, '.', "", NO_COMCHAR, 0, NULL, FALSE,
 		      FALSE, 0, FALSE,	 FALSE};
     data.NAstrings = R_NilValue;
 
@@ -344,10 +342,10 @@ SEXP countfields(SEXP args)
 
     if (isString(quotes)) {
 	const char *sc = translateChar(STRING_ELT(quotes, 0));
-	if (strlen(sc)) data.quoteset = strdup(sc);
-	else data.quoteset = "";
+	if (strlen(sc)) strcpy(data.quoteset, sc);
+	else strcpy(data.quoteset, "");
     } else if (isNull(quotes))
-	data.quoteset = "";
+	strcpy(data.quoteset, "");
     else
 	error(_("invalid quote symbol set"));
 
@@ -483,7 +481,6 @@ SEXP countfields(SEXP args)
     for (i = 0; i <= nlines; i++)
 	INTEGER(bns)[i] = INTEGER(ans)[i];
     UNPROTECT(1);
-    if (data.quoteset[0]) free(data.quoteset);
     return bns;
 }
 
@@ -546,7 +543,9 @@ static void ruleout_types(const char *s, Typecvt_Info *typeInfo, LocalData *data
    the result is a character string if as.is == TRUE
    or a factor if as.is == FALSE. */
 
-
+/* Called from R's  type.convert.default()
+ * as  .External2(C_typeconvert,
+ *                x, na.strings, as.is, dec, match.arg(numerals), tryLogical) */
 SEXP typeconvert(SEXP call, SEXP op, SEXP args, SEXP env)
 {
     SEXP cvec, a, dup, levs, dims, names, dec, numerals;
@@ -555,12 +554,12 @@ SEXP typeconvert(SEXP call, SEXP op, SEXP args, SEXP env)
     Rboolean done = FALSE, exact;
     char *endp;
     const char *tmp = NULL;
-    LocalData data = {NULL, 0, 0, '.', NULL, NO_COMCHAR, 0, NULL, FALSE,
+    LocalData data = {NULL, 0, 0, '.', "", NO_COMCHAR, 0, NULL, FALSE,
 		      FALSE, 0, FALSE, FALSE};
     Typecvt_Info typeInfo;      /* keep track of possible types of cvec */
-    typeInfo.islogical = TRUE;  /* we can't rule anything out initially */
+    /* we can't rule anything out initially,  but set 'islogical' below */
     typeInfo.isinteger = TRUE;
-    typeInfo.isreal = TRUE;
+    typeInfo.isreal    = TRUE;
     typeInfo.iscomplex = TRUE;
     data.NAstrings = R_NilValue;
 
@@ -574,7 +573,7 @@ SEXP typeconvert(SEXP call, SEXP op, SEXP args, SEXP env)
 	error(_("invalid '%s' argument"), "na.strings");
 
     asIs = asLogical(CADDR(args));
-    if (asIs == NA_LOGICAL) asIs = 0;
+    if (asIs == NA_LOGICAL) asIs = FALSE;
 
     dec = CADDDR(args);
     if (isString(dec) || isNull(dec)) {
@@ -604,6 +603,10 @@ SEXP typeconvert(SEXP call, SEXP op, SEXP args, SEXP env)
 	exact = FALSE;
     }
 
+    Rboolean tryLogical = asLogical(CAD5R(args));
+    if (tryLogical == NA_LOGICAL) tryLogical = FALSE;
+    typeInfo.islogical = tryLogical;
+
     cvec = CAR(args);
     len = length(cvec);
 
@@ -765,7 +768,7 @@ SEXP menu(SEXP choices)
     int c, j;
     double first;
     char buffer[MAXELTSIZE], *bufp = buffer;
-    LocalData data = {NULL, 0, 0, '.', NULL, NO_COMCHAR, 0, NULL, FALSE,
+    LocalData data = {NULL, 0, 0, '.', "", NO_COMCHAR, 0, NULL, FALSE,
 		      FALSE, 0, FALSE, FALSE};
     data.NAstrings = R_NilValue;
 
@@ -809,7 +812,7 @@ SEXP readtablehead(SEXP args)
 	blskip, skipNul;
     const char *p; char *buf;
     Rboolean empty, skip, firstnonwhite;
-    LocalData data = {NULL, 0, 0, '.', NULL, NO_COMCHAR, 0, NULL, FALSE,
+    LocalData data = {NULL, 0, 0, '.', "", NO_COMCHAR, 0, NULL, FALSE,
 		      FALSE, 0, FALSE, FALSE, FALSE};
     data.NAstrings = R_NilValue;
 
@@ -829,10 +832,11 @@ SEXP readtablehead(SEXP args)
     if (isString(quotes)) {
 	const char *sc = translateChar(STRING_ELT(quotes, 0));
 	/* FIXME: will leak memory at long jump */
-	if (strlen(sc)) data.quoteset = strdup(sc);
-	else data.quoteset = "";
+	// strdup allocates and can fail
+	if (strlen(sc)) strcpy(data.quoteset, sc);
+	else strcpy(data.quoteset, "");
     } else if (isNull(quotes))
-	data.quoteset = "";
+	strcpy(data.quoteset, "");
     else
 	error(_("invalid quote symbol set"));
 
@@ -869,7 +873,7 @@ SEXP readtablehead(SEXP args)
     if(!buf)
 	error(_("cannot allocate buffer in 'readTableHead'"));
 
-    PROTECT(ans = allocVector(STRSXP, nlines));
+    ans = PROTECT(allocVector(STRSXP, nlines));
     for(nread = 0; nread < nlines; ) {
 	nbuf = 0; empty = TRUE; skip = FALSE; firstnonwhite = TRUE;
 	if (data.ttyflag)
@@ -882,7 +886,6 @@ SEXP readtablehead(SEXP args)
 		char *tmp = (char *) realloc(buf, buf_size);
 		if(!tmp) {
 		    free(buf);
-		    if (data.quoteset[0]) free(data.quoteset);
 		    error(_("cannot allocate buffer in 'readTableHead'"));
 		} else buf = tmp;
 	    }
@@ -896,7 +899,6 @@ SEXP readtablehead(SEXP args)
 		    c = scanchar(TRUE, &data);
 		    if(c == R_EOF) {
 			free(buf);
-			if (data.quoteset[0]) free(data.quoteset);
 			error(_("\\ followed by EOF"));
 		    }
 		    buf[nbuf++] = (char) c;
@@ -943,7 +945,6 @@ SEXP readtablehead(SEXP args)
     UNPROTECT(1);
     free(buf);
     if(!data.wasopen) data.con->close(data.con);
-    if (data.quoteset[0]) free(data.quoteset);
     return ans;
 
 no_more_lines:
@@ -954,7 +955,6 @@ no_more_lines:
 		    data.con->description);
 	} else {
 	    free(buf);
-	    if (data.quoteset[0]) free(data.quoteset);
 	    error(_("incomplete final line found by readTableHeader on '%s'"),
 		  data.con->description);
 	}
@@ -964,7 +964,6 @@ no_more_lines:
     for(i = 0; i < nread; i++)
 	SET_STRING_ELT(ans2, i, STRING_ELT(ans, i));
     UNPROTECT(2);
-    if (data.quoteset[0]) free(data.quoteset);
     return ans2;
 }
 
@@ -1054,7 +1053,7 @@ static void wt_cleanup(void *data)
 	    else
 	        warning(_("Problem closing connection"));
 	}
-    }	
+    }
     R_FreeStringBuffer(ld->buf);
     R_print.digits = ld->savedigits;
 }
@@ -1118,7 +1117,8 @@ SEXP writetable(SEXP call, SEXP op, SEXP args, SEXP env)
     }
     R_AllocStringBuffer(0, &strBuf);
     PrintDefaults();
-    wi.savedigits = R_print.digits; R_print.digits = DBL_DIG;/* MAX precision */
+    wi.savedigits = R_print.digits;
+    R_print.digits = DBL_DIG; /* MAX precision */
     wi.con = con;
     wi.wasopen = wasopen;
     wi.buf = &strBuf;
@@ -1142,7 +1142,10 @@ SEXP writetable(SEXP call, SEXP op, SEXP args, SEXP env)
 	}
 
 	for(int i = 0; i < nr; i++) {
-	    if(i % 1000 == 999) R_CheckUserInterrupt();
+	    if(i % 1000 == 999) {
+		R_CheckUserInterrupt();
+		R_print.digits = DBL_DIG; /* MAX precision, see PR#18384 */
+	    }
 	    if(!isNull(rnames))
 		Rconn_printf(con, "%s%s",
 			     EncodeElement2(rnames, i, quote_rn, qmethod,
@@ -1186,7 +1189,10 @@ SEXP writetable(SEXP call, SEXP op, SEXP args, SEXP env)
 	    error(_("corrupt matrix -- dims do not match length"));
 
 	for(int i = 0; i < nr; i++) {
-	    if(i % 1000 == 999) R_CheckUserInterrupt();
+	    if(i % 1000 == 999) {
+		R_CheckUserInterrupt();
+		R_print.digits = DBL_DIG; /* MAX precision, see PR#18384 */
+	    }
 	    if(!isNull(rnames))
 		Rconn_printf(con, "%s%s",
 			     EncodeElement2(rnames, i, quote_rn, qmethod,
