@@ -1,6 +1,6 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
- *  Copyright (C) 1998-2020   The R Core Team
+ *  Copyright (C) 1998-2022   The R Core Team
  *  Copyright (C) 2002-2015   The R Foundation
  *  Copyright (C) 1995, 1996  Robert Gentleman and Ross Ihaka
  *
@@ -26,7 +26,7 @@
 #include <Defn.h>
 #include <Internal.h>
 #include <Rmath.h>
-#include <R_ext/RS.h>     /* for Calloc/Free, F77_CALL */
+#include <R_ext/RS.h>     /* for R_Calloc/R_Free, F77_CALL */
 #include <R_ext/BLAS.h>
 #include <R_ext/Itermacros.h>
 
@@ -63,7 +63,8 @@ SEXP GetColNames(SEXP dimnames)
 	return R_NilValue;
 }
 
-SEXP attribute_hidden do_matrix(SEXP call, SEXP op, SEXP args, SEXP rho)
+// .Internal(matrix(data, nrow, ncol, byrow, dimnames, missing(nrow), missing(ncol)))
+attribute_hidden SEXP do_matrix(SEXP call, SEXP op, SEXP args, SEXP rho)
 {
     SEXP vals, ans, snr, snc, dimnames;
     int nr = 1, nc = 1, byrow, miss_nr, miss_nc;
@@ -96,6 +97,11 @@ SEXP attribute_hidden do_matrix(SEXP call, SEXP op, SEXP args, SEXP rho)
     miss_nr = asLogical(CAR(args)); args = CDR(args);
     miss_nc = asLogical(CAR(args));
 
+    static int nowarn = -1;
+    if (nowarn == -1) {
+	char *p = getenv("_R_CHECK_MATRIX_DATA_");
+	nowarn = (p && StringTrue(p)) ? 1 : 0;
+    }
     if (!miss_nr) {
 	if (!isNumeric(snr)) error(_("non-numeric matrix extent"));
 	nr = asInteger(snr);
@@ -116,36 +122,37 @@ SEXP attribute_hidden do_matrix(SEXP call, SEXP op, SEXP args, SEXP rho)
 	if (lendat > INT_MAX) error("data is too long");
 	nr = (int) lendat;
     } else if (miss_nr) {
-	if (lendat > (double) nc * INT_MAX) error("data is too long");
-	// avoid division by zero
-	if (nc == 0) {
-	    if (lendat) error(_("nc = 0 for non-null data"));
-	    else nr = 0;
-	} else
+	if (lendat > (double) nc * INT_MAX) error("data is too long"); // incl lendat > nc == 0
+	if (nc == 0) // as lendat <= nc, have lendat == 0
+	    nr = 0;
+	else
 	    nr = (int) ceil((double) lendat / (double) nc);
     } else if (miss_nc) {
-	if (lendat > (double) nr * INT_MAX) error("data is too long");
-	// avoid division by zero
-	if (nr == 0) {
-	    if (lendat) error(_("nr = 0 for non-null data"));
-	    else nc = 0;
-	} else
+	if (lendat > (double) nr * INT_MAX) error("data is too long"); // incl lendat > nr == 0
+	if (nr == 0) // then lendat == 0
+	    nc = 0;
+	else
 	    nc = (int) ceil((double) lendat / (double) nr);
     }
 
-    if(lendat > 0) {
+    if (lendat > 0) {
 	R_xlen_t nrc = (R_xlen_t) nr * nc;
-	if (lendat > 1 && nrc % lendat != 0) {
+	if (lendat > 1 && (nrc % lendat) != 0) { // ==> nrc > 0
 	    if (((lendat > nr) && (lendat / nr) * nr != lendat) ||
 		((lendat < nr) && (nr / lendat) * lendat != nr))
 		warning(_("data length [%d] is not a sub-multiple or multiple of the number of rows [%d]"), lendat, nr);
 	    else if (((lendat > nc) && (lendat / nc) * nc != lendat) ||
 		     ((lendat < nc) && (nc / lendat) * lendat != nc))
 		warning(_("data length [%d] is not a sub-multiple or multiple of the number of columns [%d]"), lendat, nc);
+	    else if (nrc != lendat) {
+		if(nowarn)
+		    error(_("data length differs from size of matrix: [%d != %d x %d]"), lendat, nr, nc);
+		else
+		    warning(_("data length differs from size of matrix: [%d != %d x %d]"), lendat, nr, nc);
+	    }
 	}
-	else if ((lendat > 1) && (nrc == 0)){
-	    warning(_("data length exceeds size of matrix"));
-	}
+	else if (lendat > 1 && nrc == 0) // for now *not* warning for e.g., matrix(NA, 0, 4)
+	    warning(_("non-empty data for zero-extent matrix"));
     }
 
 #ifndef LONG_VECTOR_SUPPORT
@@ -289,11 +296,8 @@ SEXP allocArray(SEXPTYPE mode, SEXP dims)
 
 SEXP DropDims(SEXP x)
 {
-    SEXP dims, dimnames, newnames = R_NilValue;
-    int i, n, ndims;
-
     PROTECT(x);
-    dims = getAttrib(x, R_DimSymbol);
+    SEXP dims = getAttrib(x, R_DimSymbol);
 
     /* Check that dropping will actually do something. */
     /* (1) Check that there is a "dim" attribute. */
@@ -302,11 +306,12 @@ SEXP DropDims(SEXP x)
 	UNPROTECT(1); /* x */
 	return x;
     }
-    ndims = LENGTH(dims);
+
+    int ndims = LENGTH(dims);
     int *dim = INTEGER(dims); // used several times
 
     /* (2) Check whether there are redundant extents */
-    n = 0;
+    int i, n = 0;
     for (i = 0; i < ndims; i++)
 	if (dim[i] != 1) n++;
     if (n == ndims) {
@@ -314,7 +319,8 @@ SEXP DropDims(SEXP x)
 	return x;
     }
 
-    PROTECT(dimnames = getAttrib(x, R_DimNamesSymbol));
+    SEXP dimnames = PROTECT(getAttrib(x, R_DimNamesSymbol)),
+	newnames = R_NilValue;
     if (n <= 1) {
 	/* We have reduced to a vector result.
 	   If that has length one, it is ambiguous which dimnames to use,
@@ -405,7 +411,7 @@ SEXP DropDims(SEXP x)
     return x;
 }
 
-SEXP attribute_hidden do_drop(SEXP call, SEXP op, SEXP args, SEXP rho)
+attribute_hidden SEXP do_drop(SEXP call, SEXP op, SEXP args, SEXP rho)
 {
     SEXP x, xdims;
     int i, n, shorten;
@@ -427,7 +433,7 @@ SEXP attribute_hidden do_drop(SEXP call, SEXP op, SEXP args, SEXP rho)
 
 /* Length of Primitive Objects */
 
-SEXP attribute_hidden do_length(SEXP call, SEXP op, SEXP args, SEXP rho)
+attribute_hidden SEXP do_length(SEXP call, SEXP op, SEXP args, SEXP rho)
 {
     checkArity(op, args);
     check1arg(args, call, "x");
@@ -458,7 +464,7 @@ SEXP attribute_hidden do_length(SEXP call, SEXP op, SEXP args, SEXP rho)
     return ScalarInteger(length(x));
 }
 
-R_len_t attribute_hidden dispatch_length(SEXP x, SEXP call, SEXP rho) {
+attribute_hidden R_len_t dispatch_length(SEXP x, SEXP call, SEXP rho) {
     R_xlen_t len = dispatch_xlength(x, call, rho);
 #ifdef LONG_VECTOR_SUPPORT
     if (len > INT_MAX) return R_BadLongVector(x, __FILE__, __LINE__);
@@ -466,7 +472,7 @@ R_len_t attribute_hidden dispatch_length(SEXP x, SEXP call, SEXP rho) {
     return (R_len_t) len;
 }
 
-R_xlen_t attribute_hidden dispatch_xlength(SEXP x, SEXP call, SEXP rho) {
+attribute_hidden R_xlen_t dispatch_xlength(SEXP x, SEXP call, SEXP rho) {
     static SEXP length_op = NULL;
     if (isObject(x)) {
         SEXP len, args;
@@ -511,7 +517,7 @@ static SEXP do_lengths_long(SEXP x, SEXP call, SEXP rho)
 }
 #endif
 
-SEXP attribute_hidden do_lengths(SEXP call, SEXP op, SEXP args, SEXP rho)
+attribute_hidden SEXP do_lengths(SEXP call, SEXP op, SEXP args, SEXP rho)
 {
     checkArity(op, args);
     SEXP x = CAR(args), ans;
@@ -572,7 +578,7 @@ SEXP attribute_hidden do_lengths(SEXP call, SEXP op, SEXP args, SEXP rho)
     return ans;
 }
 
-SEXP attribute_hidden do_rowscols(SEXP call, SEXP op, SEXP args, SEXP rho)
+attribute_hidden SEXP do_rowscols(SEXP call, SEXP op, SEXP args, SEXP rho)
 {
     checkArity(op, args);
     SEXP dim = CAR(args);
@@ -811,7 +817,7 @@ static void matprod(double *x, int nrx, int ncx,
 	F77_CALL(dgemv)(transT, &nry, &ncy, &one, y,
 			&nry, x, &ione, &zero, z, &ione FCONE);
     else /* matrix-matrix or outer product */
-	F77_CALL(dgemm)(transN, transN, &nrx, &ncy, &ncx, &one, x, 
+	F77_CALL(dgemm)(transN, transN, &nrx, &ncy, &ncx, &one, x,
 			&nrx, y, &nry, &zero, z, &nrx FCONE FCONE);
 }
 
@@ -1225,20 +1231,24 @@ static void tccrossprod(Rcomplex *x, int nrx, int ncx,
 
 
 /* "%*%" (op = 0), crossprod (op = 1) or tcrossprod (op = 2) */
-SEXP attribute_hidden do_matprod(SEXP call, SEXP op, SEXP args, SEXP rho)
+attribute_hidden SEXP do_matprod(SEXP call, SEXP op, SEXP args, SEXP rho)
 {
     int ldx, ldy, nrx, ncx, nry, ncy, mode;
     SEXP x = CAR(args), y = CADR(args), xdims, ydims, ans;
     Rboolean sym;
 
     if (PRIMVAL(op) == 0 && /* %*% is primitive, the others are .Internal() */
-       (IS_S4_OBJECT(x) || IS_S4_OBJECT(y))
-       && R_has_methods(op)) {
+	(OBJECT(x) || OBJECT(y))) {
 	SEXP s, value;
 	/* Remove argument names to ensure positional matching */
 	for(s = args; s != R_NilValue; s = CDR(s)) SET_TAG(s, R_NilValue);
-	value = R_possible_dispatch(call, op, args, rho, FALSE);
-	if (value) return value;
+
+	if ((IS_S4_OBJECT(x) || IS_S4_OBJECT(y)) && R_has_methods(op)){
+	    value = R_possible_dispatch(call, op, args, rho, FALSE);
+	    if (value) return value;
+	}
+	else if (DispatchGroup("matrixOps", call, op, args, rho, &ans))
+	    return ans;
     }
 
     checkArity(op, args);
@@ -1541,7 +1551,7 @@ SEXP attribute_hidden do_matprod(SEXP call, SEXP op, SEXP args, SEXP rho)
 }
 #undef YDIMS_ET_CETERA
 
-SEXP attribute_hidden do_transpose(SEXP call, SEXP op, SEXP args, SEXP rho)
+attribute_hidden SEXP do_transpose(SEXP call, SEXP op, SEXP args, SEXP rho)
 {
     SEXP a, r, dims, dimnames, dimnamesnames = R_NilValue,
 	ndimnamesnames, rnames, cnames;
@@ -1688,7 +1698,7 @@ SEXP attribute_hidden do_transpose(SEXP call, SEXP op, SEXP args, SEXP rho)
 	lj += iip[itmp] * stride[itmp];
 
 /* aperm (a, perm, resize = TRUE) */
-SEXP attribute_hidden do_aperm(SEXP call, SEXP op, SEXP args, SEXP rho)
+attribute_hidden SEXP do_aperm(SEXP call, SEXP op, SEXP args, SEXP rho)
 {
     SEXP a, perm, r, dimsa, dimsr, dna;
     int i, j, n, itmp;
@@ -1868,7 +1878,7 @@ SEXP attribute_hidden do_aperm(SEXP call, SEXP op, SEXP args, SEXP rho)
 }
 
 /* colSums(x, n, p, na.rm) and friends */
-SEXP attribute_hidden do_colsum(SEXP call, SEXP op, SEXP args, SEXP rho)
+attribute_hidden SEXP do_colsum(SEXP call, SEXP op, SEXP args, SEXP rho)
 {
     SEXP x, ans = R_NilValue;
     int type;
@@ -1957,8 +1967,8 @@ SEXP attribute_hidden do_colsum(SEXP call, SEXP op, SEXP args, SEXP rho)
 	    R_CheckStack2(n * sizeof(LDOUBLE));
 	    rans = (LDOUBLE *) alloca(n * sizeof(LDOUBLE));
 	    Memzero(rans, n);
-	} else rans = Calloc(n, LDOUBLE);
-	if (!keepNA && OP == 3) Cnt = Calloc(n, int);
+	} else rans = R_Calloc(n, LDOUBLE);
+	if (!keepNA && OP == 3) Cnt = R_Calloc(n, int);
 
 	for (R_xlen_t j = 0; j < p; j++) {
 	    LDOUBLE *ra = rans;
@@ -2014,8 +2024,8 @@ SEXP attribute_hidden do_colsum(SEXP call, SEXP op, SEXP args, SEXP rho)
 	}
 	for (R_xlen_t i = 0; i < n; i++) REAL(ans)[i] = (double) rans[i];
 
-	if (!keepNA && OP == 3) Free(Cnt);
-	if(n > 10000) Free(rans);
+	if (!keepNA && OP == 3) R_Free(Cnt);
+	if(n > 10000) R_Free(rans);
     }
 
     UNPROTECT(1);
@@ -2041,7 +2051,7 @@ SEXP attribute_hidden do_colsum(SEXP call, SEXP op, SEXP args, SEXP rho)
 */
 
 /* array(data, dim, dimnames) */
-SEXP attribute_hidden do_array(SEXP call, SEXP op, SEXP args, SEXP rho)
+attribute_hidden SEXP do_array(SEXP call, SEXP op, SEXP args, SEXP rho)
 {
     SEXP vals, ans, dims, dimnames;
     R_xlen_t lendat, i, nans;
@@ -2155,7 +2165,7 @@ SEXP attribute_hidden do_array(SEXP call, SEXP op, SEXP args, SEXP rho)
     return ans;
 }
 
-SEXP attribute_hidden do_diag(SEXP call, SEXP op, SEXP args, SEXP rho)
+attribute_hidden SEXP do_diag(SEXP call, SEXP op, SEXP args, SEXP rho)
 {
     SEXP ans, x, snr, snc;
     int nr = 1, nc = 1, nprotect = 1;
@@ -2250,7 +2260,7 @@ SEXP attribute_hidden do_diag(SEXP call, SEXP op, SEXP args, SEXP rho)
 
 
 /* backsolve(r, b, k, upper.tri, transpose) */
-SEXP attribute_hidden do_backsolve(SEXP call, SEXP op, SEXP args, SEXP rho)
+attribute_hidden SEXP do_backsolve(SEXP call, SEXP op, SEXP args, SEXP rho)
 {
     int nprot = 1;
     checkArity(op, args);
@@ -2296,7 +2306,7 @@ SEXP attribute_hidden do_backsolve(SEXP call, SEXP op, SEXP args, SEXP rho)
 }
 
 /* max.col(m, ties.method) */
-SEXP attribute_hidden do_maxcol(SEXP call, SEXP op, SEXP args, SEXP rho)
+attribute_hidden SEXP do_maxcol(SEXP call, SEXP op, SEXP args, SEXP rho)
 {
     checkArity(op, args);
     SEXP m = CAR(args);
