@@ -1,6 +1,6 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
- *  Copyright (C) 1997--2021  The R Core Team
+ *  Copyright (C) 1997--2022  The R Core Team
  *  Copyright (C) 1995, 1996  Robert Gentleman and Ross Ihaka
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -71,7 +71,7 @@ abbreviate chartr make.names strtrim tolower toupper give error.
 # include <config.h>
 #endif
 
-/* Used to indicate that we can safely converted marked UTF-8 strings
+/* Used to indicate that we can safely convert marked UTF-8 strings
    to wchar_t* -- not currently used.
 */
 #if defined(Win32) || defined(__STDC_ISO_10646__) || defined(__APPLE__) || defined(__FreeBSD__) || defined(__sun)
@@ -85,7 +85,7 @@ abbreviate chartr make.names strtrim tolower toupper give error.
 #include <Defn.h>
 #include <Internal.h>
 #include <errno.h>
-#include <R_ext/RS.h>  // for Calloc/Free
+#include <R_ext/RS.h>  // for R_Calloc/R_Free
 #include <R_ext/Itermacros.h>
 #include <rlocale.h>   // overrides iswxxxx on some platforms.
 
@@ -107,7 +107,7 @@ static R_StringBuffer cbuff = {NULL, 0, MAXELTSIZE};
 /* primitive, nzchar(x, keepNA = FALSE) where the second argument is optional.
    Encoding of x is immaterial.
 */
-SEXP attribute_hidden do_nzchar(SEXP call, SEXP op, SEXP args, SEXP env)
+attribute_hidden SEXP do_nzchar(SEXP call, SEXP op, SEXP args, SEXP env)
 {
     int nargs = length(args);
 
@@ -275,8 +275,8 @@ int R_nchar(SEXP string, nchar_type type_,
 		if (msg_name)
 		    R_FreeStringBufferL(&cbuff);
 		vmaxset(vmax);
-		return (nci18n < 1) ? nc : nci18n;
-	    } else if (allowNA) {
+		return (nci18n < 0) ? nc : nci18n;
+	    } else if (!allowNA) {
 		if (msg_name)
 		    error(_("invalid multibyte string, %s"), msg_name);
 		else
@@ -292,7 +292,7 @@ int R_nchar(SEXP string, nchar_type type_,
     return NA_INTEGER; // -Wall
 } // R_nchar()
 
-SEXP attribute_hidden do_nchar(SEXP call, SEXP op, SEXP args, SEXP env)
+attribute_hidden SEXP do_nchar(SEXP call, SEXP op, SEXP args, SEXP env)
 {
     SEXP d, s, x, stype, ans;
     int nargs = length(args);
@@ -384,7 +384,7 @@ static void substr(const char *str, int len, int ienc, int sa, int so,
     if (ienc == CE_UTF8) {
 	if (!assumevalid && !utf8Valid(str)) {
 	    char msg[30];
-	    sprintf(msg, "element %ld", (long)idx+1);
+	    snprintf(msg, 30, "element %ld", (long)idx+1);
 	    error(_("invalid multibyte string, %s"), msg);
 	}
 	for (i = 0; i < sa - 1 && str < end; i++)
@@ -419,7 +419,7 @@ static void substr(const char *str, int len, int ienc, int sa, int so,
     }
 }
 
-SEXP attribute_hidden
+attribute_hidden SEXP
 do_substr(SEXP call, SEXP op, SEXP args, SEXP env)
 {
     SEXP s, x;
@@ -475,7 +475,7 @@ do_substr(SEXP call, SEXP op, SEXP args, SEXP env)
 
 // .Internal( startsWith(x, prefix) )  and
 // .Internal( endsWith  (x, suffix) )
-SEXP attribute_hidden
+attribute_hidden SEXP
 do_startsWith(SEXP call, SEXP op, SEXP args, SEXP env)
 {
     checkArity(op, args);
@@ -499,7 +499,7 @@ do_startsWith(SEXP call, SEXP op, SEXP args, SEXP env)
 	} else {
 	    // ASCII matching will do for ASCII Xfix except in non-UTF-8 MBCS
 	    Rboolean need_translate = TRUE;
-	    if (strIsASCII(CHAR(el)) && (utf8locale || !mbcslocale))
+	    if (IS_ASCII(el) && (utf8locale || !mbcslocale))
 		need_translate = FALSE;
 	    cp y0 = need_translate ? translateCharUTF8(el) : CHAR(el);
 	    int ylen = (int) strlen(y0);
@@ -527,8 +527,8 @@ do_startsWith(SEXP call, SEXP op, SEXP args, SEXP env)
 	cp *x0 = (cp *) R_alloc(n1, sizeof(char *));
 	cp *y0 = (cp *) R_alloc(n2, sizeof(char *));
 	// and record lengths, -1 for NA
-	int *x1 = (int *) R_alloc(n1, sizeof(int *));
-	int *y1 = (int *) R_alloc(n2, sizeof(int *));
+	int *x1 = (int *) R_alloc(n1, sizeof(int));
+	int *y1 = (int *) R_alloc(n2, sizeof(int));
 	for (R_xlen_t i = 0; i < n1; i++) {
 	    SEXP el = STRING_ELT(x, i);
 	    if (el == NA_STRING)
@@ -589,19 +589,18 @@ substrset(char *buf, const char *const str, cetype_t ienc, int sa, int so,
     if (ienc == CE_UTF8) {
 	if (!utf8Valid(buf)) {
 	    char msg[30];
-	    sprintf(msg, "element %ld", (long)xidx+1);
+	    snprintf(msg, 30, "element %ld", (long)xidx+1);
 	    error(_("invalid multibyte string, %s"), msg);
 	}
 	if (!utf8Valid(str)) {
 	    char msg[30];
-	    sprintf(msg, "value element %ld", (long)vidx+1);
+	    snprintf(msg, 30, "value element %ld", (long)vidx+1);
 	    error(_("invalid multibyte string, %s"), msg);
 	}
 	for (i = 1; i < sa; i++) buf += utf8clen(*buf);
-	for (i = sa; i <= so && in < strlen(str); i++) {
+	for (i = sa; i <= so && buf[out] && str[in]; i++) {
 	    in +=  utf8clen(str[in]);
 	    out += utf8clen(buf[out]);
-	    if (!str[in]) break;
 	}
 	if (in != out) memmove(buf+in, buf+out, strlen(buf+out)+1);
 	memcpy(buf, str, in);
@@ -612,12 +611,16 @@ substrset(char *buf, const char *const str, cetype_t ienc, int sa, int so,
     } else {
 	/* This cannot work for stateful encodings */
 	if (mbcslocale) {
-	    for (i = 1; i < sa; i++) buf += Mbrtowc(NULL, buf, R_MB_CUR_MAX, NULL);
+	    mbstate_t mb_st_in;
+	    mbs_init(&mb_st_in);
+	    for (i = 1; i < sa; i++)
+		buf += Mbrtowc(NULL, buf, R_MB_CUR_MAX, &mb_st_in);
 	    /* now work out how many bytes to replace by how many */
-	    for (i = sa; i <= so && in < strlen(str); i++) {
-		in += (int) Mbrtowc(NULL, str+in, R_MB_CUR_MAX, NULL);
-		out += (int) Mbrtowc(NULL, buf+out, R_MB_CUR_MAX, NULL);
-		if (!str[in]) break;
+	    mbstate_t mb_st_out;
+	    mbs_init(&mb_st_out);
+	    for (i = sa; i <= so && buf[out] && str[in]; i++) {
+		in  += (int) Mbrtowc(NULL, str+in,  R_MB_CUR_MAX, &mb_st_in);
+		out += (int) Mbrtowc(NULL, buf+out, R_MB_CUR_MAX, &mb_st_out);
 	    }
 	    if (in != out) memmove(buf+in, buf+out, strlen(buf+out)+1);
 	    memcpy(buf, str, in);
@@ -629,7 +632,7 @@ substrset(char *buf, const char *const str, cetype_t ienc, int sa, int so,
     }
 }
 
-SEXP attribute_hidden do_substrgets(SEXP call, SEXP op, SEXP args, SEXP env)
+attribute_hidden SEXP do_substrgets(SEXP call, SEXP op, SEXP args, SEXP env)
 {
     SEXP s, x, sa, so, value, el, v_el;
     R_xlen_t i, len;
@@ -674,7 +677,7 @@ SEXP attribute_hidden do_substrgets(SEXP call, SEXP op, SEXP args, SEXP env)
 	    ss = CHAR(el);
 	    slen = strlen(ss);
 	    if (start < 1) start = 1;
-	    if (stop > slen) stop = (int) slen; /* SBCS optimization */
+	    if (stop > (int) slen) stop = (int) slen; /* SBCS optimization */
 	    if (start > stop) {
 		/* just copy element across */
 		SET_STRING_ELT(s, i, STRING_ELT(x, i));
@@ -685,7 +688,7 @@ SEXP attribute_hidden do_substrgets(SEXP call, SEXP op, SEXP args, SEXP env)
 		   FIXME: could re-encode to UTF-8 rather than to native.
 		 */
 		venc = getCharCE(v_el);
-		if (venc != ienc && !strIsASCII(v_ss)) {
+		if (venc != ienc && !IS_ASCII(v_el)) {
 		    ss = translateChar(el);
 		    slen = strlen(ss);
 		    v_ss = translateChar(v_el);
@@ -701,6 +704,8 @@ SEXP attribute_hidden do_substrgets(SEXP call, SEXP op, SEXP args, SEXP env)
 	}
 	R_FreeStringBufferL(&cbuff);
     }
+    SHALLOW_DUPLICATE_ATTRIB(s, x);
+    /* This copied the class, if any */
     UNPROTECT(1);
     return s;
 }
@@ -906,12 +911,12 @@ donewsc:
     char *cbuf = CallocCharBuf(nb);
     wcstoutf8(cbuf, wc, nb);
     SEXP ans = mkCharCE(cbuf, CE_UTF8);
-    Free(cbuf);
+    R_Free(cbuf);
     return ans;
 }
 
 
-SEXP attribute_hidden do_abbrev(SEXP call, SEXP op, SEXP args, SEXP env)
+attribute_hidden SEXP do_abbrev(SEXP call, SEXP op, SEXP args, SEXP env)
 {
     checkArity(op,args);
     SEXP x = CAR(args);
@@ -935,7 +940,7 @@ SEXP attribute_hidden do_abbrev(SEXP call, SEXP op, SEXP args, SEXP env)
 	    SET_STRING_ELT(ans, i, NA_STRING);
 	else {
 	    const char *s = CHAR(el);
-	    if (strIsASCII(s)) {
+	    if (IS_ASCII(el)) {
 		if(strlen(s) > minlen) {
 		    R_AllocStringBuffer(strlen(s)+1, &cbuff);
 		    SET_STRING_ELT(ans, i, stripchars(s, minlen, usecl));
@@ -962,7 +967,7 @@ SEXP attribute_hidden do_abbrev(SEXP call, SEXP op, SEXP args, SEXP env)
     return ans;
 }
 
-SEXP attribute_hidden do_makenames(SEXP call, SEXP op, SEXP args, SEXP env)
+attribute_hidden SEXP do_makenames(SEXP call, SEXP op, SEXP args, SEXP env)
 {
     SEXP arg, ans;
     R_xlen_t i, n;
@@ -1008,18 +1013,18 @@ SEXP attribute_hidden do_makenames(SEXP call, SEXP op, SEXP args, SEXP env)
 	    } else if (!isalpha(0xff & (int) This[0])) need_prefix = TRUE;
 	}
 	if (need_prefix) {
-	    tmp = Calloc(l+2, char);
+	    tmp = R_Calloc(l+2, char);
 	    strcpy(tmp, "X");
 	    strcat(tmp, translateChar(STRING_ELT(arg, i)));
 	} else {
-	    tmp = Calloc(l+1, char);
+	    tmp = R_Calloc(l+1, char);
 	    strcpy(tmp, translateChar(STRING_ELT(arg, i)));
 	}
 	if (mbcslocale) {
 	    /* This cannot lengthen the string, so safe to overwrite it. */
 	    int nc = (int) mbstowcs(NULL, tmp, 0);
 	    if (nc >= 0) {
-		wchar_t *wstr = Calloc(nc+1, wchar_t);
+		wchar_t *wstr = R_Calloc(nc+1, wchar_t);
 		mbstowcs(wstr, tmp, nc+1);
 		for (wchar_t * wc = wstr; *wc; wc++) {
 		    if (*wc == L'.' || (allow_ && *wc == L'_'))
@@ -1027,7 +1032,7 @@ SEXP attribute_hidden do_makenames(SEXP call, SEXP op, SEXP args, SEXP env)
 		    else if (!iswalnum((int)*wc)) *wc = L'.';
 		}
 		wcstombs(tmp, wstr, strlen(tmp)+1);
-		Free(wstr);
+		R_Free(wstr);
 	    } else error(_("invalid multibyte string %d"), i+1);
 	} else {
 	    for (p = tmp; *p; p++) {
@@ -1044,9 +1049,9 @@ SEXP attribute_hidden do_makenames(SEXP call, SEXP op, SEXP args, SEXP env)
 	    strcpy(cbuf, tmp);
 	    strcat(cbuf, ".");
 	    SET_STRING_ELT(ans, i, mkChar(cbuf));
-	    Free(cbuf);
+	    R_Free(cbuf);
 	}
-	Free(tmp);
+	R_Free(tmp);
 	vmaxset(vmax);
     }
     UNPROTECT(1);
@@ -1054,7 +1059,7 @@ SEXP attribute_hidden do_makenames(SEXP call, SEXP op, SEXP args, SEXP env)
 }
 
 
-SEXP attribute_hidden do_tolower(SEXP call, SEXP op, SEXP args, SEXP env)
+attribute_hidden SEXP do_tolower(SEXP call, SEXP op, SEXP args, SEXP env)
 {
     SEXP x, y;
     R_xlen_t i, n;
@@ -1155,7 +1160,7 @@ SEXP attribute_hidden do_tolower(SEXP call, SEXP op, SEXP args, SEXP env)
 			wcstombs(cbuf, wc, nb + 1);
 			SET_STRING_ELT(y, i, markKnown(cbuf, el));
 		    }
-		    Free(cbuf);
+		    R_Free(cbuf);
 		} else {
 		    error(_("invalid multibyte string %d"), i+1);
 		}
@@ -1175,7 +1180,7 @@ SEXP attribute_hidden do_tolower(SEXP call, SEXP op, SEXP args, SEXP env)
 		for (p = xi; *p != '\0'; p++)
 		    *p = (char) (ul ? toupper(*p) : tolower(*p));
 		SET_STRING_ELT(y, i, markKnown(xi, STRING_ELT(x, i)));
-		Free(xi);
+		R_Free(xi);
 	    }
 	    vmaxset(vmax);
 	}
@@ -1208,7 +1213,7 @@ wtr_build_spec(const wchar_t *s, struct wtr_spec *trs) {
 
     This = trs;
     for (i = 0; i < len - 2; ) {
-	_new = Calloc(1, struct wtr_spec);
+	_new = R_Calloc(1, struct wtr_spec);
 	_new->next = NULL;
 	if (s[i + 1] == L'-') {
 	    _new->type = WTR_RANGE;
@@ -1226,7 +1231,7 @@ wtr_build_spec(const wchar_t *s, struct wtr_spec *trs) {
 	This = This->next = _new;
     }
     for ( ; i < len; i++) {
-	_new = Calloc(1, struct wtr_spec);
+	_new = R_Calloc(1, struct wtr_spec);
 	_new->next = NULL;
 	_new->type = WTR_CHAR;
 	_new->u.c = s[i];
@@ -1240,7 +1245,7 @@ wtr_free_spec(struct wtr_spec *trs) {
     This = trs;
     while(This) {
 	next = This->next;
-	Free(This);
+	R_Free(This);
 	This = next;
     }
 }
@@ -1294,7 +1299,7 @@ tr_build_spec(const char *s, struct tr_spec *trs) {
 
     This = trs;
     for (i = 0; i < len - 2; ) {
-	_new = Calloc(1, struct tr_spec);
+	_new = R_Calloc(1, struct tr_spec);
 	_new->next = NULL;
 	if (s[i + 1] == '-') {
 	    _new->type = TR_RANGE;
@@ -1312,7 +1317,7 @@ tr_build_spec(const char *s, struct tr_spec *trs) {
 	This = This->next = _new;
     }
     for ( ; i < len; i++) {
-	_new = Calloc(1, struct tr_spec);
+	_new = R_Calloc(1, struct tr_spec);
 	_new->next = NULL;
 	_new->type = TR_CHAR;
 	_new->u.c = s[i];
@@ -1326,7 +1331,7 @@ tr_free_spec(struct tr_spec *trs) {
     This = trs;
     while(This) {
 	next = This->next;
-	Free(This);
+	R_Free(This);
 	This = next;
     }
 }
@@ -1393,7 +1398,7 @@ static R_INLINE int xtable_key_comp(const void *a, const void *b)
 
 #define COMPRESS(_base,_num,_TYPE,_comp)                       \
 {                                                              \
-/* supress even c_old. last use */                             \
+/* suppress even c_old. last use */                             \
     int _i,_j ;                                                \
     for ( _i = 0 ; _i < (*(_num)) - 1 ; _i++ ){                \
 	int rc = (*_comp)(_base+_i, _base+_i+1);               \
@@ -1429,7 +1434,7 @@ static R_INLINE int xtable_key_comp(const void *a, const void *b)
     }                                                          \
 }
 
-SEXP attribute_hidden do_chartr(SEXP call, SEXP op, SEXP args, SEXP env)
+attribute_hidden SEXP do_chartr(SEXP call, SEXP op, SEXP args, SEXP env)
 {
     SEXP old, _new, x, y;
     R_xlen_t i, n;
@@ -1479,10 +1484,10 @@ SEXP attribute_hidden do_chartr(SEXP call, SEXP op, SEXP args, SEXP env)
 	struct wtr_spec *trs_new, **trs_new_ptr;
 
 	/* Initialize the old and new wtr_spec lists. */
-	trs_old = Calloc(1, struct wtr_spec);
+	trs_old = R_Calloc(1, struct wtr_spec);
 	trs_old->type = WTR_INIT;
 	trs_old->next = NULL;
-	trs_new = Calloc(1, struct wtr_spec);
+	trs_new = R_Calloc(1, struct wtr_spec);
 	trs_new->type = WTR_INIT;
 	trs_new->next = NULL;
 	/* Build the old and new wtr_spec lists. */
@@ -1506,7 +1511,7 @@ SEXP attribute_hidden do_chartr(SEXP call, SEXP op, SEXP args, SEXP env)
 	    mbstowcs(wc, s, nc + 1);
 	}
 	wtr_build_spec(wc, trs_old);
-	trs_cnt = Calloc(1, struct wtr_spec);
+	trs_cnt = R_Calloc(1, struct wtr_spec);
 	trs_cnt->type = WTR_INIT;
 	trs_cnt->next = NULL;
 	wtr_build_spec(wc, trs_cnt); /* use count only */
@@ -1536,17 +1541,17 @@ SEXP attribute_hidden do_chartr(SEXP call, SEXP op, SEXP args, SEXP env)
 	   wtr_spec lists and retrieving the next chars from the lists.
 	*/
 
-	trs_cnt_ptr = Calloc(1, struct wtr_spec *);
+	trs_cnt_ptr = R_Calloc(1, struct wtr_spec *);
 	*trs_cnt_ptr = trs_cnt->next;
 	for (xtable_cnt = 0 ; wtr_get_next_char_from_spec(trs_cnt_ptr);
 	      xtable_cnt++) ;
 	wtr_free_spec(trs_cnt);
-	Free(trs_cnt_ptr);
+	R_Free(trs_cnt_ptr);
 	xtable = (xtable_t *) R_alloc(xtable_cnt+1, sizeof(xtable_t));
 
-	trs_old_ptr = Calloc(1, struct wtr_spec *);
+	trs_old_ptr = R_Calloc(1, struct wtr_spec *);
 	*trs_old_ptr = trs_old->next;
-	trs_new_ptr = Calloc(1, struct wtr_spec *);
+	trs_new_ptr = R_Calloc(1, struct wtr_spec *);
 	*trs_new_ptr = trs_new->next;
 	for (i = 0; ; i++) {
 	    c_old = wtr_get_next_char_from_spec(trs_old_ptr);
@@ -1564,7 +1569,7 @@ SEXP attribute_hidden do_chartr(SEXP call, SEXP op, SEXP args, SEXP env)
 	/* Free the memory occupied by the wtr_spec lists. */
 	wtr_free_spec(trs_old);
 	wtr_free_spec(trs_new);
-	Free(trs_old_ptr); Free(trs_new_ptr);
+	R_Free(trs_old_ptr); R_Free(trs_new_ptr);
 
 	ISORT(xtable, xtable_cnt, xtable_t , xtable_comp);
 	COMPRESS(xtable, &xtable_cnt, xtable_t, xtable_comp);
@@ -1607,7 +1612,7 @@ SEXP attribute_hidden do_chartr(SEXP call, SEXP op, SEXP args, SEXP env)
 		    wcstombs(cbuf, wc, nb + 1);
 		    SET_STRING_ELT(y, i, markKnown(cbuf, el));
 		}
-		Free(cbuf);
+		R_Free(cbuf);
 	    }
 	    vmaxset(vmax);
 	}
@@ -1621,10 +1626,10 @@ SEXP attribute_hidden do_chartr(SEXP call, SEXP op, SEXP args, SEXP env)
 	    xtable[ii] = (unsigned char) ii;
 
 	/* Initialize the old and new tr_spec lists. */
-	trs_old = Calloc(1, struct tr_spec);
+	trs_old = R_Calloc(1, struct tr_spec);
 	trs_old->type = TR_INIT;
 	trs_old->next = NULL;
-	trs_new = Calloc(1, struct tr_spec);
+	trs_new = R_Calloc(1, struct tr_spec);
 	trs_new->type = TR_INIT;
 	trs_new->next = NULL;
 	/* Build the old and new tr_spec lists. */
@@ -1633,9 +1638,9 @@ SEXP attribute_hidden do_chartr(SEXP call, SEXP op, SEXP args, SEXP env)
 	/* Initialize the pointers for walking through the old and new
 	   tr_spec lists and retrieving the next chars from the lists.
 	*/
-	trs_old_ptr = Calloc(1, struct tr_spec *);
+	trs_old_ptr = R_Calloc(1, struct tr_spec *);
 	*trs_old_ptr = trs_old->next;
-	trs_new_ptr = Calloc(1, struct tr_spec *);
+	trs_new_ptr = R_Calloc(1, struct tr_spec *);
 	*trs_new_ptr = trs_new->next;
 	for (;;) {
 	    c_old = tr_get_next_char_from_spec(trs_old_ptr);
@@ -1650,7 +1655,7 @@ SEXP attribute_hidden do_chartr(SEXP call, SEXP op, SEXP args, SEXP env)
 	/* Free the memory occupied by the tr_spec lists. */
 	tr_free_spec(trs_old);
 	tr_free_spec(trs_new);
-	Free(trs_old_ptr); Free(trs_new_ptr);
+	R_Free(trs_old_ptr); R_Free(trs_new_ptr);
 
 	n = LENGTH(x);
 	PROTECT(y = allocVector(STRSXP, n));
@@ -1665,7 +1670,7 @@ SEXP attribute_hidden do_chartr(SEXP call, SEXP op, SEXP args, SEXP env)
 		for (p = (unsigned char *) cbuf; *p != '\0'; p++)
 		    *p = xtable[*p];
 		SET_STRING_ELT(y, i, markKnown(cbuf, STRING_ELT(x, i)));
-		Free(cbuf);
+		R_Free(cbuf);
 	    }
 	}
 	vmaxset(vmax);
@@ -1677,7 +1682,7 @@ SEXP attribute_hidden do_chartr(SEXP call, SEXP op, SEXP args, SEXP env)
     return(y);
 }
 
-SEXP attribute_hidden do_strtrim(SEXP call, SEXP op, SEXP args, SEXP env)
+attribute_hidden SEXP do_strtrim(SEXP call, SEXP op, SEXP args, SEXP env)
 {
     SEXP s, x, width;
     R_xlen_t i, len;
@@ -1758,7 +1763,7 @@ static int strtoi(SEXP s, int base)
 	: (int) res;
 }
 
-SEXP attribute_hidden do_strtoi(SEXP call, SEXP op, SEXP args, SEXP env)
+attribute_hidden SEXP do_strtoi(SEXP call, SEXP op, SEXP args, SEXP env)
 {
     SEXP ans, x, b;
     R_xlen_t i, n;
@@ -1786,7 +1791,7 @@ SEXP attribute_hidden do_strtoi(SEXP call, SEXP op, SEXP args, SEXP env)
 /* creates a new STRSXP which is a suffix of string, starting
    with given index; the result is returned unprotected  */
 
-SEXP attribute_hidden stringSuffix(SEXP string, int fromIndex) {
+attribute_hidden SEXP stringSuffix(SEXP string, int fromIndex) {
 
     int origLen = LENGTH(string);
     int newLen = origLen - fromIndex;
@@ -1801,7 +1806,7 @@ SEXP attribute_hidden stringSuffix(SEXP string, int fromIndex) {
     return res;
 }
 
-SEXP attribute_hidden do_strrep(SEXP call, SEXP op, SEXP args, SEXP env)
+attribute_hidden SEXP do_strrep(SEXP call, SEXP op, SEXP args, SEXP env)
 {
     SEXP d, s, x, n, el;
     R_xlen_t is, ix, in, ns, nx, nn;
@@ -1849,7 +1854,7 @@ SEXP attribute_hidden do_strrep(SEXP call, SEXP op, SEXP args, SEXP env)
 		buf += nc;
 	    }
 	    SET_STRING_ELT(s, is, mkCharCE(cbuf, getCharCE(el)));
-	    Free(cbuf);
+	    R_Free(cbuf);
 	    vmaxset(vmax);
 	}
 	ix = (++ix == nx) ? 0 : ix;
