@@ -1,6 +1,6 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
- *  Copyright (C) 1997--2021  The R Core Team
+ *  Copyright (C) 1997--2023  The R Core Team
  *  Copyright (C) 1995, 1996  Robert Gentleman and Ross Ihaka
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -132,6 +132,7 @@ typedef struct {
     Rboolean startline; /* = TRUE; */
     int indent;
     SEXP strvec;
+    int left;
 
     DeparseBuffer buffer;
 
@@ -163,7 +164,7 @@ static void linebreak(Rboolean *lbreak, LocalParseData *);
 static void deparse2(SEXP, SEXP, LocalParseData *);
 
 // .Internal(deparse(expr, width.cutoff, backtick, .deparseOpts(control), nlines))
-SEXP attribute_hidden do_deparse(SEXP call, SEXP op, SEXP args, SEXP rho)
+attribute_hidden SEXP do_deparse(SEXP call, SEXP op, SEXP args, SEXP rho)
 {
     checkArity(op, args);
 
@@ -232,17 +233,28 @@ static SEXP deparse1WithCutoff(SEXP call, Rboolean abbrev, int cutoff,
     SEXP svec;
     int savedigits;
     Rboolean need_ellipses = FALSE;
-    LocalParseData localData =
-	{/* linenumber */ 0,
-	 0, 0, 0, /*startline = */TRUE, 0,
-	 NULL,
-	 /* DeparseBuffer= */ {NULL, 0, BUFSIZE},
-	 DEFAULT_Cutoff, FALSE, 0, TRUE,
+    LocalParseData localData = {
+	.linenumber = 0,
+	.len = 0,
+	.incurly = 0,
+	.inlist = 0,
+	.startline = TRUE,
+	.indent = 0,
+	.strvec = NULL,
+	.left = 0,
+	.buffer = { NULL, 0, BUFSIZE },
+	.cutoff = DEFAULT_Cutoff,
+	.backtick = FALSE,
+	.opts = 0,
+	.sourceable = TRUE,
 #ifdef longstring_WARN
-	 FALSE,
+	.longstring = FALSE,
 #endif
-	 /* maxlines = */ INT_MAX,
-	 /* active = */TRUE, 0, FALSE};
+	.maxlines = INT_MAX,
+	.active = TRUE,
+	.isS4 = 0,
+	.fnarg = FALSE
+    };
     localData.cutoff = cutoff;
     localData.backtick = backtick;
     localData.opts = opts;
@@ -349,7 +361,7 @@ SEXP deparse1line(SEXP call, Rboolean abbrev)
 
 
 // called only from ./errors.c  for calls in warnings and errors :
-SEXP attribute_hidden deparse1s(SEXP call)
+attribute_hidden SEXP deparse1s(SEXP call)
 {
    Rboolean backtick=TRUE;
    return
@@ -366,7 +378,7 @@ static void con_cleanup(void *data)
 }
 
 // .Internal(dput(x, file, .deparseOpts(control)))
-SEXP attribute_hidden do_dput(SEXP call, SEXP op, SEXP args, SEXP rho)
+attribute_hidden SEXP do_dput(SEXP call, SEXP op, SEXP args, SEXP rho)
 {
     checkArity(op, args);
     SEXP tval = CAR(args);
@@ -420,7 +432,7 @@ SEXP attribute_hidden do_dput(SEXP call, SEXP op, SEXP args, SEXP rho)
 }
 
 // .Internal(dump(list, file, envir, opts, evaluate))
-SEXP attribute_hidden do_dump(SEXP call, SEXP op, SEXP args, SEXP rho)
+attribute_hidden SEXP do_dump(SEXP call, SEXP op, SEXP args, SEXP rho)
 {
     checkArity(op, args);
     SEXP names = CAR(args),
@@ -560,7 +572,8 @@ curlyahead(SEXP s)
    mainop is a unary or binary operator,
    arg is an argument to it, on the left if left == 1 */
 
-static Rboolean needsparens(PPinfo mainop, SEXP arg, unsigned int left)
+static Rboolean needsparens(PPinfo mainop, SEXP arg, unsigned int left,
+			    unsigned int deepLeft)
 {
     PPinfo arginfo;
     if (TYPEOF(arg) == LANGSXP) {
@@ -568,45 +581,60 @@ static Rboolean needsparens(PPinfo mainop, SEXP arg, unsigned int left)
 	    if ((TYPEOF(SYMVALUE(CAR(arg))) == BUILTINSXP) ||
 		(TYPEOF(SYMVALUE(CAR(arg))) == SPECIALSXP)) {
 		arginfo = PPINFO(SYMVALUE(CAR(arg)));
+
+		/* Not all binary ops are binary! */
 		switch(arginfo.kind) {
-		case PP_BINARY:	      /* Not all binary ops are binary! */
+		case PP_BINARY:
 		case PP_BINARY2:
 		    switch(length(CDR(arg))) {
 		    case 1:
-			if (!left)
-			    return FALSE;
-			if (arginfo.precedence == PREC_SUM)   /* binary +/- precedence upgraded as unary */
+			/* binary +/- precedence upgraded as unary */
+			if (arginfo.precedence == PREC_SUM)
 			    arginfo.precedence = PREC_SIGN;
+			arginfo.kind = PP_UNARY;
+			break;
 		    case 2:
-			if (mainop.precedence == PREC_COMPARE &&
-			    arginfo.precedence == PREC_COMPARE)
-		          return TRUE;     /*   a < b < c   is not legal syntax */
 			break;
 		    default:
 			return FALSE;
 		    }
+		default:
+		    break;
+		}
+
+		switch(arginfo.kind) {
 		case PP_SUBSET:
-		    if (mainop.kind == PP_DOLLAR)
-		    	return FALSE;
-		    /* fall through, don't break... */
+		    switch (mainop.kind) {
+		    case PP_DOLLAR:
+		    case PP_SUBSET:
+			if (mainop.precedence > arginfo.precedence)
+			    return FALSE;
+			/* else fall through */
+		    default:
+			break;
+		    }
+		case PP_BINARY:
+		case PP_BINARY2:
+		    if (mainop.precedence == PREC_COMPARE &&
+			arginfo.precedence == PREC_COMPARE)
+			return TRUE;     /*   a < b < c   is not legal syntax */
+		    /* else fall through */
 		case PP_ASSIGN:
 		case PP_ASSIGN2:
-		case PP_UNARY:
 		case PP_DOLLAR:
-		    /* Same as other unary operators above */
-		    if (arginfo.precedence == PREC_NOT && !left)
-			return FALSE;
 		    if (mainop.precedence > arginfo.precedence
 			|| (mainop.precedence == arginfo.precedence && left == mainop.rightassoc)) {
 			return TRUE;
 		    }
 		    break;
+		case PP_UNARY:
+		    return (left && mainop.precedence > arginfo.precedence)
+			|| (deepLeft && deepLeft > arginfo.precedence);
 		case PP_FOR:
 		case PP_IF:
 		case PP_WHILE:
 		case PP_REPEAT:
-		    return left == 1;
-		    break;
+		    return left || deepLeft;
 		default:
 		    return FALSE;
 		}
@@ -752,19 +780,19 @@ static void attr2(SEXP s, LocalParseData *d, Rboolean not_names)
 	   !(TAG(a) == R_NamesSymbol && not_names)) {
 	    print2buff(", ", d);
 	    if(TAG(a) == R_DimSymbol) {
-		print2buff(".Dim", d);
+		print2buff("dim", d); // was .Dim
 	    }
 	    else if(TAG(a) == R_DimNamesSymbol) {
-		print2buff(".Dimnames", d);
+		print2buff("dimnames", d); // was .Dimnames
 	    }
 	    else if(TAG(a) == R_NamesSymbol) {
-		print2buff(".Names", d);
+		print2buff("names", d); // was .Names
 	    }
 	    else if(TAG(a) == R_TspSymbol) {
-		print2buff(".Tsp", d);
+		print2buff("tsp", d); // was .Tsp
 	    }
 	    else if(TAG(a) == R_LevelsSymbol) {
-		print2buff(".Label", d);
+		print2buff("levels", d); // was .Label
 	    }
 	    else {
 		/* TAG(a) might contain spaces etc */
@@ -853,10 +881,14 @@ static void deparse2buff(SEXP s, LocalParseData *d)
 
     d->fnarg = FALSE;
 
+    /* This flag should only be set when recursing through the LHS
+       of binary ops, so by default we reset to zero */
+    int prevLeft = d->left;
+    d->left = 0;
+
     if (!d->active) return;
 
-    Rboolean hasS4_t = TYPEOF(s) == S4SXP;
-    if (IS_S4_OBJECT(s) || hasS4_t) {
+    if (IS_S4_OBJECT(s)) {
 	d->isS4 = TRUE;
 	/* const void *vmax = vmaxget(); */
 	SEXP class = getAttrib(s, R_ClassSymbol),
@@ -879,6 +911,7 @@ static void deparse2buff(SEXP s, LocalParseData *d)
 	    UNPROTECT(2); // (e, cl_def)
 	    int n;
 	    Rboolean has_Data = FALSE;// does it have ".Data" slot?
+	    Rboolean hasS4_t = TYPEOF(s) == S4SXP;
 	    if(TYPEOF(slotNms) == STRSXP && (n = LENGTH(slotNms))) {
 		PROTECT(slotNms);
 		SEXP slotlist = PROTECT(allocVector(VECSXP, n));
@@ -1081,6 +1114,7 @@ static void deparse2buff(SEXP s, LocalParseData *d)
 		d->opts &= ~QUOTEEXPRESSIONS;
 	    }
 	}
+
 	if (TYPEOF(op) == SYMSXP) {
 	    int userbinop = 0;
 	    if ((TYPEOF(SYMVALUE(op)) == BUILTINSXP) ||
@@ -1100,7 +1134,9 @@ static void deparse2buff(SEXP s, LocalParseData *d)
 			fop.kind = PP_FUNCALL;
 		} else
 		    fop = PPINFO(SYMVALUE(op));
-		if (fop.kind == PP_BINARY) {
+
+		switch (fop.kind) {
+		case PP_BINARY:
 		    switch (length(s)) {
 		    case 1:
 			fop.kind = PP_UNARY;
@@ -1114,12 +1150,26 @@ static void deparse2buff(SEXP s, LocalParseData *d)
 			fop.kind = PP_FUNCALL;
 			break;
 		    }
-		}
-		else if (fop.kind == PP_BINARY2) {
+		    break;
+		case PP_BINARY2:
 		    if (length(s) != 2)
 			fop.kind = PP_FUNCALL;
 		    else if (userbinop)
 			fop.kind = PP_BINARY;
+		    break;
+		case PP_DOLLAR: {
+		    if (length(s) != 2) {
+			fop.kind = PP_FUNCALL;
+			break;
+		    }
+		    SEXP rhs = CADR(s);
+		    if (TYPEOF(rhs) != SYMSXP && !(isValidString(rhs)
+						   && STRING_ELT(rhs, 0) != NA_STRING))
+			fop.kind = PP_FUNCALL;
+		    break;
+		}
+		default:
+		    break;
 		}
 		switch (fop.kind) {
 		case PP_IF:
@@ -1128,7 +1178,7 @@ static void deparse2buff(SEXP s, LocalParseData *d)
 		    deparse2buff(CAR(s), d);
 		    print2buff(") ", d);
 		    if (d->incurly && !d->inlist ) {
-			lookahead = curlyahead(CAR(CDR(s)));
+			lookahead = curlyahead(CADR(s));
 			if (!lookahead) {
 			    writeline(d);
 			    d->indent++;
@@ -1136,7 +1186,7 @@ static void deparse2buff(SEXP s, LocalParseData *d)
 		    }
 		    /* need to find out if there is an else */
 		    if (length(s) > 2) {
-			deparse2buff(CAR(CDR(s)), d);
+			deparse2buff(CADR(s), d);
 			if (d->incurly && !d->inlist) {
 			    writeline(d);
 			    if (!lookahead)
@@ -1145,10 +1195,10 @@ static void deparse2buff(SEXP s, LocalParseData *d)
 			else
 			    print2buff(" ", d);
 			print2buff("else ", d);
-			deparse2buff(CAR(CDDR(s)), d);
+			deparse2buff(CADDR(s), d);
 		    }
 		    else {
-			deparse2buff(CAR(CDR(s)), d);
+			deparse2buff(CADR(s), d);
 			if (d->incurly && !lookahead && !d->inlist )
 			    d->indent--;
 		    }
@@ -1165,7 +1215,7 @@ static void deparse2buff(SEXP s, LocalParseData *d)
 		    print2buff(" in ", d);
 		    deparse2buff(CADR(s), d);
 		    print2buff(") ", d);
-		    deparse2buff(CADR(CDR(s)), d);
+		    deparse2buff(CADDR(s), d);
 		    break;
 		case PP_REPEAT:
 		    print2buff("repeat ", d);
@@ -1191,7 +1241,7 @@ static void deparse2buff(SEXP s, LocalParseData *d)
 		    print2buff(")", d);
 		    break;
 		case PP_SUBSET:
-		    if ((parens = needsparens(fop, CAR(s), 1)))
+		    if ((parens = needsparens(fop, CAR(s), 1, prevLeft)))
 			print2buff("(", d);
 		    deparse2buff(CAR(s), d);
 		    if (parens)
@@ -1249,26 +1299,30 @@ static void deparse2buff(SEXP s, LocalParseData *d)
 		    Rboolean outerparens = fnarg && !strcmp(CHAR(PRINTNAME(op)), "=");
 		    if (outerparens)
 		    	print2buff("(", d);
-		    if ((parens = needsparens(fop, CAR(s), 1)))
+		    if ((parens = needsparens(fop, CAR(s), 1, prevLeft)))
 			print2buff("(", d);
+		    d->left = parens ? 0 : fop.precedence;
 		    deparse2buff(CAR(s), d);
 		    if (parens)
 			print2buff(")", d);
 		    print2buff(" ", d);
 		    print2buff(CHAR(PRINTNAME(op)), d); /* ASCII */
 		    print2buff(" ", d);
-		    if ((parens = needsparens(fop, CADR(s), 0)))
+		    if ((parens = needsparens(fop, CADR(s), 0, prevLeft)))
 			print2buff("(", d);
+		    d->left = parens ? 0 : prevLeft;
 		    deparse2buff(CADR(s), d);
 		    if (parens)
 			print2buff(")", d);
 		    if (outerparens)
 		    	print2buff(")", d);
+		    d->left = 0;
 		    break;
 		}
 		case PP_DOLLAR:
-		    if ((parens = needsparens(fop, CAR(s), 1)))
+		    if ((parens = needsparens(fop, CAR(s), 1, prevLeft)))
 			print2buff("(", d);
+		    d->left = parens ? 0 : fop.precedence;
 		    deparse2buff(CAR(s), d);
 		    if (parens)
 			print2buff(")", d);
@@ -1278,16 +1332,19 @@ static void deparse2buff(SEXP s, LocalParseData *d)
 			isValidName(CHAR(STRING_ELT(CADR(s), 0))))
 			deparse2buff(STRING_ELT(CADR(s), 0), d);
 		    else {
-			if ((parens = needsparens(fop, CADR(s), 0)))
+			if ((parens = needsparens(fop, CADR(s), 0, prevLeft)))
 			    print2buff("(", d);
+			d->left = parens ? 0 : prevLeft;
 			deparse2buff(CADR(s), d);
 			if (parens)
 			    print2buff(")", d);
 		    }
+		    d->left = 0;
 		    break;
 		case PP_BINARY:
-		    if ((parens = needsparens(fop, CAR(s), 1)))
+		    if ((parens = needsparens(fop, CAR(s), 1, prevLeft)))
 			print2buff("(", d);
+		    d->left = parens ? 0 : fop.precedence;
 		    deparse2buff(CAR(s), d);
 		    if (parens)
 			print2buff(")", d);
@@ -1295,8 +1352,10 @@ static void deparse2buff(SEXP s, LocalParseData *d)
 		    print2buff(CHAR(PRINTNAME(op)), d); /* ASCII */
 		    print2buff(" ", d);
 		    linebreak(&lbreak, d);
-		    if ((parens = needsparens(fop, CADR(s), 0)))
+
+		    if ((parens = needsparens(fop, CADR(s), 0, prevLeft)))
 			print2buff("(", d);
+		    d->left = parens ? 0 : prevLeft;
 		    deparse2buff(CADR(s), d);
 		    if (parens)
 			print2buff(")", d);
@@ -1304,27 +1363,34 @@ static void deparse2buff(SEXP s, LocalParseData *d)
 			d->indent--;
 			lbreak = FALSE;
 		    }
+		    d->left = 0;
 		    break;
 		case PP_BINARY2:	/* no space between op and args */
-		    if ((parens = needsparens(fop, CAR(s), 1)))
+		    if ((parens = needsparens(fop, CAR(s), 1, prevLeft)))
 			print2buff("(", d);
+		    d->left = parens ? 0 : fop.precedence;
 		    deparse2buff(CAR(s), d);
 		    if (parens)
 			print2buff(")", d);
+
 		    print2buff(CHAR(PRINTNAME(op)), d); /* ASCII */
-		    if ((parens = needsparens(fop, CADR(s), 0)))
+		    if ((parens = needsparens(fop, CADR(s), 0, prevLeft)))
 			print2buff("(", d);
+		    d->left = parens ? 0 : prevLeft;
 		    deparse2buff(CADR(s), d);
 		    if (parens)
 			print2buff(")", d);
+		    d->left = 0;
 		    break;
 		case PP_UNARY:
 		    print2buff(CHAR(PRINTNAME(op)), d); /* ASCII */
-		    if ((parens = needsparens(fop, CAR(s), 0)))
+		    if ((parens = needsparens(fop, CAR(s), 0, prevLeft)))
 			print2buff("(", d);
+		    d->left = parens ? 0 : prevLeft;
 		    deparse2buff(CAR(s), d);
 		    if (parens)
 			print2buff(")", d);
+		    d->left = 0;
 		    break;
 		case PP_BREAK:
 		    print2buff("break", d);
@@ -1448,6 +1514,8 @@ static void deparse2buff(SEXP s, LocalParseData *d)
 	d->sourceable = FALSE;
 	UNIMPLEMENTED_TYPE("deparse2buff", s);
     }
+
+    d->left = prevLeft;
 }
 
 
