@@ -1,7 +1,7 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
  *  Copyright (C) 1995, 1996  Robert Gentleman and Ross Ihaka
- *  Copyright (C) 1998-2019   The R Core Team.
+ *  Copyright (C) 1998-2020   The R Core Team.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -42,7 +42,7 @@
 #include <config.h>
 #endif
 
-#include "Defn.h"
+#include <Defn.h>
 
 
 /* used in subscript.c and subassign.c */
@@ -141,7 +141,7 @@ static SEXP matchPar_int(const char *tag, SEXP *list, Rboolean exact)
 }
 
 /* unused outside this file */
-SEXP attribute_hidden matchPar(const char *tag, SEXP * list)
+attribute_hidden SEXP matchPar(const char *tag, SEXP * list)
 {
     return matchPar_int(tag, list, FALSE);
 }
@@ -152,7 +152,7 @@ SEXP attribute_hidden matchPar(const char *tag, SEXP * list)
 /* Returns the first partially matching tag found. */
 /* Pattern is a symbol. */
 
-SEXP attribute_hidden matchArg(SEXP tag, SEXP * list)
+attribute_hidden SEXP matchArg(SEXP tag, SEXP * list)
 {
     return matchPar(CHAR(PRINTNAME(tag)), list);
 }
@@ -162,7 +162,7 @@ SEXP attribute_hidden matchArg(SEXP tag, SEXP * list)
 /* Returns the first exactly matching tag found. */
 /* Pattern is a symbol. */
 
-SEXP attribute_hidden matchArgExact(SEXP tag, SEXP * list)
+attribute_hidden SEXP matchArgExact(SEXP tag, SEXP * list)
 {
       return matchPar_int(CHAR(PRINTNAME(tag)), list, TRUE);
 }
@@ -181,7 +181,7 @@ SEXP attribute_hidden matchArgExact(SEXP tag, SEXP * list)
 /* Renamed to matchArgs_NR to reflect that it returns a
    non-reference-tracking list */
 
-SEXP attribute_hidden matchArgs_NR(SEXP formals, SEXP supplied, SEXP call)
+attribute_hidden SEXP matchArgs_NR(SEXP formals, SEXP supplied, SEXP call)
 {
     Rboolean seendots;
     int i, arg_i = 0;
@@ -353,42 +353,29 @@ SEXP attribute_hidden matchArgs_NR(SEXP formals, SEXP supplied, SEXP call)
 	}
     } else {
 	/* Check that all arguments are used */
-	SEXP unused = R_NilValue, last = R_NilValue;
-	for (b = supplied; b != R_NilValue; b = CDR(b))
-	    if (!ARGUSED(b)) {
-		if(last == R_NilValue) {
-		    PROTECT(unused = CONS(CAR(b), R_NilValue));
-		    SET_TAG(unused, TAG(b));
-		    last = unused;
-		} else {
-		    SETCDR(last, CONS(CAR(b), R_NilValue));
+	for (b = supplied; b != R_NilValue && ARGUSED(b); b = CDR(b));
+
+	if (b != R_NilValue) {
+            /* show bad arguments in call without evaluating them */
+	    SEXP carB = CAR(b);
+	    if (TYPEOF(carB) == PROMSXP) carB = PREXPR(carB);
+	    SEXP unused = PROTECT(CONS(carB, R_NilValue));
+	    SET_TAG(unused, TAG(b));
+	    SEXP last = unused;
+
+            for(b = CDR(b) ; b != R_NilValue ; b = CDR(b))
+		if (!ARGUSED(b)) {
+		    SEXP carB = CAR(b);
+		    if (TYPEOF(carB) == PROMSXP) carB = PREXPR(carB);
+		    SETCDR(last, CONS(carB, R_NilValue));
 		    last = CDR(last);
 		    SET_TAG(last, TAG(b));
 		}
-	    }
-
-	if(last != R_NilValue) {
-            /* show bad arguments in call without evaluating them */
-            SEXP unusedForError = R_NilValue, last = R_NilValue;
-
-            for(b = unused ; b != R_NilValue ; b = CDR(b)) {
-                SEXP tagB = TAG(b), carB = CAR(b) ;
-                if (TYPEOF(carB) == PROMSXP) carB = PREXPR(carB) ;
-                if (last == R_NilValue) {
-                    PROTECT(last = CONS(carB, R_NilValue));
-                    SET_TAG(last, tagB);
-                    unusedForError = last;
-                } else {
-                    SETCDR(last, CONS(carB, R_NilValue));
-                    last = CDR(last);
-                    SET_TAG(last, tagB);
-                }
-            }
 	    errorcall(call /* R_GlobalContext->call */,
 		      ngettext("unused argument %s",
 			       "unused arguments %s",
-			       (unsigned long) length(unusedForError)),
-		      strchr(CHAR(asChar(deparse1line(unusedForError, 0))), '('));
+			       (unsigned long) length(unused)),
+		      strchr(CHAR(asChar(deparse1line(unused, 0))), '('));
 	}
     }
     UNPROTECT(1);
@@ -396,7 +383,7 @@ SEXP attribute_hidden matchArgs_NR(SEXP formals, SEXP supplied, SEXP call)
 }
 
 /* Use matchArgs_RC if the result might escape into R. */
-SEXP attribute_hidden matchArgs_RC(SEXP formals, SEXP supplied, SEXP call)
+attribute_hidden SEXP matchArgs_RC(SEXP formals, SEXP supplied, SEXP call)
 {
     SEXP args = matchArgs_NR(formals, supplied, call);
     /* it would be better not to build this arglist with CONS_NR in
@@ -446,7 +433,7 @@ void patchArgument(SEXP suppliedSlot, SEXP name, fstype_t *farg, SEXP cloenv) {
     SETCAR(suppliedSlot, mkPROMISE(name, cloenv));
 }
 
-SEXP attribute_hidden
+attribute_hidden SEXP
 patchArgsByActuals(SEXP formals, SEXP supplied, SEXP cloenv)
 {
     int i, seendots, farg_i;
