@@ -1,8 +1,8 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
+ *  Copyright (C) 1999-2023  The R Core Team.
+ *  Copyright (C) 2002-2023  The R Foundation
  *  Copyright (C) 1995, 1996  Robert Gentleman and Ross Ihaka
- *  Copyright (C) 2002-2017  The R Foundation
- *  Copyright (C) 1999-2018  The R Core Team.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -29,7 +29,7 @@
 #define R_USE_SIGNALS 1
 #include <Defn.h>
 #include <Internal.h>
-#include <R_ext/RS.h> /* for Calloc, Realloc and for S4 object bit */
+#include <R_ext/RS.h> /* for R_Calloc, R_Realloc and for S4 object bit */
 
 static SEXP GetObject(RCNTXT *cptr)
 {
@@ -233,9 +233,6 @@ SEXP R_LookupMethod(SEXP method, SEXP rho, SEXP callrho, SEXP defrho)
 {
     SEXP val, top = R_NilValue;	/* -Wall */
     static SEXP s_S3MethodsTable = NULL;
-    static int lookup_baseenv_after_globalenv = -1;
-    static int lookup_report_search_path_uses = -1;
-    char *lookup;
     PROTECT_INDEX validx;
 
     if (TYPEOF(callrho) != ENVSXP) {
@@ -253,18 +250,6 @@ SEXP R_LookupMethod(SEXP method, SEXP rho, SEXP callrho, SEXP defrho)
 	    error(_("bad generic definition environment"));
     }
 
-    if(lookup_baseenv_after_globalenv == -1) {
-	lookup = getenv("_R_S3_METHOD_LOOKUP_BASEENV_AFTER_GLOBALENV_");
-	lookup_baseenv_after_globalenv = 
-	    ((lookup != NULL) && StringFalse(lookup)) ? 0 : 1;
-    }
-
-    if(lookup_report_search_path_uses == -1) {
-	lookup = getenv("_R_S3_METHOD_LOOKUP_REPORT_SEARCH_PATH_USES_");
-	lookup_report_search_path_uses = 
-	    ((lookup != NULL) && StringTrue(lookup)) ? 1 : 0;
-    }
-
     /* This evaluates promises */
     PROTECT(top = topenv(R_NilValue, callrho));
     val = findFunInEnvRange(method, callrho, top);
@@ -295,34 +280,12 @@ SEXP R_LookupMethod(SEXP method, SEXP rho, SEXP callrho, SEXP defrho)
 	}
     } 
 
-    if(lookup_baseenv_after_globalenv) {
-	if (top == R_GlobalEnv)
-	    top = R_BaseEnv;
-	else
-	    top = ENCLOS(top);
-	REPROTECT(val = findFunWithBaseEnvAfterGlobalEnv(method, top),
-	          validx);
-    }
-    else if(lookup_report_search_path_uses) {
-	if(top != R_GlobalEnv) 
-	    REPROTECT(val = findFunInEnvRange(method, ENCLOS(top),
-	                                      R_GlobalEnv), validx);
-	if(val == R_UnboundValue) {
-	    REPROTECT(val = findFunInEnvRange(method, ENCLOS(R_GlobalEnv),
-	                                      R_EmptyEnv), validx);
-	    if((val != R_UnboundValue) && 
-	       (CLOENV(val) != R_BaseNamespace) &&
-	       (CLOENV(val) != R_BaseEnv)) {
-		/* Note that we do not really know where on the search
-		   path we found the method. */
-		REprintf("S3 method lookup found '%s' on search path \n",
-			 CHAR(PRINTNAME(method)));
-	    }
-	}
-    }
+    if (top == R_GlobalEnv)
+	top = R_BaseEnv;
     else
-	REPROTECT(val = findFunInEnvRange(method, ENCLOS(top), R_EmptyEnv),
-	          validx);
+	top = ENCLOS(top);
+    REPROTECT(val = findFunWithBaseEnvAfterGlobalEnv(method, top),
+	      validx);
 
     UNPROTECT(2); /* top, val */
     return val;
@@ -499,42 +462,29 @@ int usemethod(const char *generic, SEXP obj, SEXP call, SEXP args,
 */
 
 /* This is a primitive SPECIALSXP */
-SEXP attribute_hidden NORET do_usemethod(SEXP call, SEXP op, SEXP args, SEXP env)
+attribute_hidden NORET
+SEXP do_usemethod(SEXP call, SEXP op, SEXP args, SEXP env)
 {
-    SEXP ans, generic = R_NilValue /* -Wall */, obj, val;
-    SEXP callenv, defenv;
-    SEXP argList;
-    RCNTXT *cptr;
     static SEXP do_usemethod_formals = NULL;
-
-    static int lookup_use_topenv_as_defenv = -1;
-    char *lookup;
-
     if (do_usemethod_formals == NULL)
 	do_usemethod_formals = allocFormalsList2(install("generic"),
 						 install("object"));
 
-    PROTECT(argList = matchArgs_NR(do_usemethod_formals, args, call));
+    SEXP argList = PROTECT(matchArgs_NR(do_usemethod_formals, args, call));
     if (CAR(argList) == R_MissingArg)
 	errorcall(call, _("there must be a 'generic' argument"));
-    else
-	PROTECT(generic = eval(CAR(argList), env));
+    // else
+    SEXP generic = PROTECT(eval(CAR(argList), env));
     if(!isString(generic) || LENGTH(generic) != 1)
 	errorcall(call, _("'generic' argument must be a character string"));
 
-    if(lookup_use_topenv_as_defenv == -1) {
-	lookup = getenv("_R_S3_METHOD_LOOKUP_USE_TOPENV_AS_DEFENV_");
-	lookup_use_topenv_as_defenv = 
-	    ((lookup != NULL) && StringFalse(lookup)) ? 0 : 1;
-    }
-
     /* get environments needed for dispatching.
        callenv = environment from which the generic was called
        defenv = environment where the generic was defined */
-    cptr = R_GlobalContext;
+    RCNTXT *cptr = R_GlobalContext;
     if ( !(cptr->callflag & CTXT_FUNCTION) || cptr->cloenv != env)
 	errorcall(call, _("'UseMethod' used in an inappropriate fashion"));
-    callenv = cptr->sysparent;
+    SEXP callenv = cptr->sysparent;
     /* We need to find the generic to find out where it is defined.
        This is set up to avoid getting caught by things like
 
@@ -547,41 +497,48 @@ SEXP attribute_hidden NORET do_usemethod(SEXP call, SEXP op, SEXP args, SEXP env
 	The generic need not be a closure (Henrik Bengtsson writes
 	UseMethod("$"), although only functions are documented.)
     */
-    if(lookup_use_topenv_as_defenv) {
-	defenv = topenv(R_NilValue, env);
-    } else {
-	val = findVar1(installTrChar(STRING_ELT(generic, 0)),
-		       ENCLOS(env), FUNSXP, TRUE); /* That has evaluated
-						    * promises */
-	if(TYPEOF(val) == CLOSXP) defenv = CLOENV(val);
-	else defenv = R_BaseNamespace;
-    }
-
-    if (CADR(argList) != R_MissingArg)
-	PROTECT(obj = eval(CADR(argList), env));
-    else
-	PROTECT(obj = GetObject(cptr));
-
+    SEXP defenv = topenv(R_NilValue, env);
+    SEXP obj = PROTECT((CADR(argList) != R_MissingArg)
+		       ? eval(CADR(argList), env)
+		       : GetObject(cptr));
+    SEXP ans;
     if (usemethod(translateChar(STRING_ELT(generic, 0)), obj, call, CDR(args),
 		  env, callenv, defenv, &ans) == 1) {
 	UNPROTECT(3); /* obj, generic, argList */
 	findcontext(CTXT_RETURN, env, ans); /* does not return */
     }
-    else {
-	SEXP klass;
-	int nclass;
-	char cl[1000];
-	PROTECT(klass = R_data_class2(obj));
-	nclass = length(klass);
-	if (nclass == 1)
-	    strcpy(cl, translateChar(STRING_ELT(klass, 0)));
-	else {
+    else { // no method found
+#define CLLEN 1024
+	char cl[CLLEN] = {0};
+	SEXP klass = PROTECT(R_data_class2(obj));
+	int nclass = length(klass);
+	if(!nclass)
+	    errorcall(call, _("illegal class of length 0 when using method for '%s'"),
+		      translateChar(STRING_ELT(generic, 0)));
+	const char *c_name = translateChar(STRING_ELT(klass, 0));
+	if (nclass == 1) {
+	    strncpy(cl, c_name, CLLEN-1); cl[CLLEN-1] = '\0';
+	} else { // nclass >= 2
+	    size_t cl_count = 3; // 3 : "c(" + \0
 	    strcpy(cl, "c('");
 	    for (int i = 0; i < nclass; i++) {
-		if (i > 0) strcat(cl, "', '");
-		strcat(cl, translateChar(STRING_ELT(klass, i)));
+		if(i > 0) {
+		    c_name = translateChar(STRING_ELT(klass, i));
+		    cl_count += 4; // "', '"
+		    if (cl_count < CLLEN) strcat(cl, "', '");
+		}
+		size_t len_nm = strlen(c_name);
+		if (cl_count + len_nm >= CLLEN) {
+		    cl_count += 2;
+		    if(cl_count < CLLEN) strcat(cl, "..");
+		    break;
+		}
+		// else
+		cl_count += len_nm;
+		strcat(cl, c_name);
 	    }
-	    strcat(cl, "')");
+	    if (++cl_count < CLLEN) strcat(cl, "'");
+	    if (++cl_count < CLLEN) strcat(cl, ")");
 	}
 	errorcall(call, _("no applicable method for '%s' applied to an object of class \"%s\""),
 		  translateChar(STRING_ELT(generic, 0)), cl);
@@ -671,7 +628,7 @@ static R_INLINE SEXP getPrimitive(SEXP symbol)
 
 
 /* This is a special .Internal */
-SEXP attribute_hidden do_nextmethod(SEXP call, SEXP op, SEXP args, SEXP env)
+attribute_hidden SEXP do_nextmethod(SEXP call, SEXP op, SEXP args, SEXP env)
 {
     const char *sb, *sg, *sk;
     SEXP ans, s, t, klass, method, matchedarg, generic;
@@ -919,7 +876,7 @@ SEXP attribute_hidden do_nextmethod(SEXP call, SEXP op, SEXP args, SEXP env)
 }
 
 /* primitive */
-SEXP attribute_hidden do_unclass(SEXP call, SEXP op, SEXP args, SEXP env)
+attribute_hidden SEXP do_unclass(SEXP call, SEXP op, SEXP args, SEXP env)
 {
     checkArity(op, args);
     check1arg(args, call, "x");
@@ -991,7 +948,8 @@ static SEXP inherits3(SEXP x, SEXP what, SEXP which)
 	PROTECT(klass = R_data_class(x, FALSE));
 
     if(!isString(what))
-	error(_("'what' must be a character vector"));
+	error(_("'what' must be a character vector "
+		"or an object with a nameOfClass() method"));
     int j, nwhat = LENGTH(what);
 
     if( !isLogical(which) || (LENGTH(which) != 1) )
@@ -1021,13 +979,46 @@ static SEXP inherits3(SEXP x, SEXP what, SEXP which)
     return rval;
 }
 
-SEXP attribute_hidden do_inherits(SEXP call, SEXP op, SEXP args, SEXP env)
+static R_INLINE SEXP nameOfClass(SEXP what, SEXP env)
+{
+    static SEXP call = NULL;
+    static SEXP Xsym = NULL;
+    if (call == NULL) {
+	Xsym = install("X");
+	call = R_ParseString("base::nameOfClass(X)");
+	R_PreserveObject(call);
+    }
+
+    SEXP callenv = PROTECT(R_NewEnv(env, FALSE, 0));
+    defineVar(Xsym, what, callenv);
+    INCREMENT_NAMED(what);
+    SEXP name = eval(call, callenv);
+    /* could remove 'what' from callenv to drop REFCNT */
+    UNPROTECT(1); /* callenv */
+
+    return name;
+}
+
+attribute_hidden SEXP do_inherits(SEXP call, SEXP op, SEXP args, SEXP env)
 {
     checkArity(op, args);
 
-    return inherits3(/* x = */ CAR(args),
-		     /* what = */ CADR(args),
-		     /* which = */ CADDR(args));
+    SEXP x = CAR(args);
+    SEXP what = CADR(args);
+    SEXP which = CADDR(args);
+
+    if (OBJECT(what) && TYPEOF(what) != STRSXP) {
+	SEXP name = nameOfClass(what, env);
+	if (name != R_NilValue) {
+	    PROTECT(name);
+	    SEXP val = inherits3(x, name, which);
+	    UNPROTECT(1); /* name */
+	    return val;
+	}
+	/* fall through */
+    }
+
+    return inherits3(x, what, which);
 }
 
 
@@ -1052,20 +1043,35 @@ SEXP attribute_hidden do_inherits(SEXP call, SEXP op, SEXP args, SEXP env)
 int R_check_class_and_super(SEXP x, const char **valid, SEXP rho)
 {
     int ans;
-    SEXP cl = PROTECT(asChar(getAttrib(x, R_ClassSymbol)));
+    SEXP clattr = getAttrib(x, R_ClassSymbol);
+    SEXP cl = PROTECT(asChar(clattr));
     const char *class = CHAR(cl);
-    for (ans = 0; ; ans++) {
-	if (!strlen(valid[ans])) // empty string
-	    break;
+    for (ans = 0; strlen(valid[ans]); ans++)
 	if (!strcmp(class, valid[ans])) {
 	    UNPROTECT(1); /* cl */
 	    return ans;
 	}
-    }
-    /* if not found directly, now search the non-virtual super classes :*/
+    /* if not found directly, then look for a match among the nonvirtual 
+       superclasses, possibly after finding the environment 'rho' in which
+       class(x) is defined */
     if(IS_S4_OBJECT(x)) {
-	/* now try the superclasses, i.e.,  try   is(x, "....");  superCl :=
-	   .selectSuperClasses(getClass("....")@contains, dropVirtual=TRUE)  */
+	if (rho == NULL) {
+	    SEXP pkg = getAttrib(clattr, R_PackageSymbol);
+	    if (!isNull(pkg)) {
+		static SEXP meth_classEnv = NULL;
+		if(!meth_classEnv)
+		    meth_classEnv = install(".classEnv");
+		/* FIXME: fails if 'methods' is not loaded */
+		SEXP clEnvCall = PROTECT(lang2(meth_classEnv, clattr));
+		rho = eval(clEnvCall, R_MethodsNamespace);
+		UNPROTECT(1); /* clEnvCall */
+		if(!isEnvironment(rho))
+		    error(_("could not find correct environment; "
+			    "please report!"));
+	    } else
+		rho = R_GlobalEnv;
+	}
+	PROTECT(rho);
 	SEXP classExts, superCl, _call;
 	static SEXP s_contains = NULL, s_selectSuperCl = NULL;
 	if(!s_contains) {
@@ -1074,32 +1080,30 @@ int R_check_class_and_super(SEXP x, const char **valid, SEXP rho)
 	}
 	SEXP classDef = PROTECT(R_getClassDef(class));
 	PROTECT(classExts = R_do_slot(classDef, s_contains));
-	/* .selectSuperClasses(getClassDef(class)@contains, dropVirtual = TRUE,
-	 *                     namesOnly = TRUE, directOnly = FALSE, simpleOnly = TRUE) :
+	/* .selectSuperClasses(getClassDef(class)@contains, 
+	 *                     dropVirtual = TRUE, namesOnly = TRUE, 
+	 *                     directOnly = FALSE, simpleOnly = TRUE):
 	 */
-	PROTECT(_call = lang6(s_selectSuperCl, classExts, ScalarLogical(1),
-			      ScalarLogical(1), ScalarLogical(0), ScalarLogical(1)));
+	PROTECT(_call = lang6(s_selectSuperCl, classExts,
+			      ScalarLogical(1), ScalarLogical(1),
+			      ScalarLogical(0), ScalarLogical(1)));
 	superCl = eval(_call, rho);
 	UNPROTECT(3); /* _call, classExts, classDef */
 	PROTECT(superCl);
-	for(int i=0; i < LENGTH(superCl); i++) {
+	for(int i = 0; i < LENGTH(superCl); i++) {
 	    const char *s_class = CHAR(STRING_ELT(superCl, i));
-	    for (ans = 0; ; ans++) {
-		if (!strlen(valid[ans]))
-		    break;
+	    for (ans = 0; strlen(valid[ans]); ans++)
 		if (!strcmp(s_class, valid[ans])) {
-		    UNPROTECT(2); /* superCl, cl */
+		    UNPROTECT(3); /* superCl, rho, cl */
 		    return ans;
 		}
-	    }
 	}
-	UNPROTECT(1); /* superCl */
+	UNPROTECT(2); /* superCl, rho */
     }
     UNPROTECT(1); /* cl */
     return -1;
 }
 
-
 /**
  * Return the 0-based index of an is() match in a vector of class-name
  * strings terminated by an empty string.  Returns -1 for no match.
@@ -1113,25 +1117,7 @@ int R_check_class_and_super(SEXP x, const char **valid, SEXP rho)
  */
 int R_check_class_etc(SEXP x, const char **valid)
 {
-    static SEXP meth_classEnv = NULL;
-    SEXP cl = getAttrib(x, R_ClassSymbol), rho = R_GlobalEnv, pkg;
-    if(!meth_classEnv)
-	meth_classEnv = install(".classEnv");
-
-    pkg = getAttrib(cl, R_PackageSymbol); /* ==R== packageSlot(class(x)) */
-    if(!isNull(pkg)) { /* find  rho := correct class Environment */
-	SEXP clEnvCall;
-	// FIXME: fails if 'methods' is not loaded.
-	PROTECT(clEnvCall = lang2(meth_classEnv, cl));
-	rho = eval(clEnvCall, R_MethodsNamespace);
-	UNPROTECT(1);
-	if(!isEnvironment(rho))
-	    error(_("could not find correct environment; please report!"));
-    }
-    PROTECT(rho);
-    int res = R_check_class_and_super(x, valid, rho);
-    UNPROTECT(1);
-    return res;
+    return R_check_class_and_super(x, valid, NULL);
 }
 
 /* standardGeneric:  uses a pointer to R_standardGeneric, to be
@@ -1258,7 +1244,7 @@ static SEXP dispatchNonGeneric(SEXP name, SEXP env, SEXP fdef)
 
 static SEXP get_this_generic(SEXP args);
 
-SEXP attribute_hidden do_standardGeneric(SEXP call, SEXP op, SEXP args, SEXP env)
+attribute_hidden SEXP do_standardGeneric(SEXP call, SEXP op, SEXP args, SEXP env)
 {
     SEXP arg, value, fdef; R_stdGen_ptr_t ptr = R_get_standardGeneric_ptr();
 
@@ -1399,11 +1385,11 @@ SEXP do_set_prim_method(SEXP op, const char *code_string, SEXP fundef,
 	if(prim_methods) {
 	    int i;
 
-	    prim_methods  = Realloc(prim_methods,  n, prim_methods_t);
-	    prim_generics = Realloc(prim_generics, n, SEXP);
-	    prim_mlist	  = Realloc(prim_mlist,	   n, SEXP);
+	    prim_methods  = R_Realloc(prim_methods,  n, prim_methods_t);
+	    prim_generics = R_Realloc(prim_generics, n, SEXP);
+	    prim_mlist	  = R_Realloc(prim_mlist,	   n, SEXP);
 
-	    /* Realloc does not clear the added memory, hence: */
+	    /* R_Realloc does not clear the added memory, hence: */
 	    for (i = maxMethodsOffset ; i < n ; i++) {
 		prim_methods[i]	 = NO_METHODS;
 		prim_generics[i] = NULL;
@@ -1411,9 +1397,9 @@ SEXP do_set_prim_method(SEXP op, const char *code_string, SEXP fundef,
 	    }
 	}
 	else {
-	    prim_methods  = Calloc(n, prim_methods_t);
-	    prim_generics = Calloc(n, SEXP);
-	    prim_mlist	  = Calloc(n, SEXP);
+	    prim_methods  = R_Calloc(n, prim_methods_t);
+	    prim_generics = R_Calloc(n, SEXP);
+	    prim_mlist	  = R_Calloc(n, SEXP);
 	}
 	maxMethodsOffset = n;
     }
@@ -1520,7 +1506,7 @@ Rboolean R_has_methods(SEXP op)
 
 static SEXP deferred_default_object;
 
-SEXP R_deferred_default_method()
+SEXP R_deferred_default_method(void)
 {
     if(!deferred_default_object)
 	deferred_default_object = install("__Deferred_Default_Marker__");
@@ -1544,7 +1530,7 @@ void R_set_quick_method_check(R_stdGen_ptr_t value)
    promises, but not from the other two: there all the arguments have
    already been evaluated.
  */
-SEXP attribute_hidden
+attribute_hidden SEXP
 R_possible_dispatch(SEXP call, SEXP op, SEXP args, SEXP rho,
 		    Rboolean promisedArgs)
 {
@@ -1586,7 +1572,7 @@ R_possible_dispatch(SEXP call, SEXP op, SEXP args, SEXP rho,
 		PROTECT(s = promiseArgs(CDR(call), rho));
 		if (length(s) != length(args)) error(_("dispatch error"));
 		for (a = args, b = s; a != R_NilValue; a = CDR(a), b = CDR(b))
-		    SET_PRVALUE(CAR(b), CAR(a));
+		    IF_PROMSXP_SET_PRVALUE(CAR(b), CAR(a));
 		value =  applyClosure(call, value, s, rho, suppliedvars);
 #ifdef ADJUST_ENVIR_REFCNTS
 		unpromiseArgs(s);
@@ -1616,7 +1602,7 @@ R_possible_dispatch(SEXP call, SEXP op, SEXP args, SEXP rho,
 	PROTECT(s = promiseArgs(CDR(call), rho));
 	if (length(s) != length(args)) error(_("dispatch error"));
 	for (a = args, b = s; a != R_NilValue; a = CDR(a), b = CDR(b))
-	    SET_PRVALUE(CAR(b), CAR(a));
+	    IF_PROMSXP_SET_PRVALUE(CAR(b), CAR(a));
 	value = applyClosure(call, fundef, s, rho, R_NilValue);
 	UNPROTECT(1);
     } else {
@@ -1745,7 +1731,7 @@ Rboolean attribute_hidden R_seemsOldStyleS4Object(SEXP object)
 	    getAttrib(klass, R_PackageSymbol) != R_NilValue) ? TRUE: FALSE;
 }
 
-SEXP attribute_hidden do_setS4Object(SEXP call, SEXP op, SEXP args, SEXP env)
+attribute_hidden SEXP do_setS4Object(SEXP call, SEXP op, SEXP args, SEXP env)
 {
     checkArity(op, args);
     SEXP object = CAR(args);
