@@ -1,7 +1,7 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
  *  Copyright (C) 1995, 1996  Robert Gentleman and Ross Ihaka
- *  Copyright (C) 1998-2020   The R Core Team.
+ *  Copyright (C) 1998-2023   The R Core Team.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -311,8 +311,14 @@ static int scanchar(Rboolean inQuote, LocalData *d)
 static void scan_cleanup(void *data)
 {
     LocalData *ld = data;
-    if(!ld->ttyflag && !ld->wasopen) ld->con->close(ld->con);
-    if (ld->quoteset[0]) free(ld->quoteset);
+    if(ld->con && !ld->ttyflag && !ld->wasopen) {
+	ld->con->close(ld->con);
+	ld->con = NULL;
+    }
+    if(ld->quoteset && ld->quoteset[0]) {
+	free(ld->quoteset);
+	ld->quoteset = NULL;
+    }
 }
 
 #include "RBufferUtils.h"
@@ -334,7 +340,7 @@ fillBuffer(SEXPTYPE type, int strip, int *bch, LocalData *d,
 */
     char *bufp;
     int c, quote, filled, nbuf = MAXELTSIZE, m, mm = 0;
-    Rboolean dbcslocale = (MB_CUR_MAX == 2) && !d->isUTF8 && !d->isLatin1;
+    Rboolean dbcslocale = (R_MB_CUR_MAX == 2) && !d->isUTF8 && !d->isLatin1;
 
     m = 0;
     filled = 1;
@@ -478,7 +484,7 @@ static R_INLINE int isNAstring(const char *buf, int mode, LocalData *d)
     return 0;
 }
 
-static R_INLINE void NORET expected(char *what, char *got, LocalData *d)
+NORET static R_INLINE void expected(char *what, char *got, LocalData *d)
 {
     int c;
     if (d->ttyflag) { /* This is safe in a MBCS */
@@ -673,7 +679,7 @@ static SEXP scanFrame(SEXP what, R_xlen_t maxitems, R_xlen_t maxlines,
     char *buffer = NULL;
     int c, strip, bch;
     R_xlen_t blksize, i, ii, j, n, nc, linesread, colsread;
-    R_xlen_t badline, nstring = 0;
+    R_xlen_t badline;
     R_StringBuffer buf = {NULL, 0, MAXELTSIZE};
 
     nc = xlength(what);
@@ -693,7 +699,6 @@ static SEXP scanFrame(SEXP what, R_xlen_t maxitems, R_xlen_t maxlines,
 	    if (!isVector(w)) {
 		error(_("invalid '%s' argument"), "what");
 	    }
-	    if(TYPEOF(w) == STRSXP) nstring++;
 	    SET_VECTOR_ELT(ans, i, allocVector(TYPEOF(w), blksize));
 	}
     }
@@ -833,7 +838,7 @@ static SEXP scanFrame(SEXP what, R_xlen_t maxitems, R_xlen_t maxlines,
     return ans;
 }
 
-SEXP attribute_hidden do_scan(SEXP call, SEXP op, SEXP args, SEXP rho)
+attribute_hidden SEXP do_scan(SEXP call, SEXP op, SEXP args, SEXP rho)
 {
     SEXP ans, file, sep, what, stripwhite, dec, quotes, comstr;
     int c, flush, fill, blskip, multiline, escapes, skipNul;
@@ -910,9 +915,16 @@ SEXP attribute_hidden do_scan(SEXP call, SEXP op, SEXP args, SEXP rho)
     else
 	error(_("invalid decimal separator"));
 
+    /* set up a context which will close the connection if there is
+       an error or user interrupt */
+    begincontext(&cntxt, CTXT_CCODE, R_GlobalContext->call, R_BaseEnv,
+		 R_BaseEnv, R_NilValue, R_NilValue);
+    cntxt.cend = &scan_cleanup;
+    cntxt.cenddata = &data;
+
     if (isString(quotes)) {
 	const char *sc = translateChar(STRING_ELT(quotes, 0));
-	if (strlen(sc)) data.quoteset = strdup(sc);
+	if (strlen(sc)) data.quoteset = Rstrdup(sc);
 	else data.quoteset = "";
     } else if (isNull(quotes))
 	data.quoteset = "";
@@ -953,20 +965,22 @@ SEXP attribute_hidden do_scan(SEXP call, SEXP op, SEXP args, SEXP rho)
 	    if(!data.con->canread)
 		error(_("cannot read from this connection"));
 	}
-	for (R_xlen_t i = 0; i < nskip; i++) /* MBCS-safe */
-	    while ((c = scanchar(FALSE, &data)) != '\n' && c != R_EOF);
+	for(R_xlen_t i = 0, j = 10000; i < nskip; i++) { /* MBCS-safe */
+	    for(;;) {
+		c = scanchar(FALSE, &data);
+		if (!j--) {
+		    R_CheckUserInterrupt();
+		    j = 10000;
+		}
+		if (c == '\n' || c == R_EOF)
+		    break;
+	    }
+	}
     }
 
     ans = R_NilValue;		/* -Wall */
     data.save = 0;
 
-    /* set up a context which will close the connection if there is
-       an error or user interrupt */
-    begincontext(&cntxt, CTXT_CCODE, R_GlobalContext->call, R_BaseEnv,
-		 R_BaseEnv, R_NilValue, R_NilValue);
-    cntxt.cend = &scan_cleanup;
-    cntxt.cenddata = &data;
-
     switch (TYPEOF(what)) {
     case LGLSXP:
     case INTSXP:
@@ -1005,7 +1019,7 @@ SEXP attribute_hidden do_scan(SEXP call, SEXP op, SEXP args, SEXP rho)
     return ans;
 }
 
-SEXP attribute_hidden do_readln(SEXP call, SEXP op, SEXP args, SEXP rho)
+attribute_hidden SEXP do_readln(SEXP call, SEXP op, SEXP args, SEXP rho)
 {
     int c;
     char buffer[MAXELTSIZE], *bufp = buffer;
