@@ -1,6 +1,6 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
- *  Copyright (C) 1995--2020  The R Core Team
+ *  Copyright (C) 1995--2022  The R Core Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -30,7 +30,7 @@
 #include <Fileio.h>
 #include <Rversion.h>
 #include <R_ext/Riconv.h>
-#include <R_ext/RS.h>           /* for CallocCharBuf, Free */
+#include <R_ext/RS.h>           /* for CallocCharBuf, R_Free */
 #include <errno.h>
 #include <ctype.h>		/* for isspace */
 #include <stdarg.h>
@@ -153,7 +153,7 @@
    any specially marked entry causes a call to the hook function with
    the reconstructed STRSXP and data value as arguments.  This should
    return the value to use for the reference object.  A reasonable
-   convention on how to use this mechanism is neded, but again the
+   convention on how to use this mechanism is needed, but again the
    format should be compatible with any reasonable convention.
 
    Eventually it may be useful to use these hooks to allow objects
@@ -195,7 +195,7 @@ static SEXP ReadBC(SEXP ref_table, R_inpstream_t stream);
 /* The default version used when a stream Init function is called with
    version = 0 */
 
-static int defaultSerializeVersion()
+static int defaultSerializeVersion(void)
 {
     static int dflt = -1;
 
@@ -349,17 +349,17 @@ static void OutString(R_outpstream_t stream, const char *s, int length)
 	char buf[128];
 	for (i = 0; i < length; i++) {
 	    switch(s[i]) {
-	    case '\n': sprintf(buf, "\\n");  break;
-	    case '\t': sprintf(buf, "\\t");  break;
-	    case '\v': sprintf(buf, "\\v");  break;
-	    case '\b': sprintf(buf, "\\b");  break;
-	    case '\r': sprintf(buf, "\\r");  break;
-	    case '\f': sprintf(buf, "\\f");  break;
-	    case '\a': sprintf(buf, "\\a");  break;
-	    case '\\': sprintf(buf, "\\\\"); break;
-	    case '\?': sprintf(buf, "\\?");  break;
-	    case '\'': sprintf(buf, "\\'");  break;
-	    case '\"': sprintf(buf, "\\\""); break;
+	    case '\n': snprintf(buf, 128, "\\n");  break;
+	    case '\t': snprintf(buf, 128, "\\t");  break;
+	    case '\v': snprintf(buf, 128, "\\v");  break;
+	    case '\b': snprintf(buf, 128, "\\b");  break;
+	    case '\r': snprintf(buf, 128, "\\r");  break;
+	    case '\f': snprintf(buf, 128, "\\f");  break;
+	    case '\a': snprintf(buf, 128, "\\a");  break;
+	    case '\\': snprintf(buf, 128, "\\\\"); break;
+	    case '\?': snprintf(buf, 128, "\\?");  break;
+	    case '\'': snprintf(buf, 128, "\\'");  break;
+	    case '\"': snprintf(buf, 128, "\\\""); break;
 	    default  :
 		/* cannot print char in octal mode -> cast to unsigned
 		   char first */
@@ -367,9 +367,9 @@ static void OutString(R_outpstream_t stream, const char *s, int length)
 		   is handled above, s[i] > 126 can't happen, but
 		   I'm superstitious...  -pd */
 		if (s[i] <= 32 || s[i] > 126)
-		    sprintf(buf, "\\%03o", (unsigned char) s[i]);
+		    snprintf(buf, 128, "\\%03o", (unsigned char) s[i]);
 		else
-		    sprintf(buf, "%c", s[i]);
+		    snprintf(buf, 128, "%c", s[i]);
 	    }
 	    stream->OutBytes(stream, buf, (int)strlen(buf));
 	}
@@ -1622,7 +1622,7 @@ ConvertChar(void *obj, char *inp, size_t inplen, cetype_t enc)
 	} else {
 	    char *buf = CallocCharBuf(buflen);
 	    if (TryConvertString(obj, inp, inplen, buf, &bufleft) == -1) {
-		Free(buf);
+		R_Free(buf);
 		if (errno == E2BIG) {
 		    buflen *= 2;
 		    continue;
@@ -1630,7 +1630,7 @@ ConvertChar(void *obj, char *inp, size_t inplen, cetype_t enc)
 		    return R_NilValue;
 	    }
 	    SEXP ans = mkCharLenCE(buf, (int)(buflen - bufleft), enc);
-	    Free(buf);
+	    R_Free(buf);
 	    return ans;
 	}
     }
@@ -1646,6 +1646,23 @@ static char *native_fromcode(R_inpstream_t stream)
     return from;
 }
 
+static void invalid_utf8_warning(const char *buf, const char *from)
+{
+    const void *vmax = vmaxget();
+    const char *native_buf;
+
+    if (utf8Valid(buf)) {
+	native_buf = reEnc3(buf, "UTF-8", "", 1);
+	warning(_("input string '%s' cannot be translated from '%s' to UTF-8, but is valid UTF-8"),
+		native_buf, from);
+    } else {
+	native_buf = reEnc(reEnc3(buf, from, "UTF-8", 1), CE_UTF8, CE_NATIVE, 2);
+	warning(_("input string '%s' cannot be translated to UTF-8, is it valid in '%s'?"),
+		native_buf, from);
+    }
+    vmaxset(vmax);
+}
+
 /* Read string into pre-allocated buffer, convert encoding if necessary, and
    return a CHARSXP */
 static SEXP
@@ -1695,9 +1712,7 @@ ReadChar(R_inpstream_t stream, char *buf, int length, int levs)
 	if (known_to_be_utf8) {
 	    /* nat2nat_obj is converting to UTF-8, no need to use nat2utf8_obj */
 	    stream->nat2utf8_obj = (void *)-1;
-	    char *from = native_fromcode(stream);
-	    warning(_("input string '%s' cannot be translated to UTF-8, is it valid in '%s'?"),
-	            buf, from);
+	    invalid_utf8_warning(buf, native_fromcode(stream));
 	}
     }
     /* try converting to UTF-8 */
@@ -1716,9 +1731,7 @@ ReadChar(R_inpstream_t stream, char *buf, int length, int levs)
 	SEXP ans = ConvertChar(stream->nat2utf8_obj, buf, length, CE_UTF8);
 	if (ans != R_NilValue)
 	    return ans;
-	char *from = native_fromcode(stream);
-	warning(_("input string '%s' cannot be translated to UTF-8, is it valid in '%s' ?"),
-	        buf, from);
+	invalid_utf8_warning(buf, native_fromcode(stream));
     }
     /* no translation possible */
     return mkCharLenCE(buf, length, CE_NATIVE); 
@@ -1881,6 +1894,7 @@ static SEXP ReadItem (SEXP ref_table, R_inpstream_t stream)
 	}
 	SETCAR(s, ReadItem(ref_table, stream));
 	R_ReadItemDepth--; /* do this early because of the recursion. */
+	R_CheckStack();
 	SETCDR(s, ReadItem(ref_table, stream));
 	/* For reading closures and promises stored in earlier versions, convert NULL env to baseenv() */
 	if      (type == CLOSXP && CLOENV(s) == R_NilValue) SET_CLOENV(s, R_BaseEnv);
@@ -1934,7 +1948,7 @@ static SEXP ReadItem (SEXP ref_table, R_inpstream_t stream)
 	    } else {
 		char *cbuf = CallocCharBuf(length);
 		PROTECT(s = ReadChar(stream, cbuf, length, levs));
-		Free(cbuf);
+		R_Free(cbuf);
 	    }
 	    break;
 	case LGLSXP:
@@ -2502,7 +2516,7 @@ static void con_cleanup(void *data)
 /* Used from saveRDS().
    This became public in R 2.13.0, and that version added support for
    connections internally */
-SEXP attribute_hidden
+attribute_hidden SEXP
 do_serializeToConn(SEXP call, SEXP op, SEXP args, SEXP env)
 {
     /* serializeToConn(object, conn, ascii, version, hook) */
@@ -2572,7 +2586,7 @@ do_serializeToConn(SEXP call, SEXP op, SEXP args, SEXP env)
 /* unserializeFromConn(conn, hook) used from readRDS().
    It became public in R 2.13.0, and that version added support for
    connections internally */
-SEXP attribute_hidden
+attribute_hidden SEXP
 do_unserializeFromConn(SEXP call, SEXP op, SEXP args, SEXP env)
 {
     /* 0 .. unserializeFromConn(conn, hook) */
@@ -2883,7 +2897,7 @@ R_serialize(SEXP object, SEXP icon, SEXP ascii, SEXP Sversion, SEXP fun)
 }
 
 
-SEXP attribute_hidden R_unserialize(SEXP icon, SEXP fun)
+attribute_hidden SEXP R_unserialize(SEXP icon, SEXP fun)
 {
     struct R_inpstream_st in;
     SEXP (*hook)(SEXP, SEXP);
@@ -2972,10 +2986,10 @@ static SEXP appendRawToFile(SEXP file, SEXP bytes)
 
 #define NC 100
 static int used = 0;
-static char names[NC][PATH_MAX];
+static char *names[NC];
 static char *ptr[NC];
 
-SEXP attribute_hidden
+attribute_hidden SEXP
 do_lazyLoadDBflush(SEXP call, SEXP op, SEXP args, SEXP env)
 {
     checkArity(op, args);
@@ -2985,8 +2999,9 @@ do_lazyLoadDBflush(SEXP call, SEXP op, SEXP args, SEXP env)
 
     /* fprintf(stderr, "flushing file %s", cfile); */
     for (i = 0; i < used; i++)
-	if(strcmp(cfile, names[i]) == 0) {
-	    strcpy(names[i], "");
+	if(names[i] != NULL && strcmp(cfile, names[i]) == 0) {
+	    free(names[i]);
+	    names[i] = NULL;
 	    free(ptr[i]);
 	    /* fprintf(stderr, " found at pos %d in cache", i); */
 	    break;
@@ -3023,7 +3038,7 @@ static SEXP readRawFromFile(SEXP file, SEXP key)
     val = allocVector(RAWSXP, len);
     /* Do we have this database cached? */
     for (i = 0; i < used; i++)
-	if(strcmp(cfile, names[i]) == 0) {icache = i; break;}
+	if(names[i] != NULL && strcmp(cfile, names[i]) == 0) {icache = i; break;}
     if (icache >= 0) {
 	memcpy(RAW(val), ptr[icache]+offset, len);
 	vmaxset(vmax);
@@ -3032,8 +3047,11 @@ static SEXP readRawFromFile(SEXP file, SEXP key)
 
     /* find a vacant slot? */
     for (i = 0; i < used; i++)
-	if(strcmp("", names[i]) == 0) {icache = i; break;}
-    if(icache < 0 && used < NC) icache = used++;
+	if(names[i] == NULL) {icache = i; break;}
+    if(icache < 0 && used < NC) {
+	icache = used++;
+	names[icache] = NULL;
+    }
 
     if(icache >= 0) {
 	if ((fp = R_fopen(cfile, "rb")) == NULL)
@@ -3044,11 +3062,13 @@ static SEXP readRawFromFile(SEXP file, SEXP key)
 	}
 	filelen = ftell(fp);
 	if (filelen < LEN_LIMIT) {
-	    char *p;
+	    char *p, *n;
 	    /* fprintf(stderr, "adding file '%s' at pos %d in cache, length %d\n",
 	       cfile, icache, filelen); */
 	    p = (char *) malloc(filelen);
-	    if (p) {
+	    n = (char *) malloc(strlen(cfile) + 1);
+	    if (p && n) {
+		names[icache] = n;
 		strcpy(names[icache], cfile);
 		ptr[icache] = p;
 		if (fseek(fp, 0, SEEK_SET) != 0) {
@@ -3060,6 +3080,10 @@ static SEXP readRawFromFile(SEXP file, SEXP key)
 		if (filelen != in) error(_("read failed on %s"), cfile);
 		memcpy(RAW(val), p+offset, len);
 	    } else {
+		if (p)
+		    free(p);
+		if (n)
+		    free(n);
 		if (fseek(fp, offset, SEEK_SET) != 0) {
 		    fclose(fp);
 		    error(_("seek failed on %s"), cfile);
@@ -3179,7 +3203,7 @@ R_lazyLoadDBinsertValue(SEXP value, SEXP file, SEXP ascii,
    from a file, optionally decompresses, and unserializes the bytes.
    If the result is a promise, then the promise is forced. */
 
-SEXP attribute_hidden
+attribute_hidden SEXP
 do_lazyLoadDBfetch(SEXP call, SEXP op, SEXP args, SEXP env)
 {
     SEXP key, file, compsxp, hook;
@@ -3214,7 +3238,7 @@ do_lazyLoadDBfetch(SEXP call, SEXP op, SEXP args, SEXP env)
     return val;
 }
 
-SEXP attribute_hidden
+attribute_hidden SEXP
 do_getVarsFromFrame(SEXP call, SEXP op, SEXP args, SEXP env)
 {
     checkArity(op, args);
@@ -3222,7 +3246,7 @@ do_getVarsFromFrame(SEXP call, SEXP op, SEXP args, SEXP env)
 }
 
 
-SEXP attribute_hidden
+attribute_hidden SEXP
 do_lazyLoadDBinsertValue(SEXP call, SEXP op, SEXP args, SEXP env)
 {
     checkArity(op, args);
@@ -3235,7 +3259,7 @@ do_lazyLoadDBinsertValue(SEXP call, SEXP op, SEXP args, SEXP env)
     return R_lazyLoadDBinsertValue(value, file, ascii, compsxp, hook);
 }
 
-SEXP attribute_hidden
+attribute_hidden SEXP
 do_serialize(SEXP call, SEXP op, SEXP args, SEXP env)
 {
     checkArity(op, args);
