@@ -1,6 +1,6 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
- *  Copyright (C) 1998-2020   The R Core Team
+ *  Copyright (C) 1998-2023   The R Core Team
  *  Copyright (C) 1995, 1996  Robert Gentleman and Ross Ihaka
  *  Copyright (C) 2004        The R Foundation
  *
@@ -26,7 +26,7 @@
 #include <Defn.h> /* => Utils.h with the protos from here; Rinternals.h */
 #include <Internal.h>
 #include <Rmath.h>
-#include <R_ext/RS.h>  /* for Calloc/Free */
+#include <R_ext/RS.h>  /* for R_Calloc/R_Free */
 #include <float.h> /* for DBL_MAX */
 #include <R_ext/Itermacros.h> /* for ITERATE_BY_REGION */
 
@@ -217,7 +217,7 @@ Rboolean isUnsorted(SEXP x, Rboolean strictly)
      ((TYPEOF(x) == INTSXP && FIRST_LAST_DIFF(x, INTEGER)) ||		\
       (TYPEOF(x) == REALSXP && FIRST_LAST_DIFF(x, REAL))))
 
-SEXP attribute_hidden do_isunsorted(SEXP call, SEXP op, SEXP args, SEXP rho)
+attribute_hidden SEXP do_isunsorted(SEXP call, SEXP op, SEXP args, SEXP rho)
 {
     if (length(args) == 2) {
 	/* This allows for old calls of the form
@@ -333,7 +333,7 @@ void R_csort(Rcomplex *x, int n)
 }
 
 /* used in platform.c */
-void attribute_hidden ssort(SEXP *x, int n)
+attribute_hidden void ssort(SEXP *x, int n)
 {
     SEXP v;
     sort_body(scmp,PROTECT,UNPROTECT(1))
@@ -408,7 +408,7 @@ void revsort(double *a, int *ib, int n)
 }
 
 
-SEXP attribute_hidden do_sort(SEXP call, SEXP op, SEXP args, SEXP rho)
+attribute_hidden SEXP do_sort(SEXP call, SEXP op, SEXP args, SEXP rho)
 {
     SEXP ans;
     Rboolean decreasing;
@@ -424,7 +424,7 @@ SEXP attribute_hidden do_sort(SEXP call, SEXP op, SEXP args, SEXP rho)
     if(TYPEOF(CAR(args)) == RAWSXP)
 	error(_("raw vectors cannot be sorted"));
 
-    /* we need consistent behaviour here, including dropping attibutes,
+    /* we need consistent behaviour here, including dropping attributes,
        so as from 2.3.0 we always duplicate. */
     PROTECT(ans = duplicate(CAR(args)));
     SET_ATTRIB(ans, R_NilValue);  /* this is never called with names */
@@ -506,7 +506,7 @@ static int makeSortEnum(int decr, int nalast) {
 }
 
 /* .Internal(sorted_fpass(x, decr, nalast)) */
-SEXP attribute_hidden do_sorted_fpass(SEXP call, SEXP op, SEXP args, SEXP rho)
+attribute_hidden SEXP do_sorted_fpass(SEXP call, SEXP op, SEXP args, SEXP rho)
 {
     checkArity(op, args);
 
@@ -774,7 +774,7 @@ Psort0(SEXP x, R_xlen_t lo, R_xlen_t hi, R_xlen_t *ind, int nind)
 
 
 /* FUNCTION psort(x, indices) */
-SEXP attribute_hidden do_psort(SEXP call, SEXP op, SEXP args, SEXP rho)
+attribute_hidden SEXP do_psort(SEXP call, SEXP op, SEXP args, SEXP rho)
 {
     checkArity(op, args);
     SEXP x = CAR(args), p = CADR(args);
@@ -1143,7 +1143,7 @@ void R_orderVector1(int *indx, int n, SEXP x,
    Also used by do_options and  ../gnuwin32/extra.c
    Called with rho != R_NilValue only from do_rank, when NAs are not involved.
  */
-void attribute_hidden
+attribute_hidden void
 orderVector1(int *indx, int n, SEXP key, Rboolean nalast, Rboolean decreasing,
 	     SEXP rho)
 {
@@ -1173,7 +1173,7 @@ orderVector1(int *indx, int n, SEXP key, Rboolean nalast, Rboolean decreasing,
 
     if(isNull(rho)) {
 	/* First sort NAs to one end */
-	isna = Calloc(n, int);
+	isna = R_Calloc(n, int);
 	switch (TYPEOF(key)) {
 	case LGLSXP:
 	case INTSXP:
@@ -1206,7 +1206,7 @@ orderVector1(int *indx, int n, SEXP key, Rboolean nalast, Rboolean decreasing,
 		sort2_with_index
 #undef less
 		if (n - numna < 2) {
-		    Free(isna);
+		    R_Free(isna);
 		    return;
 		}
 		if (nalast) hi -= numna; else lo += numna;
@@ -1274,7 +1274,7 @@ orderVector1(int *indx, int n, SEXP key, Rboolean nalast, Rboolean decreasing,
 #undef less
 	}
     }
-    if(isna) Free(isna);
+    if(isna) R_Free(isna);
 }
 
 /* version for long vectors */
@@ -1310,7 +1310,7 @@ orderVector1l(R_xlen_t *indx, R_xlen_t n, SEXP key, Rboolean nalast,
 
     if(isNull(rho)) {
 	/* First sort NAs to one end */
-	isna = Calloc(n, int);
+	isna = R_Calloc(n, int);
 	switch (TYPEOF(key)) {
 	case LGLSXP:
 	case INTSXP:
@@ -1343,7 +1343,7 @@ orderVector1l(R_xlen_t *indx, R_xlen_t n, SEXP key, Rboolean nalast,
 		sort2_with_index
 #undef less
 		if (n - numna < 2) {
-		    Free(isna);
+		    R_Free(isna);
 		    return;
 		}
 		if (nalast) hi -= numna; else lo += numna;
@@ -1411,12 +1411,12 @@ orderVector1l(R_xlen_t *indx, R_xlen_t n, SEXP key, Rboolean nalast,
 #undef less
 	}
     }
-    if(isna) Free(isna);
+    if(isna) R_Free(isna);
 }
 #endif
 
 /* FUNCTION order(...) */
-SEXP attribute_hidden do_order(SEXP call, SEXP op, SEXP args, SEXP rho)
+attribute_hidden SEXP do_order(SEXP call, SEXP op, SEXP args, SEXP rho)
 {
     SEXP ap, ans = R_NilValue /* -Wall */;
     int narg = 0;
@@ -1487,7 +1487,7 @@ SEXP attribute_hidden do_order(SEXP call, SEXP op, SEXP args, SEXP rho)
 }
 
 /* FUNCTION: rank(x, length, ties.method) */
-SEXP attribute_hidden do_rank(SEXP call, SEXP op, SEXP args, SEXP rho)
+attribute_hidden SEXP do_rank(SEXP call, SEXP op, SEXP args, SEXP rho)
 {
     SEXP rank, x;
     int *ik = NULL /* -Wall */;
@@ -1504,7 +1504,7 @@ SEXP attribute_hidden do_rank(SEXP call, SEXP op, SEXP args, SEXP rho)
     SEXP sn = CADR(args);
     R_xlen_t n;
     if (TYPEOF(sn) == REALSXP)  {
-	double d = REAL(x)[0];
+	double d = REAL(sn)[0];
 	if(ISNAN(d)) error(_("vector size cannot be NA/NaN"));
 	if(!R_FINITE(d)) error(_("vector size cannot be infinite"));
 	if(d > R_XLEN_T_MAX) error(_("vector size specified is too large"));
@@ -1570,7 +1570,7 @@ SEXP attribute_hidden do_rank(SEXP call, SEXP op, SEXP args, SEXP rho)
 		switch(ties_kind) {
 		case AVERAGE:
 		    for (k = i; k <= j; k++)
-			rk[in[k]] = (i + j + 2) / 2.;
+			rk[in[k]] = ((double)i + j + 2) / 2.;
 		    break;
 		case MAX:
 		    for (k = i; k <= j; k++) ik[in[k]] = j+1;
@@ -1586,7 +1586,7 @@ SEXP attribute_hidden do_rank(SEXP call, SEXP op, SEXP args, SEXP rho)
     return rank;
 }
 
-SEXP attribute_hidden do_xtfrm(SEXP call, SEXP op, SEXP args, SEXP rho)
+attribute_hidden SEXP do_xtfrm(SEXP call, SEXP op, SEXP args, SEXP rho)
 {
     SEXP fn, prargs, ans;
 
@@ -1598,7 +1598,7 @@ SEXP attribute_hidden do_xtfrm(SEXP call, SEXP op, SEXP args, SEXP rho)
     /* otherwise dispatch the default method */
     PROTECT(fn = findFun(install("xtfrm.default"), rho));
     PROTECT(prargs = promiseArgs(args, R_GlobalEnv));
-    SET_PRVALUE(CAR(prargs), CAR(args));
+    IF_PROMSXP_SET_PRVALUE(CAR(prargs), CAR(args));
     ans = applyClosure(call, fn, prargs, rho, R_NilValue);
     UNPROTECT(2);
     return ans;
