@@ -1,6 +1,6 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
- *  Copyright (C) 1997--2021  The R Core Team
+ *  Copyright (C) 1997--2024  The R Core Team
  *  Copyright (C) 1995, 1996  Robert Gentleman and Ross Ihaka
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -74,6 +74,9 @@ struct _HashData {
     int nomatch;
     Rboolean useUTF8;
     Rboolean useCache;
+    Rboolean useCloEnv;
+    Rboolean extptrAsRef;
+    Rboolean inHashtab;
 };
 
 #define HTDATA_INT(d) (INTEGER0((d)->HashTable))
@@ -179,22 +182,49 @@ static hlen chash(SEXP x, R_xlen_t indx, HashData *d)
 #endif
 }
 
-/* Hash CHARSXP by address.  Hash values are int, For 64bit pointers,
- * we do (upper ^ lower) */
-static R_INLINE hlen cshash(SEXP x, R_xlen_t indx, HashData *d)
+/* Pointer hashing as used here isn't entirely portable (we do it in
+   several other places, sometimes in slightly different ways) but it
+   could be made so by computing a unique value based on the
+   allocation page and position in the page.
+
+   Pointer hashes will not be valid if serialized and unserialized in
+   another process.
+
+   Hash values are int, For 64bit pointers, we do (upper ^ lower) */
+static R_INLINE unsigned int PTRHASH(void *x)
 {
-    intptr_t z = (intptr_t) STRING_ELT(x, indx);
+    intptr_t z = (intptr_t) x;
     unsigned int z1 = (unsigned int)(z & 0xffffffff), z2 = 0;
 #if SIZEOF_LONG == 8
     z2 = (unsigned int)(z/0x100000000L);
 #endif
-    return scatter(z1 ^ z2, d);
+    return z1 ^ z2;
+}
+
+/* Hash CHARSXP by address. */
+static R_INLINE hlen cshash(SEXP x, R_xlen_t indx, HashData *d)
+{
+    return scatter(PTRHASH(STRING_ELT(x, indx)), d);
 }
 
 static R_INLINE hlen shash(SEXP x, R_xlen_t indx, HashData *d)
 {
     unsigned int k;
     const char *p;
+    if (d->inHashtab) {
+	SEXP xi = STRING_ELT(x, indx);
+	int noTrans = (IS_BYTES(xi) || IS_ASCII(xi)) ? TRUE : FALSE;
+	if (d->useCache && noTrans)
+	    return scatter(PTRHASH(xi), d);
+	const void *vmax = vmaxget();
+	p = noTrans ? CHAR(xi) : translateCharUTF8(xi);
+	k = 0;
+	while (*p++)
+	    /* multiplier was 8 but 11 isn't a power of 2 */
+	    k = 11 * k + (unsigned int) *p;
+	vmaxset(vmax); /* discard any memory used by translateChar */
+	return scatter(k, d);
+    }
     if(!d->useUTF8 && d->useCache) return cshash(x, indx, d);
     const void *vmax = vmaxget();
     /* Not having d->useCache really should not happen anymore. */
@@ -284,13 +314,23 @@ static int rawequal(SEXP x, R_xlen_t i, SEXP y, R_xlen_t j)
     return (RAW_ELT(x, i) == RAW_ELT(y, j));
 }
 
+static hlen vhash_one(SEXP _this, HashData *d);
 static hlen vhash(SEXP x, R_xlen_t indx, HashData *d)
 {
-    int i;
-    unsigned int key;
     SEXP _this = VECTOR_ELT(x, indx);
+    return vhash_one(_this, d);
+}
 
-    key = OBJECT(_this) + 2*TYPEOF(_this) + 100U*(unsigned int) length(_this);
+static hlen vhash_one(SEXP _this, HashData *d)
+{
+    /* Handle environments by pointer hashing. Previously,
+       environments were hashed based only on length, which is not
+       very effective and could be expensive to compute. */
+    if (TYPEOF(_this) == ENVSXP)
+	return scatter(PTRHASH(_this), d);
+
+    int i;
+    unsigned int key = OBJECT(_this) + 2*TYPEOF(_this) + 100U*(unsigned int) length(_this);
     /* maybe we should also look at attributes, but that slows us down */
     switch (TYPEOF(_this)) {
     case LGLSXP:
@@ -330,12 +370,52 @@ static hlen vhash(SEXP x, R_xlen_t indx, HashData *d)
 	    key *= 97;
 	}
 	break;
+    case EXPRSXP:
     case VECSXP:
+	R_CheckStack();
 	for(i = 0; i < LENGTH(_this); i++) {
 	    key ^= vhash(_this, i, d);
 	    key *= 97;
 	}
 	break;
+    case LANGSXP:
+    case LISTSXP:
+	R_CheckStack();
+	/* all attributes are ignored */
+	/* might be good to consider environments on formulas */
+	for (SEXP next = _this; next != R_NilValue; next = CDR(next)) {
+	    key ^= vhash_one(CAR(next), d);
+	    key *= 97;
+	}
+	break;
+    case CLOSXP:
+	/* all attributes are ignored */
+	key ^= vhash_one(BODY_EXPR(_this), d);
+	key *= 97;
+	if (d->useCloEnv) {
+	    key ^= vhash_one(CLOENV(_this), d);
+	    key *= 97;
+	}
+	break;
+    case SYMSXP:
+	key *= PTRHASH(_this);
+	key *= 97;
+	break;
+    case CHARSXP:
+	if(!d->useUTF8 && d->useCache) key *= PTRHASH(_this);
+	/**** otherwise, do nothing for now */
+	/* this should only happen in C-leve hash tables */
+	/* eventually this should do what shash does */
+	key *= 97;
+	break;
+    case EXTPTRSXP:
+	if (d->extptrAsRef)
+	    key ^= PTRHASH(_this);
+	else
+	    /* identical() considers only the EXTPTR_PTR values ... */
+	    key ^= PTRHASH(EXTPTR_PTR(_this));
+	key *= 97;
+	break;
     default:
 	break;
     }
@@ -501,14 +581,24 @@ static Rboolean duplicatedInit(SEXP x, HashData *d)
 		break;
 	    }
 	}
-    } else if (TYPEOF(x) == VECSXP) {
+    } else if (TYPEOF(x) == VECSXP || TYPEOF(x) == EXPRSXP) {
 	R_xlen_t i, n = XLENGTH(x);
 	for(i = 0; i < n; i++)
 	    if (duplicatedInit(VECTOR_ELT(x, i), d)) {
 		stop = TRUE;
 		break;
 	    }
-    }
+    } else if (TYPEOF(x) == LANGSXP || TYPEOF(x) == LISTSXP) {
+	for(SEXP head = x; head != R_NilValue; head = CDR(head))
+	    if (duplicatedInit(CAR(head), d)) {
+		stop = TRUE;
+		break;
+	    }
+
+    } else if (TYPEOF(x) == CLOSXP) {
+	if (duplicatedInit(BODY_EXPR(x), d))
+	    stop = TRUE;
+    }	
     return stop;
 }
 
@@ -541,7 +631,7 @@ static void removeEntry(SEXP table, SEXP x, R_xlen_t indx, HashData *d)
 }
 
 #define DUPLICATED_INIT						\
-    HashData data;						\
+    HashData data = { 0 };					\
     HashTableSetup(x, &data, nmax);				\
     data.useUTF8 = FALSE; data.useCache = TRUE;			\
     duplicatedInit(x, &data);
@@ -999,7 +1089,7 @@ R_xlen_t any_duplicated3(SEXP x, SEXP incomp, Rboolean from_last)
    .Internal(       unique(x, incomparables, fromLast, nmax))  [op=1]
    .Internal(anyDuplicated(x, incomparables, fromLast      ))  [op=2]
 */
-SEXP attribute_hidden do_duplicated(SEXP call, SEXP op, SEXP args, SEXP env)
+attribute_hidden SEXP do_duplicated(SEXP call, SEXP op, SEXP args, SEXP env)
 {
     SEXP x, dup, ans;
     R_xlen_t i, k, n;
@@ -1190,14 +1280,23 @@ static SEXP match_transform(SEXP s, SEXP env)
 {
     if(OBJECT(s)) {
 	if(inherits(s, "factor")) return asCharacterFactor(s);
-	else if(inherits(s, "POSIXlt")) { /* and maybe more classes in the future:
-					   * Call R's (generic)	 as.character(s) : */
+	/*
+	else if(inherits(s, "POSIXlt")) { // and maybe more classes in the future:
+					  // Call R's (generic) as.character(s): 
 	    SEXP call, r;
 	    PROTECT(call = lang2(R_AsCharacterSymbol, s));
 	    r = eval(call, env);
 	    UNPROTECT(1);
 	    return r;
 	}
+	*/
+	else {
+	    SEXP call, r;
+	    PROTECT(call = lang2(install("mtfrm"), s));
+	    r = eval(call, env);
+	    UNPROTECT(1);
+	    return r;
+	}
     }
     /* else */
     return duplicate(s);
@@ -1317,7 +1416,7 @@ SEXP match5(SEXP itable, SEXP ix, int nmatch, SEXP incomp, SEXP env)
       PROTECT(ans = ScalarInteger(val)); nprot++;
     }
     else { // regular case
-	HashData data;
+	HashData data = { 0 };
 	if (incomp) { PROTECT(incomp = coerceVector(incomp, type)); nprot++; }
 	data.nomatch = nmatch;
 	HashTableSetup(table, &data, NA_INTEGER);
@@ -1386,12 +1485,12 @@ SEXP match(SEXP itable, SEXP ix, int nmatch)
 
 
 // .Internal(match(x, table, nomatch, incomparables)) :
-SEXP attribute_hidden do_match(SEXP call, SEXP op, SEXP args, SEXP env)
+attribute_hidden SEXP do_match(SEXP call, SEXP op, SEXP args, SEXP env)
 {
     checkArity(op, args);
 
-    if ((!isVector(CAR(args)) && !isNull(CAR(args)))
-	|| (!isVector(CADR(args)) && !isNull(CADR(args))))
+    if ((!isVector(CAR (args)) && !isNull(CAR (args))) ||
+	(!isVector(CADR(args)) && !isNull(CADR(args))))
 	error(_("'match' requires vector arguments"));
 
     int nomatch = asInteger(CADDR(args));
@@ -1419,35 +1518,32 @@ SEXP attribute_hidden do_match(SEXP call, SEXP op, SEXP args, SEXP env)
  * Empty strings are unmatched			      BDR 2000/2/16
  */
 
-SEXP attribute_hidden do_pmatch(SEXP call, SEXP op, SEXP args, SEXP env)
+// .Internal(pmatch(x, table, nomatch, duplicates.ok))
+attribute_hidden SEXP do_pmatch(SEXP call, SEXP op, SEXP args, SEXP env)
 {
-    SEXP ans, input, target;
-    int mtch, n_target, mtch_count, dups_ok, no_match;
-    size_t temp;
-    int *used = NULL, *ians;
-    const char **in, **tar;
-    Rboolean no_dups;
-    Rboolean useBytes = FALSE, useUTF8 = FALSE;
-
     checkArity(op, args);
-    input = CAR(args);
+    SEXP input  = CAR(args), // = x     in R
+	target = CADR(args), // = table  "
+	ans;
     R_xlen_t n_input = XLENGTH(input);
-    target = CADR(args);
-    n_target = LENGTH(target); // not allowed to be long
-    no_match = asInteger(CADDR(args));
-    dups_ok = asLogical(CADDDR(args));
+
+    int n_target = LENGTH(target), // not allowed to be long
+	no_match = asInteger(CADDR(args)),
+	dups_ok  = asLogical(CADDDR(args));
     if (dups_ok == NA_LOGICAL)
 	error(_("invalid '%s' argument"), "duplicates.ok");
-    no_dups = !dups_ok;
+    Rboolean no_dups = !dups_ok;
 
     if (!isString(input) || !isString(target))
 	error(_("argument is not of mode character"));
 
+    int *used = NULL;
     if(no_dups) {
 	used = (int *) R_alloc((size_t) n_target, sizeof(int));
 	for (int j = 0; j < n_target; j++) used[j] = 0;
     }
 
+    Rboolean useBytes = FALSE, useUTF8 = FALSE;
     for(R_xlen_t i = 0; i < n_input; i++) {
 	if(IS_BYTES(STRING_ELT(input, i))) {
 	    useBytes = TRUE;
@@ -1469,10 +1565,11 @@ SEXP attribute_hidden do_pmatch(SEXP call, SEXP op, SEXP args, SEXP env)
 	}
     }
 
-    in = (const char **) R_alloc((size_t) n_input, sizeof(char *));
+    const char **in, **tar;
+    in  = (const char **) R_alloc((size_t) n_input,  sizeof(char *));
     tar = (const char **) R_alloc((size_t) n_target, sizeof(char *));
     PROTECT(ans = allocVector(INTSXP, n_input));
-    ians = INTEGER0(ans);
+    int *ians = INTEGER0(ans);
     if(useBytes) {
 	for(R_xlen_t i = 0; i < n_input; i++) {
 	    in[i] = CHAR(STRING_ELT(input, i));
@@ -1514,7 +1611,7 @@ SEXP attribute_hidden do_pmatch(SEXP call, SEXP op, SEXP args, SEXP env)
 	    }
 	}
     } else {
-	HashData data;
+	HashData data = { 0 };
 	HashTableSetup(target, &data, NA_INTEGER);
 	data.useUTF8 = useUTF8;
 	data.nomatch = 0;
@@ -1536,11 +1633,13 @@ SEXP attribute_hidden do_pmatch(SEXP call, SEXP op, SEXP args, SEXP env)
 	    const char *ss;
 	    if (ians[i]) continue;
 	    ss = in[i];
-	    temp = strlen(ss);
+	    size_t temp = strlen(ss);
 	    if (temp == 0) continue;
-	    mtch = 0;
-	    mtch_count = 0;
+	    int mtch = 0,
+		mtch_count = 0;
 	    for (int j = 0; j < n_target; j++) {
+		if (!(((size_t)i * n_target + j) & 0x1fff))
+		    R_CheckUserInterrupt();
 		if (no_dups && used[j]) continue;
 		if (strncmp(ss, tar[j], temp) == 0) {
 		    mtch = j + 1;
@@ -1565,7 +1664,7 @@ SEXP attribute_hidden do_pmatch(SEXP call, SEXP op, SEXP args, SEXP env)
 /* Partial Matching of Strings */
 /* Based on Therneau's charmatch. */
 
-SEXP attribute_hidden do_charmatch(SEXP call, SEXP op, SEXP args, SEXP env)
+attribute_hidden SEXP do_charmatch(SEXP call, SEXP op, SEXP args, SEXP env)
 {
     SEXP ans, input, target;
     const char *ss, *st;
@@ -1728,7 +1827,7 @@ static SEXP subDots(SEXP rho)
 }
 
 
-SEXP attribute_hidden do_matchcall(SEXP call, SEXP op, SEXP args, SEXP env)
+attribute_hidden SEXP do_matchcall(SEXP call, SEXP op, SEXP args, SEXP env)
 {
     SEXP formals, actuals, rlist;
     SEXP funcall, f, b, rval, sysp, t1, t2, tail;
@@ -1847,7 +1946,7 @@ rowsum(SEXP x, SEXP g, SEXP uniqueg, SEXP snarm, SEXP rn)
     SEXP matches,ans;
     int n, p, ng, narm;
     R_xlen_t offset = 0, offsetg = 0;
-    HashData data;
+    HashData data = { 0 };
     data.nomatch = 0;
 
     n = LENGTH(g);
@@ -1921,7 +2020,7 @@ rowsum_df(SEXP x, SEXP g, SEXP uniqueg, SEXP snarm, SEXP rn)
 {
     SEXP matches,ans,col,xcol;
     int p, narm;
-    HashData data;
+    HashData data = { 0 };
     data.nomatch = 0;
 
     R_xlen_t n = XLENGTH(g);
@@ -1988,7 +2087,7 @@ rowsum_df(SEXP x, SEXP g, SEXP uniqueg, SEXP snarm, SEXP rn)
     return ans;
 }
 
-SEXP attribute_hidden do_rowsum(SEXP call, SEXP op, SEXP args, SEXP env)
+attribute_hidden SEXP do_rowsum(SEXP call, SEXP op, SEXP args, SEXP env)
 {
     checkArity(op, args);
     if(PRIMVAL(op) == 1)
@@ -2035,12 +2134,12 @@ static SEXP duplicated2(SEXP x, HashData *d)
     return ans;
 }
 
-SEXP attribute_hidden do_makeunique(SEXP call, SEXP op, SEXP args, SEXP env)
+attribute_hidden SEXP do_makeunique(SEXP call, SEXP op, SEXP args, SEXP env)
 {
     SEXP names, sep, ans, dup, newx;
     int i, cnt, *cnts, dp;
     int n, len, maxlen = 0;
-    HashData data;
+    HashData data = { 0 };
     const char *csep, *ss;
     const void *vmax;
 
@@ -2063,8 +2162,9 @@ SEXP attribute_hidden do_makeunique(SEXP call, SEXP op, SEXP args, SEXP env)
     }
     if(n > 1) {
 	/* +2 for terminator and rounding error */
-	char buf[maxlen + (int) strlen(csep)
-		 + (int) (log((double)n)/log(10.0)) + 2];
+	size_t sz = maxlen + (int) strlen(csep)
+	    + (int) (log((double)n)/log(10.0)) + 2;
+	char buf[sz];
 	if(n < 10000) {
 	    R_CheckStack2((size_t)n * sizeof(int));
 	    cnts = (int *) alloca(((size_t) n) * sizeof(int));
@@ -2085,7 +2185,7 @@ SEXP attribute_hidden do_makeunique(SEXP call, SEXP op, SEXP args, SEXP env)
 	    ss = translateChar(STRING_ELT(names, i));
 	    /* Try appending 1,2,3, ..., n-1 until it is not already in use */
 	    for(cnt = cnts[dp - 1]; cnt < n; cnt++) {
-		sprintf(buf, "%s%s%d", ss, csep, cnt);
+		snprintf(buf, sz, "%s%s%d", ss, csep, cnt);
 		SET_STRING_ELT(newx, 0, mkChar(buf));
 		if(Lookup(ans, newx, 0, &data) == data.nomatch) break;
 	    }
@@ -2126,7 +2226,7 @@ SEXP Rf_csduplicated(SEXP x)
     if(TYPEOF(x) != STRSXP)
 	error("C function 'csduplicated' not called on a STRSXP");
     R_xlen_t n = XLENGTH(x);
-    HashData data;
+    HashData data = { 0 };
     HashTableSetup1(x, &data);
     PROTECT(data.HashTable);
     SEXP ans = PROTECT(allocVector(LGLSXP, n));
@@ -2141,17 +2241,21 @@ SEXP Rf_csduplicated(SEXP x)
 #include <R_ext/Random.h>
 
 // sample.int(.) --> .Internal(sample2(n, size)) :
-SEXP attribute_hidden do_sample2(SEXP call, SEXP op, SEXP args, SEXP env)
+attribute_hidden SEXP do_sample2(SEXP call, SEXP op, SEXP args, SEXP env)
 {
     checkArity(op, args);
     SEXP ans;
     double dn = asReal(CAR(args));
-    int k = asInteger(CADR(args));
-    if (!R_FINITE(dn) || dn < 0 || dn > 4.5e15 || (k > 0 && dn == 0))
+    SEXP sk = CADR(args);
+    if (length(sk) != 1)
+	error(_("invalid '%s' argument"), "size");
+    int k = asInteger(sk);
+    if  (length(CAR(args)) != 1 || !R_FINITE(dn) || dn < 0
+		|| dn > 4.5e15 || (k > 0 && dn == 0))
 	error(_("invalid first argument"));
-    if (k < 0) error(_("invalid '%s' argument"), "size");
+    if (k < 0) error(_("invalid '%s' argument"), "size"); // includes NA
     if (k > dn/2) error("This algorithm is for size <= n/2");
-    HashData data;
+    HashData data = { 0 };
     GetRNGstate();
     if (dn > INT_MAX) {
 	ans = PROTECT(allocVector(REALSXP, k));
@@ -2178,3 +2282,367 @@ SEXP attribute_hidden do_sample2(SEXP call, SEXP op, SEXP args, SEXP env)
     UNPROTECT(2);
     return ans;
 }
+
+/***
+ *** Experimental Hath Table Implementation
+ ***
+ *** The interface is based loosely on the design of the Common Lisp
+ *** hash table support. Two equality tests are supported: identical()
+ *** and pointer equality.
+ ***/
+
+/**
+ ** C Level Interface
+ **/
+
+/*
+ * Low Level Functions
+ */
+
+static int hash_identical(SEXP x, int K, int useCloEnv)
+{
+    /* using 31 seems to work reasonably */
+    if (K == 0 || K > 31) K = 31;
+
+    HashData d = { .K = K, .useUTF8 = FALSE, .useCache = TRUE };
+    d.useCloEnv = useCloEnv;
+    d.extptrAsRef = TRUE;
+    d.inHashtab = TRUE;
+
+    int val = (int) vhash_one(x, &d);
+    if (val == NA_INTEGER) val = 0;
+    if (val < 0) val = -val;
+    return val;
+}
+
+static int hash_address(SEXP x, int K)
+{
+    if (K == 0 || K > 31) K = 31;
+
+    HashData d = { .K = K };
+
+    int val = (int) scatter(PTRHASH(x), &d);
+    if (val == NA_INTEGER) val = 0;
+    if (val < 0) val = -val;
+    return val;
+}
+
+/* allow for compiling with NAMED */
+static R_INLINE SEXP INC_NMD(SEXP x) {
+    INCREMENT_NAMED(x);
+    return x;
+}
+
+#define HT_SEXP(h) (h).cell
+#define HT_META_SIZE 3
+#define HT_META(h) R_ExternalPtrTag(HT_SEXP(h))
+
+#define HT_TABLE(h) R_ExternalPtrProtected(HT_SEXP(h))
+#define SET_HT_TABLE(h, table) R_SetExternalPtrProtected(HT_SEXP(h), table)
+#define HT_TABLE_SIZE(h) LENGTH(HT_TABLE(h))
+
+#define HT_COUNT(h) (INTEGER(HT_META(h))[0])
+#define HT_TYPE(h) (INTEGER(HT_META(h))[1])
+#define HT_TABLE_K(h) (INTEGER(HT_META(h))[2])
+
+#define HT_IS_VALID(h) (R_ExternalPtrAddr(HT_SEXP(h)) != NULL)
+#define HT_VALIDATE(h) R_SetExternalPtrAddr(HT_SEXP(h), HT_SEXP(h))
+
+static R_INLINE int HT_HASH(R_hashtab_type h, SEXP key)
+{
+    switch(HT_TYPE(h)) {
+    case HT_TYPE_IDENTICAL:
+	return hash_identical(key, HT_TABLE_K(h), TRUE);
+    case HT_TYPE_ADDRESS:
+	return hash_address(key, HT_TABLE_K(h));
+    default:
+	error("bad hash table type");
+    }
+}
+
+static R_INLINE int HT_EQUAL(R_hashtab_type h, SEXP x, SEXP y)
+{
+    switch(HT_TYPE(h)) {
+    case HT_TYPE_IDENTICAL:
+	{
+	    /* IDENT_USE_CLOENV corresponds to the default in identical().
+	       IDENT_EXTPTR_AS_REF ensures that EXTPTRSXP objects in
+	       keys unserialize sensibly. */
+	    int flags = IDENT_USE_CLOENV | IDENT_EXTPTR_AS_REF;
+	    return R_compute_identical(x, y, flags);
+	}
+    case HT_TYPE_ADDRESS:
+	return x == y;
+    default:
+	error("bad hash table type");
+    }
+}
+
+static void rehash(R_hashtab_type h, int resize)
+{
+    /* If the meta-data structure is changed then a saved hash table
+       will be invalid. For now, just check and reject incompatible
+       oned. For future changes would be better to try to
+       rewrite/repair the meta data. */
+    if (TYPEOF(HT_META(h)) != INTSXP || LENGTH(HT_META(h)) != HT_META_SIZE)
+	error("invalid hash table meta data");
+
+    SEXP old_table = PROTECT(HT_TABLE(h));
+    int old_size = LENGTH(old_table);
+
+    R_xlen_t new_size = resize ? 2 * old_size : old_size;
+    if (new_size > INT_MAX)
+	error("hash size would exceed limit");
+
+    HT_COUNT(h) = 0;
+    HT_VALIDATE(h);
+    
+    SET_HT_TABLE(h, allocVector(VECSXP, new_size));
+    if (resize) HT_TABLE_K(h)++;
+
+    for (int i = 0; i < old_size; i++)
+	for (SEXP cell = VECTOR_ELT(old_table, i);
+	     cell != R_NilValue;
+	     cell = CDR(cell))
+	    R_sethash(h, TAG(cell), CAR(cell));
+
+    UNPROTECT(1); /* old_table */
+}
+
+static SEXP getcell(R_hashtab_type h, SEXP key, int *pidx)
+{
+    SEXP table = HT_TABLE(h);
+
+    if (! HT_IS_VALID(h))
+	rehash(h, FALSE);
+
+    int idx = HT_HASH(h, key);
+    *pidx = idx;
+
+    SEXP chain = VECTOR_ELT(table, idx);
+    while (chain != R_NilValue) {
+	if (HT_EQUAL(h, TAG(chain), key))
+	    return chain;
+	chain = CDR(chain);
+    }
+    return R_NilValue;
+}    
+
+
+/*
+ * Higer Level Public Functions
+ */
+
+/* initial size = 2 ^ HT_INIT_K */
+#define HT_INIT_K 3
+
+R_hashtab_type R_mkhashtab(int type, int K)
+{
+    if (K < 3 || K > 30) K = 3;
+    int size = 1;
+    for (int i = 0; i < K; i++) size *= 2;
+
+    switch(type) {
+    case HT_TYPE_IDENTICAL:
+    case HT_TYPE_ADDRESS: break;
+    default: error("bad hash table type");
+    }
+    SEXP table = PROTECT(allocVector(VECSXP, size));
+    SEXP meta = PROTECT(allocVector(INTSXP, HT_META_SIZE));
+    R_hashtab_type val = { .cell = R_MakeExternalPtr(NULL, meta, table) };
+    HT_VALIDATE(val);
+    HT_COUNT(val) = 0;
+    HT_TYPE(val) = type;
+    HT_TABLE_K(val) = K;
+    UNPROTECT(2); /* table, meta */
+    return val;
+}
+
+SEXP R_gethash(R_hashtab_type h, SEXP key, SEXP nomatch)
+{
+    int idx;
+    PROTECT(HT_SEXP(h));
+    PROTECT(key);
+    PROTECT(nomatch);
+    SEXP cell = getcell(h, key, &idx);
+    UNPROTECT(3); /* h, key, nomatch */
+    if (cell == R_NilValue)
+	return nomatch;
+    else
+	return CAR(cell);
+}
+
+SEXP R_sethash(R_hashtab_type h, SEXP key, SEXP value)
+{
+    int idx;
+    PROTECT(HT_SEXP(h));
+    PROTECT(key);
+    PROTECT(value);
+    SEXP cell = getcell(h, key, &idx);
+    if (cell == R_NilValue) {
+	int new_count = HT_COUNT(h) + 1;
+	if (new_count > 0.5 * HT_TABLE_SIZE(h)) {
+	    /* rehash() will fail if new_count would be too large */
+	    rehash(h, TRUE);
+	    /* recalculate the hash bin, cell will still be R_NilValue */
+	    (void) getcell(h, key, &idx);
+	}
+	SEXP table = HT_TABLE(h);
+	SEXP chain = CONS(INC_NMD(value), VECTOR_ELT(table, idx));
+	SET_TAG(chain, INC_NMD(key));
+	SET_VECTOR_ELT(table, idx, chain);
+	HT_COUNT(h) = new_count;
+    }
+    else {
+	SETCAR(cell, value);
+	INCREMENT_NAMED(value);
+    }
+    UNPROTECT(3); /* h, key, value */
+    return value;
+}
+
+int R_remhash(R_hashtab_type h, SEXP key)
+{
+    int idx;
+    PROTECT(HT_SEXP(h));
+    PROTECT(key);
+    SEXP cell = getcell(h, key, &idx);
+    UNPROTECT(2); /* h, key */
+
+    if (cell == R_NilValue)
+	return FALSE;
+    else {
+	SEXP table = HT_TABLE(h);
+	if (cell == VECTOR_ELT(table, idx))
+	    SET_VECTOR_ELT(table, idx, CDR(cell));
+	else {
+	    SEXP prev = VECTOR_ELT(table, idx);
+	    while (CDR(prev) !=  cell)
+		prev = CDR(prev);
+	    SETCDR(prev, CDR(cell));
+	}
+	HT_COUNT(h)--;
+	SETCAR(cell, R_NilValue);  // drop REFCNT on old value
+	SET_TAG(cell, R_NilValue); // drop REFCNT on old key
+	return TRUE;
+    }
+}
+
+int R_numhash(R_hashtab_type h) { return HT_COUNT(h); }
+int R_typhash(R_hashtab_type h) { return HT_TYPE(h); }
+
+static R_INLINE void defvar(SEXP sym, SEXP val, SEXP env)
+{
+    defineVar(sym, INC_NMD(val), env);
+}
+
+SEXP R_maphash(R_hashtab_type h, SEXP FUN)
+{
+    PROTECT(HT_SEXP(h));
+    PROTECT(FUN);
+    SEXP FUN_sym = install("FUN");
+    SEXP key_sym = install("key");
+    SEXP val_sym = install("value");
+
+    SEXP env = PROTECT(R_NewEnv(R_GlobalEnv, FALSE, 0));
+    SEXP call = PROTECT(lang3(FUN_sym, key_sym, val_sym));
+    defvar(FUN_sym, FUN, env);
+    
+    SEXP table = PROTECT(HT_TABLE(h)); // PROTECT in case FUN causes a rehash
+    int size = LENGTH(table);
+    for (int i = 0; i < size; i++) {
+	SEXP cell = VECTOR_ELT(table, i);
+	while (cell != R_NilValue) {
+	    SEXP next = PROTECT(CDR(cell));
+	    defvar(key_sym, TAG(cell), env); // key is PROTECTed by env
+	    defvar(val_sym, CAR(cell), env); // val is PROTECTed by env
+	    eval(call, env);
+	    cell = next;
+	    UNPROTECT(1); /* next */
+	}
+    }
+    UNPROTECT(5); /* h, FUN, env, call, table */
+    return R_NilValue;
+}
+
+void R_maphashC(R_hashtab_type h, void (*FUN)(SEXP, SEXP, void *), void *data)
+{
+    PROTECT(HT_SEXP(h));
+    SEXP table = PROTECT(HT_TABLE(h)); // PROTECT in case FUN causes a rehash
+    int size = LENGTH(table);
+    for (int i = 0; i < size; i++) {
+	SEXP cell = VECTOR_ELT(table, i);
+	while (cell != R_NilValue) {
+	    SEXP next = PROTECT(CDR(cell));
+	    SEXP key = PROTECT(TAG(cell));
+	    SEXP val = PROTECT(CAR(cell));
+	    FUN(key, val, data);
+	    cell = next;
+	    UNPROTECT(3); /* next, key, val */
+	}
+    }
+    UNPROTECT(2); /* h, table */
+}
+
+void R_clrhash(R_hashtab_type h)
+{
+    SEXP table = HT_TABLE(h);
+    int size = LENGTH(table);
+    for (int i = 0; i < size; i++) {
+	for (SEXP cell = VECTOR_ELT(table, i);
+	     cell != R_NilValue;
+	     cell = CDR(cell)) {
+	    SETCAR(cell, R_NilValue);  // drop REFCNT on old value
+	    SET_TAG(cell, R_NilValue); // drop REFCNT on old key
+	}
+	SET_VECTOR_ELT(table, i, R_NilValue);
+    }
+    HT_COUNT(h) = 0;
+    /* could also drop table size back down to HT_INIT_SIZE */
+}
+
+
+/**
+ ** R Level Interface Support
+ **/
+
+R_hashtab_type R_asHashtable(SEXP h)
+{
+    if (TYPEOF(h) != VECSXP || LENGTH(h) != 1 || ! inherits(h, "hashtab"))
+	error("not a proper hash table object");
+    SEXP p = VECTOR_ELT(h, 0);
+    if (TYPEOF(p) != EXTPTRSXP)
+	error("hash table object is corrupted");
+    R_hashtab_type val = { .cell = p };
+    return val;
+}
+
+SEXP R_HashtabSEXP(R_hashtab_type  h)
+{
+    return HT_SEXP(h);
+}
+
+int R_isHashtable(SEXP h)
+{
+    if (TYPEOF(h) != VECSXP ||
+	LENGTH(h) != 1 ||
+	! inherits(h, "hashtab") ||
+	TYPEOF(VECTOR_ELT(h, 0)) != EXTPTRSXP)
+	return FALSE;
+    else
+	return TRUE;
+}
+
+/* This allows experimenting with creating hash tables at the R level. */
+attribute_hidden SEXP do_vhash(SEXP call, SEXP op, SEXP args, SEXP env)
+{
+    SEXP x = CAR(args);
+    SEXP sK = CADR(args);
+    SEXP sUseCloEnv = CADDR(args);
+
+    int K = sK == R_NilValue ? 31 : asInteger(sK);
+    int useCloEnv = sUseCloEnv == R_NilValue ? TRUE : asLogical(sUseCloEnv);
+
+    int val = hash_identical(x, K, useCloEnv);
+    return ScalarInteger(val);
+}
