@@ -1,10 +1,10 @@
 /*
  *  AUTHOR
- *    Claus Ekstrm, ekstrom@dina.kvl.dk
+ *    Claus Ekstrøm, ekstrom@dina.kvl.dk
  *    July 15, 2003.
  *
  *  Merge in to R:
- *	Copyright (C) 2003 The R Foundation
+ *	Copyright (C) 2003-2015 The R Foundation
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -17,8 +17,8 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+ *  along with this program; if not, a copy is available at
+ *  https://www.R-project.org/Licenses/
  *
  *
  *  NOTE
@@ -32,13 +32,14 @@
  *
  * DESCRIPTION
  *
- *    The non-central t density is
+ *    From Johnson, Kotz and Balakrishnan (1995) [2nd ed.; formula (31.15), p.516],
+ *    the non-central t density is
+ *
+ *      f(x, df, ncp) =
+ *
+ *        exp(-.5*ncp^2) * gamma((df+1)/2) / (sqrt(pi*df)* gamma(df/2)) * (df/(df+x^2))^((df+1)/2) *
+ *          sum_{j=0}^Inf  gamma((df+j+1)/2)/(factorial(j)* gamma((df+1)/2)) * (x*ncp*sqrt(2)/sqrt(df+x^2))^ j
  *
- *	   f(x, df, ncp) =
- *		df^(df/2) * exp(-.5*ncp^2) /
- *		(sqrt(pi)*gamma(df/2)*(df+x^2)^((df+1)/2)) *
- *		sum_{k=0}^Inf  gamma((df + k + df)/2)*ncp^k /
- *				prod(1:k)*(2*x^2/(df+x^2))^(k/2)
  *
  *    The functional relationship
  *
@@ -54,6 +55,8 @@
  *
  *    All calculations are done on log-scale to increase stability.
  *
+ * FIXME: pnt() is known to be inaccurate in the (very) left tail and for ncp > 38
+ *       ==> use a direct log-space summation formula in that case
  */
 
 #include "nmath.h"
@@ -68,7 +71,9 @@ double dnt(double x, double df, double ncp, int give_log)
 #endif
 
     /* If non-positive df then error */
-    if (df <= 0) ML_ERR_return_NAN;
+    if (df <= 0.0) ML_WARN_return_NAN;
+
+    if(ncp == 0.0) return dt(x, df, give_log);
 
     /* If x is infinite then return 0 */
     if(!R_FINITE(x))
@@ -92,7 +97,7 @@ double dnt(double x, double df, double ncp, int give_log)
     }
     else {  /* x ~= 0 : -> same value as for  x = 0 */
 	u = lgammafn((df+1)/2) - lgammafn(df/2)
-	    - .5*(log(M_PI) + log(df) + ncp*ncp);
+	    - (M_LN_SQRT_PI + .5*(log(df) + ncp*ncp));
     }
 
     return (give_log ? u : exp(u));
