@@ -1,6 +1,6 @@
 /*
  *  Mathlib : A C Library of Special Functions
- *  Copyright (C) 1998-2020  The R Core Team
+ *  Copyright (C) 1998-2022  The R Core Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -70,6 +70,7 @@ double	Rf_gamma_cody(double);
 # define MATHLIB_WARNING3(fmt,x,x2,x3)	warning(fmt,x,x2,x3)
 # define MATHLIB_WARNING4(fmt,x,x2,x3,x4) warning(fmt,x,x2,x3,x4)
 # define MATHLIB_WARNING5(fmt,x,x2,x3,x4,x5) warning(fmt,x,x2,x3,x4,x5)
+# define MATHLIB_WARNING6(fmt,x,x2,x3,x4,x5,x6) warning(fmt,x,x2,x3,x4,x5,x6)
 
 #include <R_ext/Arith.h>
 #define ML_POSINF	R_PosInf
@@ -107,6 +108,7 @@ void R_CheckUserInterrupt(void);
 #define MATHLIB_WARNING3(fmt,x,x2,x3)	printf(fmt,x,x2,x3)
 #define MATHLIB_WARNING4(fmt,x,x2,x3,x4) printf(fmt,x,x2,x3,x4)
 #define MATHLIB_WARNING5(fmt,x,x2,x3,x4,x5) printf(fmt,x,x2,x3,x4,x5)
+#define MATHLIB_WARNING6(fmt,x,x2,x3,x4,x5,x6) printf(fmt,x,x2,x3,x4,x5,x6)
 
 #define ISNAN(x) (isnan(x)!=0)
 // Arith.h defines it
@@ -140,7 +142,7 @@ int R_finite(double);
 #define ME_PRECISION	8
 /*	does not have "full" precision */
 #define ME_UNDERFLOW	16
-/*	and underflow occured (important for IEEE)*/
+/*	and underflow occurred (important for IEEE)*/
 
 
 #define ML_WARN_return_NAN { ML_WARNING(ME_DOMAIN, ""); return ML_NAN; }
@@ -187,6 +189,7 @@ int R_finite(double);
 
 /* always remap internal functions */
 #define bd0       	Rf_bd0
+#define ebd0       	Rf_ebd0
 #define chebyshev_eval	Rf_chebyshev_eval
 #define chebyshev_init	Rf_chebyshev_init
 #define gammalims	Rf_gammalims
@@ -213,6 +216,7 @@ double  attribute_hidden stirlerr(double);  /* Stirling expansion "error" */
 double	attribute_hidden lfastchoose(double, double);
 
 double  attribute_hidden bd0(double, double);
+void    attribute_hidden ebd0(double, double, double*, double*);
 
 double  attribute_hidden pnchisq_raw(double, double, double, double, double,
 				     int, Rboolean, Rboolean);
