@@ -1,3 +1,4 @@
+#include "Mdefines.h"
 #include "subscript.h"
 
 #define F_X( _X_)  (_X_)
@@ -7,7 +8,8 @@
 #define AR21_UP(i, j, m) i + j + (        j * (    j - 1)) / 2
 #define AR21_LO(i, j, m) i +     (j * m + j * (m - j - 1)) / 2
 
-static SEXP unpackedMatrix_subscript_1ary(SEXP x, SEXP w, const char *cl)
+static
+SEXP unpackedMatrix_subscript_1ary(SEXP x, SEXP w, const char *cl)
 {
 
 #define SUB1_START(_SEXPTYPE_) \
@@ -38,7 +40,7 @@ static SEXP unpackedMatrix_subscript_1ary(SEXP x, SEXP w, const char *cl)
 		} \
 	}
 
-	SUB1_START_EXTRA(kind2type(cl[0]));
+	SUB1_START_EXTRA(kindToType(cl[0]));
 
 #define SUB1_CASES(_SUB1_N_, _SUB1_X_, _F_N_, _F_X_) \
 	do { \
@@ -141,9 +143,10 @@ static SEXP unpackedMatrix_subscript_1ary(SEXP x, SEXP w, const char *cl)
 	return res;
 }
 
-static SEXP packedMatrix_subscript_1ary(SEXP x, SEXP w, const char *cl)
+static
+SEXP packedMatrix_subscript_1ary(SEXP x, SEXP w, const char *cl)
 {
-	SUB1_START_EXTRA(kind2type(cl[0]));
+	SUB1_START_EXTRA(kindToType(cl[0]));
 
 #define SUB1_LOOP(_NA_SUBSCRIPT_, _NA_, _ZERO_, _ONE_, _F_, _INT_) \
 	do { \
@@ -196,9 +199,10 @@ static SEXP packedMatrix_subscript_1ary(SEXP x, SEXP w, const char *cl)
 	return res;
 }
 
-static SEXP CsparseMatrix_subscript_1ary(SEXP x, SEXP w, const char *cl)
+static
+SEXP CsparseMatrix_subscript_1ary(SEXP x, SEXP w, const char *cl)
 {
-	SUB1_START(kind2type(cl[0]));
+	SUB1_START(kindToType(cl[0]));
 
 	SEXP p = PROTECT(GET_SLOT(x, Matrix_pSym)),
 		i = PROTECT(GET_SLOT(x, Matrix_iSym));
@@ -262,9 +266,10 @@ static SEXP CsparseMatrix_subscript_1ary(SEXP x, SEXP w, const char *cl)
 	return res;
 }
 
-static SEXP RsparseMatrix_subscript_1ary(SEXP x, SEXP w, const char *cl)
+static
+SEXP RsparseMatrix_subscript_1ary(SEXP x, SEXP w, const char *cl)
 {
-	SUB1_START(kind2type(cl[0]));
+	SUB1_START(kindToType(cl[0]));
 
 	SEXP p = PROTECT(GET_SLOT(x, Matrix_pSym)),
 		j = PROTECT(GET_SLOT(x, Matrix_jSym));
@@ -328,9 +333,10 @@ static SEXP RsparseMatrix_subscript_1ary(SEXP x, SEXP w, const char *cl)
 	return res;
 }
 
-static SEXP diagonalMatrix_subscript_1ary(SEXP x, SEXP w, const char *cl)
+static
+SEXP diagonalMatrix_subscript_1ary(SEXP x, SEXP w, const char *cl)
 {
-	SUB1_START(kind2type(cl[0]));
+	SUB1_START(kindToType(cl[0]));
 
 	SEXP diag = PROTECT(GET_SLOT(x, Matrix_diagSym));
 	int nonunit = *CHAR(STRING_ELT(diag, 0)) == 'N';
@@ -360,7 +366,8 @@ static SEXP diagonalMatrix_subscript_1ary(SEXP x, SEXP w, const char *cl)
 	return res;
 }
 
-static SEXP indMatrix_subscript_1ary(SEXP x, SEXP w)
+static
+SEXP indMatrix_subscript_1ary(SEXP x, SEXP w)
 {
 	SUB1_START(LGLSXP);
 
@@ -451,7 +458,8 @@ SEXP R_subscript_1ary(SEXP x, SEXP i)
 	}
 }
 
-static SEXP unpackedMatrix_subscript_1ary_mat(SEXP x, SEXP w, const char *cl)
+static
+SEXP unpackedMatrix_subscript_1ary_mat(SEXP x, SEXP w, const char *cl)
 {
 
 #define SUB1_START(_SEXPTYPE_) \
@@ -482,7 +490,7 @@ static SEXP unpackedMatrix_subscript_1ary_mat(SEXP x, SEXP w, const char *cl)
 		} \
 	}
 
-	SUB1_START_EXTRA(kind2type(cl[0]));
+	SUB1_START_EXTRA(kindToType(cl[0]));
 
 	Matrix_int_fast64_t i_, j_;
 
@@ -528,9 +536,10 @@ static SEXP unpackedMatrix_subscript_1ary_mat(SEXP x, SEXP w, const char *cl)
 	return res;
 }
 
-static SEXP packedMatrix_subscript_1ary_mat(SEXP x, SEXP w, const char *cl)
+static
+SEXP packedMatrix_subscript_1ary_mat(SEXP x, SEXP w, const char *cl)
 {
-	SUB1_START_EXTRA(kind2type(cl[0]));
+	SUB1_START_EXTRA(kindToType(cl[0]));
 
 	Matrix_int_fast64_t i_, j_;
 
@@ -583,9 +592,10 @@ static SEXP packedMatrix_subscript_1ary_mat(SEXP x, SEXP w, const char *cl)
 	return res;
 }
 
-static SEXP CsparseMatrix_subscript_1ary_mat(SEXP x, SEXP w, const char *cl)
+static
+SEXP CsparseMatrix_subscript_1ary_mat(SEXP x, SEXP w, const char *cl)
 {
-	SUB1_START(kind2type(cl[0]));
+	SUB1_START(kindToType(cl[0]));
 
 	SEXP p = PROTECT(GET_SLOT(x, Matrix_pSym)),
 		i = PROTECT(GET_SLOT(x, Matrix_iSym));
@@ -645,9 +655,10 @@ static SEXP CsparseMatrix_subscript_1ary_mat(SEXP x, SEXP w, const char *cl)
 	return res;
 }
 
-static SEXP RsparseMatrix_subscript_1ary_mat(SEXP x, SEXP w, const char *cl)
+static
+SEXP RsparseMatrix_subscript_1ary_mat(SEXP x, SEXP w, const char *cl)
 {
-	SUB1_START(kind2type(cl[0]));
+	SUB1_START(kindToType(cl[0]));
 
 	SEXP p = PROTECT(GET_SLOT(x, Matrix_pSym)),
 		j = PROTECT(GET_SLOT(x, Matrix_jSym));
@@ -707,9 +718,10 @@ static SEXP RsparseMatrix_subscript_1ary_mat(SEXP x, SEXP w, const char *cl)
 	return res;
 }
 
-static SEXP diagonalMatrix_subscript_1ary_mat(SEXP x, SEXP w, const char *cl)
+static
+SEXP diagonalMatrix_subscript_1ary_mat(SEXP x, SEXP w, const char *cl)
 {
-	SUB1_START(kind2type(cl[0]));
+	SUB1_START(kindToType(cl[0]));
 
 	SEXP diag = PROTECT(GET_SLOT(x, Matrix_diagSym));
 	int nonunit = *CHAR(STRING_ELT(diag, 0)) == 'N';
@@ -737,7 +749,8 @@ static SEXP diagonalMatrix_subscript_1ary_mat(SEXP x, SEXP w, const char *cl)
 	return res;
 }
 
-static SEXP indMatrix_subscript_1ary_mat(SEXP x, SEXP w)
+static
+SEXP indMatrix_subscript_1ary_mat(SEXP x, SEXP w)
 {
 	SUB1_START(LGLSXP);
 
@@ -826,7 +839,8 @@ SEXP R_subscript_1ary_mat(SEXP x, SEXP i)
 	}
 }
 
-static int keep_tr(int *pi, int *pj, int n, int upper, int nonunit, int checkNA)
+static
+int keep_tr(int *pi, int *pj, int n, int upper, int nonunit, int checkNA)
 {
 	int k, ident = memcmp(pi, pj, n * sizeof(int)) == 0;
 	if (checkNA) {
@@ -897,7 +911,8 @@ static int keep_tr(int *pi, int *pj, int n, int upper, int nonunit, int checkNA)
 	}
 }
 
-static int keep_sy(int *pi, int *pj, int n, int upper, int checkNA)
+static
+int keep_sy(int *pi, int *pj, int n, int upper, int checkNA)
 {
 	if (memcmp(pi, pj, n * sizeof(int)) != 0)
 		return 0;
@@ -927,7 +942,8 @@ static int keep_sy(int *pi, int *pj, int n, int upper, int checkNA)
 	return 2 * r;
 }
 
-static int keep_di(int *pi, int *pj, int n, int nonunit, int checkNA, int lwork)
+static
+int keep_di(int *pi, int *pj, int n, int nonunit, int checkNA, int lwork)
 {
 	int k, ident = memcmp(pi, pj, n * sizeof(int)) == 0;
 	if (checkNA) {
@@ -970,7 +986,8 @@ static int keep_di(int *pi, int *pj, int n, int nonunit, int checkNA, int lwork)
 	}
 }
 
-static void sort_cr(SEXP obj, const char *cl)
+static
+void sort_cr(SEXP obj, const char *cl)
 {
 	SEXP dim = PROTECT(GET_SLOT(obj, Matrix_DimSym));
 	int *pdim = INTEGER(dim),
@@ -1135,8 +1152,8 @@ static void sort_cr(SEXP obj, const char *cl)
 	 ? *(_X_ + AR21_LO(_I_, _J_, _M_)) \
 	 : *(_X_ + AR21_LO(_J_, _I_, _M_)))
 
-static SEXP unpackedMatrix_subscript_2ary(SEXP x, SEXP i, SEXP j,
-                                          const char *cl)
+static
+SEXP unpackedMatrix_subscript_2ary(SEXP x, SEXP i, SEXP j, const char *cl)
 {
 
 #define SUB2_START \
@@ -1187,7 +1204,7 @@ static SEXP unpackedMatrix_subscript_2ary(SEXP x, SEXP i, SEXP j,
 			} \
 		} \
 	} \
-	SEXP res = PROTECT(NEW_OBJECT_OF_CLASS(cl_)); \
+	SEXP res = PROTECT(newObject(cl_)); \
 	 \
 	PROTECT(dim = GET_SLOT(res, Matrix_DimSym)); \
 	pdim = INTEGER(dim); \
@@ -1389,8 +1406,8 @@ static SEXP unpackedMatrix_subscript_2ary(SEXP x, SEXP i, SEXP j,
 	return res;
 }
 
-static SEXP packedMatrix_subscript_2ary(SEXP x, SEXP i, SEXP j,
-                                        const char *cl)
+static
+SEXP packedMatrix_subscript_2ary(SEXP x, SEXP i, SEXP j, const char *cl)
 {
 	SUB2_START_EXTRA('e', 'p', 'p', 1);
 
@@ -1498,8 +1515,8 @@ static SEXP packedMatrix_subscript_2ary(SEXP x, SEXP i, SEXP j,
 	return res;
 }
 
-static SEXP CsparseMatrix_subscript_2ary(SEXP x, SEXP i, SEXP j,
-                                         const char *cl)
+static
+SEXP CsparseMatrix_subscript_2ary(SEXP x, SEXP i, SEXP j, const char *cl)
 {
 	SUB2_START_EXTRA('C', 'C', 'C', 0);
 
@@ -1651,8 +1668,8 @@ static SEXP CsparseMatrix_subscript_2ary(SEXP x, SEXP i, SEXP j,
 	return res;
 }
 
-static SEXP RsparseMatrix_subscript_2ary(SEXP x, SEXP i, SEXP j,
-                                         const char *cl)
+static
+SEXP RsparseMatrix_subscript_2ary(SEXP x, SEXP i, SEXP j, const char *cl)
 {
 	SUB2_START_EXTRA('R', 'R', 'R', 0);
 
@@ -1799,8 +1816,8 @@ static SEXP RsparseMatrix_subscript_2ary(SEXP x, SEXP i, SEXP j,
 	return res;
 }
 
-static SEXP diagonalMatrix_subscript_2ary(SEXP x, SEXP i, SEXP j,
-                                          const char *cl)
+static
+SEXP diagonalMatrix_subscript_2ary(SEXP x, SEXP i, SEXP j, const char *cl)
 {
 	SUB2_START;
 
@@ -1813,12 +1830,13 @@ static SEXP diagonalMatrix_subscript_2ary(SEXP x, SEXP i, SEXP j,
 	cl_[0] = cl[0];
 	if (!(mi || mj) && ni == nj) {
 		keep = keep_di(pi, pj, ni, nonunit, 0, m);
-		if (keep) {
+		if (keep > 0) {
 			cl_[1] = 'd';
 			cl_[2] = 'i';
 		}
 	}
-	SEXP res = PROTECT(NEW_OBJECT_OF_CLASS(cl_));
+
+	SEXP res = PROTECT(newObject(cl_));
 
 	PROTECT(dim = GET_SLOT(res, Matrix_DimSym));
 	pdim = INTEGER(dim);
@@ -1833,7 +1851,7 @@ static SEXP diagonalMatrix_subscript_2ary(SEXP x, SEXP i, SEXP j,
 		SET_STRING_ELT(diag, 0, diag_);
 		UNPROTECT(2); /* diag_, diag */
 
-	} else if (keep) {
+	} else if (keep > 0) {
 
 		SEXP x0 = PROTECT(GET_SLOT(x, Matrix_xSym)),
 			x1 = PROTECT(allocVector(TYPEOF(x0), ni));
@@ -1858,31 +1876,73 @@ static SEXP diagonalMatrix_subscript_2ary(SEXP x, SEXP i, SEXP j,
 
 	} else {
 
+		SEXP x0 = PROTECT(GET_SLOT(x, Matrix_xSym));
+		char *work = NULL;
+		int j_;
+
+		if (nonunit) {
+
+			Matrix_Calloc(work, n, char);
+
+#define SUB2_WORK(_CTYPE_, _PTR_, _ISNZ_) \
+			do { \
+				_CTYPE_ *px0 = _PTR_(x0); \
+				for (j_ = 0; j_ < n; ++j_) \
+					work[j_] = _ISNZ_(px0[j_]); \
+			} while (0)
+
+			switch (cl[0]) {
+			case 'n':
+				SUB2_WORK(int, LOGICAL, ISNZ_PATTERN);
+				break;
+			case 'l':
+				SUB2_WORK(int, LOGICAL, ISNZ_LOGICAL);
+				break;
+			case 'i':
+				SUB2_WORK(int, INTEGER, ISNZ_INTEGER);
+				break;
+			case 'd':
+				SUB2_WORK(double, REAL, ISNZ_REAL);
+				break;
+			case 'z':
+				SUB2_WORK(Rcomplex, COMPLEX, ISNZ_COMPLEX);
+				break;
+			default:
+				break;
+			}
+
+#undef SUB2_WORK
+
+		}
+
 		SEXP p1 = PROTECT(allocVector(INTSXP, (R_xlen_t) nj + 1));
-		int *pp1 = INTEGER(p1), j_;
+		int *pp1 = INTEGER(p1);
 		*(pp1++) = 0;
 
 		for (kj = 0; kj < nj; ++kj) {
 			pp1[kj] = 0;
 			j_ = (mj) ? kj : pj[kj] - 1;
-			if (mi) {
-				for (ki = 0; ki < ni; ++ki)
-					if (ki == j_)
-						++pp1[kj];
-			} else {
-				for (ki = 0; ki < ni; ++ki)
-					if (pi[ki] - 1 == j_)
-						++pp1[kj];
+			if (!nonunit || work[j_]) {
+				if (mi) {
+					for (ki = 0; ki < ni; ++ki)
+						if (ki == j_)
+							++pp1[kj];
+				} else {
+					for (ki = 0; ki < ni; ++ki)
+						if (pi[ki] - 1 == j_)
+							++pp1[kj];
+				}
+				if (pp1[kj] > INT_MAX - pp1[kj - 1]) {
+					if (nonunit)
+						Matrix_Free(work, n);
+					error(_("%s too dense for %s; would have more than %s nonzero entries"),
+					      "x[i,j]", "[CR]sparseMatrix", "2^31-1");
+				}
 			}
-			if (pp1[kj] > INT_MAX - pp1[kj - 1])
-				error(_("%s too dense for %s; would have more than %s nonzero entries"), \
-				      "x[i,j]", "[CR]sparseMatrix", "2^31-1"); \
-
 			pp1[kj] += pp1[kj-1];
 		}
 
 		SEXP i1 = PROTECT(allocVector(INTSXP, pp1[nj - 1])),
-			x0 = PROTECT(GET_SLOT(x, Matrix_xSym)),
 			x1 = PROTECT(allocVector(TYPEOF(x0), pp1[nj - 1]));
 		int *pi1 = INTEGER(i1);
 
@@ -1891,10 +1951,12 @@ static SEXP diagonalMatrix_subscript_2ary(SEXP x, SEXP i, SEXP j,
 			_CTYPE_ *px0 = _PTR_(x0), *px1 = _PTR_(x1); \
 			for (kj = 0; kj < nj; ++kj) { \
 				j_ = (mj) ? kj : pj[kj] - 1; \
-				for (ki = 0; ki < ni; ++ki) { \
-					if (((mi) ? ki : pi[ki] - 1) == j_) { \
-						*(pi1++) = ki; \
-						*(px1++) = (nonunit) ? px0[j_] : _ONE_; \
+				if (!nonunit || work[j_]) { \
+					for (ki = 0; ki < ni; ++ki) { \
+						if (((mi) ? ki : pi[ki] - 1) == j_) { \
+							*(pi1++) = ki; \
+							*(px1++) = (nonunit) ? px0[j_] : _ONE_; \
+						} \
 					} \
 				} \
 			} \
@@ -1909,14 +1971,17 @@ static SEXP diagonalMatrix_subscript_2ary(SEXP x, SEXP i, SEXP j,
 		SET_SLOT(res, Matrix_xSym, x1);
 		UNPROTECT(4); /* x1, x0, i1, p1 */
 
+		if (nonunit)
+			Matrix_Free(work, n);
+
 	}
 
 	UNPROTECT(1); /* res */
 	return res;
 }
 
-static SEXP indMatrix_subscript_2ary(SEXP x, SEXP i, SEXP j,
-                                     const char *cl)
+static
+SEXP indMatrix_subscript_2ary(SEXP x, SEXP i, SEXP j, const char *cl)
 {
 	PROTECT_INDEX pidA;
 	PROTECT_WITH_INDEX(x, &pidA);
@@ -1965,7 +2030,7 @@ static SEXP indMatrix_subscript_2ary(SEXP x, SEXP i, SEXP j,
 			Matrix_Free(work, m);
 		}
 
-		x = NEW_OBJECT_OF_CLASS((isP) ? "pMatrix" : "indMatrix");
+		x = newObject((isP) ? "pMatrix" : "indMatrix");
 		REPROTECT(x, pidA);
 
 		PROTECT(dim = GET_SLOT(x, Matrix_DimSym));
@@ -2009,7 +2074,7 @@ static SEXP indMatrix_subscript_2ary(SEXP x, SEXP i, SEXP j,
 			Matrix_Free(work, nj);
 		}
 
-		x = NEW_OBJECT_OF_CLASS((isP)
+		x = newObject((isP)
 		                        ? "pMatrix"
 		                        : ((!mg) ? "ngCMatrix" : "ngRMatrix"));
 		REPROTECT(x, pidA);
