@@ -1,4 +1,3 @@
-/* $Id: survfit4.c 11080 2008-10-24 03:47:51Z therneau $ */
 /*
 ** C routine to do a small computation that is hard in Splus
 **
@@ -16,8 +15,9 @@
 */
 
 #include "survS.h"
+#include "survproto.h"
 
-void survfit4(Sint *n,	Sint *dd,  double *x1,  double *x2) {
+void survfit4(int *n,	int *dd,  double *x1,  double *x2) {
     double temp, temp1, temp2;
     int i,j;
     double d;
