@@ -1,7 +1,7 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
  *  Copyright (C) 1995, 1996  Robert Gentleman and Ross Ihaka
- *  Copyright (C) 1997--2020  The R Core Team
+ *  Copyright (C) 1997--2023  The R Core Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -48,7 +48,7 @@
 #include "Startup.h"
 #include <R_ext/Riconv.h>
 #include <R_ext/Print.h> // for REprintf
-#include <R_ext/RS.h> // for Calloc
+#include <R_ext/RS.h> // for R_Calloc
 
 #define __SYSTEM__
 /* includes <sys/select.h> and <sys/time.h> */
@@ -67,7 +67,7 @@ extern FILE* ifp; /* from system.c */
  *  1) FATAL MESSAGES AT STARTUP
  */
 
-void attribute_hidden Rstd_Suicide(const char *s)
+attribute_hidden void Rstd_Suicide(const char *s)
 {
     REprintf("Fatal error: %s\n", s);
     /* Might be called before translation is running */
@@ -102,11 +102,11 @@ void attribute_hidden Rstd_Suicide(const char *s)
 
 static SIGJMP_BUF seljmpbuf;
 
-static RETSIGTYPE (*oldSigintHandler)(int) = SIG_DFL;
+static void (*oldSigintHandler)(int) = SIG_DFL;
 
 typedef void (*sel_intr_handler_t)(void);
 
-static RETSIGTYPE NORET handleSelectInterrupt(int dummy)
+NORET static void handleSelectInterrupt(int dummy)
 {
     signal(SIGINT, oldSigintHandler);
     SIGLONGJMP(seljmpbuf, 1);
@@ -138,8 +138,8 @@ int R_SelectEx(int  n,  fd_set  *readfds,  fd_set  *writefds,
 	    myintr();
 
 	    if (timeout != NULL) {
-		/* Ajdust timeout for elapsed complete seconds; ignore
-		   microseconde for now. This modifies the data pointed to
+		/* Adjust timeout for elapsed complete seconds; ignore
+		   microseconds for now. This modifies the data pointed to
 		   by timeval, which is what select() on Linux does as
 		   well. */
 		double new_time = currentTime();
@@ -163,16 +163,15 @@ int R_SelectEx(int  n,  fd_set  *readfds,  fd_set  *writefds,
 	       context. */
 	    R_interrupts_suspended = FALSE;
 
+	    /* check for and handle any pending interrupt registered
+	       by the standard handler. */
+	    if (R_interrupts_pending)
+		myintr();
+
 	    /* install a temporary signal handler for breaking out of
 	       a blocking select */
 	    oldSigintHandler = signal(SIGINT, handleSelectInterrupt);
 
-	    /* once the new sinal handler is in place we need to check
-	       for and handle any pending interrupt registered by the
-	       standard handler. */
-	    if (R_interrupts_pending)
-		myintr();
-
 	    /* now do the (possibly blocking) select, restore the
 	       signal handler, and return the result of the select. */
 	    val = select(n, readfds, writefds, exceptfds, timeout);
@@ -227,7 +226,7 @@ addInputHandler(InputHandler *handlers, int fd, InputHandlerProc handler,
 {
     InputHandler *input, *tmp;
 //    input = (InputHandler*) calloc(1, sizeof(InputHandler));
-    input = Calloc(1, InputHandler);
+    input = R_Calloc(1, InputHandler);
 
     input->activity = activity;
     input->fileDescriptor = fd;
@@ -268,7 +267,7 @@ removeInputHandler(InputHandler **handlers, InputHandler *it)
 
     if(*handlers == it) {
 	*handlers = (*handlers)->next;
-	Free(it); // use Free to match allocation with Calloc
+	R_Free(it); // use R_Free to match allocation with R_Calloc
 	return(1);
     }
 
@@ -277,7 +276,7 @@ removeInputHandler(InputHandler **handlers, InputHandler *it)
     while(tmp) {
 	if(tmp->next == it) {
 	    tmp->next = it->next;
-	    Free(it); // use Free to match allocation with Calloc
+	    R_Free(it); // use R_Free to match allocation with R_Calloc
 	    return(1);
 	}
 	tmp = tmp->next;
@@ -477,8 +476,10 @@ getSelectedHandler(InputHandler *handlers, fd_set *readMask)
 
 # include <readline/readline.h>
 
-/* For compatibility with pre-readline-4.2 systems, 
-   also missing in Apple's emulation via the NetBSD editline library.*/
+/* For compatibility with pre-readline-4.2 systems, also missing in
+   Apple's emulation via the NetBSD editline library, aka libedit.
+   _RL_FUNCTION_TYPEDEF is not currently defined anywhere.
+*/
 # if !defined (_RL_FUNCTION_TYPEDEF)
 typedef void rl_vcpfunc_t (char *);
 # endif /* _RL_FUNCTION_TYPEDEF */
@@ -496,14 +497,14 @@ typedef void rl_vcpfunc_t (char *);
 attribute_hidden
 char *R_ExpandFileName_readline(const char *s, char *buff)
 {
-#if defined(__APPLE__)
-    char *s2 = tilde_expand_word((char *)s);
-#else
     char *s2 = tilde_expand_word(s);
-#endif
+    size_t len = strlen(s2);
 
-    strncpy(buff, s2, PATH_MAX);
-    if(strlen(s2) >= PATH_MAX) buff[PATH_MAX-1] = '\0';
+    strncpy(buff, s2, R_PATH_MAX);
+    if(len >= R_PATH_MAX) {
+	buff[R_PATH_MAX-1] = '\0';
+	warning(_("expanded path length %d would be too long for\n%s\n"), len, s);
+    }
     free(s2);
     return buff;
 }
@@ -519,8 +520,8 @@ char *R_ExpandFileName_readline(const char *s, char *buff)
 
 /*
 
-There has been a general problem with asynchonous calls to browser and
-anything that uses the standard console reading facilties asynchronously
+There has been a general problem with asynchronous calls to browser and
+anything that uses the standard console reading facilities asynchronously
 (e.g. scan(), parse(), menu()).  The basic problem is as follows.  We
 are in the usual input loop awaiting characters typed by the user.  Then
 asynchronously, we enter the browser due to a callback that is invoked
@@ -571,7 +572,7 @@ static struct {
 #ifdef NEED_INT_HANDLER
 static volatile Rboolean caught_sigwinch = FALSE;
 
-static RETSIGTYPE
+static void
 R_readline_sigwinch_handler(int sig)
 {
     caught_sigwinch = TRUE;
@@ -659,6 +660,15 @@ static void readline_handler(char *line)
 {
     R_size_t buflen = rl_top->readline_len;
 
+#if defined(RL_READLINE_VERSION) && RL_READLINE_VERSION >= 0x0802
+    /* In this version of readline, rl_redisplay called from
+       popReadline -> rl_free_line_state (-> rl_clear_message -> rl_redisplay)
+       wipes the prompt and moves the cursor accordingly, causing
+       undesirable indentation here when the input is empty, but the
+       cursor has been moved to the next line.
+       As a work-around, clear the prompt. */
+    if (line && !line[0]) rl_set_prompt("");
+#endif
     popReadline();
 
     if ((rl_top->readline_eof = !line)) /* Yes, I don't mean ==...*/
@@ -796,7 +806,7 @@ static void initialize_rlcompletion(void)
     /* Tell the completer that we want a crack first. */
     rl_attempted_completion_function = R_custom_completion;
 
-// This was added in readline 6.0
+// This was added in readline 6.0: default is 1 (and was in earlier versions)
 #ifdef HAVE_RL_SORT_COMPLETION_MATCHES
     rl_sort_completion_matches = 0;
 #endif
@@ -891,8 +901,7 @@ static char *R_completion_generator(const char *text, int state)
        completeToken(), and retrieving the completions. */
 
     if (!state) {
-	int i;
-	SEXP completions,
+	SEXP
 	    assignCall = PROTECT(lang2(RComp_assignTokenSym, mkString(text))),
 	    completionCall = PROTECT(lang1(RComp_completeTokenSym)),
 	    retrieveCall = PROTECT(lang1(RComp_retrieveCompsSym));
@@ -900,7 +909,7 @@ static char *R_completion_generator(const char *text, int state)
 
 	eval(assignCall, rcompgen_rho);
 	eval(completionCall, rcompgen_rho);
-	PROTECT(completions = eval(retrieveCall, rcompgen_rho));
+	SEXP completions = PROTECT(eval(retrieveCall, rcompgen_rho));
 	list_index = 0;
 	ncomp = length(completions);
 	if (ncomp > 0) {
@@ -909,8 +918,16 @@ static char *R_completion_generator(const char *text, int state)
 		UNPROTECT(4);
 		return (char *)NULL;
 	    }
-	    for (i = 0; i < ncomp; i++)
-		compstrings[i] = strdup(translateChar(STRING_ELT(completions, i)));
+	    for (int i = 0; i < ncomp; i++) {
+		compstrings[i] =
+		    strdup(translateChar(STRING_ELT(completions, i)));
+		if (!compstrings[i]) {
+		    UNPROTECT(4);
+		    for (int j = 0; j < i; j++) free(compstrings[j]);
+		    free(compstrings);
+		    return (char *)NULL;
+		}
+	    }
 	}
 	UNPROTECT(4);
 	vmaxset(vmax);
@@ -945,7 +962,7 @@ handleInterrupt(void)
 /* Fill a text buffer from stdin or with user typed console input. */
 static void *cd = NULL;
 
-int attribute_hidden
+attribute_hidden int
 Rstd_ReadConsole(const char *prompt, unsigned char *buf, int len,
 		 int addtohistory)
 {
@@ -1094,7 +1111,7 @@ Rstd_ReadConsole(const char *prompt, unsigned char *buf, int len,
 	/* Write a text buffer to the console. */
 	/* All system output is filtered through this routine (unless R_Consolefile is used). */
 
-void attribute_hidden Rstd_WriteConsole(const char *buf, int len)
+attribute_hidden void Rstd_WriteConsole(const char *buf, int len)
 {
     printf("%s", buf);
     fflush(stdout);
@@ -1102,7 +1119,7 @@ void attribute_hidden Rstd_WriteConsole(const char *buf, int len)
 
 /* The extended version allows the distinction of errors and warnings.
    It is not enabled by default unless pretty-printing is desired. */
-void attribute_hidden Rstd_WriteConsoleEx(const char *buf, int len, int otype)
+attribute_hidden void Rstd_WriteConsoleEx(const char *buf, int len, int otype)
 {
     if (otype)
       printf("\033[1m%s\033[0m", buf);
@@ -1114,21 +1131,21 @@ void attribute_hidden Rstd_WriteConsoleEx(const char *buf, int len, int otype)
 
 	/* Indicate that input is coming from the console */
 
-void attribute_hidden Rstd_ResetConsole()
+attribute_hidden void Rstd_ResetConsole(void)
 {
 }
 
 
 	/* Stdio support to ensure the console file buffer is flushed */
 
-void attribute_hidden Rstd_FlushConsole()
+attribute_hidden void Rstd_FlushConsole(void)
 {
     /* fflush(stdin);  really work on Solaris on pipes */
 }
 
 	/* Reset stdin if the user types EOF on the console. */
 
-void attribute_hidden Rstd_ClearerrConsole()
+attribute_hidden void Rstd_ClearerrConsole(void)
 {
     clearerr(stdin);
 }
@@ -1137,7 +1154,7 @@ void attribute_hidden Rstd_ClearerrConsole()
  *  3) ACTIONS DURING (LONG) COMPUTATIONS
  */
 
-void attribute_hidden Rstd_Busy(int which)
+attribute_hidden void Rstd_Busy(int which)
 {
 }
 
@@ -1156,26 +1173,8 @@ void attribute_hidden Rstd_Busy(int which)
    If ask = SA_SUICIDE, no save, no .Last, possibly other things.
  */
 
-
-
-void R_CleanTempDir(void)
-{
-    char buf[1024];
-
-    if((Sys_TempDir)) {
-// Only __sun is neeed on Solaris >= 10 (2005).
-#if defined(__sun) || defined(sun)
-	/* On Solaris the working directory must be outside this one */
-	chdir(R_HomeDir());
-#endif
-	snprintf(buf, 1024, "rm -Rf %s", Sys_TempDir);
-	buf[1023] = '\0';
-	R_system(buf);
-    }
-}
-
-
-void attribute_hidden NORET Rstd_CleanUp(SA_TYPE saveact, int status, int runLast)
+attribute_hidden NORET
+void Rstd_CleanUp(SA_TYPE saveact, int status, int runLast)
 {
     if(saveact == SA_DEFAULT) /* The normal case apart from R_Suicide */
 	saveact = SaveAction;
@@ -1253,7 +1252,7 @@ void attribute_hidden NORET Rstd_CleanUp(SA_TYPE saveact, int status, int runLas
 
 # include <errno.h>
 
-int attribute_hidden
+attribute_hidden int
 Rstd_ShowFiles(int nfile,		/* number of files */
 	       const char **file,		/* array of filenames */
 	       const char **headers,	/* the `headers' args of file.show.
@@ -1320,8 +1319,9 @@ Rstd_ShowFiles(int nfile,		/* number of files */
     */
 
 
+#include <ctype.h>  /* for isspace */
 
-int attribute_hidden Rstd_ChooseFile(int _new, char *buf, int len)
+attribute_hidden int Rstd_ChooseFile(int _new, char *buf, int len)
 {
     size_t namelen;
     char *bufp;
@@ -1334,13 +1334,13 @@ int attribute_hidden Rstd_ChooseFile(int _new, char *buf, int len)
 }
 
 
-void attribute_hidden Rstd_ShowMessage(const char *s)
+attribute_hidden void Rstd_ShowMessage(const char *s)
 {
     REprintf("%s\n", s);
 }
 
 
-void attribute_hidden Rstd_read_history(const char *s)
+attribute_hidden void Rstd_read_history(const char *s)
 {
 #if defined(HAVE_LIBREADLINE) && defined(HAVE_READLINE_HISTORY_H)
     if(R_Interactive && UsingReadline) {
@@ -1349,17 +1349,17 @@ void attribute_hidden Rstd_read_history(const char *s)
 #endif
 }
 
-void attribute_hidden Rstd_loadhistory(SEXP call, SEXP op, SEXP args, SEXP env)
+attribute_hidden void Rstd_loadhistory(SEXP call, SEXP op, SEXP args, SEXP env)
 {
     SEXP sfile;
-    char file[PATH_MAX];
+    char file[R_PATH_MAX];
     const char *p;
 
     sfile = CAR(args);
     if (!isString(sfile) || LENGTH(sfile) < 1)
 	errorcall(call, _("invalid '%s' argument"), "file");
     p = R_ExpandFileName(translateCharFP(STRING_ELT(sfile, 0)));
-    if(strlen(p) > PATH_MAX - 1)
+    if(strlen(p) > R_PATH_MAX - 1)
 	errorcall(call, _("'file' argument is too long"));
     strcpy(file, p);
 #if defined(HAVE_LIBREADLINE) && defined(HAVE_READLINE_HISTORY_H)
@@ -1372,17 +1372,17 @@ void attribute_hidden Rstd_loadhistory(SEXP call, SEXP op, SEXP args, SEXP env)
 #endif
 }
 
-void attribute_hidden Rstd_savehistory(SEXP call, SEXP op, SEXP args, SEXP env)
+attribute_hidden void Rstd_savehistory(SEXP call, SEXP op, SEXP args, SEXP env)
 {
     SEXP sfile;
-    char file[PATH_MAX];
+    char file[R_PATH_MAX];
     const char *p;
 
     sfile = CAR(args);
     if (!isString(sfile) || LENGTH(sfile) < 1)
 	errorcall(call, _("invalid '%s' argument"), "file");
     p = R_ExpandFileName(translateCharFP(STRING_ELT(sfile, 0)));
-    if(strlen(p) > PATH_MAX - 1)
+    if(strlen(p) > R_PATH_MAX - 1)
 	errorcall(call, _("'file' argument is too long"));
     strcpy(file, p);
 #if defined(HAVE_LIBREADLINE) && defined(HAVE_READLINE_HISTORY_H)
@@ -1403,7 +1403,7 @@ void attribute_hidden Rstd_savehistory(SEXP call, SEXP op, SEXP args, SEXP env)
 #endif
 }
 
-void attribute_hidden Rstd_addhistory(SEXP call, SEXP op, SEXP args, SEXP env)
+attribute_hidden void Rstd_addhistory(SEXP call, SEXP op, SEXP args, SEXP env)
 {
     SEXP stamp;
     int i;
