@@ -1,7 +1,7 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
  *  Copyright (C) 1995, 1996  Robert Gentleman and Ross Ihaka
- *  Copyright (C) 1997--2020  The R Core Team
+ *  Copyright (C) 1997--2024  The R Core Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -67,7 +67,7 @@
 attribute_hidden
 FILE *R_OpenInitFile(void)
 {
-    char buf[PATH_MAX], *home, *p = getenv("R_PROFILE_USER");
+    char buf[R_PATH_MAX], *home, *p = getenv("R_PROFILE_USER");
     FILE *fp;
 
     fp = NULL;
@@ -80,7 +80,7 @@ FILE *R_OpenInitFile(void)
 	    return fp;
 	if((home = getenv("HOME")) == NULL)
 	    return NULL;
-	snprintf(buf, PATH_MAX, "%s/.Rprofile", home);
+	snprintf(buf, R_PATH_MAX, "%s/.Rprofile", home);
 	if((fp = R_fopen(buf, "r")))
 	    return fp;
     }
@@ -116,7 +116,7 @@ static const char *R_ExpandFileName_unix(const char *s, char *buff)
     if(s[0] != '~') return s;
 
     const char *user, *temp, *s2, *home;
-    char buff2[PATH_MAX];
+    char buff2[R_PATH_MAX];
     struct passwd *pass;
 
     temp = strchr(s + 1, '/');
@@ -154,9 +154,16 @@ static const char *R_ExpandFileName_unix(const char *s, char *buff)
     if (temp == NULL) { // ~name
 	strcpy(buff, home);
     } else { // ~name/path
-	size_t len = strlen(home) + 1 + strlen(s2) + 1;
-	if (len >= PATH_MAX) return s;
-	(void)snprintf(buff, len, "%s/%s", home, s2);
+	// ask snprintf to compute the length, as GCC 12 complains otherwise.
+	size_t len = snprintf(NULL, 0, "%s/%s", home, s2);
+	// buff is passed from R_ExpandFileName, uses static array of
+	// size R_PATH_MAX.
+	if (len >= R_PATH_MAX) {
+	    warning(_("expanded path length %d would be too long for\n%s\n"),
+		       len, s);
+	    return s;
+	}
+	(void)snprintf(buff, len + 1,  "%s/%s", home, s2);
     }
 
     return buff;
@@ -174,7 +181,7 @@ static const char *R_ExpandFileName_unix(const char *s, char *buff)
 */
 
 extern Rboolean UsingReadline;
-static char newFileName[PATH_MAX];
+static char newFileName[R_PATH_MAX];
 
 const char *R_ExpandFileName(const char *s)
 {
@@ -194,7 +201,7 @@ const char *R_ExpandFileName(const char *s)
  *  7) PLATFORM DEPENDENT FUNCTIONS
  */
 
-SEXP attribute_hidden do_machine(SEXP call, SEXP op, SEXP args, SEXP env)
+attribute_hidden SEXP do_machine(SEXP call, SEXP op, SEXP args, SEXP env)
 {
     checkArity(op, args);
     return mkString("Unix");
@@ -331,7 +338,7 @@ static struct {
 } tost;
 
 static void timeout_handler(int sig);
-static void timeout_init()
+static void timeout_init(void)
 {
     tost.child_pid = 0;
     tost.timedout = 0;
@@ -373,7 +380,7 @@ static void timeout_cleanup_set(sigset_t *ss)
     sigaddset(ss, SIGCHLD);
 }
 
-static void timeout_cleanup()
+static void timeout_cleanup(void)
 {
     sigset_t ss;
     timeout_cleanup_set(&ss);
@@ -470,7 +477,7 @@ static void timeout_cend(void *data)
 /* Fork with blocked SIGCHLD to make sure that tost.child_pid is set
    in the parent before the signal is received. Also makes sure
    SIGCHLD is unblocked in the parent after the call. */
-static void timeout_fork()
+static void timeout_fork(void)
 {
     sigset_t css; 
     sigemptyset(&css);
@@ -572,13 +579,13 @@ int R_pclose_timeout(FILE *fp)
 	/* should not happen */
 	error("Invalid file pointer in pclose");
 
-    /* Do not use fclose, because on Solaris it sets errno to "Invalid seek"
-       when the pipe is already closed (e.g. because of timeout). fclose would
-       not return an error, but it would set errno and the non-zero errno would
-       then be reported by R's "system" function. */
-    int fd = fileno(fp);
-    if (fd >= 0)
-	close(fd);
+    int saveerrno = errno;
+    if (!fclose(fp))
+	/* On Solaris, fclose sets errno to "Invalid seek" when the pipe is
+           already closed (e.g.  because of timeout).  fclose would not
+           return an error, but it would set errno and the non-zero errno
+           would then be reported by R's "system" function.  */
+	errno = saveerrno;
 
     pid_t wres;
     int wstatus;
@@ -666,15 +673,37 @@ static void warn_status(const char *cmd, int res)
 	   an error here (CERT ERR30-C).*/
 	/* on Solaris, if the command ends with non-zero status and timeout
 	   is 0, "Illegal seek" error is reported; the timeout version
-	   works this around by using close(fileno) */
+	   works this around by restoring errno on success */
 	warning(_("running command '%s' had status %d and error message '%s'"),
 		cmd, res, strerror(errno));
     else
 	warning(_("running command '%s' had status %d"), cmd, res);
 }
 
+NORET static void cmdError(const char *cmd, const char *format, ...)
+{
+    SEXP call = R_CurrentExpression;
+    int nextra = errno ? 3 : 1;
+
+    va_list(ap);
+    va_start(ap, format);
+    SEXP cond = R_vmakeErrorCondition(call, "cmdError", NULL,
+				      nextra, format, ap);
+    va_end(ap);
+
+    PROTECT(cond);
+    R_setConditionField(cond, 2, "cmd", mkString(cmd));
+    if (errno) {
+	R_setConditionField(cond, 3, "errno", ScalarInteger(errno));
+	R_setConditionField(cond, 4, "error", mkString(strerror(errno)));
+    }
+
+    R_signalErrorCondition(cond, call);
+    UNPROTECT(1); /* cond; not reached */
+}
+
 #define INTERN_BUFSIZE 8096
-SEXP attribute_hidden do_system(SEXP call, SEXP op, SEXP args, SEXP rho)
+attribute_hidden SEXP do_system(SEXP call, SEXP op, SEXP args, SEXP rho)
 {
     SEXP tlist = R_NilValue;
     int intern = 0;
@@ -730,8 +759,8 @@ SEXP attribute_hidden do_system(SEXP call, SEXP op, SEXP args, SEXP rho)
 	else
 	    fp = R_popen_timeout(cmd, x, timeout);
 	if(!fp)
-	    error(_("cannot popen '%s', probable reason '%s'"),
-		  cmd, strerror(errno));
+	    cmdError(cmd, _("cannot popen '%s', probable reason '%s'"),
+		     cmd, strerror(errno));
 #ifdef HAVE_GETLINE
         size_t read;
         for(i = 0; (read = getline(&buf, &buf_len, fp)) != (size_t)-1; i++) {
@@ -790,9 +819,10 @@ SEXP attribute_hidden do_system(SEXP call, SEXP op, SEXP args, SEXP rho)
 #endif
 	if ((res & 0xff)  == 127) {/* 127, aka -1 */
 	    if (errno)
-		error(_("error in running command: '%s'"), strerror(errno));
+		cmdError(cmd, _("error in running command: '%s'"),
+			 strerror(errno));
 	    else
-		error(_("error in running command"));
+		cmdError(cmd, _("error in running command"));
 	}
 
 	if (timeout && tost.timedout) {
@@ -855,7 +885,7 @@ SEXP attribute_hidden do_system(SEXP call, SEXP op, SEXP args, SEXP rho)
 #  include <pwd.h>
 # endif
 
-SEXP attribute_hidden do_sysinfo(SEXP call, SEXP op, SEXP args, SEXP rho)
+attribute_hidden SEXP do_sysinfo(SEXP call, SEXP op, SEXP args, SEXP rho)
 {
     SEXP ans, ansnames;
     struct utsname name;
@@ -931,27 +961,8 @@ void R_ProcessEvents(void)
     if (ptr_R_ProcessEvents) ptr_R_ProcessEvents();
 #endif
     R_PolledEvents();
-    if (cpuLimit > 0.0 || elapsedLimit > 0.0) {
-	double cpu, data[5];
-	R_getProcTime(data);
-	cpu = data[0] + data[1] + data[3] + data[4];
-	if (elapsedLimit > 0.0 && data[2] > elapsedLimit) {
-	    cpuLimit = elapsedLimit = -1;
-	    if (elapsedLimit2 > 0.0 && data[2] > elapsedLimit2) {
-		elapsedLimit2 = -1.0;
-		error(_("reached session elapsed time limit"));
-	    } else
-		error(_("reached elapsed time limit"));
-	}
-	if (cpuLimit > 0.0 && cpu > cpuLimit) {
-	    cpuLimit = elapsedLimit = -1;
-	    if (cpuLimit2 > 0.0 && cpu > cpuLimit2) {
-		cpuLimit2 = -1.0;
-		error(_("reached session CPU time limit"));
-	    } else
-		error(_("reached CPU time limit"));
-	}
-    }
+    if (cpuLimit > 0.0 || elapsedLimit > 0.0)
+	R_CheckTimeLimits();
 }
 
 
@@ -990,17 +1001,17 @@ void fpu_setup(Rboolean start)
 #if defined(__ARM_ARCH) && defined(__ARM_32BIT_STATE) && defined(__ARM_FP)
     uint32_t fpscr;
 
-    asm volatile("vmrs %0, fpscr" : "=r"(fpscr));
+    __asm__ volatile("vmrs %0, fpscr" : "=r"(fpscr));
     /* clear/disable DN (default NaN) and FZ (flush to zero) bits */
     fpscr = fpscr & 0xfcffffff;
-    asm volatile("vmsr fpscr, %0" : : "r"(fpscr));
+    __asm__ volatile("vmsr fpscr, %0" : : "r"(fpscr));
 #elif defined(__ARM_ARCH) && defined(__ARM_64BIT_STATE) && defined(__ARM_FP)
     uint64_t fpcr;
 
-    asm volatile("mrs %0, fpcr" : "=r"(fpcr));
+    __asm__ volatile("mrs %0, fpcr" : "=r"(fpcr));
     /* clear/disable DN (default NaN) and FZ (flush to zero) bits */
     fpcr = fpcr & 0xfffffffffcffffff;
-    asm volatile("msr fpcr, %0" : : "r"(fpcr));
+    __asm__ volatile("msr fpcr, %0" : : "r"(fpcr));
 #endif
     } else {
 #ifdef __FreeBSD__
