@@ -1,7 +1,7 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
  *  Copyright (C) 1995, 1996  Robert Gentleman and Ross Ihaka
- *  Copyright (C) 1997--2020  The R Core Team
+ *  Copyright (C) 1997--2023  The R Core Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -66,7 +66,7 @@ Rboolean UsingReadline = TRUE;  /* used in sys-std.c & ../main/platform.c
 
 /* call pointers to allow interface switching */
 
-void NORET R_Suicide(const char *s) {
+NORET void R_Suicide(const char *s) {
     ptr_R_Suicide(s);
     // This should not have returned, but belt-and-braces
     exit(2); // same status as Rstd_Suicide
@@ -82,7 +82,7 @@ void R_FlushConsole(void) { ptr_R_FlushConsole(); }
 #endif
 void R_ClearerrConsole(void) { ptr_R_ClearerrConsole(); }
 void R_Busy(int which) { ptr_R_Busy(which); }
-void NORET R_CleanUp(SA_TYPE saveact, int status, int runLast)
+NORET void R_CleanUp(SA_TYPE saveact, int status, int runLast)
 {
     ptr_R_CleanUp(saveact, status, runLast);
     // This should not have returned, but belt-and-braces
@@ -116,7 +116,7 @@ void R_FlushConsole(void) {
 #endif
 
 
-void R_setupHistory()
+void R_setupHistory(void)
 {
     int value, ierr;
     char *p;
@@ -170,6 +170,9 @@ static char* unescape_arg(char *p, char* avp) {
 	} else if(*q == '~' && *(q+1) == 'n' && *(q+2) == '~') {
 	    q += 2;
 	    *p++ = '\n';
+	} else if(*q == '~' && *(q+1) == 't' && *(q+2) == '~') {
+	    q += 2;
+	    *p++ = '\t';
 	} else *p++ = *q;
     }
     return p;
@@ -333,7 +336,7 @@ int Rf_initialize_R(int ac, char **av)
     process_system_Renviron();
 
     R_setStartTime();
-    R_DefParams(Rp);
+    R_DefParamsEx(Rp, RSTART_VERSION);
     /* Store the command line arguments before they are processed
        by the R option handler.
      */
@@ -409,7 +412,12 @@ int Rf_initialize_R(int ac, char **av)
 #define R_INIT_TREAT_F(_AV_)						\
 		Rp->R_Interactive = FALSE;				\
 		if(strcmp(_AV_, "-")) {					\
-		    char path[PATH_MAX], *p = path;			\
+		    if(strlen(_AV_) >= R_PATH_MAX) {			\
+			snprintf(msg, 1024,				\
+				 _("path given in -f/--file is too long"));	\
+			R_Suicide(msg);					\
+		    }							\
+		    char path[R_PATH_MAX], *p = path;			\
 		    p = unescape_arg(p, _AV_);				\
 		    *p = '\0';						\
 		    ifp = R_fopen(path, "r");				\
@@ -459,7 +467,7 @@ int Rf_initialize_R(int ac, char **av)
     if(strlen(cmdlines)) { /* had at least one -e option */
 	size_t res;
 	char *tm;
-	static char ifile[PATH_MAX] = "\0";
+	static char ifile[R_PATH_MAX] = "\0";
 	int ifd;
 
 	if(ifp) R_Suicide(_("cannot use -e with -f or --file"));    
@@ -474,9 +482,9 @@ int Rf_initialize_R(int ac, char **av)
 		    tm = "/tmp";
 	    }
 	}
-	snprintf(ifile, PATH_MAX, "%s/Rscript%x.XXXXXX", tm, getpid());
+	snprintf(ifile, R_PATH_MAX, "%s/Rscript%x.XXXXXX", tm, getpid());
 	ifd = mkstemp(ifile);
-	if (ifd > 0)
+	if (ifd >= 0) /* -1 on error, can be 0 if stdin is closed */
 	    ifp = fdopen(ifd, "w+");
 	if(!ifp) R_Suicide(_("creating temporary file for '-e' failed"));
 	unlink(ifile);
@@ -583,7 +591,7 @@ int R_EditFiles(int nfile, const char **file, const char **title,
 
 /* Returns the limit on the number of open files. On error or when no
    limit is known, returns a negative number. */
-int R_GetFDLimit() {
+int R_GetFDLimit(void) {
 
 #if defined(HAVE_SYS_RESOURCE_H) && defined(HAVE_GETRLIMIT)
     struct rlimit rlim;
