@@ -9,7 +9,7 @@ options(contrasts=c('contr.treatment', 'contr.poly')) #ensure constrast type
 test2 <- data.frame(start=c(1, 2, 5, 2, 1, 7, 3, 4, 8, 8),
                     stop =c(2, 3, 6, 7, 8, 9, 9, 9,14,17),
                     event=c(1, 1, 1, 1, 1, 1, 1, 0, 0, 0),
-                    x    =c(1, 0, 0, 1, 0, 1, 1, 1, 0, 0) )
+                    x    =c(1, 0, 0, 1, 0, 1, 1, 1, 0, 0))
 
 byhand <- function(beta, newx=0) {
     r <- exp(beta)
@@ -83,7 +83,8 @@ fit1 <- coxph(Surv(start, stop, event) ~x, test2, iter=1, ties="breslow")
 aeq(beta1, coef(fit1))
 
 truth <- byhand(-0.084526081, 0)
-fit <- coxph(Surv(start, stop, event) ~x, test2, eps=1e-8, method='breslow')
+fit <- coxph(Surv(start, stop, event) ~x, test2, eps=1e-8, method='breslow',
+             nocenter= NULL)
 aeq(truth$loglik, fit$loglik[2])
 aeq(1/truth$imat, fit$var)
 aeq(truth$mart, fit$resid)
