@@ -109,7 +109,7 @@ except <- c("call", "switch", ".C", ".Fortran", ".Call", ".External",
             ".External2", ".Call.graphics", ".External.graphics",
             ".subset", ".subset2", ".primTrace", ".primUntrace",
             "lazyLoadDBfetch", ".Internal", ".Primitive", "^", "|",
-            "%*%", "rep", "seq.int", "forceAndCall",
+            "::", ":::", "%*%", "rep", "seq.int", "forceAndCall",
             ## these may not be enabled
             "tracemem", "retracemem", "untracemem")
 
@@ -141,6 +141,6 @@ for(f in ls(.ArgsEnv, all.names=TRUE))
     } else a <- list(zZ=NULL)
     res <- try(do.call(f, a), silent = TRUE)
     m <- geterrmessage()
-    if(!grepl('does not match|unused argument|requires 0|native symbol', m))
+    if(!grepl('does not match|unused argument|requires 0|native symbol|valid .* object', m))
         stop("failure on ", f)
 }
