@@ -139,7 +139,7 @@ if(availableGB > 60) withAutoprint({
     system.time(ii <- ii + 0) #  6.726  17.558  24.450 (slow!, seen faster)
     system.time(i2 <- ii[-n]) # 14.267  23.532  37.918 (slow!, seen slower: el.= 51)
     ##
-    ## NB: keep n, i, i2 for "below"
+    ## NB: keep n, ii, i2 for "below"
 })
 ## In R <= 3.4.1 :
 ## Program received signal SIGSEGV, Segmentation fault.
@@ -160,7 +160,7 @@ if(availableGB > 99) withAutoprint({
 	length(ap1$x) == 50
 	all.equal(ap1$y, sin(pi*ap1$x), tol= 1e-9)
     })
-    rm(ap1) # keep x,y,n,i2
+    rm(ap1) # keep x,y,n,ii,i2
     gc()     # --> max used: 92322 Mb
 })
 
@@ -320,7 +320,7 @@ mkDat <- function(n) {
     data.frame(x = x, y = sin(pi*x^2) * exp(-x/2) + rnorm(n)/8)
 }
 set.seed(1); dat <- mkDat(n = 42000)
-system.time( # 14.5 sec (on lynne ~ 2019)
+system.time( # 12.7 sec (lynne ~ 2021)
     fit <- loess(y~x, data=dat)
 )
 r <- tools::assertError(
@@ -336,9 +336,64 @@ system.time(x <- raw(i)) # ~ 0.8 sec ; needs 2 GB
 x [i]  <- r1 <- as.raw(1); stopifnot(x [i]  == r1)
 x[[i]] <- r2 <- as.raw(2); stopifnot(x[[i]] == r2)
 x[[i]] <- r3 <- as.raw(3); stopifnot(x[[i]] == r3)
-## failed in R <= 0.4.3 {even with large vectors}
+## last two failed in R <= 4.0.n {even with large vectors}
+
+
+## print()ing {up to max.print only!} of long vectors;
+## including named and "generic" (= list):
+stopifnot((n <- 2^31 + 352) > .Machine$integer.max)
+system.time(L <- integer(n))        #   5.8 sec {ada-20}
+system.time(LL <- vector("list", n))# ~15   sec {ada-20}
+system.time(nm <- c(LETTERS, letters, rep("xx", length(L) - 2*26)))
+## between 55 and 76 sec {ada-20, 2022-01-07}  user  system elapsed
+Ln <- L
+## FIXME? takes about 2 secs, but these are *not* seen by system.time (!!)
+system.time(names(Ln) <- nm)
+## user  system elapsed
+##    0       0       0
+op <- options(max.print = 300)
+L
+## now (after using %lld) gives
+## [ reached getOption("max.print") -- omitted 2147483700 entries ]
+## before, it gave   ..... -- omitted -2147483596 entries
+##                                   ^^^
+Ln # gave  Error: long vectors not supported yet: ...
+LL # gave  Error: long vect...
+options(op)
+rm(Ln, L)
+
+## PR#17977 --- x[<fractional>] behavior should fulfill x[i] === x[as.integer(i)]
+## large (no overflow in index computations!) -- needs 2 GB
+LL <- matrix(as.raw(1:2), 2, 2^30)
+ca.half <- 0.5+ (eps <- unique(sort(outer(2^-c(16, 21, 26, 30), -1:1))))
+print(eps, digits=3)
+LL[cbind(2, ca.half)]   # should be of length 0, too: ca.half ~= 0.5
+LL[cbind(1, 1+ca.half)] # should be constantly == raw(1L) '01'
+LL[cbind(2+ca.half, 1)] # all 02 -- failed in R <= 4.1.x
+LL[cbind(-ca.half, 1)]  # raw(0) --  "      "    "
+stopifnot(exprs = {
+    length(LL[cbind(2, ca.half)]) == 0
+    LL[cbind(1, 1+ca.half)] == as.raw(1L)
+    LL[cbind(2+ca.half, 1)] == as.raw(2L)
+    length(LL[cbind( -ca.half, 1)]) == 0
+})
+
+
+x <- -1:2^31 # (immediate: ALTREP)
+system.time( r <- rank(x) )# Error about invalid length() -- PR#18617, in R <= 4.3.2
+## seen 260 sec (!)
+head(r)
+stopifnot(r[1:6] == 1:6)
+rm(r,x)
+
+## rank(<largish) -- PR#18630
+vals <- 1:1475000000 # (immedate, thanks to ALTREP)
+system.time( ranks <- rank(vals) ) # ~ 130 sec
+head(ranks, 11)
+stopifnot(ranks[1:11] == 1:11, min(ranks) == 1)
+## min(ranks) was -1073741824, in R <= 4.3.2
 
 
 
 gc() # NB the "max used"
-proc.time() # total  [ ~ 40 minutes in full case, 2019-04-12]
+proc.time() # total  [ ~ 50 minutes in full case, 2024-01; was 40' in 2019-04-12]
