@@ -736,12 +736,12 @@ y <- pi + (-9:9)*2^-53
 z <- c(1:2,2:1) ; names(z) <- nz <- letters[seq_along(z)]
 of <- ordered(4:1)
 stopifnot(identical(factor(c(2, 1:2), labels = L),
-		    structure(c(2L, 1:2), .Label = L, class="factor")),
+		    structure(c(2L, 1:2), levels = L, class="factor")),
 	  identical(factor(x),
-		    structure(5:1, .Label = lx, class="factor")),
+		    structure(5:1, levels = lx, class="factor")),
 	  length(levels(factor(y))) == 1, length(unique(y)) == 5,
 	  identical(factor(z),
-		    structure(z, .Names = nz, .Label = c("1","2"),
+		    structure(z, names = nz, levels = c("1","2"),
 			      class="factor")),
 	  identical(of, factor(of)))
 ## partly failed in R <= 2.9.0, partly in R-devel(2.10.0)
@@ -1213,6 +1213,13 @@ stopifnot(nchar(char.dec) == nchar(char))
 ## short in R <= 2.11.0
 
 
+## right-to-left assignment/removal of columns (PR#14263)
+X <- data.frame(A1 = 1, A2 = 2, A3 = 3, A4 = 4)
+X[3:2] <- list(NULL)
+stopifnot(identical(names(X), c("A1", "A4")))
+## R <= 2.11.0 removed columns 2 and 4
+
+
 ## rbeta() with mass very close to 1 -- bug PR#14291
 set.seed(1)
 if(any(ii <- is.na(rbeta(5000, 100, 0.001))))
